@@ -17,23 +17,34 @@ then
 
   cd libjhead; make clean
   rm -r .ui .moc .obj ../../../lib/libjhead*
-  rm Makefile Makefile.interface core*
+  rm Makefile* Makefile core*
 
   cd jheadGui; make clean
   rm -r .ui .moc .obj jheadGui
-  rm Makefile core*
+  rm Makefile* core*
 
 else
+  CORES=`cat /proc/cpuinfo | grep processor | wc -l`
+  if [ $# -gt 0 ]; then
+    if [ $1 -eq $1 2> /dev/null ]; then
+      CORES=$1
+    fi;
+  fi;
 
   QMAKE=$QTDIR/bin/qmake
   if [ ! -e $QMAKE ]; then
     QMAKE=qmake
   fi;
+
   cd jhead/libjhead
-  $QMAKE && make
+  $QMAKE -o Makefile.tmp
+  sed "10 s,^,QTDIR = $QTDIR\n," Makefile.tmp > Makefile
+  make -j $CORES
 
   cd jheadGui
-  $QMAKE && make 
+  $QMAKE -o Makefile.tmp
+  sed "10 s,^,QTDIR = $QTDIR\n," Makefile.tmp > Makefile
+  make -j $CORES
 
 fi;
 
