@@ -230,7 +230,7 @@ void Configurator::slotCompile ( )
 		listEnv.append  ( "QT_LIB=qt" );
 
 	QString qsPath = getenv ( "PATH" );
-	qsPath = "PATH=" + qsQtDir + "/bin/:" + qsPath;
+	qsPath = "PATH=" + qsQtDir + "/bin/:" + qsPath + "/usr/local/bin/";
 	m_qsUsersPath = qsPath;
 	listEnv.append ( qsPath );
 	// This var will tell the qplayer.pro file to not to create qmake_image_collection ...
@@ -240,12 +240,13 @@ void Configurator::slotCompile ( )
 	textOutput->clear ( );
 	for ( QStringList::Iterator it = listEnv.begin ( ); it != listEnv.end ( ); ++it )  {
 		textOutput->append ( *it + QString ( "\n" ) );
+		putenv ( (char *)( (const char *)*it ) );
 	}
 
 	// let us set the env of the calling process rather then just
 	// create our own environment with only the environment we set.
-	for ( uint t=0; t<listEnv.count ( ); t++ )
-		putenv ( (char *)( (const char *)listEnv [t] ) );
+//	for ( uint t=0; t<listEnv.count ( ); t++ )
+//		putenv ( (char *)( (const char *)listEnv [t] ) );
 
 	// qdvdauthor.pro needs those files to physically be there ... Go figure !
 	iRet = system ( "touch ../bin/qdvdauthor ../bin/qslideshow ../bin/qplayer ../bin/qrender" );
@@ -253,6 +254,16 @@ void Configurator::slotCompile ( )
 	if ( hasMediaEngineSelectionChanged ( ) )
 		iRet = system ( "rm Makefile ../qrender/Makefile ../qdvdauthor/Makefile ../qdvdauthor/qplayer/Makefile ../qdvdauthor/qslideshow/Makefile" );
 	m_pNewProcess->setWorkingDirectory (QDir ("../"));
+
+	// Next we create the Makefiles
+	iRet = system ( QString ( "cd ../qdvdauthor; %1/bin/qmake qdvdauthor.pro; cd .." ).arg ( qsQtDir ) );
+	if ( m_pCheckQPlayer->isChecked ( ) )
+		iRet = system ( QString ( "cd ../qdvdauthor/qplayer; %1/bin/qmake qplayer.pro; cd ../.." ).arg ( qsQtDir ) );
+	if ( m_pCheckQSlideshow->isChecked ( ) ) 
+		iRet = system ( QString ( "cd ../qdvdauthor/qslideshow; %1/bin/qmake qslideshow.pro; cd ../.." ).arg ( qsQtDir ) );
+//	if ( ( m_pCheckQRender->isChecked ( ) ) && ( ! m_pCheckLocalFFMpeg->isChecked ( ) ) )
+//		iRet = system ( QString ( "cd ../qrender; %1/bin/qmake qrender.pro; cd .." ).arg ( editQtDir3->text ( ) ) );
+
 	// And here we generate the make file and compile the application.
 	m_pNewProcess->addArgument ( QString ( "%1/bin/qmake" ).arg ( qsQtDir ) );
 	m_pNewProcess->addArgument ( "all.pro" );
