@@ -1,8 +1,10 @@
 /****************************************************************************
 ** Interface to the GUI.
 **
-** Created: Mon Jun 10 22:48:00 2005
-**      by: Varol Okan using kate editor
+**   Created : Mon Jun 10 22:48:00 2005
+**        by : Varol Okan using kate editor
+** Copyright : (c) Varol Okan
+**   License : GPL v 2.0
 **
 **
 ****************************************************************************/
@@ -11,14 +13,16 @@
 #define ANIMATIONATTRIBUTE_H
 
 #include <qstring.h>
-#include <qcolor.h>
 #include <qpoint.h>
 
+#include "rgba.h"
+
 #define TYPE_COLOR_RED   0
 #define TYPE_COLOR_GREEN 1 
 #define TYPE_COLOR_BLUE  2
-#define TYPE_POSITION_X  3
-#define TYPE_POSITION_Y  4
+#define TYPE_COLOR_ALPHA 3
+#define TYPE_POSITION_X  4
+#define TYPE_POSITION_Y  5
 
 // Needed to access the function of the stored function pointer to a function of a class
 class CallbackClass
@@ -62,16 +66,18 @@ public:
       (m_obj.*m_fn)( n );
     }
     else if ( (m_iWhichType >= TYPE_COLOR_RED) && (m_iWhichType <= TYPE_COLOR_BLUE) )	{// Special handling for QColor
-      QColor &theColor = ((QColor &)m_obj);
-      int r = theColor.red  ();
-      int g = theColor.green();
-      int b = theColor.blue ();
+      Rgba &theColor = ((Rgba &)m_obj);
+      int r = theColor.red   ( );
+      int g = theColor.green ( );
+      int b = theColor.blue  ( );
+	  int a = theColor.alpha ( );
       switch (m_iWhichType)	{
       case TYPE_COLOR_RED:   r = n; break;
       case TYPE_COLOR_GREEN: g = n; break;
       case TYPE_COLOR_BLUE:  b = n; break;
+	  case TYPE_COLOR_ALPHA: a = n; break;
       }
-      theColor.setRgb(r, g, b);
+      theColor.setRgb ( r, g, b, a );
       //		(m_obj.*m_colorFn)( n, ((QColor &)m_obj).green(), ((QColor &)m_obj).blue() );
     }
     else if ( ( m_iWhichType == TYPE_POSITION_X ) || (  m_iWhichType == TYPE_POSITION_Y ) ) {
@@ -116,16 +122,21 @@ public:
 
 	void    initMe     (long, QString);
 	void    append     (long, QString);
-	const   QString name   ();
-	void    setValue   (long);
+	const   QString name  ( );
+	bool    setValue   (long);
 	int     iValue     (long);
 	bool    bValue     (long);
 	double  dValue     (long);
 	float   fValue     (long);
 	QString qsValue    (long);
-	long    maxFrames  ();;
+	long    maxFrames  ( );
+
+	// the following two functions indicate that the Image needs to be reloaded etc.
+	bool    needRefresh    ( );
+	void    setNeedRefresh ( bool );
 
 private:
+	bool           m_bNeedRefresh;
 	int           *m_piValue;	// Pointer to the real deal (if int)
 	bool          *m_pbValue;	// Pointer to the real deal (if int)
 	float         *m_pfValue;	// pointer to the real deal (if float)
