@@ -35,11 +35,10 @@ ButtonPreview::ButtonPreview(QWidget * parent, const char * name, WFlags f)
     m_pActiveObject ( NULL )
 {
   // Here we have the undo buffer ...
-  m_pUndoBuffer = new UndoBuffer(MAX_UNDO_DEPTH);
-  
-  m_iCursorStyle = MousePointerCursor;
-  m_pActiveObject = NULL;
-  m_bMouseEventDone = false;
+  m_pUndoBuffer        = new UndoBuffer ( MAX_UNDO_DEPTH );
+  m_iCursorStyle       = MousePointerCursor;
+  m_pActiveObject      = NULL;
+  m_bMouseEventDone    = false;
   m_bDrawVisibleRegion = false;	// This is set in the derived calss MenuPreview - only ... (No need for the buttonPreview class to play with)
 }
 
@@ -88,30 +87,36 @@ ButtonPreview &ButtonPreview::operator= ( ButtonPreview &theOther )
   return *this;
 }
 
-bool ButtonPreview::removeObject(MenuObject *pTheObject)
+bool ButtonPreview::removeObject ( MenuObject *pTheObject )
 {
-	bool bReturn = m_listMenuObjects.remove(pTheObject);
-	update ();
-	return bReturn;
+  if ( m_pActiveObject == pTheObject )
+       m_pActiveObject  = NULL;
+
+  bool bReturn = m_listMenuObjects.remove ( pTheObject );
+
+  update ( );
+  return bReturn;
+}
+
+void ButtonPreview::addMenuObject ( MenuObject *pNewObject )
+{
+  // Adds the menuObject to the list of MenuObjects.
+  m_listMenuObjects.append ( pNewObject );
+  update ( );
 }
 
 void ButtonPreview::slotDeleteObject ( MenuObject *pTheObject )
 {
   // object not found. Maybe it is in a ButtonObject ...
-  if ( m_listMenuObjects.remove ( pTheObject ) == false )
-    return;
-
   if ( m_pActiveObject == pTheObject )
        m_pActiveObject  = NULL;
 
-  delete pTheObject;
-  update ( );
-}
+  if ( m_listMenuObjects.remove ( pTheObject ) == false )
+    return;
+
+  undoBuffer ( )->push ( new DVDMenuUndoObject ( DVDMenuUndoObject::DELETED_OBJECT, pTheObject, this ) );
+//  delete pTheObject;
 
-void ButtonPreview::addMenuObject(MenuObject *pNewObject)
-{
-	// Adds the menuObject to the list of MenuObjects.
-  m_listMenuObjects.append ( pNewObject );
   update ( );
 }
 
@@ -157,7 +162,7 @@ MenuObject *ButtonPreview::menuObject (uint iNr)
 
 uint ButtonPreview::objectCount()
 {
-	return m_listMenuObjects.count();
+	return m_listMenuObjects.count ( );
 }
 
 UndoBuffer *ButtonPreview::undoBuffer()
@@ -165,7 +170,7 @@ UndoBuffer *ButtonPreview::undoBuffer()
 	return m_pUndoBuffer;
 }
 
-void ButtonPreview::toTop(MenuObject *pObject)
+void ButtonPreview::toTop ( MenuObject *pObject )
 {
   // This function pushes the currently selected child to the top of the stack as to draw it last (on the top).
   m_listMenuObjects.remove (pObject);
@@ -234,7 +239,7 @@ void ButtonPreview::slotMoveOnStack(MenuObject *pObject, int iHowMany)
 	m_listMenuObjects.remove (pObject);
 
 	m_listMenuObjects.insert (iIndex, pObject);
-	updatePixmap ();
+	updatePixmap ( );
 }
 
 void ButtonPreview::mousePressEvent ( QMouseEvent *pEvent )
@@ -243,14 +248,14 @@ void ButtonPreview::mousePressEvent ( QMouseEvent *pEvent )
   // in which case the Object will take over the mouse handling ...
   QPoint thePos  = pEvent->pos ( );
   MenuObject *pObject = childContains ( thePos );
-  if (pObject != NULL)	{
+  if ( pObject )  {
     MenuObject::enState currentState = pObject->setObjectState ( thePos, pEvent->state ( ) );
     setObjectCursor ( currentState );
     m_bMouseEventDone = pObject->mousePressEvent ( pEvent );
     // Here we protect the action toTop for creation of Buttons, which delete the returned pObject
-    if (!m_bMouseEventDone)	{
-      toTop(pObject);
-      undoBuffer()->push (new DVDMenuUndoObject(DVDMenuUndoObject::MOVED_OBJECT, pObject));
+    if ( ! m_bMouseEventDone )  {
+      toTop ( pObject );
+      undoBuffer ( )->push ( new DVDMenuUndoObject ( pObject, currentState ) );
     }
   }
 }
@@ -271,28 +276,75 @@ void ButtonPreview::mouseReleaseEvent (QMouseEvent *pEvent)
   //      delete pUndo;
   //    }
   //  }
-  for ( uint t=0; t<m_listMenuObjects.count();t++ ) {
-    if ( m_listMenuObjects.at ( t )->isActive ( ) ) {
-      m_bMouseEventDone = m_listMenuObjects.at ( t )->mouseReleaseEvent(pEvent);
+  for ( uint t=0; t<m_listMenuObjects.count( );t++ )  {
+    if ( m_listMenuObjects.at  ( t )->isActive ( ) )  {
+      m_bMouseEventDone = m_listMenuObjects.at ( t )->mouseReleaseEvent ( pEvent );
       // here we get the last object on the undoBuffer
-      DVDMenuUndoObject *pUndo = (DVDMenuUndoObject *) undoBuffer()->last();
-      if (pUndo && !pUndo->hasMoved())	{
-	undoBuffer()->removeLast();
-	delete pUndo;
+      DVDMenuUndoObject *pUndo = (DVDMenuUndoObject *) undoBuffer ( )->last ( );
+      if ( pUndo && ! pUndo->hasMoved ( ) )  {
+        undoBuffer  ( )->removeLast   ( );
+        delete pUndo;
       }
     }
   }
   unsetCursor  ( );
 }
 
-void ButtonPreview::mouseDoubleClickEvent	(QMouseEvent *pEvent)
+void ButtonPreview::mouseDoubleClickEvent ( QMouseEvent *pEvent )
 {
-	// Here we check if the mouse click appeared withtin one of the buttons,
-	// in which case the button will take over ...
-	QPoint thePos = pEvent->pos();
-	MenuObject *pObject = childContains(thePos);
-	if (pObject != NULL)
-		m_bMouseEventDone = pObject->mouseDoubleClickEvent(pEvent);
+  // Here we check if the mouse click appeared withtin one of the buttons,
+  // in which case the button will take over ...
+  QPoint thePos = pEvent->pos ( );
+  MenuObject *pObject = childContains ( thePos );
+  if ( pObject != NULL )
+    m_bMouseEventDone = pObject->mouseDoubleClickEvent ( pEvent );
+}
+
+void ButtonPreview::keyPressEvent ( QKeyEvent *pEvent )
+{
+  QLabel::keyPressEvent ( pEvent );
+
+  if ( ( pEvent->key ( ) == Qt::Key_Up    ) ||
+       ( pEvent->key ( ) == Qt::Key_Down  ) ||
+       ( pEvent->key ( ) == Qt::Key_Left  ) ||
+       ( pEvent->key ( ) == Qt::Key_Right ) )  {
+
+    QPoint pos = mapFromGlobal ( QCursor::pos ( ) );
+    MenuObject *pObject = childContains ( pos );
+
+    if ( ! pObject )
+      return;
+
+    pos = pObject->position ( );
+
+    if ( pEvent->key      ( ) == Qt::Key_Up    )
+      pos.setY ( pos.y ( ) - 1 );
+    else if ( pEvent->key ( ) == Qt::Key_Down  )
+      pos.setY ( pos.y ( ) + 1 );
+    else if ( pEvent->key ( ) == Qt::Key_Left  )
+      pos.setX ( pos.x ( ) - 1 );
+    else if ( pEvent->key ( ) == Qt::Key_Right )
+      pos.setX ( pos.x ( ) + 1 );
+
+    pObject->setPosition ( pos );
+    updatePixmap ( );
+
+    bool bNeedUndo = true;
+    DVDMenuUndoObject *pUndo = (DVDMenuUndoObject *) undoBuffer ( )->last ( );
+    if ( pUndo && ( pUndo->getMenuObject ( ) ==  pObject ) )
+         bNeedUndo = ( ! pUndo->isMoving ( pEvent->key ( ) ) );
+    if ( bNeedUndo )
+         undoBuffer ( )->push ( new DVDMenuUndoObject ( DVDMenuUndoObject::MOVED_OBJECT, pObject, NULL, pEvent->key ( ) ) );
+  }
+  if ( pEvent->key ( ) == Qt::Key_Delete )  {
+
+    QPoint pos = mapFromGlobal ( QCursor::pos ( ) );
+    MenuObject *pObject = childContains ( pos );
+
+    if ( ! pObject )
+      return;
+    slotDeleteObject ( pObject );
+  }
 }
 
 QPixmap &ButtonPreview::renderMenu (CDVDMenuInterface *pInterface, long iRenderFrameNumber, long iTotalFramesToRender)
@@ -443,14 +495,14 @@ void ButtonPreview::drawContents ( QPainter *painter )
     erase ( 5, 5, width ( ) - 10, height ( ) - 10 );
     drawVisibleRegion ( painter );
     for ( t=0; t<m_listMenuObjects.count ( ); t++ )  {
-      m_listMenuObjects.at(t)->drawContents (painter);
+      m_listMenuObjects.at ( t )->drawContents ( painter );
     }
     return;
   }
 
-  // This will draw the currentObject on top of the 'copy' we already painted into the pixmap. 
+  // This will draw the currentObject on top of the 'copy' we already painted into the pixmap.
   if ( m_pActiveObject )
-       m_pActiveObject->drawContents(painter);
+       m_pActiveObject->drawContents ( painter );
 }
 
 void ButtonPreview::drawVisibleRegion(QPainter *pPainter)
@@ -475,40 +527,37 @@ void ButtonPreview::drawVisibleRegion(QPainter *pPainter)
 	}
 }
 
-void ButtonPreview::setPixmap (const QPixmap &pixmap)
+void ButtonPreview::setPixmap ( const QPixmap &pixmap )
 {
-	// This is an overloaded function from QLabel
-	// We are going to handle the double buffer here ...
-	m_backgroundPixmap = pixmap;
-	// Here we get rid of the checkered - background pixmap, No need after a real pixmap has been set ...
-//	if (!pixmap.isNull())
-//		setPaletteBackgroundPixmap (NULL);
+  // This is an overloaded function from QLabel
+  // We are going to handle the double buffer here ...
+  m_backgroundPixmap = pixmap;
 
-	updatePixmap ();
+  updatePixmap ( );
 }
 
-void ButtonPreview::updatePixmap()
+void ButtonPreview::updatePixmap ( )
 {
-	if (m_backgroundPixmap.isNull())
+	if ( m_backgroundPixmap.isNull ( ) )
 		return;
 	// Set the waiting cursor ...
-	QCursor myCursor (QCursor::WaitCursor);
-	setCursor(myCursor);
+	QCursor myCursor ( QCursor::WaitCursor );
+	setCursor ( myCursor );
 
 	// This is the function, which updates the background Pixmap
 	QPixmap thePixmap;
 	drawBackground ( thePixmap );
 
 	// This will check if we load the transparent background (extensions are much smaller)
-	if ( (thePixmap.width() < 100) || (thePixmap.height() < 100) )
-		setFixedSize(720, 480);
+	if ( (thePixmap.width ( )  < 100 ) || ( thePixmap.height ( ) < 100 ) )
+		setFixedSize  ( 720, 480 );
 	else	// Otherwise we set the original image size.
-		setFixedSize(thePixmap.width(), thePixmap.height());
+		setFixedSize  ( thePixmap.width ( ), thePixmap.height ( ) );
 
 	setBackground ( thePixmap );
 	// re-set the cursor ...
-	myCursor = QCursor(QCursor::ArrowCursor);
-	setCursor(myCursor);
+	myCursor = QCursor ( QCursor::ArrowCursor );
+	setCursor ( myCursor );
 }
 
 void ButtonPreview::setBackground ( QPixmap &thePixmap )
