@@ -29,13 +29,12 @@ class CDVDMenuInterface;
 
 class ButtonPreview : public QLabel
 {
-	Q_OBJECT
+  Q_OBJECT
 
 public:
-  // m_iPointerShape
-  enum enPointerShape {                     MousePointerCursor=Qt::ArrowCursor, 
-    MouseCrossCursor  =Qt::CrossCursor,     MouseMoveCursor   =Qt::SizeAllCursor, 
-    MouseXResizeCursor=Qt::SizeHorCursor,   MouseYResizeCursor=Qt::SizeVerCursor, 
+  enum enPointerShape {                     MousePointerCursor=Qt::ArrowCursor,
+    MouseCrossCursor  =Qt::CrossCursor,     MouseMoveCursor   =Qt::SizeAllCursor,
+    MouseXResizeCursor=Qt::SizeHorCursor,   MouseYResizeCursor=Qt::SizeVerCursor,
     MouseZoomCursor   =Qt::SizeFDiagCursor, MouseRotateCursor =Qt::PointingHandCursor,
     MouseLZoomCursor  =Qt::SizeFDiagCursor, MouseRZoomCursor  =Qt::SizeBDiagCursor,
   };
@@ -76,7 +75,8 @@ protected:
   virtual void mouseReleaseEvent     ( QMouseEvent * );
   virtual void mouseDoubleClickEvent ( QMouseEvent * );
   virtual void setObjectCursor       ( MenuObject::enState );
-  virtual void drawContents          ( QPainter * );
+  virtual void drawContents          ( QPainter    * );
+  virtual void keyPressEvent         ( QKeyEvent   * );
 
 private:
   void drawVisibleRegion             ( QPainter * );
@@ -85,8 +85,8 @@ protected:
   // Here we have the undo buffer ...
   UndoBuffer *m_pUndoBuffer;
 
-  QPtrList<MenuObject>	 m_listMenuObjects;
-  QRect       m_rectViewport;		// This is the viewport set in drawContents()
+  QPtrList<MenuObject>   m_listMenuObjects;
+  QRect       m_rectViewport;  // This is the viewport set in drawContents()
   QPixmap     m_backgroundPixmap;
   MenuObject *m_pActiveObject;
   // tells the process to skip further handling of the MouseEvents since they were already taken care of.
