@@ -425,6 +425,8 @@ bool Thumbs::ImageEntry::loadImages ( ) // load images from cache or from the fi
        freeImages  ( );
 
   QImage theImage;
+  QFileInfo fileInfo ( qsFileName );
+  QString qsExt = fileInfo.extension ( FALSE ).lower ( );
   QString qsFilePath, qsHashBaseName, qsFullName;
   arrayOfThumbs = new QImage *[1];
 
@@ -438,6 +440,12 @@ bool Thumbs::ImageEntry::loadImages ( ) // load images from cache or from the fi
     iScanStatus      = 2;
     iNumberOfThumbs  = 1;
   }
+  else if ( ( qsExt != "jpg" ) && ( qsExt != "jpeg" ) && theImage.load ( qsFileName ) )  {
+    // This is for images which are NOT jpegs AND are < 680x480
+    arrayOfThumbs[0] = new QImage ( theImage );
+    iScanStatus      = 2;
+    iNumberOfThumbs  = 1;
+  }
   else if ( pPlugin )  {
     jhead::JHead::Info *pInfo = pPlugin->getFileInfo ( qsFileName );
     if ( pInfo )  {
@@ -1449,9 +1457,24 @@ unsigned long long Thumbs::hashFromFile ( QString &qsFileName )
   if ( ! fileInfo.exists ( ) )
     return 0LL;
 
-  uint      iSize = fileInfo.size    ( );
-  QDateTime  time = fileInfo.created ( );
-  qsFingerprint = qsFileName + QString ( "%1" ).arg ( iSize ) + time.toString ( "yyyyMMddhhmmss" );
+  uint     iSize = fileInfo.size    ( );
+  QDateTime time = fileInfo.created ( );
+  qsFingerprint  = qsFileName + QString ( "%1" ).arg ( iSize ) + time.toString ( "yyyyMMddhhmmss" );
+
+  return SDBMHash ( qsFingerprint );
+}
+
+unsigned long long Thumbs::hashFromPath ( QString &qsPathName )
+{
+  QString   qsFingerprint;
+  QFileInfo theDir ( qsPathName );
+
+  // Sanity check.
+  if ( ! theDir.exists ( ) )
+    return 0LL;
+
+  QDateTime time = theDir.created ( );
+  qsFingerprint  = qsPathName + time.toString ( "yyyyMMddhhmmss" );
 
   return SDBMHash ( qsFingerprint );
 }
