@@ -95,8 +95,10 @@ void DialogAudio::initMe ( SourceFileInfo *pInfo, SourceFileEntry *pEntry, QStri
   m_pSourceFileEntry     = new SourceFileEntry;
   *m_pSourceFileEntry    = *pEntry; // make a deep copy.
   m_pSourceFileInfo      = m_pSourceFileEntry->listFileInfos [ t ];
-  
-  // VAROL 
+  // The original SourceFileEntry might hold outdated data for the SourceFileInfo in question
+  *m_pSourceFileInfo     = *m_pOrigSourceFileInfo;
+
+  // VAROL
   /*
   SourceFileEntry *p = m_pSourceFileEntry;
   printf ( "%s::%s : %d > ORIG > ", __FILE__, __FUNCTION__, __LINE__ );
@@ -239,10 +241,10 @@ void DialogAudio::initAudioInfo ( SourceFileInfo *pInfo )
   TranscodeInterface *pInterface = pInfo->pTranscodeInterface;
 
   if ( ! m_pSourceFileInfo )
-    m_pSourceFileInfo = new SourceFileInfo;
+         m_pSourceFileInfo = new SourceFileInfo;
 
   *m_pSourceFileInfo = *pInfo;
-  
+
   initAudioInfo ( pInterface );
 }
 
@@ -288,14 +290,14 @@ void DialogAudio::slotAudioEntryChanged ( int )
   Audio          *pAudio = NULL;
   QFileInfo fileInfo;
   QString   qsFile = m_pComboAudioEntry->currentText ( );
-  
+
   if ( m_pDVDMenu ) {
     for ( t=0; t<m_pDVDMenu->getInterface ( )->listSoundEntries.count ( ); t++ ) {
       pAudio   = m_pDVDMenu->getInterface ( )->listSoundEntries[t];
       fileInfo.setFile ( pAudio->qsFileName );
-      if ( fileInfo.fileName ( ) == qsFile ) {
-	initAudioInfo ( pAudio );
-	break;
+      if ( fileInfo.fileName  ( ) == qsFile ) {
+        initAudioInfo  ( pAudio );
+        break;
       }
     }
   }
@@ -304,8 +306,8 @@ void DialogAudio::slotAudioEntryChanged ( int )
       pInfo = m_pOrigSourceFileEntry->listFileInfos[t];
       fileInfo.setFile ( pInfo->qsFileName );
       if ( fileInfo.fileName ( ) == qsFile ) {
-	initAudioInfo ( pInfo );
-	break;
+        initAudioInfo  ( pInfo );
+        break;
       }
     }
   }
@@ -477,7 +479,7 @@ void DialogAudio::slotDelete ( )
     return;
 
   int iAudioNumber = pItem->text ( 0 ).toInt ( );
-  if ( iAudioNumber == 0 ) {
+  if ( ( iAudioNumber == 0 ) ) { // no longer a restriction, as we could have elementary streams.
     MessageBox::warning ( this, tr ( "Can't delete main audio track." ), 
 			  tr ( "You can not delete Audio track number 0.\nEither Edit or choose a different track." ), 
 			  QMessageBox::Ok, QMessageBox::NoButton );
@@ -582,7 +584,7 @@ void DialogAudio::setAudioInfo ( SourceFileInfo *pOrig, SourceFileInfo *pNew )
          pOrig->arrayAudio[ t ] = NULL;
     }
     if ( pNew->arrayAudio [ t ] ) {
-        pOrig->arrayAudio [ t ] =   new Audio;
+        pOrig->arrayAudio [ t ] = new Audio;
        *pOrig->arrayAudio [ t ] = *pNew->arrayAudio [ t ];
     }
   }
@@ -616,7 +618,7 @@ void DialogAudio::accept ( )
   }
 
   iAudioNumber = getSelectedNumber ( );
-  if ( ( iAudioNumber < 1 ) && ( m_pSourceFileInfo ) ) {
+  if ( ( iAudioNumber < 1 ) && ( m_pSourceFileInfo && m_pSourceFileInfo->bHasAudioTrack ) ) {
     MessageBox::warning ( this, tr ("Can not select track 0" ), tr ( "You can not select track number 0.\nThis track holds the original audio track from the movie." ) );
     return;
   }
@@ -639,14 +641,14 @@ void DialogAudio::accept ( )
     pAudio->qsFileName   = m_qsFileName;
 
     pAudio->pTranscodeInterface = getTranscode ( );
-    if ( ( pAudio->iAudioNumber > -1 ) && ( pAudio->iAudioNumber < 8 ) ) {
+    if ( ( pAudio->iAudioNumber > -1 ) && ( pAudio->iAudioNumber < MAX_AUDIO ) ) {
       if ( m_pSourceFileInfo->arrayAudio [ pAudio->iAudioNumber ] )
-	delete m_pSourceFileInfo->arrayAudio [ pAudio->iAudioNumber ];
-      
+        delete m_pSourceFileInfo->arrayAudio [ pAudio->iAudioNumber ];
+
       m_pSourceFileInfo->arrayAudio [ pAudio->iAudioNumber ] = pAudio;
-      
+
       for ( t=0; t<(int)m_pOrigSourceFileEntry->listFileInfos.count ( ); t++ )
-	setAudioInfo  ( m_pOrigSourceFileEntry->listFileInfos [ t ], m_pSourceFileEntry->listFileInfos [ t ] );
+        setAudioInfo  ( m_pOrigSourceFileEntry->listFileInfos [ t ], m_pSourceFileEntry->listFileInfos [ t ] );
       setAudioInfo ( m_pOrigSourceFileInfo, m_pSourceFileInfo );
     }
   }
@@ -655,25 +657,25 @@ void DialogAudio::accept ( )
     for ( t=0; t<(int)m_pOrigSourceFileEntry->listFileInfos.count ( ); t++ ) {
       pInfo1 = m_pOrigSourceFileEntry->listFileInfos [ t ];
       pInfo2 = NULL;
-      if ( (int)m_pSourceFileEntry->listFileInfos.count ( ) >= t )	
-	pInfo2 = m_pSourceFileEntry->listFileInfos [ t ];
+      if (  (int)m_pSourceFileEntry->listFileInfos.count ( ) >= t )
+        pInfo2 = m_pSourceFileEntry->listFileInfos [ t ];
 
       // Next we move the Audio objects over ...
       if ( pInfo1 && pInfo2 ) {
-	for ( i=0; i<MAX_AUDIO; i++ ) {
-	  // First we delete the original Audio - object
-	  if ( pInfo1->arrayAudio [ i ] )
-	    delete pInfo1->arrayAudio [ i ];
-	  pInfo1->arrayAudio [ i ] = NULL;
-	  // ... and then move the Audio - object from the tempSourceFileEntry over
-	  if ( pInfo2->arrayAudio [ i ] )
-	    pInfo1->arrayAudio [ i ] = pInfo2->arrayAudio [ i ];	  
-	  pInfo2->arrayAudio [ i ] = NULL;
-	}
+        for ( i=0; i<MAX_AUDIO; i++ ) {
+          // First we delete the original Audio - object
+          if ( pInfo1->arrayAudio [ i ] )
+            delete pInfo1->arrayAudio [ i ];
+          pInfo1->arrayAudio [ i ] = NULL;
+          // ... and then move the Audio - object from the tempSourceFileEntry over
+          if ( pInfo2->arrayAudio [ i ] )
+               pInfo1->arrayAudio [ i ] = pInfo2->arrayAudio [ i ];	  
+          pInfo2->arrayAudio [ i ] = NULL;
+        }
       }
     }
   }
-  // VAROL 
+
   /*
   SourceFileEntry *p = m_pOrigSourceFileEntry;
   printf ( "%s::%s : %d > ORIG > ", __FILE__, __FUNCTION__, __LINE__ );
