@@ -46,13 +46,17 @@
 #include "buttonobject.h"
 #include "buttonpreview.h"
 #include "sourcefileentry.h"
+#include "dialogbuttontrans.h"
+#include "render_client.h"
+#include "cachethumbs.h"
+#include "dvdmenuundoobject.h" //undobuffer.h"
 
 #define BUTTON_BASIC    "<<< Basic"
 #define BUTTON_ADVANCED "Advanced >>>"
 #define MANUAL_CHANGE   255, 180, 60
 
-DialogButton::DialogButton(QWidget * parent, const char * name, WFlags f)
-	: uiDialogButton (parent, name, f)
+DialogButton::DialogButton ( QWidget * parent, const char * name, WFlags f )
+	: uiDialogButton   ( parent, name, f )
 {
   // Standard is set to false. This indicates that we create a button as
   // opposed to modify an existing one.
@@ -165,9 +169,10 @@ void DialogButton::initMe ( ButtonObject *pButton, MenuPreview *pView )
 	}
 
 	// Now we proceed to initialize the GUI objects (LineEdits, ComboBoxes etc.)
-	initNext     ( );
-	initMultiple ( );
-	initSystem   ( pView, pButton );
+	initNext       ( );
+//	initMultiple   ( ); called from initTransition ( )
+	initTransition ( );
+//	initSystem     ( pView, pButton ); // must be initialized last (timer)
 
 	// Here we set the text fields for the bounding rectangle ...
 	m_pEditX     ->setText ( QString ("%1").arg( pButton->getNormal(0)->boundingRect().x ( ) ) );
@@ -220,6 +225,7 @@ void DialogButton::initMe ( ButtonObject *pButton, MenuPreview *pView )
 	connect (m_pButtonSColor2,    SIGNAL ( clicked (      ) ), this, SLOT ( slotChangeColor (   ) ) );
 	connect (m_pButtonSColor3,    SIGNAL ( clicked (      ) ), this, SLOT ( slotChangeColor (   ) ) );
 	connect (m_pButtonSColor4,    SIGNAL ( clicked (      ) ), this, SLOT ( slotChangeColor (   ) ) );
+	connect (m_pButtonTransition, SIGNAL ( clicked (      ) ), this, SLOT ( slotTransition  (   ) ) );
 
 	connect (m_pRadioHighlightedFrame, SIGNAL ( toggled ( bool ) ), this, SLOT ( slotUseFrame ( bool ) ) );
 	connect (m_pRadioHighlightedMask,  SIGNAL ( toggled ( bool ) ), this, SLOT ( slotUseMask  ( bool ) ) );
@@ -234,6 +240,7 @@ void DialogButton::initMe ( ButtonObject *pButton, MenuPreview *pView )
 void DialogButton::slotAdjustSize ( )
 {
   resize  ( width ( ), 530 );
+  initSystem ( m_pMenuPreview, m_pButtonObject );
 }
 
 void DialogButton::initNext()
@@ -276,6 +283,32 @@ void DialogButton::initMultiple ( )
   if ( ! m_pClonedButton )
     return;
 
+  QString qsText;
+  QStringList list = m_pClonedButton->getPlayMultipleList ( );
+  if (   m_pClonedButton->getTransition ( ) ) {
+    if ( m_pClonedButton->getTransition ( )->bUserFile )
+      qsText = tr ( "User Transition" );
+    else
+      qsText = tr ( "Transition, %1 Sec" ).arg ( m_pClonedButton->getTransition ( )->fDuration );
+  }
+  if ( list.count ( ) > 0 ) {
+    if ( qsText.length ( ) > 0 )
+         qsText += "<br>";
+    QString qsLoopMenu;
+    int iTargetCount = list.count ( );
+    QString qsLast   = list.last  ( );
+    QStringList lastList = QStringList::split ( STRING_SEPARATOR, qsLast );
+    qsLoopMenu += m_pClonedButton->getLoopMultiple ( ) ? tr (  ", Loop"   ) : "";
+    if ( lastList.count ( ) == 2 )  { // Last target is to a Menu
+      qsLoopMenu += tr ( ", to Menu" );
+      iTargetCount --;
+    }
+    qsText += tr ( "Play %1" ).arg ( iTargetCount ) + qsLoopMenu;
+    m_pLabelMultiple->setText ( qsText );
+  }
+  m_pLabelMultiple->setText ( "<FONT COLOR=\"RED\"><B>" + qsText + "</B></FONT>" );
+
+/*
   QStringList list = m_pClonedButton->getPlayMultipleList ( );
   if ( list.count ( ) > 0 ) {
     int iTargetCount = list.count ( );
@@ -293,7 +326,18 @@ void DialogButton::initMultiple ( )
   }
   else
     m_pLabelMultiple->setText ( "" );
+*/
+}
 
+void DialogButton::initTransition ( )
+{
+  // Simply sets the text of the transition Button
+  // and re-write the Multiple label.
+  initMultiple ( );
+  if ( m_pClonedButton->getTransition ( ) )
+    m_pButtonTransition->setText ( tr ( "Remove Transition" ) );
+  else
+    m_pButtonTransition->setText ( tr ( "Add Transition" ) );
 }
 
 void DialogButton::initColors ( )
@@ -314,7 +358,7 @@ void DialogButton::initColors ( )
 
   Rgba clrHighlighted = m_pClonedButton->getMaskColor ( true  );
   Rgba clrSelected    = m_pClonedButton->getMaskColor ( false );
-  
+
   for ( t=0; t<8; t++ ) {
     i = t%4;
     if ( t < MAX_MASK_COLORS ) {
@@ -377,10 +421,10 @@ void DialogButton::initActionState ()
   qsAction3 = listActions[2];
   if (listActions.count() > 3)
     qsAction4 = listActions[3];
-  
+
   if (!qsAction2.isEmpty())
     m_pComboSourceEntry->setCurrentText(qsAction2);
-  
+
   // Okay here we generate the information in the Combo boxes
   slotSourceEntry(m_pComboSourceEntry->currentItem());
 
@@ -403,24 +447,47 @@ void DialogButton::initActionState ()
   }
 }
 
+void DialogButton::initButtons ( ButtonObject *pButton )
+{
+  DVDMenu *pDVDMenu = NULL;
+  if ( pButton )  {
+    pDVDMenu = getSelectedDVDMenu ( ); //pButton->dvdMenu ( );
+    if ( ! pDVDMenu )  // The button does not point to a DVDMenu
+           pDVDMenu = Global::pApp->getCurrentSubMenu ( );
+  }
+  initButtons ( pDVDMenu );
+}
+
+void DialogButton::initButtons ( DVDMenu *pDVDMenu )
+{
+  // Note: pDVDMenu is either the menu the button is defined in OR
+  //       it is the menu the button points to.
+  ButtonObject *pButton = NULL;
+  m_pComboButton->clear ( );
+  m_pComboButton->insertItem ( "Button -Def-" );
+
+  if ( pDVDMenu )  {
+    QValueList<ButtonObject *> list = pDVDMenu->getButtons ( );
+    QValueList<ButtonObject *>::iterator it  =  list.begin ( );
+    while ( it != list.end ( ) )  {
+      pButton = *it++;
+      m_pComboButton->insertItem ( pButton->name ( ) );
+    }
+  }
+//  slotSystem ( 0 );
+}
+
 void DialogButton::initSystem ( MenuPreview *pMenuPreview, ButtonObject *pButton )
 {
-  uint t, iItem;
-  bool bOkay;
+  uint t, i, iItem;
+  bool bOkay, bFound;
   MenuObject *pMenuObject = NULL;
   QString     qsTemp, qsPreAction;
 
   if ( pButton )
     qsPreAction = pButton->preAction ( );
-  m_pComboButton->clear ( );
-  m_pComboButton->insertItem ( "Button -Def-" );
-  if ( pMenuPreview && pButton ) {
-    for ( t=0; t<pMenuPreview->objectCount ( );t++ ) {
-      pMenuObject = pMenuPreview->menuObject ( t );
-      if ( pMenuObject && pMenuObject->objectType ( ) == pButton->objectType ( ) )
-	m_pComboButton->insertItem ( pMenuObject->name ( ) );
-    }
-  }
+
+  initButtons ( pButton );
 
   // Next we initialize the combo boxes.
   QRegExp rx;
@@ -460,18 +527,29 @@ void DialogButton::initSystem ( MenuPreview *pMenuPreview, ButtonObject *pButton
       m_pComboRegion->setCurrentItem ( ++iItem );
   }
 
+  MenuPreview *pButtonsMenuPreview = pMenuPreview;
   qsPreAction.replace ( "\"", "&quot;" ); // this to make sure we catch either way " and &quot; ...
   rx.setPattern ( "button=&quot;.*&quot;" );
-  if ( rx.search ( qsPreAction ) > -1 ) {
-    QString qsButtonName = rx.cap ( 0 );
-    qsButtonName.remove ( "button=" );
-    qsButtonName.remove ( "&quot;" );
-    for ( t=0; t<pMenuPreview->objectCount ( ); t++ ) {
-      pMenuObject = pMenuPreview->menuObject ( t );
-      if ( pMenuObject->name ( ) == qsButtonName ) {
-	m_pComboButton->setCurrentItem ( t + 1 );
-	break; // exit for - loop
+  if ( rx.search ( qsPreAction ) > -1 )  {
+    if ( pButton->dvdMenu ( ) ) // Button to another menu should check there 
+      pMenuPreview = pButton->dvdMenu ( )->getMenuPreview ( );
+    bFound = false;
+    for ( i=0; i<2 && !bFound; i++ )  {
+      if ( pMenuPreview )  {
+        QString qsButtonName = rx.cap ( 0 );
+        qsButtonName.remove ( "button=" );
+        qsButtonName.remove ( "&quot;" );
+        for ( t=0; t<pMenuPreview->objectCount ( ); t++ )  {
+          pMenuObject = pMenuPreview->menuObject ( t );
+          if ( pMenuObject->name ( ) == qsButtonName ) {
+            if ( (uint)m_pComboButton->count      ( )  > t + 1 )
+                       m_pComboButton->setCurrentItem  ( t + 1 );
+            bFound = true;
+            break; // exit for - loop
+          }
+        }
       }
+      pMenuPreview = pButtonsMenuPreview;
     }
   }
   qsPreAction.replace ( "&quot;", "\"" ); // this to make sure we catch either way " and &quot; ...
@@ -577,6 +655,19 @@ void DialogButton::setMenuButtons (QStringList listMenuButtons)
 	initNext();
 }
 
+DVDMenu *DialogButton::getSelectedDVDMenu ( )
+{
+  DVDMenu *pMenu     = NULL;
+  QString qsMenuName = m_pComboSourceEntry->currentText ( );
+  if ( qsMenuName == "vmgm" )
+       qsMenuName  = VMGM_LABEL;
+
+  if ( qsMenuName.length ( ) > 0 )
+       pMenu = Global::pApp->getSubMenuByName( qsMenuName );
+
+  return pMenu;
+}
+
 SourceFileEntry *DialogButton::getSelectedSourceEntry()
 {
 	uint t;
@@ -633,7 +724,7 @@ void DialogButton::slotEditActionChanged(const QString &qsText)
 	}
 }
 
-void DialogButton::slotAction1(int)
+void DialogButton::slotAction1 ( int )
 {
 	// Okay, here we prepare the available actions ...
 	// if (Action1->text() == jump , then we generate a movie list for Action2, and Action3 has chapters according to the Movie.
@@ -676,11 +767,8 @@ void DialogButton::slotAction1(int)
 		for (t=0;t<m_listSubMenus.count();t++)
 			m_pComboSourceEntry->insertItem ( m_listSubMenus[t] );
 	}
-//	else	{	// resume ...
-//			m_pComboSourceEntry->insertItem(m_pClonedButton->action());
-//	}
 
-	slotSourceEntry(0);
+	slotSourceEntry ( 0 );
 }
 
 void DialogButton::slotSourceEntry(int)
@@ -689,11 +777,11 @@ void DialogButton::slotSourceEntry(int)
   bool bHideStatus = true;
   QFileInfo fileInfo;
   SourceFileEntry *pEntry;
-  
+
   m_pComboChapters->clear();
-  // Since we have also menus in the combo box we need to get the iCurrentEntry, which is the correct index 
+  // Since we have also menus in the combo box we need to get the iCurrentEntry, which is the correct index
   // to the m_listSourceFileEntries - list.
-  pEntry = getSelectedSourceEntry ();
+  pEntry = getSelectedSourceEntry ( );
   if (pEntry)	{
     if (pEntry->listFileInfos.count() > 1)	{
       // Okay we found a movie SourceEntry with multiple source files.
@@ -730,15 +818,17 @@ void DialogButton::slotSourceEntry(int)
     slotChapters(0);
   // Finally we show or hide the appropriate combo boxes ...
   if (bHideStatus)	{
-    m_pComboSourceFile->hide();
-    m_pComboChapters2->hide();
-    m_pComboChapters->show();
+    m_pComboSourceFile->hide ( );
+    m_pComboChapters2->hide  ( );
+    m_pComboChapters->show   ( );
   }
   else	{	// The default is to show only one line of ComboBoxes ...
-    m_pComboSourceFile->show();
-    m_pComboChapters2->show();
-    m_pComboChapters->hide();
+    m_pComboSourceFile->show ( );
+    m_pComboChapters2->show  ( );
+    m_pComboChapters->hide   ( );
   }
+
+  initButtons ( m_pButtonObject );
 }
 
 void DialogButton::slotSourceFile(int)
@@ -801,8 +891,8 @@ void DialogButton::slotChapters2(int)
 	uint t;
 	int  iChapterCounter = 1;
 	QFileInfo fileInfo;
-	SourceFileEntry *pEntry = getSelectedSourceEntry();
-	SourceFileInfo *pInfo = NULL;
+	SourceFileEntry *pEntry = getSelectedSourceEntry ( );
+	SourceFileInfo  *pInfo  = NULL;
 	// Okay this here is generating the text for m_pEditAction.
 	QString qsAction1 = m_pComboAction1->currentText();
 	QString qsAction2 = m_pComboSourceFile->currentText();
@@ -854,6 +944,23 @@ void DialogButton::slotHelp()
 		QMessageBox::Ok, QMessageBox::NoButton);
 }
 
+void DialogButton::slotTransition ( )
+{
+  // Add a transition from the menu to the Movie or the next menu.
+  // every button can max hold 1 transition
+  // Required parameters:
+  if ( m_pClonedButton->getTransition (   )  )
+       m_pClonedButton->setTransition ( NULL );
+  else  {
+    DialogButtonTrans dialog ( m_pClonedButton->getTransition ( ) );
+    if ( dialog.exec ( ) == QDialog::Rejected )
+      return;
+
+    m_pClonedButton->setTransition ( dialog.getTransition ( ) );
+  }
+  initTransition ( );
+}
+
 void DialogButton::slotMultiple ( )
 {
   // Open the selection dialog of all available objects.
@@ -1226,7 +1333,7 @@ void DialogButton::slotCancel ( )
   reject ( );
 }
 
-void DialogButton::slotOkay()
+void DialogButton::slotOkay ( )
 {
   // Here we take care of the information we obtained ...
   QString qsPreAction = m_pEditSystem->text        ( );
@@ -1244,16 +1351,16 @@ void DialogButton::slotOkay()
     // jump my_movie1.mpg chapter 3
     // this needs to convert to something like ...
     // jump titleset 2 chapter 3
-    SourceFileEntry *pEntry = getSelectedSourceEntry ();
-    if ( (pEntry) && (pEntry->listFileInfos.count() > 1) )
+    SourceFileEntry *pEntry = getSelectedSourceEntry   ( );
+    if ( pEntry && ( pEntry->listFileInfos.count ( ) > 1 ) )
       qsAction = m_pComboAction1->currentText ( ) + QString (STRING_SEPARATOR) +
-	m_pComboSourceEntry->currentText( ) + QString (STRING_SEPARATOR) +
-	m_pComboSourceFile->currentText ( ) + QString (STRING_SEPARATOR) +
-	m_pComboChapters2->currentText  ( );
+    m_pComboSourceEntry->currentText( ) + QString (STRING_SEPARATOR) +
+    m_pComboSourceFile->currentText ( ) + QString (STRING_SEPARATOR) +
+    m_pComboChapters2->currentText  ( );
     else
       qsAction = m_pComboAction1->currentText ( ) + QString (STRING_SEPARATOR) +
-	m_pComboSourceEntry->currentText ( ) + QString (STRING_SEPARATOR) +
-	m_pComboChapters->currentText    ( );
+    m_pComboSourceEntry->currentText ( ) + QString (STRING_SEPARATOR) +
+    m_pComboChapters->currentText    ( );
   }
 
   uint t, iIdx;
@@ -1280,60 +1387,108 @@ void DialogButton::slotOkay()
       delete m_pSelectedMask[t];
   }
 
-  m_pClonedButton->setPreAction   ( qsPreAction );
-  m_pClonedButton->setAction      ( qsAction    );
-  m_pClonedButton->setName        ( qsName      );
+  m_pClonedButton->setPreAction ( qsPreAction );
+  m_pClonedButton->setAction    ( qsAction    );
+  m_pClonedButton->setName      ( qsName      );
   //  m_pClonedButton->setLoopMultiple( m_pCheckLoop->isChecked  ( ) );
-  m_pClonedButton->setNext        ( NEXT_BUTTON_UP,    qsUp    );
-  m_pClonedButton->setNext        ( NEXT_BUTTON_DOWN,  qsDown  );
-  m_pClonedButton->setNext        ( NEXT_BUTTON_LEFT,  qsLeft  );
-  m_pClonedButton->setNext        ( NEXT_BUTTON_RIGHT, qsRight );
+  m_pClonedButton->setNext      ( NEXT_BUTTON_UP,    qsUp    );
+  m_pClonedButton->setNext      ( NEXT_BUTTON_DOWN,  qsDown  );
+  m_pClonedButton->setNext      ( NEXT_BUTTON_LEFT,  qsLeft  );
+  m_pClonedButton->setNext      ( NEXT_BUTTON_RIGHT, qsRight );
 
   // Here we associate the new SourceFileEntry to the Cloned Button ...
   m_pClonedButton->setSourceFileEntry(NULL);
   for (t=0;t<m_listSourceFileEntries.count();t++)
     if (m_listSourceFileEntries[t]->qsDisplayName == m_pComboSourceEntry->currentText())
       m_pClonedButton->setSourceFileEntry(m_listSourceFileEntries[t]);
-  
+
   // We should only touch the rect if the user has indeed changed something ...	
   QRect rect (m_pEditX->text().toInt(), m_pEditY->text().toInt(), m_pEditWidth->text().toInt(), m_pEditHeight->text().toInt());
-  if (rect != m_pButtonObject->getNormal(0)->boundingRect())	{
-    m_pClonedButton->setRect(rect);
-    for (t=0;t<m_pClonedButton->getNormalCount();t++)
-      m_pClonedButton->getNormal(t)->setRect(rect);
-    for (t=0;t<m_pClonedButton->getHighlightedCount();t++)
-      m_pClonedButton->getHighlighted(t)->setRect(rect);
-    for (t=0;t<m_pClonedButton->getSelectedCount();t++)
-      m_pClonedButton->getSelected(t)->setRect(rect);
+  if ( rect != m_pButtonObject->getNormal ( 0 )->boundingRect ( ) )  {
+    m_pClonedButton->setRect ( rect );
+    for ( t=0; t<m_pButtonObject->getNormalCount ( );    t++ )
+      m_pButtonObject->getNormal      ( t )->setRect ( rect  );
+//    for ( t=0; t<m_pClonedButton->getNormalCount ( );    t++ )
+//      m_pClonedButton->getNormal      ( t )->setRect ( rect  );
+    for ( t=0; t<m_pClonedButton->getHighlightedCount( );t++ )
+      m_pClonedButton->getHighlighted ( t )->setRect ( rect  );
+    for ( t=0; t<m_pClonedButton->getSelectedCount   ( );t++ )
+      m_pClonedButton->getSelected    ( t )->setRect ( rect  );
   }
-  
+
+  MenuObject *pObject = NULL;
+  // We want to preserve the original object to keep Undo functionality intact.
+  QValueList<MenuObject *> list;
+  for ( t=0; t<m_pButtonObject->getNormalCount ( ); t++ )  {
+    pObject =  m_pButtonObject->getNormal ( t );
+    list.append ( pObject );
+  }
+  for ( t=0; t<m_pButtonObject->getNormalCount ( ); t++ )
+    m_pButtonObject->removeNormal ( (uint)0 );
+
   // And last but not least we equalize the original with the cloned brother ...
   *m_pButtonObject = *m_pClonedButton;
-  MenuObject *pObject;
+
   for ( t=0; t<m_pButtonObject->getNormalCount ( ); t++ )  {
-    pObject = m_pButtonObject->getNormal( t );
-    m_pClonedButton->removeNormal ( pObject );
-    m_pMenuPreview->connectStdSlots  ( pObject, true );
-  }
-  for (t=0;t<m_pButtonObject->getHighlightedCount(); t++ )  {
-    pObject = m_pButtonObject->getHighlighted ( t );
-    m_pClonedButton->removeHighlighted  ( pObject );
+    pObject =  m_pButtonObject->getNormal ( t );
+    m_pClonedButton->removeNormal   ( pObject );
+    pObject->disconnect ( );
     m_pMenuPreview->connectStdSlots ( pObject, true );
   }
-  for (t=0;t<m_pButtonObject->getSelectedCount ( ); t++ )  {
+  for (t=0; t<m_pButtonObject->getHighlightedCount ( ); t++ )  {
+    pObject = m_pButtonObject->getHighlighted( t );
+    m_pClonedButton->removeHighlighted ( pObject );
+    pObject->disconnect ( );
+    m_pMenuPreview->connectStdSlots    ( pObject, true );
+  }
+  for (t=0; t<m_pButtonObject->getSelectedCount ( ); t++ )  {
     pObject = m_pButtonObject->getSelected ( t );
     m_pClonedButton->removeSelected  ( pObject );
+    pObject->disconnect ( );
     m_pMenuPreview->connectStdSlots  ( pObject, true );
   }
   delete m_pClonedButton;
 
-  m_pFramePreviewNormal->clearObjectList  ( );
-  m_pFrameHighlighted->clearObjectList  ( );
-  m_pFrameSelected->clearObjectList  ( );
+  // Then delete the newly created objects
+  for ( t=0;  t<m_pButtonObject->getNormalCount  ( ); t++ )
+    delete m_pButtonObject->removeNormal ( (uint)0 );
+  // and copy the originals over
+  for ( t=0; t<list.count ( );  t++ )
+    m_pButtonObject->appendNormal   ( list[t] );
+  m_pButtonObject->reParentChildren ( );
+
+  m_pFramePreviewNormal->clearObjectList ( );
+  m_pFrameHighlighted->clearObjectList   ( );
+  m_pFrameSelected->clearObjectList      ( );
 
   emit   ( signalUpdateStructure ( ) );
   emit   ( signalUpdatePixmap    ( ) );
+
+  ButtonTransition *pTransition = m_pButtonObject->getTransition ( );
+  if ( pTransition )  { // Initialize the transitions name, and video file name
+    pTransition->bFinishedRendering = false;
+    DVDMenu *pDVDMenu     = Global::pApp->getCurrentSubMenu ( );
+    if ( pDVDMenu )  {
+      Utils theUtils;
+      QString qsTempPath   = theUtils.getTempFile ( "" );
+      CDVDMenuInterface    * pInterface = pDVDMenu->getInterface ( );
+      QString qsName       = pInterface->qsMenuName + "/" + m_pButtonObject->name ( );
+      pTransition->iFormat = pInterface->iFormat;
+      pTransition->qsName  = qsName; // E.g. "Main VMGM/Button 1"
+      if ( ! pTransition->bUserFile )  {
+        theUtils.recMkdir   ( qsTempPath + qsName );
+        qsName += "/transition.vob";
+        pTransition->qsTransitionVideoFileName = qsTempPath + qsName; // E.g. "/tmp/TestProjec/Main VMGM/Button 1/transition.vob"
+      }
+    }
+  }
+
+  if ( m_bButtonCreation )  {
+    // This button was just created. Let us add an UndoObject
+    DVDMenu *pMenu = Global::pApp->getCurrentSubMenu ( );
+    if ( pMenu )
+         pMenu->getUndoBuffer ( )->push  ( new DVDMenuUndoObject ( DVDMenuUndoObject::NEW_BUTTON, m_pButtonObject, m_pButtonObject->getNormal ( 0 ) ) );
+  }
+
   accept ( );
 }
-
-
