@@ -20,24 +20,26 @@
 
 class QFrame;
 class QPaintEvent;
+
+class DVDMenu;
+class MenuPreview;
 class ButtonObject;
 class ButtonPreview;
 class SourceFileEntry;
 class SourceFileInfo;
-class MenuPreview;
 
 class DialogButton : public uiDialogButton
 {
 	Q_OBJECT
 public:
-	 DialogButton (QWidget * parent, const char * name = 0, WFlags f = 0 );
-	~DialogButton ();
+	 DialogButton ( QWidget *, const char *p=0, WFlags f=0 );
+	~DialogButton ( );
 
-	void initMe(ButtonObject *, MenuPreview *);
-	void setSourceFiles(QValueList<SourceFileEntry *>);
-	void setSubMenus (QStringList);
-	void setMenuButtons (QStringList);
-	void setButtonCreation (bool);
+	void initMe            ( ButtonObject *, MenuPreview * );
+	void setSourceFiles    ( QValueList<SourceFileEntry *> );
+	void setSubMenus       ( QStringList );
+	void setMenuButtons    ( QStringList );
+	void setButtonCreation ( bool        );
 
 protected:
 	void initNext         ( );
@@ -45,9 +47,14 @@ protected:
 	void initColors       ( );
 	void initActions      ( );
 	void initActionState  ( );
-	void initSystem ( MenuPreview  *, ButtonObject * );
+	void initTransition   ( );
+	void initSystem  ( MenuPreview  *, ButtonObject * );
+	void initButtons ( ButtonObject * );
+	void initButtons ( DVDMenu      * );
+
 	SourceFileEntry *getSelectedSourceEntry  ( );
 	SourceFileInfo  *getSelectedSourceFile   ( );
+	DVDMenu         *getSelectedDVDMenu      ( );
 	MenuObject *createMissingMask ( bool, bool );
 	void removeFromString ( QString &, QString );
 	bool setObjectColor   ( MenuObject*, Rgba &);
@@ -77,6 +84,7 @@ protected slots:
 	virtual void slotOkay       ( );
 	virtual void slotCancel     ( );
 	virtual void slotAdvanced   ( );
+	virtual void slotTransition ( );
 	virtual void slotTabChanged ( QWidget * );
 	virtual void slotEditActionChanged ( const QString & );
 
