@@ -1025,6 +1025,7 @@ bool DialogCreateTemplate::createBackground ( QString &qsTemplatePath )
   // Here we take care of 
   // - background/background00000.jpg
   // - thumbnail.png
+  // - thumbnail.mpg ( if animated menu )
   // - preview.png
   // - and in case of a MovieBackground, copy the Movie over to background/MovieName.ext
   bool                    bFound;
@@ -1043,6 +1044,17 @@ bool DialogCreateTemplate::createBackground ( QString &qsTemplatePath )
   theThumbnail = *m_pFrameThumbnail->paletteBackgroundPixmap ( );
   theThumbnail.save ( qsFileName, "PNG", 100 );
   
+  if ( m_pComboType->currentText  ( ) == QString ( "Animated" ) )  {
+    QString qsMenuName = m_pDVDMenu ? m_pDVDMenu->getInterface ( )->qsMenuName : "";
+    QString qsMenuPath = Global::qsTempPath + QString ("/") +  Global::qsProjectName + QString ("/") + qsMenuName;
+    QFileInfo fileInfo ( qsMenuPath + "/menu.m2v" );
+    if ( fileInfo.exists ( ) )    {
+      QString qsCommand = QString ( "ffmpeg -i \"%1\" -t 10 -s 200x134 -y \"%2\" >/dev/null 2>&1" ).arg ( qsMenuPath + "/menu.m2v" ).arg ( qsTemplatePath + "/thumbnail.mpg" );
+      int iRet = system ( qsCommand );
+      iRet = iRet;
+    }
+  }
+
   iWidth  = 720;
   iHeight = 480;
   if ( ! pInterface->backgroundImage.isNull ( ) ) {
