@@ -1,8 +1,10 @@
 /****************************************************************************
 ** Interface to the GUI.
 **
-** Created: Wed Jun 7 22:48:00 2006
-**      by: Varol Okan using xemacs
+**   Created : Wed Jun 7 22:48:00 2006
+**        by : Varol Okan using xemacs
+** Copyright : (c) Varol Okan
+**   License : GPL v 2.0
 **
 **
 ****************************************************************************/
@@ -77,17 +79,23 @@ public:
   void initMe       ( DVDMenu * );
 
 protected:
-  QString createTemplate  ( );
-  QString getTypePath     ( );
+  QString createTemplate   ( );
+  QString getTypePath      ( );
 
-  bool createTemplateXml  ( QString & );
-  bool createBackground   ( QString & );
-  bool createIndexXml     ( );
-  bool createWebHtml      ( );
+  bool createTemplateXml   ( QString & );
+  bool createBackground    ( QString & );
+  bool createIndexXml      ( );
+  bool createWebHtml       ( );
 
-  QString findGuiSU       ( );
-  void    installToSystem ( QString &, bool );
-  QString handleButtonXml ( int, QString & );
+  QString findGuiSU        ( );
+  void    installToSystem  ( QString &, bool );
+  QString handleButtonXml  ( int, QString  & );
+
+  void handleMenuImages    ( QString &, QString & );
+  void handleMenuBackground( QString & );
+  void handleMenuDropZones ( QString & );
+  void handleMenuMovie     ( QString & );
+  void handleMenuAudio     ( QString &, QString & );
 
 protected slots:
   virtual void polish                ( );
