@@ -62,6 +62,8 @@ DialogExecute::DialogExecute(QValueList<ExecuteInterface *> listExecuteBlocks, b
 	m_iTranscodeOffset = 0;
 	m_pSubtitlesParent = NULL;
 	m_pSubtitlesLayout = NULL;
+	m_pSlideshowParent = NULL;
+	m_pSlideshowLayout = NULL;
 	m_iSubtitlesOffset = 0;
 
 	// Here we concat all commands together ...
@@ -113,14 +115,14 @@ void DialogExecute::slotResize ( )
   slotCheckBlock ( true );
 }
 
-void DialogExecute::setInit(QDVDAuthorInit *pInit)
+void DialogExecute::setInit ( QDVDAuthorInit *pInit )
 {
 	m_pInitObject = pInit;
 	// Here we save over the ToolsPaths. This list will be removed from the InitObject ...
-	m_listToolsPaths = pInit->getToolsPaths ();
+	m_listToolsPaths = pInit->getToolsPaths ( );
 }
 
-void DialogExecute::createMainForm ()
+void DialogExecute::createMainForm ( )
 {
 	setName( "DialogExecute" );
 	setSizeGripEnabled( TRUE );
@@ -152,13 +154,13 @@ void DialogExecute::createMainForm ()
 	m_pTabWidget = new QTabWidget( this, "m_pTabWidget" );
 	m_pTabWidget->setEnabled( TRUE );
 
-		// signals and slots connections
-	connect( buttonOk     , SIGNAL( clicked() ), this, SLOT( slotOkay  () ) );
-	connect( buttonCancel , SIGNAL( clicked() ), this, SLOT( reject    () ) );
-	connect( buttonExport , SIGNAL( clicked() ), this, SLOT( slotExport() ) );
-	connect( buttonHelp ,   SIGNAL( clicked() ), this, SLOT( slotHelp  () ) );
-	connect( m_pTabWidget , SIGNAL( currentChanged  (QWidget*)), this, SLOT(slotTabChanged(QWidget *)));
-	connect(&m_doSomething, SIGNAL( signalRenderMenu(QString )), this, SLOT(slotRenderMenu(QString  )));
+	// signals and slots connections
+	connect ( buttonOk     , SIGNAL ( clicked() ), this, SLOT( slotOkay  ( ) ) );
+	connect ( buttonCancel , SIGNAL ( clicked() ), this, SLOT( reject    ( ) ) );
+	connect ( buttonExport , SIGNAL ( clicked() ), this, SLOT( slotExport( ) ) );
+	connect ( buttonHelp ,   SIGNAL ( clicked() ), this, SLOT( slotHelp  ( ) ) );
+	connect ( m_pTabWidget , SIGNAL ( currentChanged   ( QWidget * ) ), this, SLOT ( slotTabChanged ( QWidget * ) ) );
+	connect (&m_doSomething, SIGNAL ( signalRenderMenu ( QString   ) ), this, SLOT ( slotRenderMenu ( QString   ) ) );
 }
 
 void DialogExecute::createPreprocessing ()
@@ -399,6 +401,18 @@ void DialogExecute::createSubtitlesBlock ( ExecuteInterface *pInterface, int t )
   createStandardBlock ( m_pSubtitlesParent, m_pSubtitlesLayout, pInterface, t );
 }
 
+void DialogExecute::createSlideshowBlock ( ExecuteInterface *pInterface, int t )
+{
+  if  ( ! m_pSlideshowLayout )	{
+    QString qsBlockName     = pInterface->qsBlockName;
+    pInterface->qsBlockName = tr ( "Slideshow Block" );
+    m_pSlideshowLayout      = createBlockFrame    ( m_pScrollFrame, m_pScrollFrameLayout, pInterface, t ); 
+    m_pSlideshowParent      = m_listGroupBox.last ( );
+    pInterface->qsBlockName = qsBlockName;
+  }
+  createStandardBlock ( m_pSlideshowParent, m_pSlideshowLayout, pInterface, t );
+}
+
 void DialogExecute::createDVDAuthorTab   (QValueList<ExecuteInterface *> listExecuteBlocks)
 {
 	uint t;
@@ -417,10 +431,13 @@ void DialogExecute::createDVDAuthorTab   (QValueList<ExecuteInterface *> listExe
 
 	for ( t=0; t<listExecuteBlocks.count ( ); t++ )  {
 	  pExecute = listExecuteBlocks[t];
-	  // Check if this is a transoding block ...
+	  // Check if this is a transcoding block ...
 	  //if ( pExecute->enType == ExecuteInterface::TYPE_TRANSCODING )//( pExecute->listSplitAt.count ( ) > 0 )
 	  if ( pExecute->listSplitAt.count ( ) > 0 )
 	    createTranscodeBlock ( pExecute, t );
+	  else if ( pExecute->enType == ExecuteInterface::TYPE_SLIDESHOW )
+	    //createStandardBlock  ( m_pScrollFrame, m_pScrollFrameLayout, pExecute, t );
+	    createSlideshowBlock ( pExecute, t );
 	  else if ( pExecute->enType == ExecuteInterface::TYPE_SUBTITLES )
 	    //createStandardBlock  ( m_pScrollFrame, m_pScrollFrameLayout, pExecute, t );
 	    createSubtitlesBlock ( pExecute, t );
@@ -768,7 +785,7 @@ void DialogExecute::createManPages(QStringList &listOfCommandsUsed)
 	pManDialog->show();
 }
 
-void DialogExecute::slotExport()
+void DialogExecute::slotExport ( )
 {
   uint t, iCounter = 0;
   int  i;
@@ -777,11 +794,11 @@ void DialogExecute::slotExport()
   GuiBlock *pGuiBlock;
   if (fileName.isNull ())
     return;
-	
+
   QFileInfo fileInfo(fileName);
   if (fileInfo.extension(FALSE).isEmpty())
     fileName += QString (".sh");
-  
+
   QFile shellFile(fileName);
   QString qsComment;
   if ( shellFile.open( IO_WriteOnly ) ) {
@@ -794,7 +811,7 @@ void DialogExecute::slotExport()
     stream << "fi;\n\n";
     stream << "while [ $# -gt 0 ] || [ $run -eq 1 ]\n";
     stream << "do\n\n";
-    
+
     for (t=0;t<m_listGui.count();t++)	{
       pGuiBlock = m_listGui[t];
       // First we check if the user wants this command to get executed after all ...
@@ -820,10 +837,26 @@ void DialogExecute::slotExport()
 	stream << "fi;\n\n";
       }
     }
+
+    if ( Global::iRegionalZone > 0 )  {
+      stream << "if [ 1 -eq $run ] || [ " << ++iCounter << " -eq $1 ];\n";
+      stream << "then\n";
+      stream << "# Here we write the Regional Zone code to the final DVD.\n";
+      stream << "#############################################################################\n";
+
+      QString qsRegionalZone = QString ( "%1" ).arg ( Global::iRegionalZone );
+      QString qsQRender = "qrender --zone " + qsRegionalZone + " --path " + Global::qsProjectPath + "\n";
+      qsQRender = getToolsPath ( qsQRender );
+      stream << qsQRender;
+      stream << "fi;\n\n";
+    }
+
+
     stream << "shift\n";
     stream << "run=0\n";
     stream << "done\n\n";
-    shellFile.close();
+
+    shellFile.close ( );
     chmod ((const char *)fileName, S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
     emit (signalGenerateMasks ());
   }
@@ -853,7 +886,7 @@ void DialogExecute::slotOkay()
       listCommands.append ( getToolsPath ( pGuiBlock->pComboCommands->currentText ( ) ) );
   }
   m_doSomething.system ( listCommands );
-  
+
   hide ();
 
   // for a blocking execution one has to start a timer, which checks m_doSomthing.isRunning()
