@@ -18,9 +18,11 @@
 
 
 #include <qlistview.h>
+#include <qmenudata.h>
 #include <qlistbox.h>
 #include <qprocess.h>
 #include <qpixmap.h>
+#include <qmap.h>
 
 #include "qplayer/mediascanner.h"
 #include "sourcefileentry.h"
@@ -30,6 +32,8 @@
 class QIconViewItem;
 class QIconDragItem;
 class QPushButton;
+class QScrollView;
+class QToolBox;
 class QTimer;
 
 class SourceToolBar;
@@ -133,6 +137,37 @@ class DialogFiles : public uiDialogFiles
     QString  qsPathName;
     bool     bReadable;
   };
+  class Manual  {
+    // This class stores manual assignments of subtitles / audio tracks to video source files.
+    // Instead of keeping this info around forever in the cache or project file, we limit it
+    // to a working session of QDVDAuthor
+  public:
+    class Change  {
+    public:
+       Change ( );
+      ~Change ( );
+
+       void clearAudioList    ( );
+       void clearSubtitleList ( );
+       void copyAudioList     ( QValueList<Audio *>         & );
+       void copySubtitleList  ( QValueList<SubtitleEntry *> & );
+       QValueList<Audio *>         copyAudioList     ( );
+       QValueList<SubtitleEntry *> copySubtitleList  ( );
+
+       QString qsFileName;
+       unsigned long long iHashValue;
+       QValueList<Audio *> audioList;
+       QValueList<SubtitleEntry *> subtitleList;
+    };
+
+     Manual ( );
+    ~Manual ( );
+
+     QString qsPath;  // The change is for a video file in this path
+     unsigned long long iHashValue; // instead of searching path names, use a map of hash values.
+     QMap<unsigned long long, Change *> mapOfChange;
+  };
+
 protected:
   // When the image is rendered the returned data is temp stored in this struct
   // to be executed in timerEvent.
@@ -177,13 +212,26 @@ protected:
      Thumbs ( );
     ~Thumbs ( );
 
+    bool findTracks    ( );
+    void clearAudio    ( );
+    void clearSubtitles( );
+    bool audioTrackEnabled    ( QString & );
+    bool subtitleTrackEnabled ( QString & );
+    void toggleAudioTrack     ( Audio          *  );
+    void toggleSubtitleTrack  ( SubtitleEntry  *  );
+    bool getManualChange      ( Manual::Change *& );
+    bool getManualChange      ( Manual *&, Manual::Change *&, unsigned long long &, unsigned long long & );
+
     QIconViewItem *pOwnerItem;
     QPixmap      **arrayOfPix; // these pixmaps are the scaled versions of pEntry->arrayOfThumbs
     uint           iNumberOfPix;
     uint           iCurrentPix;
+
     // we need a pointer to the corresponding cache object in here
     Cache::Thumbs::Entry *pCache;
-    SourceFileInfo *pSourceFileInfo; // only set if the item is created through a drop from a GroupView.
+    SourceFileInfo *pSourceFileInfo;          // only set if the item is created through a drop from a GroupView.
+    QValueList<Audio *>         audioList;    // the list of associated Audio tracks
+    QValueList<SubtitleEntry *> subtitleList; // the list of associated subtitle tracks
   };
 
 public:
@@ -200,25 +248,31 @@ public: // public strctures ...
   DialogFiles ( QWidget *, bool b=false, const char *n=0, WFlags f=0 );
   virtual ~DialogFiles ( );
 
+  static void clearMapOfChange ( );
+
   void initMe        ( );
   void initWork      ( QIconViewItem *, QImage *, bool );
   bool initScanDV    ( Thumbs * );
   void addProgress   ( int );
   uint thumbnailSize ( );
+  static bool isMyself ( DialogFiles * );
 
   SourceFileInfo *findSourceFileInfo ( QIconViewItem * );
   QValueList<SourceFileEntry *> &getGroups ( );
 
 protected:
   void initGroups        ( );
+  void createScrollView  ( );
   void copyEntry         ( SourceFileEntry *, SourceFileEntry * );
   GroupView *addGroup    ( QString &, QStringList & );
   void addSourceToGroup  ( GroupView * );
   QString uniqueGroupName( QString );
   bool findDesktopDir    ( QString );
   void initVirtualFolders( );
+  void initMapOfChange   ( );
   void initDirectories   ( );
   void createIconStars   ( );
+  bool isValid           ( Thumbs * );
   bool isValid           ( QIconViewItem * );
   bool removeThumb       ( QIconViewItem * );
   void addPlayButton     ( QIconViewItem *, QPixmap * );
@@ -241,16 +295,20 @@ protected:
   void updateTitlebar    ( );
   void resizeIcons       ( int );
   Thumbs *createFromCache( Cache::Thumbs::Entry * );
-  void currentSourceToPreview    ( );
-  void allSourcesToPreview       ( );
-  void stopThumbing              ( );
-  void play              ( Cache::Thumbs::Entry *, QIconViewItem *p=NULL );
-  void lock              ( );
-  void unlock            ( );
-  bool lockDialog        ( );
-  void unlockDialog      ( );
-  bool isImageDialog     ( );
-  bool isImageDialog     ( QString );
+  QPixmap &createMenuHeader   ( QString,  QSize   );
+  void currentSourceToPreview ( );
+  void allSourcesToPreview    ( );
+  void stopThumbing           ( );
+  void play                ( Cache::Thumbs::Entry *, QIconViewItem *p=NULL );
+  void lock                ( );
+  void unlock              ( );
+  bool lockDialog          ( );
+  void unlockDialog        ( );
+  bool isImageDialog       ( );
+  bool isImageDialog       ( QString );
+  void applyManualChange   ( SourceFileEntry  * );
+  void mergeAudioTracks    ( QValueList<Audio *> &, SourceFileInfo * );
+  void mergeSubtitleTracks ( QValueList<SubtitleEntry *> &, SourceFileInfo * );
 
   QValueList<QListViewItem *> getSelectedVirtual ( );
 
@@ -286,6 +344,7 @@ protected slots:
   virtual void slotSelectByStars         ( );
   virtual void slotSizeReleased          ( );
   virtual void slotSizeChanged           ( int );
+  virtual void slotAudioSubtitle         ( int );
   virtual void slotRegisterRequests      ( );
   virtual void slotAddGroup              ( );
   virtual void slotEditGroup             ( );
@@ -297,6 +356,7 @@ protected slots:
   virtual void slotPlay                  ( );
   virtual void accept                    ( );
   virtual void reject                    ( );
+  virtual void slotResizeScrollView      ( );
   virtual void slotAutoGroup             ( );
   virtual void slotReload                ( );
   virtual void slotRefresh               ( );
@@ -311,13 +371,16 @@ protected:
   static bool m_bDate;
   static bool m_bLength;
   static int  m_iThumbnailSize;
-  bool        m_bTooSmallForText; // less then 50x50 which is too small to display text
+  static QMap<unsigned long long, Manual *> m_mapOfManualChange; // keep manual changes around for the duration of ths session
+  bool        m_bTooSmallForText;    // less then 50x50 which is too small to display text
+  bool        m_bExitScannFunction;  // For the ImageDialog to exit slotCheckWorker
 
-  QTimer    *m_pTimerAutoOpen;
-  QTimer    *m_pTimerScanDir;
-  QTimer    *m_pTimerThumbing;
-  QListBox  *m_pVirtualFolderDialog;
-  Previewer *m_pPreviewer;
+  QTimer     *m_pTimerAutoOpen;
+  QTimer     *m_pTimerScanDir;
+  QTimer     *m_pTimerThumbing;
+  QListBox   *m_pVirtualFolderDialog;
+  Previewer  *m_pPreviewer;
+  QPopupMenu *m_pContextMenu;
 
   QString   m_qsProjectDir;
   QString   m_qsDocumentsDir;
@@ -341,7 +404,7 @@ protected:
   QMutex    m_mutexDialog;
   QMutex    m_mutexRequest;
 
-  DialogFiles *m_pIAmAlive;
+  static DialogFiles *m_pIAmAlive;
 
   QValueList<ScanDVDate  *>       m_listOfDVScansToDo;
   QValueList<WorkWaiting *>       m_listOfWorkToDo;
@@ -352,6 +415,10 @@ protected:
   QValueList<SourceFileEntry *>   m_listOfGroups;
 
   friend class GroupView;
+
+  // Create the ScrollView or the Groups
+  QScrollView *m_pScrollView;
+  QToolBox    *m_pToolbox;
 };
 
 }; // end namespace Input
