@@ -110,8 +110,9 @@ void DialogFrame::slotRefresh ( )
 void DialogFrame::slotColor ()
 {
   bool bOkay;
-  Rgba transparentColor = Rgba ( TRANSPARENT_COLOR );
-  QRgb rgbColor = QColorDialog::getRgba ( transparentColor.rgb ( ), &bOkay, this );
+//  Rgba transparentColor = Rgba ( TRANSPARENT_COLOR );
+//  QRgb rgbColor = QColorDialog::getRgba ( transparentColor.rgb ( ), &bOkay, this );
+  QRgb rgbColor = QColorDialog::getRgba ( m_colorFrame.rgb ( ), &bOkay, this );
   if ( ! bOkay )
     return;
 
