@@ -1,8 +1,10 @@
 /****************************************************************************
 ** Interface to the GUI.
 **
-** Created: Wed Mar 18 22:48:00 2004
-**      by: Varol Okan using kate editor
+**   Created : Wed Mar 18 22:48:00 2004
+**        by : Varol Okan using kate editor
+** Copyright : (c) Varol Okan
+**   License : GPL v 2.0
 **
 **
 ****************************************************************************/
@@ -36,11 +38,11 @@ protected:
 	DialogFrame *m_pDialogFrame;
 };
 */
+#include "rgba.h"
 #include "uidialogframe.h"
 #include "frameobject.h"
 #include "dialoganimation.h"
 
-class QColor;
 class QPaintEvent;
 class QPainter;
 
@@ -48,34 +50,39 @@ class DialogFrame : public uiDialogFrame
 {
 	Q_OBJECT
 public:
-	 DialogFrame (QWidget * parent, const char * name = 0, WFlags f = 0 );
-	~DialogFrame ();
+	 DialogFrame ( QWidget *, const char *n=0, WFlags f=0 );
+	~DialogFrame ( );
 
-	void initMe(FrameObject *);
+	void initMe  ( );
+	void initMe  ( FrameObject * );
+	void initMe  ( int, int, int, Rgba );
 
-	int      frameWidth ();
-	int      style      ();
-	int      join       ();
-	QColor   color      ();
-	QString &animation  ();
+	int      frameWidth ( );
+	int      style      ( );
+	int      join       ( );
+	Rgba     color      ( );
+	QString &animation  ( );
 
 protected:
 	virtual void paintEvent  (QPaintEvent *);
-	void   drawPreview();
+	void   drawPreview  ( );
+	virtual void polish ( );
 
 protected slots:
-	virtual void slotColor     ();
-	virtual void slotImage     ();
-	virtual void slotPicture   ();
-	virtual void slotAnimation ();
-	virtual void slotImage     (bool);
-	virtual void slotPicture   (bool);
-	virtual void slotWidth     (int);
-	virtual void slotStyle     (int);
-	virtual void slotJoin      (int);
+	virtual void slotColor     ( );
+	virtual void slotImage     ( );
+	virtual void slotPicture   ( );
+	virtual void slotAnimation ( );
+	virtual void slotImage     ( bool );
+	virtual void slotPicture   ( bool );
+	virtual void slotWidth     ( int  );
+	virtual void slotStyle     ( int  );
+	virtual void slotJoin      ( int  );
+	virtual void slotRefresh   ( );
 
 private:
-	QColor  m_colorFrame;
+	FrameObject *m_pOrigFrame;
+	Rgba    m_colorFrame;
 	int     m_iFrameWidth;
 	int     m_iFrameStyle;
 	int     m_iFrameJoin;
