@@ -35,9 +35,10 @@
 #include "previewer.h"
 #include "qdvdauthor.h"
 #include "dialogimages.h"
+#include "xml_slideshow.h"
+#include "slidedefaults.h"
 #include "dialogslideshow.h"
 #include "qplayer/mediacreator.h" // for MEDIASCANNER_EVENT - id
-#include "xml_slideshow.h"
 
 
 #define MAX_THREADS          8
@@ -100,8 +101,7 @@ void DialogImages::initMe ( )
   if ( pGroupTab )
     m_pTabWidget->setTabLabel ( pGroupTab, QString ( "Slideshows" ) );
 
-// m_iMultiThumbNumber = 1;
-  m_iMaxThreads       = 2; // default to 2 worker threads
+  m_iMaxThreads = 2; // default to 2 worker threads
   if ( getenv ( "MAX_IMG_RENDER_THREADS" ) )  {
     uint iEnv = QString ( getenv ( "MAX_IMG_RENDER_THREADS" ) ).toUInt ( );
     if ( ( iEnv > 0 ) && ( iEnv <= MAX_THREADS ) )
@@ -122,9 +122,7 @@ void DialogImages::slotEditGroup ( )
   if ( ! pSlideshow )  {
     pSlideshow = new CXmlSlideshow;
     pSlideshow->slideshow_name = qsLabel;
-    pSlideshow->delay          = 5;
-    pSlideshow->filter_delay   = 3;
-    pSlideshow->background     = "#000000";
+    Global::pSlideDefaults->initSlideshow ( pSlideshow );
     pGroupView->setSlideshow ( pSlideshow );
   }
   DialogSlideshow theDialog ( this );
@@ -139,8 +137,10 @@ void DialogImages::slotEditGroup ( )
     pSlideshow->intro_page       = theDialog.m_pCheckIntroPage->isChecked( );
     pSlideshow->audio_list       = theDialog.audioList  ( );
     pSlideshow->validFilterNames = theDialog.filterList ( );
-    pSlideshow->yres             =(theDialog.m_pComboFormat->currentItem ( ) == 0) ? 480 : 576;
+    int iFormat = theDialog.m_pComboFormat->currentItem ( );
+    pSlideshow->yres             = ( ( iFormat == 0 ) || ( iFormat == 2 ) ) ? 480 : 576;
     pSlideshow->xres             = 720;
+    pSlideshow->aspect           = ( ( iFormat == 0 ) || ( iFormat == 1 ) ) ? 0 : 1;  // 4:3 or 16:9
     pSlideshow->setImgBkgImg ( theDialog.m_pRadioImgBkgImg->isChecked ( ) );
   }
 }
@@ -290,11 +290,18 @@ void DialogImages::slotCheckWorker ( )
     if ( ( ! pScan ) || ( ! pScan->pEntry ) )
       continue;
 
-    pScan->createPix ( ); // Moved to the main GUI thread to avoid locking X
+    if ( m_bExitScannFunction )
+      break; // User clicked on a different directory.
+
     pEntry = pScan->pEntry;
+//    if ( pEntry->pItem && ( ! isValid ( pEntry->pItem ) ) )
+//      continue;
+//    pScan->createPix ( ); // Moved to the main GUI thread to avoid locking X
+
     QIconViewItem *pValid = m_pPreview->firstItem ( );
     while ( pValid ) {
       if  ( pEntry->pItem == pValid ) {
+        pScan->createPix ( ); // Moved to the main GUI thread to avoid locking X
         setIconText ( pValid );
         pValid->setPixmap ( pEntry->thePixmap );
 
@@ -320,6 +327,8 @@ void DialogImages::slotCheckWorker ( )
       iCount = 0;
     }
   }
+  m_bExitScannFunction = false;
+
   if ( (int)m_iProgress == m_pProgressBar->totalSteps ( )-1 )
     m_pPreview->arrangeItemsInGrid ( TRUE );
   qApp->processEvents ( 100 );
@@ -345,6 +354,8 @@ void DialogImages::newThumbnailRequest ( QIconViewItem *pItem )
   lock   ( );
     m_listOfImageScansToDo.append       ( pImageScan );
   unlock ( );
+
+  m_bExitScannFunction = false;
   if ( m_listOfWorker.count  ( ) < m_iMaxThreads )  {
        m_listOfWorker.append ( new Worker ( this ) );
        QTimer::singleShot ( 50, this, SLOT( slotCheckWorker ( ) ) );
@@ -523,6 +534,9 @@ void DialogImages::setIconText ( DialogFiles::Thumbs *pThumbs )
   if ( ! pThumbs )
     return;
 
+  if ( ! isValid ( pThumbs->pOwnerItem ) )
+    return;
+
   if ( m_bTooSmallForText )  {
     pThumbs->pOwnerItem->setText ( QString::null );
     return;
@@ -659,8 +673,8 @@ void DialogImages::showContextMenu ( const QPoint &pos, QIconViewItem * )
 {
   int t, iID, iIDs[10], iStarRating = -2;
   bool bVidsSelected = false;
-  Thumbs *pThumbs = NULL;
-  QPoint globalPos  = pos, globalPos2 = pos;
+  Thumbs *pThumbs    = NULL;
+  QPoint globalPos   = pos, globalPos2 = pos;
 
   QIconViewItem *pIcon = m_pPreview->firstItem ( );
   QValueList<QIconViewItem *>listIcons;
@@ -995,10 +1009,8 @@ void DialogImages::accept ( )
       pGroupItem = (GroupView::Item *)pGroupView->firstChild ( );
       if ( pGroupItem ) {
         if ( ! pSlideshow )  {
-          pSlideshow  = new CXmlSlideshow;
-          pSlideshow->delay        = 5;
-          pSlideshow->filter_delay = 3;
-          pSlideshow->background   = "#000000";
+          pSlideshow = new CXmlSlideshow;
+          Global::pSlideDefaults->initSlideshow ( pSlideshow );
         }
         pSlideshow->slideshow_name = m_pToolbox->itemLabel ( t );
         // We need to take the slideshow object away from the GroupView object
@@ -1017,7 +1029,7 @@ void DialogImages::accept ( )
         pSlideshow->clearObjects ( );
         pSlideshow->audio_list       = tempSlideshow.audio_list;
         pSlideshow->validFilterNames = tempSlideshow.validFilterNames;
-        while ( pGroupItem )  {
+        while ( pGroupItem &&   pGroupItem->pCache )  {
           if  ( isImageDialog ( pGroupItem->pCache->qsFileName ) )  {
             pXmlImg       = pSlideshow->addImg ( );
             pXmlImg->src  = pGroupItem->pCache->qsFileName;
