@@ -21,7 +21,8 @@ class QIconView;
 class DialogLibrary : public uiDialogLibrary
 {
     Q_OBJECT
-    
+
+protected:
     class LibraryPart {
     public:
       LibraryPart ( QIconView *, QString );
@@ -30,15 +31,15 @@ class DialogLibrary : public uiDialogLibrary
     };
 
 public:
-   DialogLibrary ( QWidget *p=0, const char *n=NULL, bool m=FALSE, WFlags f=0 );
-  ~DialogLibrary ( );
+           DialogLibrary ( QWidget *p=0, const char *n=NULL, bool m=FALSE, WFlags f=0 );
+  virtual ~DialogLibrary ( );
 
-  void initMe    ( );
   void createTab ( QString, QString, QStringList & );
 
   static bool m_bCreateButtonMask;
 
 protected:
+  void initMe ( );
   void setTransparentColor ( QImage & );
 
 protected slots:
@@ -47,7 +48,7 @@ protected slots:
   virtual void slotChecked   ( );
   virtual void slotAfterInit ( );
 
-private:
+protected:
   int m_iWidth;
   int m_iHeight;
   QValueList<LibraryPart *>  m_listLibrary;
