@@ -13,6 +13,7 @@
 **
 *****************************************************************************/
 #include <qwmatrix.h>
+#include <qtimer.h>
 
 #include "menuobject.h"
 #include "imageobject.h"
@@ -20,19 +21,19 @@
 
 void DialogMatrix::slotClose()
 {
-    accept();
+  accept ( );
+  QTimer::singleShot ( 10, m_pParent, SLOT ( slotDeleteMtxDlg ( ) ) );
 }
 
 
-void DialogMatrix::slotZoom(int iPos)
+void DialogMatrix::slotZoom ( int iPos )
 {
-	double fZoom   = (double)(iPos / 1000.0);
-	if (fZoom > 1.0)
-		fZoom = fZoom * 2.0 - 1.0;
-	m_pEditZoom->setText (QString ("%1").arg(fZoom));
-	m_pParent->modifiers()->fScaleX = fZoom;
-	m_pParent->modifiers()->fScaleY = fZoom;
-	m_pParent->update();
+  double fZoom   = (double) ( iPos / 1000.0 );
+  m_pEditZoom->setText ( QString ("%1").arg ( fZoom ) );
+  m_pParent->modifiers ( )->fScaleX = fZoom;
+  m_pParent->modifiers ( )->fScaleY = fZoom;
+
+  m_pParent->update    ( );
 }
 
 void DialogMatrix::slotRotate(int iPos)
@@ -43,7 +44,6 @@ void DialogMatrix::slotRotate(int iPos)
 	m_pParent->update();
 }
 
-
 void DialogMatrix::slotShearX(int iPos)
 {
 	double fShear = (double)(iPos / 1000.0);
@@ -62,35 +62,36 @@ void DialogMatrix::slotShearY (int iPos)
 }
 
 
-void DialogMatrix::initMe(MenuObject *pParent)
+void DialogMatrix::initMe ( MenuObject *pParent )
 {
 	m_pParent = pParent;
 	// Here we set the matrix values ...
-	m_pSliderRotate->setRange (-1800, +1800);
-	m_pSliderZoom  ->setRange (0, +2000);
-	m_pSliderShearX->setRange (-2000, +2000);
-	m_pSliderShearY->setRange (-2000, +2000);
-	m_pSliderRotate->setValue ((int)(pParent->modifiers()->fRotate * 10.0)); // 0
-	m_pSliderZoom  ->setValue ((int)(pParent->modifiers()->fScaleX * 1000.0)); // 1000
-	m_pSliderShearX->setValue ((int)(pParent->modifiers()->fShearX * 1000.0)); // 1000
-	m_pSliderShearY->setValue ((int)(pParent->modifiers()->fShearY * 1000.0)); // 1000
+	m_pSliderRotate->setRange ( -1800, +1800 );
+	m_pSliderZoom  ->setRange (     0, +2000 );
+	m_pSliderShearX->setRange ( -2000, +2000 );
+	m_pSliderShearY->setRange ( -2000, +2000 );
+	m_pSliderRotate->setValue ( (int)( pParent->modifiers ( )->fRotate *   10.0 ) ); // 0
+	m_pSliderZoom  ->setValue ( (int)( pParent->modifiers ( )->fScaleX * 1000.0 ) ); // 1000
+	m_pSliderShearX->setValue ( (int)( pParent->modifiers ( )->fShearX * 1000.0 ) ); // 1000
+	m_pSliderShearY->setValue ( (int)( pParent->modifiers ( )->fShearY * 1000.0 ) ); // 1000
 }
 
-void DialogMatrix::slotSliderPressed()
+void DialogMatrix::slotSliderPressed ( )
 {
-	// Initiate the fast drawing algol of the MenuObject ...
-	QMouseEvent fakeMouseEvent (QEvent::MouseButtonPress, QPoint(), Qt::LeftButton, Qt::LeftButton);
-	m_pParent->mousePressEvent(&fakeMouseEvent);
+  // Initiate the fast drawing algol of the MenuObject ...
+  QMouseEvent fakeMouseEvent ( QEvent::MouseButtonPress, QPoint ( ), Qt::LeftButton, Qt::LeftButton );
+  m_pParent->mousePressEvent ( &fakeMouseEvent );
 }
 
-void DialogMatrix::slotSliderReleased()
+void DialogMatrix::slotSliderReleased ( )
 {
-	QMouseEvent fakeMouseEvent (QEvent::MouseButtonPress, QPoint(), Qt::LeftButton, Qt::LeftButton);
-	m_pParent->mouseReleaseEvent(&fakeMouseEvent);
-	m_pParent->update();
+	QMouseEvent fakeMouseEvent   ( QEvent::MouseButtonPress, QPoint(), Qt::LeftButton, Qt::LeftButton );
+	m_pParent->mouseReleaseEvent ( &fakeMouseEvent );
+	m_pParent->update ( );
+
 	ImageObject theImage;
 	MovieObject theMovie;
-	if (m_pParent->objectType() == theImage.objectType())	{
+	if (m_pParent->objectType() == theImage.objectType ( ) )  {
 		ImageObject *pImage = (ImageObject *)m_pParent;
 		pImage->updatePixmap();
 	}
@@ -102,21 +103,17 @@ void DialogMatrix::slotSliderReleased()
 
 void DialogMatrix::slotDeleteObject( MenuObject * )
 {
-  accept();
+  accept ( );
 }
 
-void DialogMatrix::slotZoomChanged()
+void DialogMatrix::slotZoomChanged ( )
 {
   bool  bOk    = false;
   float fValue = m_pEditZoom->text().toFloat ( &bOk );
   if ( bOk ) {
-    m_pParent->modifiers()->fScaleX = fValue;
-    m_pParent->modifiers()->fScaleY = fValue;
-    m_pParent->update();
-    
-    if (fValue > 1.0)
-        fValue = fValue * 2.0 - 1.0;
-
+    m_pParent->modifiers ( )->fScaleX = fValue;
+    m_pParent->modifiers ( )->fScaleY = fValue;
+    m_pParent->update    ( );
     m_pSliderZoom->setValue ( (int) ( fValue * 1000.0  ) );
   }
 }
@@ -153,3 +150,4 @@ void DialogMatrix::slotShearYChanged()
     m_pSliderShearY->setValue ( (int) ( fValue * 1000.0 ) );
   }
 }
+
