@@ -7,6 +7,8 @@
 **   License : GPL v 2.0
 **
 ****************************************************************************/
+#include <stdlib.h>
+
 #include <qlabel.h>
 #include <qimage.h>
 #include <qtimer.h>
@@ -37,6 +39,7 @@
 #include "qplayer/engines/mediainterface.h"
 
 #include "importsrt.h"
+#include "importssa.h"
 #include "messagebox.h"
 #include "movieobject.h"
 #include "dialogmovie.h"
@@ -47,7 +50,7 @@
 #include "uidialogsubtitle.h"
 #include "smalldialogaudio.h"
 #include "filepreviewdialog.h"
-
+#include "dialogbuttontrans.h"
 
 #define PROPERTIES_TAB  0
 #define SUBTITLE_TAB    1
@@ -79,29 +82,31 @@ void ListViewColorItem::paintCell ( QPainter *p, const QColorGroup & cg, int col
 DialogMovie::DialogMovie(QWidget * parent, const char * name, WFlags f)
 	: uiDialogMovie (parent, name, f)
 {
+	srand ( time ( NULL ) );
 	m_pMediaInterface   = NULL;
 	m_pStartStopControl = NULL;
 	m_pTranscodeTab     = NULL;
 	m_pMovieObject      = NULL;
 	m_iMovieLength      =    1;// 1ms default, so we never have 0
+	m_iTransitionForAll =   -1;
 	m_pDVDMenu          = NULL;
 	m_pSelectItem       = NULL;
 	m_pParentSourceFileEntry = NULL; // needed to get correct audio / subtitle setting for a SourceFileInfo
-	connect (m_pEditWidth,       SIGNAL(lostFocus ()), this, SLOT (slotWidth          ()));
-	connect (m_pEditHeight,      SIGNAL(lostFocus ()), this, SLOT (slotHeight         ()));
-	connect (m_pButtonPlay,      SIGNAL(clicked   ()), this, SLOT (slotPlay           ()));
-	connect (m_pButtonStop,      SIGNAL(clicked   ()), this, SLOT (slotStop           ()));
-	connect (m_pButtonStart,     SIGNAL(clicked   ()), this, SLOT (slotStart          ()));
-	connect (m_pButtonEnd,       SIGNAL(clicked   ()), this, SLOT (slotEnd            ()));
-        connect (m_pButtonThumbnail, SIGNAL(clicked   ()), this, SLOT (slotSetThumbnail   ()));
-	connect (m_pButtonTranscode, SIGNAL(clicked   ()), this, SLOT (slotTranscode      ()));
-	connect (m_pButtonEncodeNow, SIGNAL(clicked   ()), this, SLOT (slotEncodeNow      ()));
-	connect (m_pButtonAnimation, SIGNAL(clicked   ()), this, SLOT (slotAnimation      ()));
-	connect (m_pButtonAddCK,     SIGNAL(clicked   ()), this, SLOT (slotAddColorKey    ()));
-	connect (m_pButtonDelCK,     SIGNAL(clicked   ()), this, SLOT (slotDelColorKey    ()));
-	connect (m_pButtonCK,        SIGNAL(clicked   ()), this, SLOT (slotBrowseColorKey ()));
-	connect (m_pButtonEdit,      SIGNAL(clicked   ()), this, SLOT (slotEditMediaInfo  ()));
-	connect (m_pCheckCreateSubtitles, SIGNAL ( toggled ( bool ) ), this, SLOT ( slotSubLangChanged ( bool ) ) );
+	connect ( m_pEditWidth,       SIGNAL ( lostFocus ( ) ), this, SLOT ( slotWidth          ( ) ) );
+	connect ( m_pEditHeight,      SIGNAL ( lostFocus ( ) ), this, SLOT ( slotHeight         ( ) ) );
+	connect ( m_pButtonPlay,      SIGNAL ( clicked   ( ) ), this, SLOT ( slotPlay           ( ) ) );
+	connect ( m_pButtonStop,      SIGNAL ( clicked   ( ) ), this, SLOT ( slotStop           ( ) ) );
+	connect ( m_pButtonStart,     SIGNAL ( clicked   ( ) ), this, SLOT ( slotStart          ( ) ) );
+	connect ( m_pButtonEnd,       SIGNAL ( clicked   ( ) ), this, SLOT ( slotEnd            ( ) ) );
+        connect ( m_pButtonThumbnail, SIGNAL ( clicked   ( ) ), this, SLOT ( slotSetThumbnail   ( ) ) );
+	connect ( m_pButtonTranscode, SIGNAL ( clicked   ( ) ), this, SLOT ( slotTranscode      ( ) ) );
+	connect ( m_pButtonEncodeNow, SIGNAL ( clicked   ( ) ), this, SLOT ( slotEncodeNow      ( ) ) );
+	connect ( m_pButtonAnimation, SIGNAL ( clicked   ( ) ), this, SLOT ( slotAnimation      ( ) ) );
+	connect ( m_pButtonAddCK,     SIGNAL ( clicked   ( ) ), this, SLOT ( slotAddColorKey    ( ) ) );
+	connect ( m_pButtonDelCK,     SIGNAL ( clicked   ( ) ), this, SLOT ( slotDelColorKey    ( ) ) );
+	connect ( m_pButtonCK,        SIGNAL ( clicked   ( ) ), this, SLOT ( slotBrowseColorKey ( ) ) );
+	connect ( m_pButtonEdit,      SIGNAL ( clicked   ( ) ), this, SLOT ( slotEditMediaInfo  ( ) ) );
+	connect ( m_pCheckCreateSubtitles, SIGNAL ( toggled ( bool ) ), this, SLOT ( slotSubLangChanged ( bool ) ) );
 }
 
 DialogMovie::~DialogMovie()
@@ -166,6 +171,7 @@ void DialogMovie::initMe ( DVDMenu *pMenu )
   disconnect ( m_pButtonCK,  SIGNAL ( clicked ( ) ), this, SLOT ( slotBrowseColorKey ( ) ) );
   connect    ( m_pButtonCK,  SIGNAL ( clicked ( ) ), this, SLOT ( slotSizeDialog     ( ) ) );
   connect    ( m_pCheckLoop, SIGNAL ( clicked ( ) ), this, SLOT ( slotMenuLoop       ( ) ) );
+  connect    ( m_pButtonTransition, SIGNAL ( clicked ( ) ), this, SLOT ( slotTransition ( ) ) );
 
   if ( pInterface->qsPost == "Loop" )  {
     m_pCheckLoop->setChecked ( true );
@@ -260,6 +266,8 @@ void DialogMovie::initMe ( SourceFileEntry *pSourceFileEntry )
   m_pEditEnding1     = NULL;
   m_pComboPreCommand->setCurrentText  ( pSourceFileEntry->qsPre  );
   m_pComboPostCommand->setCurrentText ( pSourceFileEntry->qsPost );
+  m_pLabelTransition->hide  ( );
+  m_pButtonTransition->hide ( );
 
   setCaption ( "Transcoding all files of the SourceFileEntry" );
   m_pTextTitle->setText (tr ("SourceFileEntry : %1 Files").arg(pSourceFileEntry->listFileInfos.count()));
@@ -293,7 +301,7 @@ void DialogMovie::initMe ( SourceFileInfo *pInfo, const void *pExistingMediaEngi
   int t, i, iThumbnailSize = pDVDAuthor->getDragNDropContainer()->sizeThumbnail.width ( );
   SourceFileEntry *pEntry  = NULL;
   for ( t=0; t<pDVDAuthor->sourceFileCount  ( ); t++ ) {
-    pEntry  = pDVDAuthor->sourceFileEntry ( t );
+    pEntry   = pDVDAuthor->sourceFileEntry ( t );
     if ( pEntry ) {
       for ( i=0; i<(int)pEntry->listFileInfos.count ( ); i++ ) {
         if ( pEntry->listFileInfos[i] == pInfo ) {
@@ -311,27 +319,20 @@ void DialogMovie::initMe ( SourceFileInfo *pInfo, const void *pExistingMediaEngi
   m_pTabWidget->removePage (pGeometry);
   m_pTabWidget->removePage (m_pTranscodeTab);
   delete pGeometry;
+  m_pLabelTransition->hide  ( );
+  m_pButtonTransition->hide ( );
   // we keep the Transcode widget around in case the user wants to transcode the file.
 
   // And next we init the chapters - tab
   int iLength = theUtil.getMsFromString ( pInfo->qsLength );
 
-  // Check if the default 5 minutes chapter creation is out of range anyways ...
-  QString qsReturn = theUtil.getStringFromMs ( iLength );
-  if (iLength < 5*60*1000)	{
-    m_pEditEvery->setText (QString ("%1").arg((int)(iLength/1000)));
-    m_pEditAdd->setText (qsReturn);
-  }
-  m_pEditEnding1->setText(qsReturn);
   m_pCheckLoop->hide ( );
 
   // Here we copy the infos over ...
   m_sourceFileInfo = *pInfo;
-  QFileInfo fileInfo(m_sourceFileInfo.qsFileName);
 
   // and now on to initializing the dialog. Length : 00:07:23.123 File : Cool Video.mpg
-  m_pTextTitle->setText (tr ("Length : <%1> File : %2").arg(qsReturn).arg(fileInfo.fileName()));
-  setCaption ( fileInfo.fileName ( ) );
+  setHeader ( pInfo );
 
   m_pListBoxChapters->clear ( );
   if ( m_sourceFileInfo.listChapters.count  ( ) == 0 )
@@ -361,14 +362,14 @@ void DialogMovie::initMe ( SourceFileInfo *pInfo, const void *pExistingMediaEngi
   // This is the new version, where I have a new WidgetControl.
   QGridLayout *pLayout = new QGridLayout      ( m_pFrameControl, 1, 1, 2, 2, "pLayout");
   m_pStartStopControl  = new StartStopControl ( m_pFrameControl );
-  m_pStartStopControl->initMe ( m_sourceFileInfo.qsFileName, iThumbnailSize, iLength);
+  m_pStartStopControl->initMe ( m_sourceFileInfo.qsFileName, iThumbnailSize, iLength );
   connect (m_pStartStopControl, SIGNAL (signalSliderValues   ( float, float ) ), this, SLOT ( slotSliderValues   ( float, float ) ) );
   connect (m_pStartStopControl, SIGNAL (signalSliderReleased ( bool ) ),         this, SLOT ( slotSliderReleased ( bool ) ) );
 
   initAudioTab ( &m_sourceFileInfo );
   m_iMovieLength = iLength;
   if ( m_iMovieLength < 1 )
-    m_iMovieLength = 1;
+       m_iMovieLength = 1;
   pLayout->addWidget( m_pStartStopControl, 0, 0 );
 
   if ( m_sourceFileInfo.pTranscodeInterface )	{
@@ -414,6 +415,30 @@ void DialogMovie::initMe ( SourceFileInfo *pInfo, const void *pExistingMediaEngi
   QTimer::singleShot ( 100, this, SLOT ( slotSetMediaInfo ( ) ) );
 }
 
+void DialogMovie::setHeader ( SourceFileInfo *pInfo )
+{
+  if ( ! pInfo )
+    return;
+
+  Utils theUtil;
+  QFileInfo fileInfo ( pInfo->qsFileName );
+
+  // And next we init the chapters - tab
+  int iLength = theUtil.getMsFromString ( pInfo->qsLength );
+
+  // Check if the default 5 minutes chapter creation is out of range anyways ...
+  QString qsReturn = theUtil.getStringFromMs ( iLength );
+  if ( iLength < 5*60*1000 )      {
+    m_pEditEvery->setText ( QString ("%1").arg ( (int)( iLength / 1000 ) ) );
+    m_pEditAdd->setText   ( qsReturn );
+  }
+  m_pEditEnding1->setText ( qsReturn );
+
+  // and now on to initializing the dialog. Length : 00:07:23.123 File : Cool Video.mpg
+  m_pTextTitle->setText ( tr ( "Length : <%1> File : %2" ).arg ( pInfo->qsLength ).arg ( fileInfo.fileName ( ) ) );
+  setCaption ( fileInfo.fileName ( ) );
+}
+
 void DialogMovie::slotSetMediaInfo ( )
 {
   // Called from initMe ( SourceFileInfo * ... )
@@ -488,9 +513,7 @@ void DialogMovie::initMe ( MovieObject *pMovieObject )
   clipTime = clipTime.addMSecs ( m_iMovieLength );
   if  ( m_iMovieLength > 0 )
         endingTime = clipTime;
-
   m_pEditEnding2->setText ( endingTime.toString  ( qsFormat ) );
-  m_pCheckLoop->hide ( );
 
   m_rectGeometry = pMovieObject->rect     ( );
   m_qsFileName   = pMovieObject->fileName ( );
@@ -642,7 +665,6 @@ void DialogMovie::initAudioTab ( SourceFileInfo *pInfo, DVDMenu *pMenu )
 {
   int t;
   Utils      theUtils;
-  Audio     *pAudio;
   Subtitles *pSubtitle;
   if ( pInfo ) {
     for ( t=0; t<MAX_SUBTITLES; t++ ) {
@@ -652,15 +674,8 @@ void DialogMovie::initAudioTab ( SourceFileInfo *pInfo, DVDMenu *pMenu )
         new QListViewItem ( m_pListViewSubtitles, QString ( "%1" ).arg ( pSubtitle->m_iSubtitleNumber ), pSubtitle->m_qsIso639, theUtils.iso639 ( pSubtitle->m_qsIso639, false ), QString ( "%1" ).arg ( pSubtitle->m_listOfSubtitles.count ( ) ) );
       }
     }
-
-    for ( t=0; t<MAX_AUDIO; t++ ) {
-      pAudio = pInfo->arrayAudio[t];
-      if ( pAudio ) {
-        // #, code, Language, file
-        new QListViewItem ( m_pListViewAudio, QString ( "%1" ).arg ( pAudio->iAudioNumber ), pAudio->qsIso639, theUtils.iso639 ( pAudio->qsIso639, false ), pAudio->qsFileName );
-      }
-    }
-
+    // The first pass we figure out the set audio tracks.
+    createAudioList ( );
   }
   else if ( pMenu ) {
     CDVDMenuInterface *pInterface = pMenu->getInterface  ( );
@@ -689,6 +704,43 @@ void DialogMovie::initAudioTab ( SourceFileInfo *pInfo, DVDMenu *pMenu )
   connect ( m_pListViewSubtitles,    SIGNAL ( pressed ( QListViewItem * )), this, SLOT ( slotSubtitleSelectionChanged ( QListViewItem * ) ) );
 }
 
+void DialogMovie::createAudioList ( )
+{
+  // Based on m_sourceFileInfo
+  int t;
+  Utils theUtils;
+  m_pListViewAudio->clear ( );
+  Audio *arrayAudio[MAX_AUDIO], *pAudio;
+  int iMaxAudio = 0; // Audio Track sohould always be displayed.
+  for ( t=0; t<MAX_AUDIO; t++ )
+    arrayAudio[t] = NULL;
+  for ( t=0; t<MAX_AUDIO; t++ )  {
+    pAudio = m_sourceFileInfo.arrayAudio[t];
+    if ( pAudio )  {
+      if ( iMaxAudio < pAudio->iAudioNumber )
+           iMaxAudio = pAudio->iAudioNumber;
+      if ( pAudio->iAudioNumber < MAX_AUDIO )
+           arrayAudio[pAudio->iAudioNumber] = pAudio;
+    }
+  }
+  iMaxAudio++;
+  // Note: m_pParentSourceFileEntry should be set.
+  for ( t=0; t<iMaxAudio; t++ ) {
+    pAudio = arrayAudio[t]; //pInfo->arrayAudio[t];
+    if ( pAudio ) {
+      // #, code, Language, file
+      new QListViewItem ( m_pListViewAudio, QString ( "%1" ).arg ( pAudio->iAudioNumber ), pAudio->qsIso639, theUtils.iso639 ( pAudio->qsIso639, false ), pAudio->qsFileName );
+    }
+    else if ( t == 0 )  {
+      QFileInfo fileInfo ( m_sourceFileInfo.qsFileName );
+      QString qsAudioName = tr ( "Audio track of : " ) + fileInfo.fileName ( );
+      new QListViewItem  ( m_pListViewAudio, QString ( "%1" ).arg ( t ), "--", "---", ( m_sourceFileInfo.bHasAudioTrack ) ? qsAudioName : tr ( "No Audio" ) );
+    }
+    else
+      new QListViewItem ( m_pListViewAudio, QString ( "%1" ).arg ( t ), "--", "---", tr ( "No Audio" ) );
+  }
+}
+
 void DialogMovie::slotAudioSelectionChanged ( QListViewItem * )
 {
   QListViewItem *pItem = m_pListViewAudio->selectedItem ( );
@@ -779,11 +831,11 @@ void DialogMovie::slotAddAudio ( )
     return;
   }
   // handling multiple audio streams for a SourceFileInfo ...
-  int t, iValue, iSelectedValue;
+  int iValue, iSelectedValue;
   Utils          theUtils;
   DialogAudio    theDialog;
   QString        qsCode, qsLang, qsFileName;
-  QListViewItem *pItem, *pSelectedItem    = NULL;
+  QListViewItem      *pSelectedItem       = NULL;
   TranscodeInterface *pTranscodeInterface = NULL;
 
   pTranscodeInterface = checkRightAudioFormat ( qsSelFileName );
@@ -797,8 +849,8 @@ void DialogMovie::slotAddAudio ( )
 
   // At this point both loal copy and original have been updated to reflect the changes 
   // made in DialogAudio.
-  
-  /* 
+
+  /*
   { // VAROL
     SourceFileEntry *p = m_pParentSourceFileEntry;
     printf ( "%s::%s : %d > iSelected<%d> ORIG > ", __FILE__, __FUNCTION__, __LINE__, iSelectedValue );
@@ -814,31 +866,18 @@ void DialogMovie::slotAddAudio ( )
   }
   */
   // we ought to reflect the changes in the ListView ...
-  m_pListViewAudio->clear ( );
-  for ( t=0; t<MAX_AUDIO; t++ ) {
-    if ( ! m_sourceFileInfo.arrayAudio [ t ] )
-      continue;
-    iValue = m_sourceFileInfo.arrayAudio [ t ]->iAudioNumber;
-    if ( iValue == -1 )
-      continue;
+  createAudioList ( );
 
-    if ( iValue == iSelectedValue ) {
-      if ( m_sourceFileInfo.arrayAudio [ t ]->pTranscodeInterface ) 
-	delete m_sourceFileInfo.arrayAudio [ t ]->pTranscodeInterface;
-      // simply copy the object pointer does the trick ...
-      m_sourceFileInfo.arrayAudio [ t ]->pTranscodeInterface = pTranscodeInterface;
-      m_sourceFileInfo.arrayAudio [ t ]->qsFileName = qsSelFileName;
-      pTranscodeInterface = NULL;
+  QListViewItem *pChild = m_pListViewAudio->firstChild ( );
+  while ( pChild )  {
+    iValue = pChild->text ( 0 ).toInt ( );
+    if ( iValue == iSelectedValue )  {
+      pSelectedItem = pChild;
+      break;
     }
-
-    qsCode     = m_sourceFileInfo.arrayAudio [ t ]->qsIso639;
-    qsFileName = m_sourceFileInfo.arrayAudio [ t ]->qsFileName;
-    qsLang     = theUtils.iso639  ( qsCode, false );
-    //qsFileName = m_sourceFileInfo.qsFileName;
-    pItem = new QListViewItem ( m_pListViewAudio, QString ( "%1" ).arg ( iValue ), qsCode, qsLang, qsFileName );
-    if ( iValue == iSelectedValue )
-      pSelectedItem = pItem;
+    pChild = pChild->nextSibling ( );
   }
+
   if ( pTranscodeInterface )
     delete pTranscodeInterface;
   // And lastly we should selected the newly added Language ...
@@ -858,6 +897,7 @@ void DialogMovie::slotEditAudio ( )
 
   unsigned int t;
   Utils    theUtils;
+
   Audio   *pAudio = NULL;
   QString  qsCode, qsLanguage = "English";
 
@@ -867,22 +907,41 @@ void DialogMovie::slotEditAudio ( )
     for ( t=0; t<m_pDVDMenu->getInterface ( )->listSoundEntries.count ( ); t++ ) {
       pAudio  =  m_pDVDMenu->getInterface ( )->listSoundEntries[t];
       if ( ! pAudio )
-	continue;
+        continue;
       if ( pAudio->qsFileName == qsDisplayName )
-	break;
+        break;
       pAudio = NULL;
     }
   }
   else { // coming from a SourceFileInfo
+    QString qsAudioNr  = pItem->text ( 0 );
+    qsCode             = pItem->text ( 1 );
     QString qsFileName = pItem->text ( 3 );
-    qsCode = pItem->text ( 1 );
-    for ( t=0; t<MAX_AUDIO; t++ ) {
+    // if video has an audio track, then we can't set the language settings for it. Sorry.
+    if ( m_sourceFileInfo.bHasAudioTrack )  {
+      if ( qsAudioNr == "0" )
+        return;
+    }
+    else  {
+      bool bHasTrackZero = false;
+      for ( t=0; t<MAX_AUDIO; t++ )  {
+        pAudio =  m_sourceFileInfo.arrayAudio[t];
+        if ( pAudio && pAudio->iAudioNumber == 0 )  {
+          bHasTrackZero  = true;
+          break;
+        }
+      }
+      if ( ! bHasTrackZero ) // Return if no track zero exists.
+        return;
+    }
+
+    for ( t=0; t<MAX_AUDIO; t++ )  {
       pAudio = m_sourceFileInfo.arrayAudio[t];
       if ( ! pAudio || pAudio->iAudioNumber == -1 )
-	continue;
-      if ( ( qsFileName == pAudio->qsFileName ) && 
-	   ( qsCode     == pAudio->qsIso639   ) )
-	break;
+        continue;
+      if ( ( qsFileName == pAudio->qsFileName ) &&
+           ( qsCode     == pAudio->qsIso639   ) )
+        break;
       pAudio = NULL;
     }
   }
@@ -898,12 +957,15 @@ void DialogMovie::slotEditAudio ( )
   theDialog.setLanguageText ( qsLanguage );
   if (  theDialog.exec ( ) == QDialog::Rejected )
     return;
-  
+
   qsLanguage = theDialog.languageText ( );
   qsLanguage = qsLanguage.remove ( QRegExp ( "<[^>]*>" ) );
   qsCode     = theUtils.iso639 ( qsLanguage, true );
-  if ( pAudio )
+  if ( pAudio )  {
+    if ( m_pParentSourceFileEntry && ( pAudio->qsIso639 != qsCode ) )
+         m_pParentSourceFileEntry->changeAudioTrack ( pAudio->iAudioNumber, qsCode );
     pAudio->qsIso639 = qsCode;
+  }
   pItem->setText  ( 1, qsCode );
   pItem->setText  ( 2, qsLanguage );
 }
@@ -926,32 +988,39 @@ void DialogMovie::slotDeleteAudio ( )
       return;
     for ( t=0; t<MAX_AUDIO; t++ ) {
       pAudio = m_sourceFileInfo.arrayAudio [ t ];
-      if ( pAudio && pAudio->iAudioNumber == iAudioNumber ) {
-	delete pAudio;
+      if ( pAudio && pAudio->iAudioNumber == iAudioNumber )  {
+        delete pAudio;
         m_sourceFileInfo.arrayAudio [ t ] = NULL;
-	break;
+        break;
       }
     }
 
+    if ( ( iAudioNumber == 0 ) && ( m_sourceFileInfo.bHasAudioTrack ) )
+      return;
+
     // remove from m_pParentSourceFileEntry if this has been the last reference to this audio track.
     bFound = false;
+    SourceFileInfo *pInfo;
     if ( m_pParentSourceFileEntry ) {
-      for ( t=0; t<(int)m_pParentSourceFileEntry->listFileInfos.count ( ) || ( ! bFound ); t++ ) {
-	for ( i=0; i<MAX_AUDIO && ( ! bFound ); i++ ) {
-	  pAudio = m_pParentSourceFileEntry->listFileInfos[t]->arrayAudio[i];
-	  if ( pAudio && pAudio->iAudioNumber == iAudioNumber ) {
-	    bFound = true;
-	    break;
-	  }
-	}
+      for ( t=0; t<(int)m_pParentSourceFileEntry->listFileInfos.count ( ) && ( ! bFound ); t++ ) {
+        pInfo = m_pParentSourceFileEntry->listFileInfos[t];
+        if ( pInfo->qsFileName == m_sourceFileInfo.qsFileName )
+          continue;
+        for ( i=0; i<MAX_AUDIO && ( ! bFound ); i++ ) {
+          pAudio = pInfo->arrayAudio[i];
+          if ( pAudio && pAudio->iAudioNumber == iAudioNumber ) {
+            bFound = true;
+            break;
+          }
+        }
       }
       if ( ! bFound ) {
-	m_pParentSourceFileEntry->arrayAudioEntries[ iAudioNumber ].iAudioNumber = -1;
-	m_pParentSourceFileEntry->arrayAudioEntries[ iAudioNumber ].qsIso639     = QString ( );
+        m_pParentSourceFileEntry->arrayAudioEntries[ iAudioNumber ].iAudioNumber = -1;
+        m_pParentSourceFileEntry->arrayAudioEntries[ iAudioNumber ].qsIso639     = QString ( );
       }
     }
     // And finally we remove the item from the ListView ...
-    delete pItem;
+    createAudioList ( );
   }
 }
 
@@ -959,10 +1028,19 @@ void DialogMovie::slotAddSubtitle ( )
 {
   int  t, iValue;
   QString qsLang ( "English" );
-  // First we import the srt - file.
-  Import::Srt theImporter;
 
-  if ( ! theImporter.import ( ) )
+  QString qsImportFile = QFileDialog::getOpenFileName ( Global::qsCurrentPath, QObject::tr ( "Subtitle files (*.srt *.SRT *.ssa *.SSA *.ass *.ASS)"), NULL, QObject::tr ( "Open Subtitles File" ) );
+
+  QFileInfo fileInfo ( qsImportFile );
+  Global::qsCurrentPath = fileInfo.dirPath ( true );
+
+  Import::Srt srtImporter;
+  Import::Ssa ssaImporter;
+  Import::BaseSubtitle *pImporter = &ssaImporter;
+  if ( fileInfo.extension ( false ) == "srt" )
+    pImporter = &srtImporter;
+
+  if ( ! pImporter->readFile ( qsImportFile ) )
     return;
 
   for ( t=0; t<MAX_SUBTITLES; t++ ) {
@@ -977,7 +1055,7 @@ void DialogMovie::slotAddSubtitle ( )
   if ( ! sanityCheckSubtitleTrack ( iValue, qsLang ) )
     return;
 
-  QString qsFileName = theImporter.getFileName ( );
+  QString qsFileName = pImporter->getFileName ( );
   if ( qsFileName.length ( )  >  1 )  {
     QFileInfo fileInfo   ( qsFileName );
     Global::qsCurrentPath = fileInfo.absFilePath ( );
@@ -986,14 +1064,12 @@ void DialogMovie::slotAddSubtitle ( )
   if ( m_sourceFileInfo.arraySubtitles [ iValue ] ) {
     QFileInfo fileInfo ( m_sourceFileInfo.qsFileName );
     // the subtitle already exists in the sourcefileinfo.
-    if ( MessageBox::question ( this, tr ( "Subtitle exists" ), tr ( "Warning, the subtitle number %1 exists already.\nDo you want to remove the current subtitle %2 from the file %3 ?" ).arg ( iValue ).arg ( iValue ).arg ( fileInfo.fileName ( ) ), 
-	QMessageBox::Yes, QMessageBox::No ) == QMessageBox::No )
+    if ( MessageBox::question ( this, tr ( "Subtitle exists" ), tr ( "Warning, the subtitle number %1 exists already.\nDo you want to remove the current subtitle %2 from the file %3 ?" ).arg ( iValue ).arg ( iValue ).arg ( fileInfo.fileName ( ) ), QMessageBox::Yes, QMessageBox::No ) == QMessageBox::No )
       return;
   }
 
-  QValueList<Subtitles::entry *> listOfEntries = theImporter.m_listOfEntries;
-  theImporter.m_listOfEntries.clear ( );
-  addSubtitle ( qsLang, iValue, listOfEntries );
+  QValueList<Subtitles::entry *> listOfEntries = pImporter->takeSubtitles ( );
+  addSubtitle ( qsLang, iValue,  listOfEntries,  pImporter->getFileName ( ) );
 }
 
 void DialogMovie::slotEditSubtitle ( )
@@ -1084,7 +1160,7 @@ bool DialogMovie::subtitleDialog ( int &iNumber, QString &qsLanguage )
   return true;
 }
 
-void DialogMovie::addSubtitle ( QString qsLang, int iValue, QValueList<Subtitles::entry *> &listOfEntries )
+void DialogMovie::addSubtitle ( QString qsLang, int iValue, QValueList<Subtitles::entry *> &listOfEntries, QString qsFileName )
 {
   QString   qsCode, qsCount, qsXmlFile;
   QFileInfo fileInfo;
@@ -1097,7 +1173,12 @@ void DialogMovie::addSubtitle ( QString qsLang, int iValue, QValueList<Subtitles
   qsXmlFile = theUtils.getTempFile ( fileInfo.baseName ( TRUE ) ) + qsXmlFile;
   if ( m_sourceFileInfo.arraySubtitles[iValue] )
     delete m_sourceFileInfo.arraySubtitles[iValue];
-  m_sourceFileInfo.arraySubtitles [ iValue ] = new Subtitles;
+
+  int iWidth  = theUtils.getWHFromResolution ( m_sourceFileInfo.qsResolution, true  );
+  int iHeight = theUtils.getWHFromResolution ( m_sourceFileInfo.qsResolution, false );
+
+  m_sourceFileInfo.arraySubtitles [ iValue ] = new Subtitles ( iWidth, iHeight );
+  m_sourceFileInfo.arraySubtitles [ iValue ]->m_qsFileName      = qsFileName;
   m_sourceFileInfo.arraySubtitles [ iValue ]->m_iSubtitleNumber = iValue;
   m_sourceFileInfo.arraySubtitles [ iValue ]->m_qsIso639        = qsCode;
   m_sourceFileInfo.arraySubtitles [ iValue ]->m_listOfSubtitles = listOfEntries;
@@ -1284,6 +1365,31 @@ void DialogMovie::slotSameAsMenuLength ( )
   }
 }
 
+void DialogMovie::slotTransition ( )
+{
+  // m_iTransitionForAll; // -1==doNothing / 0=remove all transitions / 1=set all transitions
+  if ( m_iTransitionForAll == 1 )  {
+       m_iTransitionForAll = 0; // remove all transitions.
+       m_pButtonTransition->setText ( tr ( "Set Transition" ) );
+       return;
+  }
+
+  DialogButtonTrans dialog ( &m_allTransitions, m_listButtonTransitions  );
+  if ( dialog.exec   ( ) ==  QDialog::Rejected )
+       return;
+
+  if ( dialog.getTransition ( ) )
+       m_allTransitions   = *dialog.getTransition  ( );
+  m_listButtonTransitions =  dialog.getEnabledList ( );
+  if ( m_listButtonTransitions.size ( ) > 0 )  {
+       m_pButtonTransition->setText ( tr ( "Unset Transition" ) );
+       m_iTransitionForAll = 1; // Set transitions
+  }
+  else
+       m_iTransitionForAll = -1; // do Nothing
+
+}
+
 void DialogMovie::slotMenuLoop ( )
 {
   // The user wants the menu video to loop.
@@ -1466,7 +1572,7 @@ void DialogMovie::slotEditMediaInfo ()
   // This will open the media info dialog, where the user can modify the auto detected values (through Xine, VLC or MPlayer)
   DialogMediaInfo theDialog ( this );
   theDialog.initMe ( &m_sourceFileInfo );
-  
+
   if ( theDialog.exec () == QDialog::Accepted ) {
     m_sourceFileInfo.qsRatio       = theDialog.m_pEditVideoAspect->text ( );
     m_sourceFileInfo.qsFPS         = theDialog.m_pEditVideoFPS->text    ( );
@@ -1486,12 +1592,13 @@ void DialogMovie::slotEditMediaInfo ()
 		  .arg ( m_sourceFileInfo.qsBits        )
 		  .arg ( m_sourceFileInfo.qsSample      )
     );
+    setHeader ( &m_sourceFileInfo );
   }
 }
 
-SourceFileInfo *DialogMovie::sourceFileInfo ()
+SourceFileInfo *DialogMovie::sourceFileInfo ( )
 {
-	return &m_sourceFileInfo;
+  return &m_sourceFileInfo;
 }
 
 void DialogMovie::slotNewVolume ( int iNewVolume )
@@ -1917,7 +2024,7 @@ void DialogMovie::slotCurrentChanged (QListBoxItem *pItem)
     int iChapter = 0;
     QTime chapterTime, zeroTime;
     QString qsChapter = m_pListBoxChapters->currentText ();
-    
+
     chapterTime = QTime::fromString  (   qsChapter );
     iChapter = zeroTime.secsTo       ( chapterTime );
     m_pMediaInterface->setPositionByTime ( iChapter );
@@ -2106,7 +2213,7 @@ void DialogMovie::slotSliderValues (float fStart, float fEnd)
   QString qsFormat ("hh:mm:ss.zzz");
   startTime = startTime.addMSecs ( (int)(fStart * m_iMovieLength ) );
   endTime   = endTime.addMSecs   ( (int)(fEnd   * m_iMovieLength ) );
-  if ( ! m_pMovieObject )
+  if ( ( ! m_pMovieObject ) && ( ! m_pDVDMenu  ) )
          m_pEditAdd->setText ( startTime.toString ( qsFormat ) );
   m_pEditStarting1 ->setText ( startTime.toString ( qsFormat ) );
   m_pEditEnding1   ->setText ( endTime.toString   ( qsFormat ) );
@@ -2192,7 +2299,51 @@ TranscodeInterface *DialogMovie::checkRightAudioFormat ( QString &qsFileName )
   return pTranscode;
 }
 
-void DialogMovie::accept ()
+void DialogMovie::applyTransitions ( )
+{
+  // m_iTransitionForAll; // -1==doNothing / 0=remove all transitions / 1=set all transitions
+  if ( ( m_iTransitionForAll == -1 ) || ! m_pDVDMenu )
+    return; // do nothing ...
+
+  QString qsMenuName, qsName, qsTempPath;
+  ButtonTransition transition = m_allTransitions;
+
+  ButtonObject   *pButton = NULL;
+  QValueList<ButtonObject *> list = m_pDVDMenu->getButtons ( );
+  QValueList<ButtonObject *>::iterator it = list.begin ( );
+  if ( m_iTransitionForAll == 0 )  {  // Remove all transitions.
+    while  ( it != list.end ( ) )  {
+      pButton = *it++;
+      pButton->setTransition ( NULL );
+    }
+    return;
+  }
+
+  if ( m_listButtonTransitions.size ( ) < 1 )
+    return;
+
+  int   iIdx;
+  Utils theUtils;
+  qsMenuName = m_pDVDMenu->getInterface ( )->qsMenuName;
+  qsTempPath = theUtils.getTempFile ( "" );
+  while  ( it != list.end ( ) )   {
+    pButton = *it++;
+
+    iIdx = rand ( )%m_listButtonTransitions.size ( );
+    transition.qsTransitionName = m_listButtonTransitions[iIdx];
+
+    qsName = qsMenuName + "/" + pButton->name ( );
+    transition.qsName = qsName;
+    theUtils.recMkdir ( qsTempPath + qsName );
+    qsName += "/transition.vob";
+
+    transition.qsTransitionVideoFileName = qsTempPath + qsName; // E.g. "/tmp/TestProjec/Main VMGM/Button
+    pButton->setTransition ( &transition );
+  }
+  m_pDVDMenu->slotUpdateStructure ( );
+}
+
+void DialogMovie::accept ( )
 {
 	uint      t;
 	long      iBitrate, iTemp;
@@ -2225,7 +2376,7 @@ void DialogMovie::accept ()
 	    pInterface->qsPost  = m_pComboPostCommand->currentText ( );
 
 	  if ( pInterface->qsMovieFileName.isEmpty ( ) )
-		startingTime = QTime ( );
+	       startingTime = QTime ( );
 	  // Here we hande the start/stop time.
 	  if ( startingTime.isValid ( ) )       {
 	    pInterface->timeOffset = startingTime;
@@ -2239,6 +2390,8 @@ void DialogMovie::accept ()
 	       pInterface->iMenu     = iSubMenu;
 	       pInterface->iTitleset = m_pSpinBoxTitleset->value ( );
 	  }
+	  // Lastly we handle the Transition settings.
+	  applyTransitions ( );
 	}
 	else if ( m_pMovieObject )	{
 		// Here we handle MovieObjects. Check out if we have set Color Keying.
@@ -2337,4 +2490,3 @@ void DialogMovie::accept ()
 
 	QDialog::accept ( );
 }
-
