@@ -16,6 +16,7 @@
 
 #include "sourcefileentry.h"
 #include "uidialogmovie.h"
+#include "buttonobject.h"
 
 class MovieObject;
 class DVDMenu;
@@ -97,6 +98,7 @@ protected slots:
 	virtual void slotCheckEnd           ( );
 	virtual void slotLoop               ( );
 	virtual void slotMenuLoop           ( );
+	virtual void slotTransition         ( );
 
 	virtual void slotAddColorKey    ( );
 	virtual void slotDelColorKey    ( );
@@ -118,20 +120,24 @@ signals:
 	void signalSetThumbnail ( long );
 
 private:
-	QString keepClean   ( const QString  & );
-	void setStartStop   ( bool             );
-	void addChapter     ( QString &        );
+	QString keepClean    ( const QString  & );
+	void setStartStop    ( bool             );
+	void addChapter      ( QString &        );
 	void initMediaInterface ( DVDMenu    * );
-	void initAudioTab   ( SourceFileInfo *, DVDMenu *p=NULL );
+	void initAudioTab    ( SourceFileInfo *, DVDMenu *p=NULL );
+	void setHeader       ( SourceFileInfo * );
 	bool sanityCheckSubtitleTrack( int   &, QString & );
-	void addSubtitle    ( QString, int, QValueList<Subtitles::entry *> & );
-	bool subtitleDialog ( int &, QString & );
-	void deleteSubtitle ( int );
-	QTime maxEndTime    (     );
-	QTime duration      (     );
+	void addSubtitle     ( QString, int, QValueList<Subtitles::entry *> &, QString );
+	bool subtitleDialog  ( int &, QString & );
+	void deleteSubtitle  ( int );
+	void createAudioList (     );
+	void applyTransitions(     );
+	QTime maxEndTime     (     );
+	QTime duration       (     );
 	TranscodeInterface *checkRightAudioFormat ( QString & );
 
 private:
+	int               m_iTransitionForAll; // -1==doNothing / 0=remove all transitions / 1=set all transitions
 	QRect             m_rectGeometry;
 	QString           m_qsFileName;
 	QString           m_qsPosition;
@@ -144,6 +150,8 @@ private:
 	SourceFileEntry  *m_pParentSourceFileEntry;
 	MovieObject      *m_pMovieObject;
 	DVDMenu          *m_pDVDMenu;
+	QStringList       m_listButtonTransitions;
+	ButtonTransition  m_allTransitions;
 
 	// the following is only used for the initializaion part
 	const void       *m_pExistingEngine;
