@@ -24,9 +24,9 @@ void CResizeDialog::slotManualResize(const QString &)
 }
 
 
-void CResizeDialog::slotResize( int iState )
+void CResizeDialog::slotResize ( int iState )
 {
-	// Here we filter out the secvond RadioButton, which gets Unset ...
+	// Here we filter out the second RadioButton, which gets Unset ...
 	if (iState != QButton::On)
 		return;
 	int iWidth = 720;
@@ -87,7 +87,6 @@ void CResizeDialog::slotResizeAlogrithm( int iState )
 
 }
 
-
 void CResizeDialog::slotStretchType( int iState )
 {
 	if (iState == QButton::On)
@@ -227,7 +226,7 @@ void CResizeDialog::slotHelp()
 
 // This function is called when there is no file associated to the image
 // I.e. a Drag'n Drop operation ...
-void CResizeDialog::setImage(QImage *pImage)
+void CResizeDialog::setImage ( QImage *pImage )
 {
 	// first we set the avail resize algos ...
 	QImageHelper temp;
@@ -238,6 +237,15 @@ void CResizeDialog::setImage(QImage *pImage)
 	slotPreview();
 }
 
+void CResizeDialog::setSize( int iWidth, int iHeight )
+{
+  QString qsText;
+  qsText = QString ( "%1" ).arg ( iWidth );
+  m_pEditWidth->setText  ( qsText );
+  qsText = QString ( "%1" ).arg ( iHeight );
+  m_pEditHeight->setText ( qsText );
+  }
+
 void CResizeDialog::setFilename( QString &qsFileName )
 {
 	// first we set the avail resize algos ...
@@ -261,3 +269,4 @@ void CResizeDialog::setResizeAlgo( QStringList &algoList )
 {
 	comboResizeAlgorithm->insertStringList (algoList);
 }
+
