@@ -34,6 +34,7 @@
 #include "xml_slideshow.h"
 #include "render_client.h"
 #include "dialogslideshow.h"
+#include "dialogalphatrans.h"
 #include "filepreviewdialog.h"
 #include "qplayer/mediainfo.h"
 
@@ -49,7 +50,9 @@ DialogSlideshow::DialogSlideshow ( QWidget *pParent )
   }
   m_iCurrentPix = 0;
   m_pTimer      = NULL;
+  m_bExitNow    = false;
   m_bAnimated   = false;
+  m_pSlideshow  = NULL;
 
   connect ( m_pButtonOkay,      SIGNAL ( clicked ( ) ), this, SLOT ( accept ( ) ) );
   connect ( m_pButtonCancel,    SIGNAL ( clicked ( ) ), this, SLOT ( reject ( ) ) );
@@ -57,6 +60,7 @@ DialogSlideshow::DialogSlideshow ( QWidget *pParent )
   connect ( m_pButtonColor,     SIGNAL ( clicked ( ) ), this, SLOT ( slotColor       ( ) ) );
   connect ( m_pButtonAdd,       SIGNAL ( clicked ( ) ), this, SLOT ( slotAddAudio    ( ) ) );
   connect ( m_pButtonDelete,    SIGNAL ( clicked ( ) ), this, SLOT ( slotDeleteAudio ( ) ) );
+  connect ( m_pButtonSelect,    SIGNAL ( clicked ( ) ), this, SLOT ( slotSelect      ( ) ) );
   connect ( m_pButtonAll,       SIGNAL ( clicked ( ) ), this, SLOT ( slotAll         ( ) ) );
   connect ( m_pButtonClear,     SIGNAL ( clicked ( ) ), this, SLOT ( slotClear       ( ) ) );
   connect ( m_pButtonAnim,      SIGNAL ( clicked ( ) ), this, SLOT ( slotAnim        ( ) ) );
@@ -71,6 +75,8 @@ DialogSlideshow::DialogSlideshow ( QWidget *pParent )
  */
 DialogSlideshow::~DialogSlideshow ( )
 {
+  m_pSlideshow = NULL;
+
   if ( m_pTimer )
     delete m_pTimer;
   m_pTimer = NULL;
@@ -83,6 +89,22 @@ DialogSlideshow::~DialogSlideshow ( )
   }
 }
 
+void DialogSlideshow::accept ( )
+{
+  m_bExitNow = true;
+  QDialog::accept     ( ); //done ( QDialog::Accepted ); //accept ( );
+  qApp->processEvents ( );
+  usleep ( 100000 );
+}
+
+void DialogSlideshow::reject ( )
+{
+  m_bExitNow = true;
+  QDialog::reject     ( ); //done ( QDialog::Accepted ); //accept ( );
+  qApp->processEvents ( );
+  usleep ( 100000 );
+}
+
 QString DialogSlideshow::background ( )
 {
   return m_qsBackground;
@@ -128,8 +150,13 @@ void DialogSlideshow::initMe ( CXmlSlideshow *pSlideshow, bool bEnableName )
   m_pEditName->setText          ( pSlideshow->slideshow_name    );
   m_pEditName->setEnabled       ( bEnableName                   );
 
-  if ( theUtils.getFormat ( pSlideshow->xres, pSlideshow->yres ) > FORMAT_NTSC4 )
-       m_pComboFormat->setCurrentItem ( 1 ); // set to PAL
+  int iFormat = 0;
+  if ( theUtils.getFormat ( pSlideshow->xres, pSlideshow->yres ) > FORMAT_NTSC4 )  // set to PAL
+    iFormat = ( pSlideshow->aspect == 0 ) ? 1 : 3;
+  else
+    iFormat = ( pSlideshow->aspect == 0 ) ? 0 : 2;
+
+  m_pComboFormat->setCurrentItem ( iFormat );
 
   bool      bBackgroundImage = false;
   QString   qsAudio;
@@ -171,9 +198,10 @@ void DialogSlideshow::initMe ( CXmlSlideshow *pSlideshow, bool bEnableName )
   }
 
   initTransitions   ( );
-  createVidsLength  ( );
-  createAudioLength ( );
-  displayLength     ( );
+  QTimer::singleShot ( 10,   this, SLOT ( createVidsLength  ( ) ) );
+  QTimer::singleShot ( 1000, this, SLOT ( createAudioLength ( ) ) );
+//  createVidsLength  ( );
+//  createAudioLength ( );
 }
 
 void DialogSlideshow::initTransitions ( )
@@ -189,6 +217,7 @@ void DialogSlideshow::initTransitions ( )
   m_pListViewTransitions->setSorting ( -1 );
 
   // We need to also init the selected transitions for a Slideshow
+  QString qsFilter;
   QStringList::iterator itFilter;
   QCheckListItem    *pItem = NULL;
   QStringList         list = Render::Manager::m_pSelf->getFilterNames ( );
@@ -196,8 +225,15 @@ void DialogSlideshow::initTransitions ( )
   bool bChecked;
 
   if ( list.count ( ) < 5 )  {
+    Utils theUtils;
+    QString qsQRender = theUtils.getToolPath ( "qrender");
+    if ( qsQRender == "qrender" )  // qrender path not found
+      qsQRender = tr ( "Could not find 'qrender'. Please make sure that 'qrender is in $PATH and executable.\n\n" );
+    else
+      qsQRender = "";
     QString qsTransitionPath = Global::qsSystemPath + QString ( "/share/qdvdauthor/slideshow/transitions/alpha/" );
-    QString qsText = tr ( "Please check if you have all transitions installed in " );
+    QString qsText = qsQRender;
+    qsText += tr ( "Please check if you have all transitions installed in " );
     qsText += tr ( "\n%1\n\nYou can donload them from:\nhttp://qdvdauthor.sourceforge.net/data/alpha_trans.tar.bz2\n\n" ).arg ( qsTransitionPath );
     qsText += tr ( "To install do the following:\n" );
 
@@ -209,8 +245,14 @@ void DialogSlideshow::initTransitions ( )
     MessageBox::html ( this, tr ( "Warning" ), qsText, theSize );
   }
 
+  QPixmap pix;
+  QImage  img ( QImage::fromMimeSource ( "please_wait.jpg" ) );
+  img  =  img.smoothScale ( 24, 24 );
+  pix.convertFromImage  ( img );
   do  {
-    pItem = new QCheckListItem ( m_pListViewTransitions, *(--it), QCheckListItem::CheckBoxController );
+    qsFilter = *(--it);
+    pItem = new QCheckListItem ( m_pListViewTransitions, qsFilter, QCheckListItem::CheckBoxController );
+    pItem->setPixmap ( 0, pix );
     bChecked = false;
     if ( m_pSlideshow->validFilterNames.count ( ) > 0 )  {
       itFilter = m_pSlideshow->validFilterNames.begin ( );
@@ -229,6 +271,43 @@ void DialogSlideshow::initTransitions ( )
   bChecked = m_pSlideshow->imgBkgImg ( );
   m_pRadioImgImg   ->setChecked ( !bChecked );
   m_pRadioImgBkgImg->setChecked (  bChecked );
+
+  QTimer::singleShot ( 1000, this, SLOT ( slotCreateListViewPix ( ) ) );
+}
+
+void DialogSlideshow::slotCreateListViewPix ( )
+{
+  QCheckListItem *pItem = (QCheckListItem *)m_pListViewTransitions->firstChild ( );
+  while ( pItem )  {
+    pItem->setPixmap    ( 0, getListViewPix ( pItem->text ( ) ) );
+    qApp->processEvents ( 500 );
+    pItem = (QCheckListItem *)pItem->nextSibling ( );
+  }
+}
+
+QPixmap DialogSlideshow::getListViewPix ( QString qsFilterName )
+{
+  QPixmap pix;
+  QString qsTransitionPath = Global::qsSystemPath + QString ( "/share/qdvdauthor/slideshow/transitions/alpha/" );
+  QString qsFileName       = qsTransitionPath + qsFilterName + ".jpg";
+
+  QFileInfo fileInfo ( qsFileName );
+  for ( int t=0; t<2;  t++ ) {
+    if ( fileInfo.exists ( ) )
+      break;
+
+    qsTransitionPath = Global::qsSystemPath + QString ( "/share/qdvdauthor/slideshow/transitions/luma/" );
+    qsFileName       = qsTransitionPath + qsFilterName + ".png";
+    fileInfo.setFile ( qsFileName );
+  }
+  if ( ! fileInfo.exists ( ) )
+    return pix; // E.g. "Cross Fade"
+
+  QImage img  (  qsFileName );
+  img =  img.scale ( 24, 24 );
+  pix.convertFromImage( img );
+
+  return pix;
 }
 
 void DialogSlideshow::createTransitions ( )
@@ -267,22 +346,36 @@ void DialogSlideshow::createVidsLength ( )
 
 void DialogSlideshow::createAudioLength ( )
 {
-  if ( ! m_pSlideshow )
+  static bool bBusy = false;
+  if ( bBusy )  {
+    m_pSlideshow->total_audio_length = 0;
+    displayLength ( );
     return;
-
-  MediaInfo *pMediaInfo = Global::pApp->getMediaInfo ( );
-  if ( ! pMediaInfo )
-    return;
-
-  QString qsFile;
-  m_pSlideshow->total_audio_length = 0;
-  QStringList::iterator it = m_listAudio.begin ( );
-  while ( it != m_listAudio.end ( ) )  {
-    qsFile = *it++;
-    pMediaInfo->setFileName ( qsFile, true );
-    m_pSlideshow->total_audio_length += (uint)( pMediaInfo->getLength ( ) / 1000.0 );
   }
+  bBusy = true;
+  if ( m_pSlideshow )  {
+    MediaInfo *pMediaInfo = Global::pApp->getMediaInfo ( );
+    if ( pMediaInfo )  {
+      QString qsFile;
+      m_pSlideshow->total_audio_length = 0;
+      QStringList tempList     = m_listAudio;
+      QStringList::iterator it = tempList.begin ( );
+      while ( it != tempList.end ( ) )  {
+        qsFile = *it++;
+        pMediaInfo->setFileName ( qsFile, true );
+        if ( ( ! m_pSlideshow ) || ( m_bExitNow ) ) {
+          displayLength ( );
+          bBusy = false;
+          m_bExitNow = false;
+          return;
+        }
+        m_pSlideshow->total_audio_length += (uint)( pMediaInfo->getLength ( ) / 1000.0 );
+      }
+    }
+  }
+
   displayLength ( );
+  bBusy = false;
 }
 
 void DialogSlideshow::displayLength ( )
@@ -352,6 +445,14 @@ void DialogSlideshow::slotSelectionChanged ( QListViewItem * )
   QString qsFileName       = qsTransitionPath + qsFilterName + ".jpg";
 
   QFileInfo fileInfo ( qsFileName );
+  for ( int t=0; t<2; t++ )  {
+    if ( fileInfo.exists ( ) )
+      break;
+
+    qsTransitionPath = Global::qsSystemPath + QString ( "/share/qdvdauthor/slideshow/transitions/luma/" );
+    qsFileName       = qsTransitionPath + qsFilterName + ".png";
+    fileInfo.setFile ( qsFileName );
+  }
   if ( ! fileInfo.exists ( ) )
     return; // E.g. "Cross Fade"
 
@@ -370,18 +471,20 @@ void DialogSlideshow::recreatePix ( )
   if ( ( ! pItem ) || ( ! m_pSlideshow ) )
     return;
 
-  if ( m_pSlideshow->countImg ( ) < 2 )
-    return;
-
+  Utils theUtils;
   QCursor myCursor     ( QCursor::WaitCursor );
   QApplication::setOverrideCursor ( myCursor );
 
   QString qsTransitionPath = Global::qsSystemPath + QString ( "/share/qdvdauthor/slideshow/transitions/alpha/" );
   QString qsFilterName     = qsTransitionPath + pItem->text (   ) + ".jpg";
-  if ( m_bAnimated )  {
-    CXmlSlideshow::img_struct *pStart = m_pSlideshow->getImg  ( 0 );
-    CXmlSlideshow::img_struct *pStop  = m_pSlideshow->getImg  ( 1 );
 
+  QFileInfo fileInfo ( qsFilterName );
+  if ( ! fileInfo.exists ( ) )  {
+    qsTransitionPath = Global::qsSystemPath + QString ( "/share/qdvdauthor/slideshow/transitions/luma/" );
+    qsFilterName     = qsTransitionPath + pItem->text (   ) + ".png";
+  }
+
+  if ( m_bAnimated )  {
     int      t, iDiff = 0;
     float    fDelta;
     bool     bImgBkgImg = m_pRadioImgBkgImg->isChecked ( );
@@ -391,8 +494,15 @@ void DialogSlideshow::recreatePix ( )
       array[t] = new QPixmap ( m_pLabelPreview->size ( ) );
 
     QImage filter ( qsFilterName );
-    QImage start  ( pStart->src  );
-    QImage stop   ( pStop->src   );
+    QImage start  = QImage::fromMimeSource ( "default.jpg" );
+    QImage stop   = QImage::fromMimeSource (  "splash.jpg" );
+    if ( m_pSlideshow->countImg ( ) >= 2 )  {
+      CXmlSlideshow::img_struct *pStart = m_pSlideshow->getImg  ( 0 );
+      CXmlSlideshow::img_struct *pStop  = m_pSlideshow->getImg  ( 1 );
+      start = QImage ( pStart->src  );
+      stop  = QImage ( pStop->src   );
+    }
+
     QImage result     ( start ); // init the size
     QImage background ( start.copy ( ) ); // init the size
     if ( bImgBkgImg ) {
@@ -416,9 +526,9 @@ void DialogSlideshow::recreatePix ( )
 
     for ( t=0; t<THUMBNAIL_COUNT; t++ )  {
       if ( bImgBkgImg )
-        augmentAlpha ( (int)( fDelta * t ), (int)(fDelta * (t-iDiff)), filter, start, stop, result, &background );
+        theUtils.augmentAlpha ( (int)( fDelta * t ), (int)(fDelta * (t-iDiff)), filter, start, stop, result, &background );
       else
-        augmentAlpha ( (int)( fDelta * t ), filter, start, stop, result );
+        theUtils.augmentAlpha ( (int)( fDelta * t ), filter, start, stop, result );
 
       array[t]->convertFromImage ( result );
     }
@@ -454,76 +564,30 @@ void DialogSlideshow::slotAnimatePix ( )
   m_pLabelPreview->setPaletteBackgroundPixmap ( *m_arrayPix[m_iCurrentPix] );
 }
 
-// Copied over from qrender -> filter.cpp
-void DialogSlideshow::augmentAlpha ( int iDeltaA, QImage &filter, QImage &start, QImage &stop, QImage &result )
+void DialogSlideshow::slotSelect ( )
 {
-  int   r, g, b, x, y;
-  float fVal;
-  QRgb  alpha, pix1, pix2;
-  // First we copy the filter to start and end ...
-  // Next we run through every pixel to increase / decrease the filters
-  for  ( y=0; y<filter.height ( ); y++ )  {
-    for ( x=0; x<filter.width ( ); x++ )  {
-      alpha = filter.pixel ( x, y );
-      pix1  = start.pixel  ( x, y );
-      pix2  = stop.pixel   ( x, y );
-      fVal = (float)( qRed ( alpha ) - 256 + iDeltaA ) / 255.0f; // represents the alpha value ...
-      if ( fVal < 0.0f )
-           fVal = 0.0f;
-      if ( fVal > 1.0f )
-           fVal = 1.0f;
-
-      r = qRed   ( pix1 ) * ( 1.0f - fVal ) + qRed   ( pix2 ) * fVal;
-      g = qGreen ( pix1 ) * ( 1.0f - fVal ) + qGreen ( pix2 ) * fVal;
-      b = qBlue  ( pix1 ) * ( 1.0f - fVal ) + qBlue  ( pix2 ) * fVal;
-
-      pix1 = qRgb     ( r, g, b );
-      result.setPixel ( x, y, pix1 );
-    }
+  QStringList listEnabled;
+  QCheckListItem *pItem = (QCheckListItem *)m_pListViewTransitions->firstChild ( );
+  while ( pItem )  {
+    if  ( pItem->isOn ( ) )
+      listEnabled.append ( pItem->text ( ) );
+    pItem = (QCheckListItem *)pItem->nextSibling ( );
   }
-}
 
-void DialogSlideshow::augmentAlpha ( int iDeltaA, int iDeltaB, QImage &filter, QImage &start, QImage &stop, QImage &result, QImage *pBackground )
-{
-  int   r, g, b, x, y;
-  float fVal1, fVal2, fVal3 = 0.0f;
-  QRgb alpha, pix1, pix2, pix3;
-  // First we copy the filter to start and end ...
-  // Next we run through every pixel to increase / decrease the filters
-  for  ( y=0; y<filter.height ( ); y++ )  {
-    for ( x=0; x<filter.width ( ); x++ )  {
-      alpha = filter.pixel ( x, y );
-      pix1  = start.pixel  ( x, y );
-      pix2  = stop.pixel   ( x, y );
-      fVal1 = (float)( qRed ( alpha ) - 256 + iDeltaA ) / 255.0f; // represents the alpha value ...
-      if ( fVal1 < 0.0f )
-           fVal1 = 0.0f;
-      if ( fVal1 > 1.0f )
-           fVal1 = 1.0f;
-      fVal2 = (float)( qRed ( alpha ) - 256 + iDeltaB ) / 255.0f; // represents the alpha value ...
-      if ( fVal2 < 0.0f )
-           fVal2 = 0.0f;
-      if ( fVal2 > 1.0f )
-           fVal2 = 1.0f;
+  DialogAlphaTrans filterDialog ( listEnabled, this );
+  if ( filterDialog.exec ( )  ==  QDialog::Rejected )
+    return;
 
-      r = qRed   ( pix1 ) * ( 1.0f - fVal1 ) + qRed   ( pix2 ) * fVal2;
-      g = qGreen ( pix1 ) * ( 1.0f - fVal1 ) + qGreen ( pix2 ) * fVal2;
-      b = qBlue  ( pix1 ) * ( 1.0f - fVal1 ) + qBlue  ( pix2 ) * fVal2;
-
-      if ( pBackground )  {
-        //fVal3 = 1.0f-fVal1 + fVal2;
-        fVal3 = fVal2 - fVal1;
-        if ( fVal3 < 0.0f )
-             fVal3 = -fVal3;
-        pix3  = pBackground->pixel ( x, y );
-        r += qRed   ( pix3 ) * fVal3;
-        g += qGreen ( pix3 ) * fVal3;
-        b += qBlue  ( pix3 ) * fVal3;
-      }
-
-      pix1 = qRgb     ( r, g, b );
-      result.setPixel ( x, y, pix1 );
-    }
+  QString qsName;
+  bool    bChecked;
+  pItem = (QCheckListItem *)m_pListViewTransitions->firstChild ( );
+  while ( pItem )  {
+    qsName = pItem->text ( );
+    bChecked = false;
+    if  ( filterDialog.isChecked ( qsName ) )
+      bChecked = true;
+    pItem->setOn ( bChecked );
+    pItem = (QCheckListItem *)pItem->nextSibling ( );
   }
 }
 
@@ -553,6 +617,9 @@ void DialogSlideshow::slotImage ( )
   if ( qsFileName.isEmpty ( ) )
     return;
 
+  QFileInfo fileInfo ( qsFileName );
+  Global::qsCurrentPath = fileInfo.dirPath ( TRUE );
+
   QImage  theImg ( qsFileName );
   QPixmap thePix;
   if ( theImg.isNull ( ) )
@@ -594,9 +661,10 @@ void DialogSlideshow::slotAddAudio ( )
     return;
 
   m_listAudio += list;
+  QFileInfo fileInfo ( list[0] );
+  Global::qsCurrentPath = fileInfo.dirPath ( TRUE );
 
   QString   qsAudio;
-  QFileInfo fileInfo;
   QStringList::iterator it = list.begin ( );
   while ( it  !=  list.end ( ) )  {
     qsAudio = *it++;
@@ -604,8 +672,8 @@ void DialogSlideshow::slotAddAudio ( )
     qsAudio = fileInfo.fileName ( );
     m_pListAudio->insertItem    ( qsAudio );
   }
+
   createAudioLength ( );
-  displayLength     ( );
 }
 
 void DialogSlideshow::slotDeleteAudio ( )
@@ -616,7 +684,7 @@ void DialogSlideshow::slotDeleteAudio ( )
 
   m_pListAudio->removeItem ( m_pListAudio->currentItem ( ) );
 
-  // The listBox only holds the file name, and not the whol path
+  // The listBox only holds the file name, and not the whole path
   // thus we need to search for the right entry.
   QString qsInList;
   QStringList::iterator it = m_listAudio.begin ( );
@@ -625,7 +693,6 @@ void DialogSlideshow::slotDeleteAudio ( )
     if ( qsInList.find   ( qsAudio  ) > -1 ) {
       m_listAudio.remove ( qsInList );
       createAudioLength  ( );
-      displayLength      ( );
       return;
     }
   }
