@@ -85,6 +85,9 @@ void DialogVFolder::initMe ( QString qsVirtualFolder, bool bImageFolder )
     pVFile = *--it;
     fileInfo.setFile ( pVFile->qsFileName );
     pEntry = Global::pThumbsCache->find ( pVFile->iFileHashValue, bImageFolder );
+    if ( !pEntry && bImageFolder )  // Special handling for ImageFolder, which could hold VIDs
+          pEntry = Global::pThumbsCache->find ( pVFile->iFileHashValue, false );
+
     pItem  = new VFolderItem ( pEntry, m_pListView, fileInfo.fileName( ) );
     if ( pEntry ) {
       if ( ! pEntry->arrayOfThumbs )
@@ -112,11 +115,12 @@ void DialogVFolder::slotAdd ( )
   QStringList    list;
   QListViewItem *pItem;
   if ( m_bIsImageFolder )  {
-    qsMovieFilter =  Global::pApp->getMovieFilter ( );
+    qsMovieFilter  =  Global::pApp->getImageFilter ( );
+    qsMovieFilter +=  Global::pApp->getVidFilter   ( );
     list          =  FilePreviewDialog::getOpenFileNames ( NULL, Global::qsCurrentPath, QString ("Movies ( ") + qsMovieFilter + QString (" );;All ( * )"));
   }
   else  {
-    qsMovieFilter =  Global::pApp->getImageFilter ( );
+    qsMovieFilter =  Global::pApp->getMovieFilter ( );
     list          =  FilePreviewDialog::getOpenFileNames ( NULL, Global::qsCurrentPath, QString ("Images ( ") + qsMovieFilter + QString (" );;All ( * )"));
   }
   if ( list.count ( ) < 1 )
