@@ -4,6 +4,10 @@
 //
 // This class is only a container class to encapsulate the 
 // variables needed for Drag'n drop support
+//   Created : 
+//        by : Varol Okan
+// Copyright : (c) Varol Okan
+//   License : GPL v 2.0
 //
 ////////////////////////////////////////////////////////////////
 
@@ -11,17 +15,17 @@
 
 DragNDropContainer::DragNDropContainer()
 {
-	sizeThumbnail   = QSize(50, 50);
-	sizeButton      = QSize (150, 150);
+	sizeThumbnail   = QSize (  50,  50 );
+	sizeButton      = QSize ( 150, 150 );
 	bImageButton    = true;	// Image or MovieButton
 	bWithText       = true;
 	iTextPosition   = TEXT_POS_BOTTOM;
-	timeStartOffset = QTime  ();
-	timeDuration    = QTime  ();
-	font            = QFont  ();
-	colorForeground = QColor ();
+	timeStartOffset = QTime ( );
+	timeDuration    = QTime ( );
+	font            = QFont ( );
+	colorForeground = Rgba  ( 255, 0, 0 ); // default to red ... ( why not )
 	bStartAfterDropEvent = true;
-	iHowNiceShouldIBe    = 0;
+	iHowNiceShouldIBe    =    0;
 }
 
 DragNDropContainer::~DragNDropContainer()
