@@ -82,8 +82,10 @@ Area::Container::Connection::~Connection ( )
   m_listLineStipples.clear ( );
 
   // Finally remove this connection from the associated Container.
+//  if ( target ( )->sources ( ).size   (      ) > 0 )
   if ( target ( )->sources ( ).find   ( this ) != target ( )->sources ( ).end ( ) ) 
        target ( )->sources ( ).remove ( this );
+//  if ( source ( )->targets ( ).size   (      ) > 0 )
   if ( source ( )->targets ( ).find   ( this ) != source ( )->targets ( ).end ( ) )
        source ( )->targets ( ).remove ( this );
 }
@@ -334,6 +336,9 @@ void Area::Container::clearTargets ( )
   while ( it != list.end ( ) )  {
     pConnection = *it++;
     delete pConnection;
+//    // At this point we could have removed a Connection from the list itself so be had better check
+//    if ( m_listTargets.size ( ) < 1 )
+//      break;
   }
   m_listTargets.clear ( );
 }
@@ -346,6 +351,9 @@ void Area::Container::clearSources ( )
   while ( it != list.end ( ) )  {
     pConnection = *it++;
     delete pConnection;
+//    // At this point we could have removed a Connection from the list itself so be had better check
+//    if ( m_listSources.size ( ) < 1 )
+//      break;
   }
   m_listSources.clear ( );
 }
@@ -469,13 +477,13 @@ QPoint Area::Container::getPos ( enSide whichSide, int iWhichNumber )
   if ( whichSide == SIDE_BOTTOM )  {
     iSize = m_pObject->width  ( );
     fDelta = (float)iSize / iTotal;
-    iX = fDelta * ( iWhichNumber + 1 ) + m_pObject->pos ( ).x ( );
+    iX = (int)( fDelta * ( iWhichNumber + 1 ) + m_pObject->pos ( ).x ( ) );
     iY = m_pObject->pos ( ).y ( )  +  m_pObject->height ( ) - 2;
   }
   else  { // SIDE
     iSize  = m_pObject->height ( );
     fDelta = (float)iSize / iTotal;
-    iY = fDelta  * ( iWhichNumber + 1 ) + m_pObject->pos ( ).y ( );
+    iY = (int)( fDelta  * ( iWhichNumber + 1 ) + m_pObject->pos ( ).y ( ) );
     iX = m_pObject->pos ( ).x ( ) - 2;
     if ( whichSide == SIDE_RIGHT )
        iX += m_pObject->width ( );
@@ -795,6 +803,13 @@ void Area::drawContainer ( QPainter &thePainter, Object *pExcept )
   }
 }
 
+void Area::drawContainer ( Object *pObject )
+{
+  Container *pContainer = findContainerFromObject ( pObject );
+  if ( pContainer )
+       drawContainer ( pContainer );
+}
+
 void Area::drawContainer ( Container *pContainer )
 {
   // Here we render only the connections to, and from this Container
@@ -905,6 +920,7 @@ void Area::createConnections ( QValueList<Object *> &list )
       findMovieTargets ( pContainer );
     else if ( pContainer->object  ( )->type ( ) == Object::TYPE_MENU  )
       findMenuTargets  ( pContainer );
+
     setReadParams ( pContainer );
   }
 
@@ -920,6 +936,10 @@ void Area::createConnections ( QValueList<Object *> &list )
         pContainer->container ( ).append ( pTarget );
     }
   }
+  // The following functin will remove double connection from one container to another.
+  // These double connections are of no improtance to the DVDLayout itself.
+  cleanListOfContainer ( );
+//for (int x=0;x<m_listOfContainer.count();x++) { if (m_listOfContainer[x]) printf ("%s::%d > %d = cnt<%d>\n",__FILE__,__LINE__,x,m_listOfContainer[x]->container().count()); }
 
   // And finally we go and assign the objects inside the Container objects to Left, Right, and Bottom
   Container *pTarget   = NULL;
@@ -936,6 +956,7 @@ void Area::createConnections ( QValueList<Object *> &list )
     }
     createConnections ( pContainer );
   }
+
   // Finally after we assigned all sides to all Container
   // we can figure out the idx of the connection on each side.
   Container::Connection *pConnection = NULL;
@@ -971,6 +992,27 @@ void Area::createConnections ( QValueList<Object *> &list )
   }
 }
 
+void Area::cleanListOfContainer ( )
+{
+  Container *pContainer = NULL;
+  Container *pTarget    = NULL;
+  QValueList<Container *> tempList;
+  QValueList<Container *>::iterator it = m_listOfContainer.end ( );
+  QValueList<Container *>::iterator it2;
+  while ( --it != m_listOfContainer.begin ( ) )  {
+    pContainer = *it;
+    it2 = pContainer->container ( ).begin ( );
+    while (  it2 != pContainer->container ( ).end ( ) )  {
+      pTarget = *it2++;
+      tempList.remove   ( pTarget );
+      if ( pTarget  != pContainer )  // Dont want to show connections from one to itself.
+        tempList.append ( pTarget );
+    }
+    pContainer->container ( ) = tempList;
+    tempList.clear ( );
+  }
+}
+
 void Area::createConnections ( Container *pContainer )
 {
   QRgb red    = 0xFF0000;
@@ -984,8 +1026,8 @@ void Area::createConnections ( Container *pContainer )
   QValueList<Container *>::iterator it2;
   QValueList<Container *>::iterator itCon = m_listOfContainer.begin ( );
 
-  listContainer = pContainer->container   ( );
-  int iPosX     = pContainer->object  ( )->pos ( ).x ( );
+  listContainer = pContainer->container ( );
+  int iPosX     = pContainer->object    ( )->pos ( ).x ( );
 
   it2 = listContainer.begin ( );
   while ( it2  != listContainer.end ( ) )  {
@@ -1109,8 +1151,9 @@ void Area::findMenuTargets ( Container *pContainer )
   while ( it != list.end ( ) )  {
     pEntry = *it++;
     pTarget = findContainerFromPtr ( pEntry );
-    if ( pTarget )
+    if ( pTarget )  {
       pContainer->container ( ).append ( pTarget );
+    }
   }
 
   // preAction ( );
@@ -1118,6 +1161,7 @@ void Area::findMenuTargets ( Container *pContainer )
   Container    *pCurrentTarget = NULL;
   DVDMenu      *pOtherMenu     = NULL;
   ButtonObject *pButton        = NULL;
+  int         iSelfCounter;
   QString     strMultiple;
   QStringList listMultiple;
   QStringList::iterator itMultiple;
@@ -1125,14 +1169,24 @@ void Area::findMenuTargets ( Container *pContainer )
   QValueList<ButtonObject *>::iterator itButtons = listButtons.begin ( );
   while ( itButtons != listButtons.end ( ) )  {
     pButton = *itButtons++;
-    listMultiple = pButton->getPlayMultipleList ( );
-    itMultiple   = listMultiple.begin ( );
+    listMultiple   = pButton->getPlayMultipleList ( );
+    itMultiple     = listMultiple.begin ( );
     pCurrentTarget = NULL;
+    iSelfCounter   = 0;
     while ( itMultiple != listMultiple.end ( ) )  {
-      strMultiple = *itMultiple++;
+      strMultiple  = *itMultiple++;
       pTarget = findContainerFromAction ( strMultiple );
-      if ( pCurrentTarget && pTarget )
-           pCurrentTarget->container ( ).append ( pTarget );
+      if ( ( pCurrentTarget && pTarget ) && ( pCurrentTarget != pTarget ) )
+             pCurrentTarget->container ( ).append ( pTarget );
+/* Does cause issues with multiple SFEs in a MultiSelection.
+      if ( pCurrentTarget && pTarget )  {
+        // We only want to see one pointer to itself ( If required )
+        if ( ( iSelfCounter < 1 ) || ( pCurrentTarget != pTarget ) )
+             pCurrentTarget->container ( ).append ( pTarget );
+        if ( pCurrentTarget != pTarget )
+             iSelfCounter++;
+      }
+*/
       pCurrentTarget = pTarget;
     }
   }
