@@ -173,21 +173,22 @@ int Execute::startCommand ()
 	
 	// Here we generate a simple shell script to execute the commands under the shell.
 	// QProcess is currently not flexible enough to handle this internal.
-	QFile file(Global::qsTempPath + QString("/execute.sh"));
+	QString qsExecute = Global::qsTempPath + QString ( "/execute.sh" );
+	QFile file ( qsExecute );
 	if ( file.open( IO_WriteOnly ) ) {
 		QTextStream stream( &file );
 		stream << Global::qsBashString << m_commandList[m_iCurrentCommand];
 		file.close();
 	}
 	// And here we grant read/write/execute permission.
-	chmod ((const char *)QString(Global::qsTempPath + QString("/execute.sh")), S_IEXEC | S_IRUSR | S_IWRITE | S_IROTH | S_IXOTH | S_IWOTH );
+	chmod ( qsExecute.utf8 ( ), S_IXGRP | S_IWGRP | S_IRGRP | S_IEXEC | S_IRUSR | S_IWRITE | S_IROTH | S_IXOTH | S_IWOTH );
 	// Increase the command counter ...
 	m_iCurrentCommand ++;
 	// Set the script to execute ...
-	clearArguments ();
-	addArgument (QString (Global::qsTempPath + QString("/execute.sh")));
+	clearArguments ( );
+	addArgument    ( qsExecute );
 	// And eh voila, ici on executer les commands.
-	start (pListEnv);
+	start ( pListEnv );
 	// return to the calling process (thread)
 	return 1;
 }
@@ -469,7 +470,7 @@ void MyDialog::processingFinished (int iSeconds)
   // First we check if qplayer is there and executable ...
   QString qsPlayer = Global::qsExternalPlayer;
   QFileInfo fileInfo    ( qsPlayer );
-  
+
   if ( !fileInfo.exists ( ) ) {
     Utils theUtils;
     qsPlayer = theUtils.checkForExe( qsPlayer );
@@ -481,16 +482,42 @@ void MyDialog::processingFinished (int iSeconds)
   if (!fileInfo.isExecutable())
     return;
   // last we check if there has been something clreated to play at all ...
-  fileInfo.setFile(Global::qsProjectPath + QString ("/VIDEO_TS/VIDEO_TS.BUP"));
-  if (!fileInfo.exists())
+  fileInfo.setFile ( Global::qsProjectPath + QString ( "/VIDEO_TS/VIDEO_TS.BUP" ) );
+  if ( ! fileInfo.exists ( ) )
     return;
-  fileInfo.setFile(Global::qsProjectPath + QString ("/VIDEO_TS/VIDEO_TS.IFO"));
-  if (!fileInfo.exists())
+  fileInfo.setFile ( Global::qsProjectPath + QString ( "/VIDEO_TS/VIDEO_TS.IFO" ) );
+  if ( ! fileInfo.exists ( ) )
     return;
-  fileInfo.setFile(Global::qsProjectPath + QString ("/VIDEO_TS/VIDEO_TS.VOB"));
-  if (!fileInfo.exists())
+  fileInfo.setFile ( Global::qsProjectPath + QString ( "/VIDEO_TS/VIDEO_TS.VOB" ) );
+  if ( ! fileInfo.exists ( ) )
     return;
-  buttonPlay->setEnabled(true);
+
+  writeZoneCode ( );
+  buttonPlay->setEnabled ( true );
+}
+
+void MyDialog::writeZoneCode ( )
+{
+  if ( ( Global::iRegionalZone == 0 ) || ( Global::iRegionalZone > 6 ) )
+    return;
+
+  const char arrayCodes[] = { 0x00, 0xFE, 0xFD, 0xFB, 0xF7, 0xEF, 0xDF };
+  // * Region 1 - The U.S., U.S. territories and Canada
+  // * Region 2 - Europe, Japan, the Middle East, Egypt, South Africa, Greenland
+  // * Region 3 - Taiwan, Korea, the Philippines, Indonesia, Hong Kong
+  // * Region 4 - Mexico, South America, Central America, Australia, New Zealand, Pacific Islands, Caribbean
+  // * Region 5 - Russia (okay, former Russia), Eastern Europe, India, most of Africa, North Korea, Mongolia
+  // * Region 6 - China 
+
+  QFile ifo ( Global::qsProjectPath + "/VIDEO_TS/VIDEO_TS.IFO" );
+  for ( int t=0; t<2; t++ )  {
+    if  ( ifo.open ( IO_ReadWrite ) == true )  {
+      if ( ifo.at  ( 0x23 ) )
+        ifo.putch  ( arrayCodes[Global::iRegionalZone] );
+    }
+    ifo.close      ( );
+    ifo.setName    ( Global::qsProjectPath + "/VIDEO_TS/VIDEO_TS.BUP" );
+  }
 }
 
 // The next three are related to the Keep Open countdown
