@@ -14,13 +14,13 @@
 #ifndef __EXPORT_DVDAUTHOR_H__
 #define __EXPORT_DVDAUTHOR_H__
 
+#include <qmap.h>
 #include <qvaluelist.h>
 
 class SourceFileEntry;
 class CXmlDVDAuthor;
 class ButtonObject;
 class CXmlSpumux;
-class QDVDAuthor;
 class Subtitles;
 class VMGMenu;
 class DVDMenu;
@@ -29,12 +29,12 @@ class DVDMenu;
 namespace Export
 {
 
-  class DVDAuthor;
+class DVDAuthor;
 
-class JumpTable 
+class JumpTable
 {
 public:
-	JumpTable ();
+	JumpTable ( );
 
 	// The jumpIndex is the number assigned to g3
 	// the menu designated to handle the jumpTable 
@@ -50,14 +50,46 @@ public:
 	ButtonObject   *pButton;
 };
 
+class ButtonTransition
+{
+  // For ButtonTransitions from a Menu to another Menu we'll create a new Titleset which only holds
+  // transition videos, followed by the post command to jump to the actual menu.
+public:
+  class Source {
+   public:
+    Source ( );
+    ButtonObject *pSourceButton;
+    QString       qsChapter;    // Only vald for Transitions to Video files
+    DVDMenu      *pSourceMenu;
+    QString       qsTransitionFileName;
+    int           iChapter;    // Only valid for video sources.
+    int           iTitle;      // Only valid for video sources.
+  };
+ public:
+  ButtonTransition ( SourceFileInfo *, DVDMenu * );
+ ~ButtonTransition ( );
+  void addSource   ( ButtonObject *, DVDMenu *, QString );
+
+  int     calcTitleIdx ( ButtonObject * );
+  Source *findButton   ( ButtonObject * );
+  Source *findChapter  ( QString );
+  QString fileName     (   int   );
+
+  SourceFileInfo     *  pTargetInfo;
+  DVDMenu            *  pTargetMenu;
+  int                   iJumpIndex; // The Jump to the actual menu after we played the transition video
+  QValueList<Source  *> listSource;
+};
+
 class ExportTitleset
 {
 public:
-	 ExportTitleset              ( DVDAuthor *       );
+	 ExportTitleset              ( DVDAuthor       * );
 	~ExportTitleset              (                   );
-	int  findMenu                ( DVDMenu *         );
 	int  findTitle               ( SourceFileInfo  * );
 	int  findTitle               ( SourceFileEntry * );
+	int  findTitle               ( ButtonTransition*, ButtonObject *p=NULL );
+	int  findMenu                ( DVDMenu *         );
 	JumpTable *findJumpTable     ( DVDMenu *         );
 	int  getTotalChapters        ( SourceFileInfo  * );
 	int  getChapters             ( SourceFileInfo  *, QString );
@@ -66,10 +98,11 @@ public:
 	int                           iTitlesetNumber;
 	bool                          bSetByUser;
 	QString                       qsFormat;
-	DVDAuthor                    *pExportDVDAuthor;
-	QValueList<DVDMenu *>         listMenus;              // all menus which fit the format.
-	QValueList<JumpTable *>       listJumpTable;          // the jump table
+	DVDAuthor                  *  pExportDVDAuthor;
+	QValueList<DVDMenu         *> listMenus;              // all menus which fit the format.
+	QValueList<JumpTable       *> listJumpTable;          // the jump table
 	QValueList<SourceFileEntry *> listSourceFileEntries;
+	QValueList<ButtonTransition*> listButtonTransitions;
 };
 
 class PlayMultiple
@@ -85,14 +118,15 @@ class PlayMultiple
   bool            bLoop;
 };
 
+
 class DVDAuthor
 {
 public:
-   DVDAuthor  ( QDVDAuthor * );
-  ~DVDAuthor  ();
+   DVDAuthor  ( );
+  ~DVDAuthor  ( );
 
-  bool exportXml ();
-  bool createXml ();
+  bool exportXml ( );
+  bool createXml ( );
   bool createXml ( QString & );
 
 private:
@@ -102,10 +136,12 @@ private:
   ExportTitleset *findTitleset ( DVDMenu * );
   ExportTitleset *findTitleset ( int );
   JumpTable      *findJumpTableEntry ( ButtonObject * );
+  ButtonTransition *findTransition   ( void *, bool b=true );
   PlayMultiple   *findMultiple ( SourceFileInfo *, bool );
   bool optimizeJumpTable       ( ExportTitleset *, JumpTable * );
   int  getHighestTitlesetNumber( );
   int  getFirstEmptyTitleset   ( );
+  void createVMGMTransitions   ( );
   bool populateListOfTitlesets ( );
   bool stowMenusAway           ( );
   bool createJumpTable         ( );
@@ -119,10 +155,12 @@ private:
   void cleanSourceFileEntris   ( );
 
   ExportTitleset *addToTitleset(SourceFileEntry *, int, bool b=false      );
+  bool addToTitleset( CXmlDVDAuthor::titleset_struct *, ButtonTransition* );
   bool addToTitleset( CXmlDVDAuthor::titleset_struct *, SourceFileEntry * );
   bool addToTitleset( CXmlDVDAuthor::titles_struct *,   SourceFileEntry * );
   bool addToTitleset( CXmlDVDAuthor::pgc_struct *,      SourceFileEntry * );
   bool addToTitleset( CXmlDVDAuthor::pgc_struct *,      SourceFileInfo  * );
+  void addTransition( CXmlDVDAuthor::pgc_struct *,      SourceFileInfo  * );
   bool addToMenu    ( CXmlDVDAuthor::titleset_struct *, ExportTitleset  * );
   bool addToMenu    ( CXmlDVDAuthor::menus_struct *,    ExportTitleset  * );
   bool addToMenu    ( CXmlDVDAuthor::pgc_struct *,      ExportTitleset  * );
@@ -131,9 +169,11 @@ private:
   bool addToMenu    ( CXmlDVDAuthor::pgc_struct *,      DVDMenu         * );
   bool addToVideo   ( CXmlDVDAuthor::video_struct *,    QString&,QString& );
   bool addToVideo   ( CXmlDVDAuthor::video_struct *,    int,int,QString & );
+  bool addToVideo   ( CXmlDVDAuthor::video_struct *,    ButtonTransition* );
   bool addToAudio   ( CXmlDVDAuthor::menus_struct *,    DVDMenu         * );
 
   CXmlDVDAuthor::pgc_struct *getPgc ( CXmlDVDAuthor::menus_struct *, DVDMenu * );
+  QString getPreAction         ( ButtonObject     * );
   DVDMenu *getMenuFromButton   ( ButtonObject     * );
   DVDMenu *getMenuFromEntry    ( SourceFileEntry  * );
   QString jumpToSubMenu        ( SourceFileEntry  * );
@@ -143,11 +183,11 @@ private:
   bool m_bOutOfSpecDialog;
   bool m_bMenuZeroDialog;
   bool m_bInsertComments;
-  QDVDAuthor *m_pDVDAuthor;
-  QValueList<DVDMenu *>         m_listOfAllMenus; 
-  QValueList<PlayMultiple *>    m_listPlayMultiple;
-  QValueList<ExportTitleset *>  m_listTitlesets;
-  QValueList<SourceFileEntry *> m_listNewEntries;
+  QMap<void *, ButtonTransition *> m_mapTransitionFiles;
+  QValueList<DVDMenu *>            m_listOfAllMenus; 
+  QValueList<PlayMultiple *>       m_listPlayMultiple;
+  QValueList<ExportTitleset *>     m_listTitlesets;
+  QValueList<SourceFileEntry *>    m_listNewEntries;
   friend class ExportTitleset;
 };
 
