@@ -26,33 +26,94 @@
 #include "dialogmatrix.h"
 #include "structuretoolbar.h"
 
-FrameObject::FrameObject (QWidget *pParent)
-	: MenuObject (pParent)
+#include "qdvdauthor.h"
+#include "dvdmenuundoobject.h"
+
+FrameObject::FrameData::FrameData ( )
+  : MenuObject::Data ( )
 {
-	m_qsObjectType = QString ( FRAME_OBJECT );
-	m_colorFrame   = QColor  ( START_FRAME_COLOR );
-	m_iFrameWidth  = 4;
-	m_iFrameJoin   = Qt::MiterJoin;
-	m_iFrameStyle  = Qt::SolidLine;
+  m_iDataType    = Data::FRAME;
+  m_colorFrame   = QColor  ( START_FRAME_COLOR );
+  m_iFrameWidth  = 4;
+  m_iFrameJoin   = Qt::MiterJoin;
+  m_iFrameStyle  = Qt::SolidLine;
+}
+
+FrameObject::FrameData::~FrameData ( )
+{
+}
+
+MenuObject::Data &FrameObject::FrameData::operator = ( Data &theOther )
+{
+  // At this point we are sure both are acutally ImageManipulator objects.
+  MenuObject::Data::operator= ( theOther );
+  if ( m_iDataType   == theOther.m_iDataType )  {
+    FrameData *pOther = (FrameData *)&theOther;
+    m_colorFrame      = pOther->m_colorFrame;
+    m_iFrameWidth     = pOther->m_iFrameWidth;
+    m_iFrameJoin      = pOther->m_iFrameJoin;
+    m_iFrameStyle     = pOther->m_iFrameStyle;
+  }
+
+  return *this;
+}
+
+bool FrameObject::FrameData::operator == ( Data &theOther )
+{
+  // At this point we are sure both are acutally ImageManipulator objects.
+  bool bReturn = ( MenuObject::Data::operator == ( theOther ) );
+  if ( ! bReturn )
+    return false;
+
+  if ( m_iDataType  == theOther.m_iDataType )  {
+    FrameData *pOther = (FrameData *)&theOther;
+    bReturn = ( ( m_colorFrame  == pOther->m_colorFrame  ) &&
+                ( m_iFrameWidth == pOther->m_iFrameWidth ) &&
+                ( m_iFrameJoin  == pOther->m_iFrameJoin  ) &&
+                ( m_iFrameStyle == pOther->m_iFrameStyle ) );
+  }
+  return bReturn;
+}
+
+MenuObject::Data *FrameObject::FrameData::clone ( )
+{
+  FrameData *pData = new FrameData;
+  *pData  = *this;
+  return pData;
+}
+
+FrameObject::FrameObject (QWidget *pParent)
+  : MenuObject (pParent)
+{
+  if ( m_pData )
+    delete m_pData;        // part of the base class
+  m_pData = new FrameData; // will be deleted in base class
+
+  m_qsObjectType = QString ( FRAME_OBJECT );
 }
 
 FrameObject::~FrameObject ()
 {
 }
 
+FrameObject::FrameData *FrameObject::frameData ( )
+{
+  return (FrameData *)m_pData;
+}
+
 void FrameObject::setFrameWidth(int iFrameWidth)
 {
-	m_iFrameWidth = iFrameWidth;
+  frameData ( )->m_iFrameWidth = iFrameWidth;
 }
 
 void FrameObject::setFrameJoin(int iFrameJoin)
 {
-	m_iFrameJoin = iFrameJoin;
+  frameData ( )->m_iFrameJoin = iFrameJoin;
 }
 
 void FrameObject::setFrameStyle(int iFrameStyle)
 {
-	m_iFrameStyle = iFrameStyle;
+  frameData ( )->m_iFrameStyle = iFrameStyle;
 }
 
 void FrameObject::replaceColor ( Rgba theColor, Rgba oldColor )
@@ -63,34 +124,34 @@ void FrameObject::replaceColor ( Rgba theColor, Rgba oldColor )
 
 void FrameObject::setFrameColor ( Rgba colorFrame )
 {
-  m_colorFrame = colorFrame;
+  frameData ( )->m_colorFrame = colorFrame;
 }
 
 int FrameObject::width ()
 {
-  return m_iFrameWidth;
+  return frameData ( )->m_iFrameWidth;
 }
 
 Rgba FrameObject::color ()
 {
-  return m_colorFrame;
+  return frameData ( )->m_colorFrame;
 }
 
 int FrameObject::style ()
 {
-	return m_iFrameStyle;
+  return frameData ( )->m_iFrameStyle;
 }
 
 int FrameObject::join ()
 {
-	return m_iFrameJoin;
+  return frameData ( )->m_iFrameJoin;
 }
 
 void FrameObject::drawContents (QPainter *painter)
 {
         if ( m_pShadow && ! m_bDrawRect )
 	     m_pShadow->drawContents ( painter );
-	if ( m_colorFrame.alpha ( ) == 0 )
+	if ( frameData ( )->m_colorFrame.alpha ( ) == 0 )
 	  return;
 
 	QWMatrix theMatrix;
@@ -101,7 +162,7 @@ void FrameObject::drawContents (QPainter *painter)
 	// Here we define the cenetered rect.
 	QRect theRect ((int)-(rect().width()/ 2.0), (int)-(rect().height()/2.0), rect().width(), rect().height());
 
-	QPen thePen (m_colorFrame.rgb ( ), m_iFrameWidth, (Qt::PenStyle)m_iFrameStyle, Qt::FlatCap, (Qt::PenJoinStyle)m_iFrameJoin);
+	QPen thePen (frameData ( )->m_colorFrame.rgb ( ), frameData ( )->m_iFrameWidth, (Qt::PenStyle)frameData ( )->m_iFrameStyle, Qt::FlatCap, (Qt::PenJoinStyle)frameData ( )->m_iFrameJoin);
 	// Set the PEN
 	painter->setPen(thePen);
 	// and make sure no Brush is used ...
@@ -163,34 +224,44 @@ bool FrameObject::mouseReleaseEvent (QMouseEvent *)
 
 bool FrameObject::mouseDoubleClickEvent (QMouseEvent *)
 {
-	QString qsAnimation;
-	DialogFrame frameDialog ( m_pParent );
-	frameDialog.initMe ( this );
-	if ( frameDialog.exec ( ) == QDialog::Rejected )
-		return false;
+  QString qsAnimation;
+  DialogFrame frameDialog ( m_pParent );
+  frameDialog.initMe ( this );
+  if ( frameDialog.exec ( ) == QDialog::Rejected )
+    return false;
 
-	setFrameWidth ( frameDialog.frameWidth ( ) );
-	setFrameStyle ( frameDialog.style      ( ) );
-	setFrameJoin  ( frameDialog.join       ( ) );
-	qsAnimation =   frameDialog.animation  ( );
-	setAnimation  ( qsAnimation );
-	if ( color ( ) != frameDialog.color ( ) )  {
-		// This emit signal is only caught when in ButtonDialog and the Selected, or Highlighted mask 
-		// was changed (I.e. This Object is in one of those two frames).
-		// Otherwise the signal is ignored.
-		emit ( signalMaskColorChanged ( frameDialog.color ( ), color ( ) ) );
-		// P.s. replaceColor (QColor, QColor) of all ButtonObjects will be called, inclusive this object)
-	}
-	setFrameColor ( frameDialog.color ( ) );
-	int x, y, width, height;
-	x      = frameDialog.m_pEditX->text ( ).toInt ( );
-	y      = frameDialog.m_pEditY->text ( ).toInt ( );
-	width  = frameDialog.m_pEditWidth->text  ( ).toInt ( );
-	height = frameDialog.m_pEditHeight->text ( ).toInt ( );
-	QRect newRect ( x, y, width, height );
-	setRect ( newRect );
-	emit    ( signalUpdatePixmap ( ) );
-	return false;
+  DVDMenuUndoObject *pUndo = new DVDMenuUndoObject ( this, MenuObject::StateUnknown );
+  setFrameWidth ( frameDialog.frameWidth ( ) );
+  setFrameStyle ( frameDialog.style      ( ) );
+  setFrameJoin  ( frameDialog.join       ( ) );
+  qsAnimation =   frameDialog.animation  ( );
+  setAnimation  ( qsAnimation );
+  if ( color ( ) != frameDialog.color ( ) )  {
+    // This emit signal is only caught when in ButtonDialog and the Selected, or Highlighted mask 
+    // was changed (I.e. This Object is in one of those two frames).
+    // Otherwise the signal is ignored.
+    emit ( signalMaskColorChanged ( frameDialog.color ( ), color ( ) ) );
+    // P.s. replaceColor (QColor, QColor) of all ButtonObjects will be called, inclusive this object)
+  }
+  setFrameColor ( frameDialog.color ( ) );
+  int x, y, width, height;
+  x      = frameDialog.m_pEditX->text ( ).toInt ( );
+  y      = frameDialog.m_pEditY->text ( ).toInt ( );
+  width  = frameDialog.m_pEditWidth->text  ( ).toInt ( );
+  height = frameDialog.m_pEditHeight->text ( ).toInt ( );
+  QRect newRect ( x, y, width, height );
+  setRect ( newRect );
+  emit    ( signalUpdatePixmap ( ) );
+
+  if ( pUndo->hasChanged ( ) )  {
+    UndoBuffer *pBuffer = Global::pApp->getUndoBuffer ( );
+    if ( pBuffer )
+         pBuffer->push ( pUndo );
+  }
+  else
+    delete pUndo;
+
+  return false;
 }
 
 bool FrameObject::readProjectFile  (QDomNode &theNode)
@@ -201,19 +272,19 @@ bool FrameObject::readProjectFile  (QDomNode &theNode)
 
 	attribute = theElement.attributeNode ( FRAME_OBJECT_COLOR );
 	if (!attribute.isNull())
-		m_colorFrame.fromString ( attribute.value ( ) );
+		frameData ( )->m_colorFrame.fromString ( attribute.value ( ) );
 
 	attribute = theElement.attributeNode ( FRAME_OBJECT_WIDTH );
 	if (!attribute.isNull())
-		m_iFrameWidth = attribute.value().toInt();
+		frameData ( )->m_iFrameWidth = attribute.value().toInt();
 
 	attribute = theElement.attributeNode ( FRAME_OBJECT_STYLE);
 	if (!attribute.isNull())
-		m_iFrameStyle = attribute.value().toInt();
+		frameData ( )->m_iFrameStyle = attribute.value().toInt();
 
 	attribute = theElement.attributeNode ( FRAME_OBJECT_JOIN );
 	if (!attribute.isNull())
-		m_iFrameJoin = attribute.value().toInt();
+		frameData ( )->m_iFrameJoin = attribute.value().toInt();
 
 	MenuObject::readProjectFile( theNode );
 	return true;
@@ -233,14 +304,14 @@ bool FrameObject::writeProjectFile (QDomElement &theElement)
 	QDomDocument xmlDoc = theElement.ownerDocument();
 	QDomElement frameNode = xmlDoc.createElement( FRAME_OBJECT );	// <FrameObject>
 	// Here we set the attributes of the <dvdauthor> tag
-	if (m_iFrameWidth != 4)
-		frameNode.setAttribute ( FRAME_OBJECT_WIDTH, m_iFrameWidth );
-	if (m_colorFrame != Rgba ( START_FRAME_COLOR ) )
-		frameNode.setAttribute ( FRAME_OBJECT_COLOR, m_colorFrame.toString ( ) );
-	if (m_iFrameStyle != (int)Qt::SolidLine)
-		frameNode.setAttribute ( FRAME_OBJECT_STYLE, m_iFrameStyle );
-	if (m_iFrameJoin != (int)Qt::MiterJoin)
-		frameNode.setAttribute ( FRAME_OBJECT_JOIN,  m_iFrameJoin );
+	if (frameData ( )->m_iFrameWidth != 4)
+		frameNode.setAttribute ( FRAME_OBJECT_WIDTH, frameData ( )->m_iFrameWidth );
+	if (frameData ( )->m_colorFrame != Rgba ( START_FRAME_COLOR ) )
+		frameNode.setAttribute ( FRAME_OBJECT_COLOR, frameData ( )->m_colorFrame.toString ( ) );
+	if (frameData ( )->m_iFrameStyle != (int)Qt::SolidLine)
+		frameNode.setAttribute ( FRAME_OBJECT_STYLE, frameData ( )->m_iFrameStyle );
+	if (frameData ( )->m_iFrameJoin != (int)Qt::MiterJoin)
+		frameNode.setAttribute ( FRAME_OBJECT_JOIN,  frameData ( )->m_iFrameJoin );
 
 	theElement.appendChild( frameNode );
 	// And here we write the base class ...
@@ -294,11 +365,21 @@ StructureItem *FrameObject::createStructure ( StructureItem *pParentItem )
 {
   StructureItem *pFrameItem, *pTemp=NULL;
 
+  if ( ! pParentItem )  {
+    if ( m_pOverlay  )
+      pTemp = m_pOverlay->createStructure ( NULL );
+    if ( m_pShadow )
+      pTemp = m_pShadow->createStructure  ( NULL );
+    modifiers ( )->createStructure  ( NULL, NULL );
+    m_pStructureItem = NULL;
+    return NULL;
+  }
+
   QString qsName ( tr("Frame (%1, %2, %3, %4)" ).
 		   arg ( rect ( ).x      ( ) ).arg ( rect ( ).y      ( ) ).
 		   arg ( rect ( ).width  ( ) ).arg ( rect ( ).height ( ) ) );
   setName ( qsName );
-  
+
   qsName  = tr ( "Frame Object" );
   QString qsInfo = QString ("%1 - (%2, %3, %4 %5)").arg ( width ( ) ).
     arg ( rect ( ).x      ( ) ).arg ( rect ( ).y      ( ) ).
@@ -306,7 +387,7 @@ StructureItem *FrameObject::createStructure ( StructureItem *pParentItem )
 
   pFrameItem = m_pStructureItem;
   if ( ! pFrameItem ) {
-    pFrameItem = new StructureItem ( this, pParentItem, pParentItem->lastChild ( ), qsName, qsInfo );
+    pFrameItem = new StructureItem ( this, StructureItem::Frame, pParentItem, pParentItem->lastChild ( ), qsName, qsInfo );
     pFrameItem->setExpandable ( TRUE );
     m_pStructureItem = pFrameItem;
   }
@@ -323,18 +404,18 @@ StructureItem *FrameObject::createStructure ( StructureItem *pParentItem )
 
   qsArgs = QString ( "%1" ).arg ( width ( ) );
   pTemp  = pFrameItem->setSibling ( this, pTemp, tr ( "Frame Width" ), qsArgs );
-  
+
   qsArgs = tr ( "color(%1, %2, %3)" ).
-    arg ( m_colorFrame.red   ( ) ).
-    arg ( m_colorFrame.green ( ) ).
-    arg ( m_colorFrame.blue  ( ) );
+    arg ( frameData ( )->m_colorFrame.red   ( ) ).
+    arg ( frameData ( )->m_colorFrame.green ( ) ).
+    arg ( frameData ( )->m_colorFrame.blue  ( ) );
 
   pTemp  = pFrameItem->setSibling ( this, pTemp, tr ( "Frame Color" ), qsArgs );
-  
+
   if ( m_pShadow )
     pTemp = m_pShadow->createStructure ( pFrameItem );
 
-  m_modifiers.createStructure ( pFrameItem, pTemp );
+  modifiers( )->createStructure ( pFrameItem, pTemp );
 
   return pFrameItem;
 }
@@ -347,9 +428,12 @@ void FrameObject::slotEdit()
 
 void FrameObject::slotMatrix()
 {
-	DialogMatrix *pDialog = new DialogMatrix(MenuObject::parent());
-	pDialog->initMe(this);
-	pDialog->show();
+  if ( m_pMatrixDialog )
+    return;
+
+  m_pMatrixDialog = new DialogMatrix ( MenuObject::parent ( ) );
+  m_pMatrixDialog->initMe ( this );
+  m_pMatrixDialog->show   ( );
 }
 
 void FrameObject::slotAddText()
@@ -403,27 +487,27 @@ AnimationAttribute *FrameObject::getSpecificAttributes (long iMaxNumberOfFrames,
 {
 	AnimationAttribute *pAnimAttr = NULL;
 	if (qsProperty == "color.red")	{
-		FunctionCallback<Rgba> *callMeBack = new FunctionCallback<Rgba>(m_colorFrame, TYPE_COLOR_RED);
-		pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_colorFrame.red(), callMeBack);
+		FunctionCallback<Rgba> *callMeBack = new FunctionCallback<Rgba>(frameData ( )->m_colorFrame, TYPE_COLOR_RED);
+		pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, frameData ( )->m_colorFrame.red(), callMeBack);
 	}
 	else if (qsProperty == "color.green")	{
-		FunctionCallback<Rgba> *callMeBack = new FunctionCallback<Rgba>(m_colorFrame, TYPE_COLOR_GREEN);
-		pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_colorFrame.green(), callMeBack);
+		FunctionCallback<Rgba> *callMeBack = new FunctionCallback<Rgba>(frameData ( )->m_colorFrame, TYPE_COLOR_GREEN);
+		pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, frameData ( )->m_colorFrame.green(), callMeBack);
 	}
 	else if (qsProperty == "color.blue")	{
-		FunctionCallback<Rgba> *callMeBack = new FunctionCallback<Rgba>(m_colorFrame, TYPE_COLOR_BLUE);
-		pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_colorFrame.blue(), callMeBack);
+		FunctionCallback<Rgba> *callMeBack = new FunctionCallback<Rgba>(frameData ( )->m_colorFrame, TYPE_COLOR_BLUE);
+		pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, frameData ( )->m_colorFrame.blue(), callMeBack);
 	}
 	else if (qsProperty == "color.alpha")	{
-		FunctionCallback<Rgba> *callMeBack = new FunctionCallback<Rgba>(m_colorFrame, TYPE_COLOR_ALPHA);
-		pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_colorFrame.alpha(), callMeBack);
+		FunctionCallback<Rgba> *callMeBack = new FunctionCallback<Rgba>(frameData ( )->m_colorFrame, TYPE_COLOR_ALPHA);
+		pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, frameData ( )->m_colorFrame.alpha(), callMeBack);
 	}
 	else if (qsProperty == "frameWidth")
-		pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_iFrameWidth, &m_iFrameWidth);
+		pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, frameData ( )->m_iFrameWidth, &frameData ( )->m_iFrameWidth);
 	else if (qsProperty == "frameStyle")
-		pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_iFrameStyle, &m_iFrameStyle);
+		pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, frameData ( )->m_iFrameStyle, &frameData ( )->m_iFrameStyle);
 	else if (qsProperty == "frameJoin")
-		pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_iFrameJoin, &m_iFrameJoin);
+		pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, frameData ( )->m_iFrameJoin, &frameData ( )->m_iFrameJoin);
 	return pAnimAttr;
 }
 
