@@ -21,6 +21,21 @@ class QPixmap;
 class FrameObject : public MenuObject
 {
 	Q_OBJECT
+	class FrameData : public MenuObject::Data
+	{
+	public:
+	           FrameData ( );
+	  virtual ~FrameData ( );
+	  virtual  Data &operator  = ( Data & );
+	  virtual  bool  operator == ( Data & );
+	  virtual  Data *clone ( );
+
+	  Rgba m_colorFrame;
+	  int  m_iFrameWidth;
+	  int  m_iFrameStyle;
+	  int  m_iFrameJoin;
+	};
+
 public:
 	// Public functions needed for the abstract base class.
 	FrameObject (QWidget *pParent = 0);
@@ -59,13 +74,10 @@ protected slots:
 	virtual void slotDefineAsButton    ( );
 
 protected:
-	virtual AnimationAttribute *getSpecificAttributes (long, QString);
+	virtual AnimationAttribute *getSpecificAttributes ( long, QString );
 
 private:
-	Rgba m_colorFrame;
-	int  m_iFrameWidth;
-	int  m_iFrameStyle;
-	int  m_iFrameJoin;
+	FrameData *frameData ( );
 };
 
-#endif	// FRAMEOBJECT_H
+#endif  // FRAMEOBJECT_H
