@@ -22,19 +22,73 @@
 #include <qdom.h>
 
 #include "xml_dvd.h"
-#include "menuobject.h"
+//#include "menuobject.h"
 #include "frameobject.h"
 #include "imageobject.h"
 #include "dialogmovie.h"
 #include "messagebox.h"
 #include "dialogmatrix.h"
+#include "overlayobject.h"
 #include "dialoggeometry.h"
 #include "dialoganimation.h"
 #include "structuretoolbar.h"
 
-ImageObject::ImageObject ( QWidget *pParent )
-	: MenuObject ( pParent )
+// The following three includes required for undo.
+#include "global.h"
+#include "qdvdauthor.h"
+#include "dvdmenuundoobject.h"
+
+ImageObject::ImageData::ImageData ( )
+  : MenuObject::Data ( )
 {
+    m_iDataType = Data::IMAGE;
+}
+
+ImageObject::ImageData::~ImageData ( )
+{
+}
+
+MenuObject::Data &ImageObject::ImageData::operator = ( Data &theOther )
+{
+  // At this point we are sure both are acutally ImageManipulator objects.
+  MenuObject::Data::operator= ( theOther );
+  if ( m_iDataType == theOther.m_iDataType )  {
+       ImageData *pOther  = (ImageData *)&theOther;
+       m_imageManipulator = pOther->m_imageManipulator;
+  }
+  if ( theOther.m_iDataType == Data::DATA )  {
+       m_imageManipulator = theOther.m_modifiers;
+  }
+  return *this;
+}
+
+bool ImageObject::ImageData::operator == ( Data &theOther )
+{
+  // At this point we are sure both are acutally ImageManipulator objects.
+  bool bReturn = ( MenuObject::Data::operator == ( theOther ) );
+  if ( ! bReturn )
+    return false;
+
+  if ( m_iDataType == theOther.m_iDataType )  {
+    ImageData *pOther = (ImageData *)&theOther;
+    bReturn = ( m_imageManipulator == pOther->m_imageManipulator );
+  }
+  return bReturn;
+}
+
+MenuObject::Data *ImageObject::ImageData::clone ( )
+{
+  ImageData *pData = new ImageData;
+  *pData  = *this;
+  return pData;
+}
+
+ImageObject::ImageObject ( QWidget *pParent )
+  : MenuObject ( pParent )
+{
+  if ( m_pData )
+    delete m_pData;        // part of the base class
+  m_pData = new ImageData; // will be deleted in base class
   m_qsObjectType = QString ( IMAGE_OBJECT );
 }
 
@@ -61,30 +115,31 @@ void ImageObject::setFile ( QString fileName )
 void ImageObject::setZoom ( float fZoom )
 {
   // For the purpose of this class the Zoom is alreaduy incorporated into m_pixmap ...
-  m_imageManipulator.fZoom = fZoom;
+  manipulator ( ).fZoom = fZoom;
 }
 
 void ImageObject::setManipulator(ImageManipulator &theOther)
 {
-  m_imageManipulator = theOther;
+  manipulator ( ) = theOther;
 }
 
 void ImageObject::setModifiers ( Modifiers &theOther )
 {
-  Modifiers *pModifier = (Modifiers *)&m_imageManipulator;
+  Modifiers *pModifier = (Modifiers *)&manipulator ( );
   *pModifier = theOther;
 }
 
 ImageManipulator &ImageObject::manipulator()
 {
-  return m_imageManipulator;
+  ImageData *pData = (ImageData *)data ( );
+  return     pData->m_imageManipulator;
 }
 
 Modifiers *ImageObject::modifiers()
 {
   // Return the base class only ...
   // Note: this is needed to use the same MatrixDialog for text and images.
-  return (Modifiers *) &m_imageManipulator;
+  return (Modifiers *) &manipulator ( );
 }
 
 QString &ImageObject::fileName()
@@ -94,17 +149,20 @@ QString &ImageObject::fileName()
 
 QImage &ImageObject::image ( )
 {
+  if ( overlay ( ) )  {
+    OverlayObject *pOverlayObject = (OverlayObject *)overlay ( );
+    return pOverlayObject->image ( m_image );
+  }
   return m_image;
 }
 
 float ImageObject::zoom ( )
 {
-	return m_imageManipulator.fZoom;
+	return manipulator ( ).fZoom;
 }
 
 void ImageObject::drawContents ( QPainter *painter )
 {
-  // printf ( "%s::%s::%d me<%p> shadow=<%p>\n", __FILE__, __FUNCTION__, __LINE__, this, m_pShadow );
 	QWMatrix theMatrix;
 	// Here we calculate the center point of gravity (rotation)
 	QPoint centerPos;
@@ -113,7 +171,7 @@ void ImageObject::drawContents ( QPainter *painter )
 	// Here we define the cenetered rect.
 	QRect theRect  ( (int)-( rect ( ).width ( )/ 2.0 ), (int)-( rect ( ).height ( ) / 2.0 ), rect ( ).width ( ), rect ( ).height ( ) );
 
-	QPen thePen ( QColor ( 255, 0, 0 ), 2, Qt::DashDotLine );
+	QPen     thePen ( QColor ( 255, 0, 0 ), 2, Qt::DashDotLine );
 	painter->setPen ( thePen );
 
 	// From the Qt documentation ...
@@ -134,7 +192,7 @@ void ImageObject::drawContents ( QPainter *painter )
 	// Here we get the rect that encompasses the button for this object.
 	m_boundingRect = theMatrix.mapRect ( theRect );
 
-	if ( m_bDrawRect )
+	if ( isActive ( ) )
 	  drawActiveFrame ( painter );
 
 	painter->setWorldMatrix ( theMatrix );
@@ -150,7 +208,7 @@ void ImageObject::drawContents ( QPainter *painter )
 	if ( manipulator ( ).fTransparency == 1.0 )
 		return;
 
-	if (!m_bDrawRect)	{
+	if ( ! isActive ( ) )  {
                 if ( m_pShadow )
 	             m_pShadow->drawContents ( painter );
 
@@ -163,12 +221,24 @@ void ImageObject::drawContents ( QPainter *painter )
 		int iTargetStartX = painter->viewport ( ).x ( ) + m_boundingRect.x ( );
 		int iTargetStartY = painter->viewport ( ).y ( ) + m_boundingRect.y ( );
 		// Use the fast method whenever possible
-		if (	( manipulator ( ).fRotate        == 0.0 ) && 
-			( manipulator ( ).fShearX        == 0.0 ) && 
-			( manipulator ( ).fShearY        == 0.0 ) &&
-			( manipulator ( ).fTransparency  == 0.0 ) && 
+//		if (	( manipulator ( ).fRotate        == 0.0 ) &&
+//			( manipulator ( ).fShearX        == 0.0 ) &&
+//			( manipulator ( ).fShearY        == 0.0 ) &&
+//			( manipulator ( ).fTransparency  == 0.0 ) &&
+//			( manipulator ( ).listColorKeys.count ( ) < 1 ) ) {
+		if (	( manipulator ( ).fTransparency  == 0.0 ) &&
 			( manipulator ( ).listColorKeys.count ( ) < 1 ) ) {
-		  //bitBlt ( painter->device ( ), iTargetStartX, iTargetStartY, &pixmap(), iSourceStartX, iSourceStartY, iSourceWidth, iSourceHeight, Qt::CopyROP, true);
+		if (	( manipulator ( ).fRotate        == 0.0 ) &&
+			( manipulator ( ).fShearX        == 0.0 ) &&
+			( manipulator ( ).fShearY        == 0.0 ) )  {
+		  QWMatrix theMatrix;
+//		  theMatrix.scale  ( manipulator ( ).fScaleX, manipulator ( ).fScaleY );
+		  theMatrix.shear  ( manipulator ( ).fShearX, manipulator ( ).fShearY );
+		  theMatrix.rotate ( manipulator ( ).fRotate );
+		  QImage temp = image ( ).xForm  ( theMatrix );
+		  painter->drawImage  ( iTargetStartX, iTargetStartY, temp, iSourceStartX, iSourceStartY, iSourceWidth, iSourceHeight, Qt::CopyROP );
+		}
+		else
 		  painter->drawImage ( iTargetStartX, iTargetStartY, image ( ), iSourceStartX, iSourceStartY, iSourceWidth, iSourceHeight, Qt::CopyROP );
 		}
 		//iSourceStartX, iSourceStartY, iSourceWidth, iSourceHeight, Qt::CopyROP, true);
@@ -269,6 +339,11 @@ void ImageObject::drawContents (QPainter *pPainter, int iRenderFrameNumber, int)
        ( ( iRenderFrameNumber > modifiers ( )->iStopFrame  ) && ( modifiers ( )->iStopFrame  != -1 ) ) ) 
     return;
 
+  if ( overlay ( ) )
+       overlay ( )->drawContents ( pPainter, iRenderFrameNumber, 0 );
+  if ( shadow  ( ) )
+       shadow  ( )->drawContents ( pPainter, iRenderFrameNumber, 0 );
+
   // And then we ought to refresh the pixmap ...
   if ( bRefresh ) {
     updatePixmap ( );
@@ -476,6 +551,9 @@ void ImageObject::resetStructureItem ( )
   if ( m_pShadow )
        m_pShadow->resetStructureItem ( );
 
+  if ( m_pOverlay )
+       m_pOverlay->resetStructureItem ( );
+
   if ( m_pStructureItem )
     delete m_pStructureItem;
   m_pStructureItem = NULL;
@@ -488,10 +566,20 @@ StructureItem *ImageObject::createStructure ( StructureItem *pParentItem )
   QFileInfo fileInfo ( m_qsFileName );
   QString qsName;
 
+  if ( ! pParentItem )  {
+    if ( m_pOverlay  )
+      pTemp = m_pOverlay->createStructure ( NULL );
+    if ( m_pShadow )
+      pTemp = m_pShadow->createStructure  ( NULL );
+    manipulator ( ).createStructure ( NULL );
+    m_pStructureItem = NULL;
+    return NULL;
+  }
+
   qsName = tr( "Image Object" );
 
   if ( ! m_pStructureItem ) {
-    m_pStructureItem = new StructureItem ( this, pParentItem, pParentItem->lastChild ( ), qsName, fileInfo.fileName ( ) );
+    m_pStructureItem = new StructureItem ( this, StructureItem::Image, pParentItem, pParentItem->lastChild ( ), qsName, fileInfo.fileName ( ) );
     m_pStructureItem->setExpandable      ( TRUE );
   }
 
@@ -502,6 +590,9 @@ StructureItem *ImageObject::createStructure ( StructureItem *pParentItem )
   pTemp = m_pStructureItem->setSibling ( this, NULL, tr ( "Geometry" ), QString ( "%1, %2, %3, %4" ).
        	         arg ( rect().x()).arg(rect().y()).arg(rect().width()).arg(rect().height()) );
 
+  if ( m_pOverlay )
+    pTemp = m_pOverlay->createStructure ( m_pStructureItem );
+
   if ( m_pShadow )
     pTemp = m_pShadow->createStructure ( m_pStructureItem );
 
@@ -512,24 +603,27 @@ StructureItem *ImageObject::createStructure ( StructureItem *pParentItem )
 
 bool ImageObject::mousePressEvent (QMouseEvent *pEvent)
 {
-	m_currentMousePos = pEvent->pos();
-	if (pEvent->button() == Qt::RightButton)	{
-		QPoint globalPos = pEvent->globalPos();
-		return createContextMenu ( globalPos );
-	}
-	else {
-		m_bDrawRect = true;
-	}
-	return false;
+  m_currentMousePos = pEvent->pos ( );
+  if ( pEvent->button ( ) == Qt::RightButton )  {
+    QPoint globalPos = pEvent->globalPos ( );
+    return createContextMenu ( globalPos );
+  }
+  else {
+    m_bDrawRect = true;
+  }
+
+  return false;
 }
 
-bool ImageObject::mouseReleaseEvent (QMouseEvent *)
+bool ImageObject::mouseReleaseEvent ( QMouseEvent * )
 {
-	m_bDrawRect = false;
-	updatePixmap ();
-	emit (signalUpdatePixmap());
-	emit (signalUpdateStructure());
-	return false;
+  UndoBuffer *pUndo = Global::pApp->getUndoBuffer ( );
+  pUndo->removeUnchanged ( );
+  m_bDrawRect = false;
+  updatePixmap ( );
+  emit ( signalUpdatePixmap    ( ) );
+  emit ( signalUpdateStructure ( ) );
+  return false;
 }
 
 bool ImageObject::mouseDoubleClickEvent (QMouseEvent *)
@@ -541,7 +635,7 @@ bool ImageObject::mouseDoubleClickEvent (QMouseEvent *)
 
 bool ImageObject::createContextMenu ( QPoint globalPos )
 {
-  QString qsShadow;
+  QString qsShadow, qsOverlay;
   QPoint globalPos2 = globalPos;
   QPopupMenu *pStackMenu = new QPopupMenu(m_pParent);
   pStackMenu->insertItem ( tr ("Cut") , this, SLOT ( slotCut  ( ) ) );
@@ -557,6 +651,11 @@ bool ImageObject::createContextMenu ( QPoint globalPos )
   if ( shadow ( ) )
     qsShadow = tr ( "Edit Shadow ..." );
 
+  qsOverlay   = tr ( "Add Mask ..."  );
+  if ( overlay ( ) )
+    qsOverlay = tr ( "Edit Mask ..." );
+
+
   if ( m_pContextMenu )
     delete m_pContextMenu;
   m_pContextMenu = new QPopupMenu ( m_pParent );
@@ -574,6 +673,10 @@ bool ImageObject::createContextMenu ( QPoint globalPos )
   if ( shadow ( ) )
     m_pContextMenu->insertItem ( tr ("Delete Shadow"), this, SLOT ( slotDeleteShadow ( ) ) );
 
+  m_pContextMenu->insertItem ( qsOverlay             , this, SLOT ( slotAddOverlay   ( ) ) );
+  if ( overlay ( ) )
+    m_pContextMenu->insertItem ( tr ("Delete Mask"),   this, SLOT ( slotDeleteOverlay( ) ) );
+
   //	m_pContextMenu->insertItem ( tr ("Add Frame")      , this, SLOT ( slotAddFrame  ( ) ) );
   //	m_pContextMenu->insertItem ( tr ("Add Text")       , this, SLOT ( slotAddText   ( ) ) );
   m_pContextMenu->insertSeparator ( );
@@ -644,22 +747,23 @@ void ImageObject::slotEdit()
 	mouseDoubleClickEvent(NULL);
 }
 
-void ImageObject::slotProperties()
+void ImageObject::slotProperties ( )
 {
   ImageManipulator *pMan= &manipulator();
 
-  DialogGeometry *pGeometry = new DialogGeometry ();
-  pGeometry->initMe(rect(), pMan);
-  connect ( pGeometry->m_pButtonAnimation, SIGNAL(clicked()), this, SLOT(slotAnimateMe ( ) ) );
-	
-  if (pGeometry->exec() == QDialog::Accepted)	{
+  DialogGeometry *pGeometry = new DialogGeometry ( );
+  pGeometry->initMe ( rect ( ), pMan );
+  connect ( pGeometry->m_pButtonAnimation, SIGNAL ( clicked ( ) ), this, SLOT ( slotAnimateMe ( ) ) );
+
+  if ( pGeometry->exec ( ) == QDialog::Accepted )  {
+    DVDMenuUndoObject *pUndo = new DVDMenuUndoObject ( this, MenuObject::StateUnknown );
     int iX, iY, iWidth, iHeight;
     iX      = pGeometry->m_pEditX->text().toInt();
     iY      = pGeometry->m_pEditY->text().toInt();
     iWidth  = pGeometry->m_pEditWidth ->text().toInt();
     iHeight = pGeometry->m_pEditHeight->text().toInt();
-    QRect rect (iX, iY, iWidth, iHeight);
-		
+    QRect rect ( iX, iY, iWidth, iHeight );
+
     ImageManipulator::colorKeying *pColorKey;
     ListViewColorItem *pItem;
     uint t;
@@ -674,17 +778,24 @@ void ImageObject::slotProperties()
     if ( pGeometry->m_pListViewCK->childCount ( ) )	{
       pItem = (ListViewColorItem *)pGeometry->m_pListViewCK->firstChild ( );
       while (pItem)	{
-	theColor.fromString ( pItem->text ( 0 ) );	// The Color name is stored in the first text
-	fDelta = pItem->text ( 1 ).toFloat ( );		// The delta value is stored in the second column.
-	pColorKey = new ImageManipulator::colorKeying ( theColor.rgb ( ), fDelta );
-	pMan->listColorKeys.append ( pColorKey );
-	pItem = (ListViewColorItem *)pItem->itemBelow ();
+        theColor.fromString ( pItem->text ( 0 ) );	// The Color name is stored in the first text
+        fDelta = pItem->text ( 1 ).toFloat ( );		// The delta value is stored in the second column.
+        pColorKey = new ImageManipulator::colorKeying ( theColor.rgb ( ), fDelta );
+        pMan->listColorKeys.append ( pColorKey );
+        pItem = (ListViewColorItem *)pItem->itemBelow ();
       }
     }
     setRect ( rect );
     updatePixmap ( );
     updateShadow ( );
     emit ( signalUpdatePixmap ( ) );
+    if   ( pUndo->hasChanged  ( ) )  {
+      UndoBuffer *pBuffer = Global::pApp->getUndoBuffer ( );
+      if ( pBuffer )
+           pBuffer->push ( pUndo );
+    }
+    else
+      delete pUndo;
   }
   delete pGeometry;
 }
@@ -725,10 +836,13 @@ void ImageObject::rotate(float fRot)
 
 void ImageObject::slotMatrix()
 {
-  DialogMatrix *pDialog = new DialogMatrix(MenuObject::parent());
-  pDialog->initMe(this);
-  connect (this, SIGNAL(signalDeleteMe(MenuObject *)), pDialog, SLOT(slotDeleteObject(MenuObject *)));
-  pDialog->show();
+  if ( m_pMatrixDialog )
+    return;
+
+  m_pMatrixDialog = new DialogMatrix(MenuObject::parent());
+  m_pMatrixDialog->initMe ( this );
+  connect ( this, SIGNAL ( signalDeleteMe ( MenuObject * ) ), m_pMatrixDialog, SLOT ( slotDeleteObject ( MenuObject * ) ) );
+  m_pMatrixDialog->show   ( );
 }
 
 void ImageObject::slotAddFrame()
@@ -763,11 +877,16 @@ MenuObject *ImageObject::clone ( QWidget *pParent, MenuObject * )
   pNewObject->setManipulator ( manipulator ( ) );
   pNewObject->setAnimation   ( animation   ( ) );
 
-  MenuObject *pShadow = shadow ( );
+  MenuObject *pShadow = shadow  ( );
   if ( pShadow )
        pShadow = pShadow->clone ( pParent, pNewObject );
   pNewObject->setShadow ( pShadow );
 
+  MenuObject *pOverlay = overlay ( );
+  if ( pOverlay )
+       pOverlay = pOverlay->clone ( pParent, pNewObject );
+  pNewObject->setOverlay ( pOverlay );
+
   return pNewObject;
 }
 
@@ -778,53 +897,53 @@ AnimationAttribute *ImageObject::getSpecificAttributes (long iMaxNumberOfFrames,
   AnimationAttribute *pAnimAttr = NULL;
 
   if (qsProperty == "zoom")
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_imageManipulator.fZoom, &m_imageManipulator.fZoom);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, manipulator ( ).fZoom, &manipulator ( ).fZoom);
   else if (qsProperty == "rotate")
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_imageManipulator.fRotate, &m_imageManipulator.fRotate);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, manipulator ( ).fRotate, &manipulator ( ).fRotate);
   else if (qsProperty == "shear.x")
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_imageManipulator.fShearX, &m_imageManipulator.fShearX);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, manipulator ( ).fShearX, &manipulator ( ).fShearX);
   else if (qsProperty == "shear.y")
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_imageManipulator.fShearY, &m_imageManipulator.fShearY);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, manipulator ( ).fShearY, &manipulator ( ).fShearY);
   else if (qsProperty == "scale.x")
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_imageManipulator.fScaleX, &m_imageManipulator.fScaleX);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, manipulator ( ).fScaleX, &manipulator ( ).fScaleX);
   else if (qsProperty == "scale.y")
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_imageManipulator.fScaleY, &m_imageManipulator.fScaleY);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, manipulator ( ).fScaleY, &manipulator ( ).fScaleY);
   else if (qsProperty == "transparency")
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_imageManipulator.fTransparency, &m_imageManipulator.fTransparency);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, manipulator ( ).fTransparency, &manipulator ( ).fTransparency);
   else if (qsProperty == "visibleFromTo.startFrame") {
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_imageManipulator.iStartFrame, &m_imageManipulator.iStartFrame);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, manipulator ( ).iStartFrame, &manipulator ( ).iStartFrame);
     pAnimAttr->setNeedRefresh ( false );
   }
   else if (qsProperty == "visibleFromTo.stopFrame") {
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_imageManipulator.iStopFrame, &m_imageManipulator.iStopFrame);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, manipulator ( ).iStopFrame, &manipulator ( ).iStopFrame);
     pAnimAttr->setNeedRefresh ( false );
   }
   //	else if (qsProperty == "backgroundFileName")
-  //		pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_imageManipulator.backgroundFileName, &m_imageManipulator.backgroundFileName);
+  //		pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, manipulator ( ).backgroundFileName, &manipulator ( ).backgroundFileName);
   //	else if (qsProperty == "showBackground")
-  //		pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_imageManipulator.bShowBackground, &m_imageManipulator.bShowBackground);
+  //		pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, manipulator ( ).bShowBackground, &manipulator ( ).bShowBackground);
   else if (qsProperty == "kenBurns.x1")
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_imageManipulator.fKenBurnsX1, &m_imageManipulator.fKenBurnsX1);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, manipulator ( ).fKenBurnsX1, &manipulator ( ).fKenBurnsX1);
   else if (qsProperty == "kenBurns.y1")
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_imageManipulator.fKenBurnsY1, &m_imageManipulator.fKenBurnsY1);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, manipulator ( ).fKenBurnsY1, &manipulator ( ).fKenBurnsY1);
   else if (qsProperty == "kenBurns.x2")
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_imageManipulator.fKenBurnsX2, &m_imageManipulator.fKenBurnsX2);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, manipulator ( ).fKenBurnsX2, &manipulator ( ).fKenBurnsX2);
   else if (qsProperty == "kenBurns.y2")
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_imageManipulator.fKenBurnsY2, &m_imageManipulator.fKenBurnsY2);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, manipulator ( ).fKenBurnsY2, &manipulator ( ).fKenBurnsY2);
   //	else if (qsProperty == "res.x")
-  //		pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_imageManipulator.iXRes, &m_imageManipulator.iXRes);
+  //		pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, manipulator ( ).iXRes, &manipulator ( ).iXRes);
   //	else if (qsProperty == "res.y")
-  //		pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_imageManipulator.iYRes, &m_imageManipulator.iYRes);
+  //		pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, manipulator ( ).iYRes, &manipulator ( ).iYRes);
   //	else if (qsProperty == "aspectRatio")
-  //		pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_imageManipulator.fAspectRatio, &m_imageManipulator.fAspectRatio);
+  //		pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, manipulator ( ).fAspectRatio, &manipulator ( ).fAspectRatio);
   else if (qsProperty == "blendColor.red")
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_imageManipulator.fRed, &m_imageManipulator.fRed);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, manipulator ( ).fRed, &manipulator ( ).fRed);
   else if (qsProperty == "blendColor.green")
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_imageManipulator.fGreen, &m_imageManipulator.fGreen);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, manipulator ( ).fGreen, &manipulator ( ).fGreen);
   else if (qsProperty == "blendColor.blue")
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_imageManipulator.fBlue, &m_imageManipulator.fBlue);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, manipulator ( ).fBlue, &manipulator ( ).fBlue);
   else if (qsProperty == "brightness")
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_imageManipulator.fBrightness, &m_imageManipulator.fBrightness);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, manipulator ( ).fBrightness, &manipulator ( ).fBrightness);
   else if (qsProperty == "fileName")
     pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_qsFileName, &m_qsFileName);
 
