@@ -16,11 +16,22 @@
 #include <qimage.h>
 #include "menuobject.h"
 
-
 class ImageObject : public MenuObject
 {
 	Q_OBJECT
 
+protected:
+	class ImageData : public MenuObject::Data
+	{
+	public:
+	           ImageData ( );
+	  virtual ~ImageData ( );
+	  virtual  Data &operator  = ( Data & );
+	  virtual  bool  operator == ( Data & );
+	  virtual  Data *clone ( );
+
+	  ImageManipulator m_imageManipulator;
+	};
 public:
 	// Public functions needed for the abstract base class.
 	ImageObject (QWidget *pParent = 0);
@@ -79,7 +90,6 @@ protected:
 	QString m_qsFileName;
 	//QPixmap m_pixmap;
 	QImage  m_image;
-	ImageManipulator m_imageManipulator;
 };
 
 #endif	// IMAGEOBJECT_H
