@@ -79,7 +79,7 @@ bool DVDAuthor::import ( )
     listFileNames = QFileDialog::getOpenFileNames ( "*.xml", Global::qsCurrentPath, NULL, "OpenDvdauthorXml", "dvdauthor and spumux xml file selection");
     if ( listFileNames.count() > 0 ) {
       theFile.setFile ( listFileNames[0] );
-      Global::qsCurrentPath = theFile.dirPath();
+      Global::qsCurrentPath = theFile.dirPath ( true );
 
       for (t=0;t<(int)listFileNames.count();t++) {
 	pSpumux = new CXmlSpumux;
