@@ -204,7 +204,7 @@ bool Kino::import ( QString fileName )
     return false;
   
   theFile.setFile ( fileName );
-  Global::qsCurrentPath = theFile.dirPath();
+  Global::qsCurrentPath = theFile.dirPath ( true );
 
   return buildFromKinoXML ( &theXmlStructure );
 }
