@@ -127,7 +127,7 @@ void ListViewItemMedia::initMe ( SourceFileEntry *pEntry, const QColor &theBaseC
 
 void ListViewItemMedia::updateItem ()
 {
-  SourceFileInfo *pInfo = sourceFileInfo ();
+  SourceFileInfo *pInfo = sourceFileInfo ( );
   if ( ! pInfo )
     return;
 
@@ -136,6 +136,10 @@ void ListViewItemMedia::updateItem ()
     m_backgroundColor = QColor (COLOR_MOVIE_FILES_BAD);
   if ( pInfo->pTranscodeInterface )
     m_backgroundColor = QColor (COLOR_MOVIE_TRANSCODE);
+
+  QFileInfo fileInfo ( pInfo->qsFileName );
+  QString qsInfo ( fileInfo.fileName () + QString ("\n") + pInfo->qsLength );
+  setText ( 1, qsInfo);
 }
 
 void ListViewItemMedia::replaceInfo (SourceFileInfo *pInfo)
