@@ -17,8 +17,10 @@
 #include <qfileinfo.h>
 #include <qdragobject.h>
 
-#include "sourcefileentry.h"
+#include "global.h"
+#include "qdvdauthor.h"
 #include "listviewmedia.h"
+#include "sourcefileentry.h"
 #include "listviewitemmedia.h"
 
 ListViewMedia::ListViewMedia (QWidget *pParent, const char *pName, WFlags f)
@@ -54,17 +56,33 @@ SourceFileInfo *ListViewMedia::draggingSourceFileInfo()
 	return m_pDraggingSourceFileInfo;
 }
 
-void ListViewMedia::dropEvent (QDropEvent *pDropEvent)
+void ListViewMedia::dragEnterEvent ( QDragEnterEvent *pEvent )
+{
+	// Tell the Widget that we accept ImageDrops ...
+	// This is necessary otherwise the dropEvent does not occur.
+	bool bAccept = QImageDrag::canDecode ( pEvent );
+	bAccept     |= QTextDrag::canDecode  ( pEvent );
+
+	pEvent->accept ( bAccept );
+}
+
+void ListViewMedia::dropEvent ( QDropEvent *pDropEvent )
 {
 	// This function handles the re-sorting of the SourceFileEntries.
 	// Meaning if the user can drag'n drop a SourceFileInfo and drop it where he want's to.
 	// This also means that we need to handle different SourceFileEntries for the SourceFileInfo.
 	// First we check if the source came from this listView, otherwise we don't really care.
-	if ( pDropEvent->source() != this )
+	QString qsText;
+	if ( QTextDrag::decode ( pDropEvent, qsText ) )  {
+		// Lets see if we have a video file
+		Global::pApp->addDroppedSource ( qsText );
+		return;
+	}
+	if ( pDropEvent->source ( ) != this )
 		return;
 
-	SourceFileEntry *pSourceFileEntry = draggingSourceFileEntry();
-	SourceFileInfo  *pSourceFileInfo  = draggingSourceFileInfo ();	// Could be NULL if we dragged a Header
+	SourceFileEntry *pSourceFileEntry = draggingSourceFileEntry ( );
+	SourceFileInfo  *pSourceFileInfo  = draggingSourceFileInfo  ( );  // Could be NULL if we dragged a Header
 
 	// If a slideshow is in the makinng, then stop right here ...
 	if ( pSourceFileEntry->bIsSlideshow )
@@ -265,13 +283,6 @@ void ListViewMedia::handleAudioTracks ( SourceFileEntry *pEntry )
   }
 }
 
-void ListViewMedia::dragEnterEvent(QDragEnterEvent *pEvent)
-{
-	// Tell the Widget that we accept ImageDrops ...
-	// This is necessary otherwise the dropEvent does not occur.
-	pEvent->accept(QImageDrag::canDecode(pEvent));
-}
-
 QDragObject *ListViewMedia::dragObject()
 {
 	ListViewItemMedia *pCurrentItem     = (ListViewItemMedia *)currentItem ();
