@@ -27,10 +27,47 @@
 
 #include "xml_dvd.h"
 #include "menuobject.h"
+#include "qdvdauthor.h"
 #include "shadowobject.h"
+#include "overlayobject.h"
 #include "structuretoolbar.h"
+#include "dvdmenuundoobject.h"
+#include "dialogmatrix.h"
 
-//void *g_pHelper = NULL;
+MenuObject::Data::Data ( )
+{
+  m_iDataType = DATA;
+}
+
+MenuObject::Data::~Data ( )
+{
+
+}
+
+MenuObject::Data &MenuObject::Data::operator = ( Data &theOther )
+{
+  m_rect        = theOther.m_rect;
+  m_qsName      = theOther.m_qsName;
+  m_modifiers   = theOther.m_modifiers;
+  m_qsAnimation = theOther.m_qsAnimation;
+  return *this;
+}
+
+bool MenuObject::Data::operator == ( Data &theOther )
+{
+  bool bReturn = ( ( m_rect        == theOther.m_rect        ) &&
+                   ( m_qsName      == theOther.m_qsName      ) &&
+                   ( m_modifiers   == theOther.m_modifiers   ) &&
+                   ( m_qsAnimation == theOther.m_qsAnimation ) );
+  return bReturn;
+}
+
+MenuObject::Data *MenuObject::Data::clone ( )
+{
+  Data *pData = new Data;
+  *pData = *this;
+  return pData;
+}
 
 MenuObject::MenuObject (QWidget *pParent)
   : QObject (pParent)
@@ -40,11 +77,14 @@ MenuObject::MenuObject (QWidget *pParent)
   m_bDrawRect      = false;
   m_pContextMenu   = NULL;
   m_pShadow        = NULL;
+  m_pOverlay       = NULL;
   m_pStructureItem = NULL;
+  m_pMatrixDialog  = NULL;
   m_objectState    = StateUnknown;
+  m_pData          = new Data;
 }
 
-MenuObject::~MenuObject ()
+MenuObject::~MenuObject ( )
 {
   uint t;
   if ( m_pContextMenu )
@@ -54,15 +94,24 @@ MenuObject::~MenuObject ()
     delete m_listAnimationAttributes[t];
 
   m_listAnimationAttributes.clear ( );
+
+  resetStructureItem ( );
   if ( m_pShadow )
     delete m_pShadow;
   m_pShadow = NULL;
-  //  resetStructureItem ( );
+
+  if ( m_pOverlay )
+    delete m_pOverlay;
+  m_pOverlay = NULL;
 
   if ( m_pStructureItem )
     delete m_pStructureItem;
   m_pStructureItem = NULL;
 
+  if ( m_pData )
+    delete m_pData;
+  m_pData = NULL;
+
   m_pParent = NULL;
 }
 
@@ -78,7 +127,10 @@ bool MenuObject::readProjectFile  (QDomNode &theNode)
 	  if ( m_pShadow )
 	    delete m_pShadow;
 	  m_pShadow = NULL;
-	  m_qsAnimation = QString ( );
+	  if ( m_pOverlay )
+	    delete m_pOverlay;
+	  m_pOverlay = NULL;
+	  m_pData->m_qsAnimation = QString ( );
 	  *modifiers () = emptyModifiers;
 	}
 
@@ -91,7 +143,7 @@ bool MenuObject::readProjectFile  (QDomNode &theNode)
 		// Okay, this is ugly but after all it is the simplest of all xml file structure.
 		// No need to get fancy ...
 		if (tagName == MENU_OBJECT)	{
-			m_qsName = nodeText;
+			m_pData->m_qsName = nodeText;
 			QRect theRect;
 			int iX, iY, iWidth, iHeight;
 			iX      = rect().x();
@@ -101,7 +153,7 @@ bool MenuObject::readProjectFile  (QDomNode &theNode)
 
 			attribute = theElement.attributeNode ( MENU_OBJECT_NAME );
 			if (!attribute.isNull())
-				m_qsName = attribute.value();
+				m_pData->m_qsName = attribute.value();
 
 			attribute = theElement.attributeNode ( MENU_OBJECT_EXTRACTION_PATH );
 			if (!attribute.isNull())
@@ -168,17 +220,22 @@ bool MenuObject::readProjectFile  (QDomNode &theNode)
 						modifiers()->fShearY = attribute.value().toFloat();
 				}
 				//else if (tagName == ANIMATION_OBJECT)	{
-				//	m_qsAnimation = theElement.text ();
+				//	m_pData->m_qsAnimation = theElement.text ();
 				//}
 				subNode = subNode.nextSibling ();
 			}
 		}
 		if ( tagName == ANIMATION_OBJECT )	{
-		  m_qsAnimation = nodeText;
+		  m_pData->m_qsAnimation = nodeText;
 		}
-		else if (tagName == SHADOW_OBJECT)	{
+		else if ( tagName == SHADOW_OBJECT )  {
 		  m_pShadow = new ShadowObject ( this );
-		  if ( ! m_pShadow->readProjectFile ( xmlMenuObject ) ) 
+		  if ( ! m_pShadow->readProjectFile ( xmlMenuObject ) )
+		    return false;
+		}
+		else if ( tagName == OVERLAY_OBJECT )  {
+		  m_pOverlay = new OverlayObject ( this );
+		  if ( ! m_pOverlay->readProjectFile ( xmlMenuObject ) )
 		    return false;
 		}
 		// So lets get the next sibling ... until we hit hte end of DVDMenu ...
@@ -209,8 +266,8 @@ bool MenuObject::writeProjectFile (QDomElement &theElement)
 	objectNode.setAttribute( RECT_HEIGHT,  rect ( ).height ( ) );
 	// And now proceed to writing the Modifiers ... If we need that is ...
 	Modifiers initialValues;
-	if (	(modifiers()->fRotate != initialValues.fRotate) || (modifiers()->fShearX != initialValues.fShearX)  || 
-		(modifiers()->fShearY != initialValues.fShearX) || (modifiers()->fZoom != initialValues.fZoom)      || 
+	if (	(modifiers()->fRotate != initialValues.fRotate) || (modifiers()->fShearX != initialValues.fShearX)  ||
+		(modifiers()->fShearY != initialValues.fShearX) || (modifiers()->fZoom != initialValues.fZoom)      ||
 		(modifiers()->fScaleX != initialValues.fScaleX) || (modifiers()->fScaleY != initialValues.fScaleY)  ||
 		(modifiers()->iStartFrame   != initialValues.iStartFrame) || (modifiers()->iStopFrame != initialValues.iStopFrame)  ||
 		(modifiers()->fTransparency != initialValues.fTransparency) )	{
@@ -236,9 +293,9 @@ bool MenuObject::writeProjectFile (QDomElement &theElement)
 			modifiersNode.setAttribute( MODIFIERS_TRANSPARENCY, QString ("%1").arg(modifiers()->fTransparency) );
 		objectNode.appendChild( modifiersNode );
 	}
-	if ( ! m_qsAnimation.isEmpty())	{
+	if ( ! m_pData->m_qsAnimation.isEmpty())	{
 		QDomElement animationNode = xmlDoc.createElement( ANIMATION_OBJECT );	// <AnimationObject>
-		QDomText text = xmlDoc.createTextNode ( m_qsAnimation );
+		QDomText text = xmlDoc.createTextNode ( m_pData->m_qsAnimation );
 		animationNode.appendChild ( text );
 		objectNode.appendChild ( animationNode );
 	}
@@ -246,6 +303,10 @@ bool MenuObject::writeProjectFile (QDomElement &theElement)
 	  if ( ! m_pShadow->writeProjectFile ( theElement ) )
 	    return false;
 	}
+	if ( m_pOverlay )  {
+	  if ( ! m_pOverlay->writeProjectFile ( theElement ) )
+	    return false;
+	}
 	theElement.appendChild( objectNode );
 	return true;
 }
@@ -262,12 +323,14 @@ void MenuObject::update ( )
 void MenuObject::resetStructureItem ( )
 {
   if ( m_pShadow )
-       m_pShadow->resetStructureItem ( );
+       m_pShadow->resetStructureItem  ( );
+  if ( m_pOverlay )
+       m_pOverlay->resetStructureItem ( );
 
   if ( m_pStructureItem )
     delete m_pStructureItem;
   m_pStructureItem = NULL;
-  m_modifiers.m_pStructureItem = NULL;
+  modifiers ( )->m_pStructureItem = NULL;
 }
 
 QString &MenuObject::objectType()
@@ -277,7 +340,7 @@ QString &MenuObject::objectType()
 
 QString &MenuObject::name()
 {
-	return m_qsName;
+	return m_pData->m_qsName;
 }
 
 QString &MenuObject::extractionPath()
@@ -292,33 +355,35 @@ void MenuObject::animationDialog ( QString &qsTempAnim, int iType )
 
 QString &MenuObject::animation()
 {
-  return m_qsAnimation;
+  return m_pData->m_qsAnimation;
 }
 
-Modifiers *MenuObject::modifiers()
+Modifiers *MenuObject::modifiers ( )
 {
-  return &m_modifiers;
+  if ( ! m_pData )
+         m_pData = new Data;
+  return &m_pData->m_modifiers;
 }
 
-void MenuObject::setModifiers(Modifiers &theModifiers)
+void MenuObject::setModifiers ( Modifiers &theModifiers )
 {
-  *modifiers() = theModifiers;
+  *modifiers ( ) = theModifiers;
 }
 
-QPoint   &MenuObject::currentMousePos()
+QPoint &MenuObject::currentMousePos ( )
 {
   return m_currentMousePos;
 }
 
-// Need to take m_rectPreview into accoutn ...
-bool MenuObject::contains (QPoint &pos)
+// Need to take m_pData->m_rectPreview into accoutn ...
+bool MenuObject::contains ( QPoint &pos )
 {
   if ( ! boundingRect( ).contains ( pos ) )
     return false;
   return true;
 }
 
-bool MenuObject::mouseMoveEvent (QMouseEvent *pEvent)
+bool MenuObject::mouseMoveEvent ( QMouseEvent *pEvent )
 {
   // This will move, scale rotatae or share the Object ...
   QRect theRect = rect();
@@ -326,19 +391,19 @@ bool MenuObject::mouseMoveEvent (QMouseEvent *pEvent)
 
   int iPos = pEvent->pos().x() - m_currentMousePos.x();
   switch ( m_objectState ) {
-  case MenuObject::StateScaleTLXY: 
-  case MenuObject::StateScaleTRXY: 
-  case MenuObject::StateScaleBLXY: 
+  case MenuObject::StateScaleTLXY:
+  case MenuObject::StateScaleTRXY:
+  case MenuObject::StateScaleBLXY:
   case MenuObject::StateScaleBRXY: {
     double fZoom   = (double)( iPos / 100.0 );
     if (fZoom > 1.0)
       fZoom = fZoom * 2.0 - 1.0;
-    
-    if ( ( m_objectState == MenuObject::StateScaleTLXY ) || 
-	 ( m_objectState == MenuObject::StateScaleBLXY )  )
+
+    if ( ( m_objectState == MenuObject::StateScaleTLXY ) ||
+         ( m_objectState == MenuObject::StateScaleBLXY )  )
       fZoom *= -1;
-    modifiers( )->fScaleX *= (1.0 + fZoom);
-    modifiers( )->fScaleY *= (1.0 + fZoom);
+    modifiers( )->fScaleX *= ( 1.0 + fZoom );
+    modifiers( )->fScaleY *= ( 1.0 + fZoom );
     update   ( );
   }
   break;
@@ -351,7 +416,7 @@ bool MenuObject::mouseMoveEvent (QMouseEvent *pEvent)
     if ( m_objectState == MenuObject::StateScaleLeftX )
       fZoom *= -1;
 
-    modifiers( )->fScaleX *= (1.0 + fZoom);
+    modifiers( )->fScaleX *= ( 1.0 + fZoom );
     update   ( );
   }
   break;
@@ -359,11 +424,11 @@ bool MenuObject::mouseMoveEvent (QMouseEvent *pEvent)
   case MenuObject::StateScaleBottomY: {
     iPos = pEvent->pos().y() - m_currentMousePos.y();
     double fZoom   = (double)( iPos / 100.0 );
-    if (fZoom > 1.0)
-      fZoom = fZoom * 2.0 - 1.0;
-    
+    if ( fZoom > 1.0 )
+         fZoom = fZoom * 2.0 - 1.0;
+
     if ( m_objectState == MenuObject::StateScaleTopY )
-      fZoom *= -1;
+         fZoom *= -1;
     modifiers( )->fScaleY *= (1.0 + fZoom);
     update   ( );
   }
@@ -384,11 +449,11 @@ bool MenuObject::mouseMoveEvent (QMouseEvent *pEvent)
     theRect.setY  ( iY );
     theRect.setWidth  ( iWidth  );
     theRect.setHeight ( iHeight );
-    
-    setRect(theRect);
+
+    setRect ( theRect );
     break;
   }
-  m_currentMousePos = pEvent->pos();
+  m_currentMousePos = pEvent->pos ( );
 
   //  char arrayState[][30] = { "StateUnknown", "StateInactive", "StateMoveObject", "StateScaleTLXY", "StateScaleTRXY", "StateScaleBLXY", "StateScaleBRXY", "StateScaleLeftX", "StateScaleRightX", "StateScaleTopY", "StateScaleBottomY", "StateRotate" };
   //  printf ( "MenuObject::mouseMoveEvent objState<%s>\n", arrayState[m_objectState] );
@@ -452,7 +517,7 @@ void MenuObject::drawActiveFrame ( QPainter *pPainter )
   QBrush theBrush ( yellow );
   QRect theRect = m_boundingRect;
   int iHeight2, iWidth2, iRight, iBottom;
-  
+
   iWidth2  = theRect.x( ) + (int)((float) theRect.width  ( )  / 2.0 );
   iHeight2 = theRect.y( ) + (int)((float) theRect.height ( )  / 2.0 );
   iRight   = theRect.x( ) + theRect.width  ( ) - HANDLE_WIDTH;
@@ -494,7 +559,7 @@ void MenuObject::setActive ( bool bActive )
   m_bDrawRect = bActive;
 }
 
-MenuObject *MenuObject::shadow ( ) 
+MenuObject *MenuObject::shadow ( )
 {
   return m_pShadow;
 }
@@ -513,39 +578,71 @@ void MenuObject::updateShadow ( )
        m_pShadow->updateShadow ( );
 }
 
-QRect &MenuObject::rect()
+MenuObject *MenuObject::overlay ( )
 {
-  return m_rect;
+  return m_pOverlay;
+}
+
+void MenuObject::setOverlay ( MenuObject *pOverlay )
+{
+  if ( m_pOverlay )
+    delete m_pOverlay;
+
+  m_pOverlay = pOverlay;
+}
+
+void MenuObject::updateOverlay ( )
+{
+  if ( m_pOverlay )
+       m_pOverlay->updateOverlay ( );
+}
+
+QRect &MenuObject::rect ( )
+{
+  return m_pData->m_rect;
+}
+
+MenuObject::Data *MenuObject::data ( )
+{
+  return m_pData;
+}
+
+void MenuObject::setData ( MenuObject::Data *pData )
+{
+  if ( m_pData && pData )
+      *m_pData = *pData;
+  if ( m_pMatrixDialog  )
+       m_pMatrixDialog->initMe ( this );
 }
 
 QPoint MenuObject::position ( )
 {
   // returns the centered position ...
   QPoint thePosition;
-  thePosition.setX ( m_rect.x ( ) + (int) ( (float)m_rect.width  ( ) / 2.0 ) );
-  thePosition.setY ( m_rect.y ( ) + (int) ( (float)m_rect.height ( ) / 2.0 ) );
+  thePosition.setX ( m_pData->m_rect.x ( ) + (int) ( (float)m_pData->m_rect.width  ( ) / 2.0 ) );
+  thePosition.setY ( m_pData->m_rect.y ( ) + (int) ( (float)m_pData->m_rect.height ( ) / 2.0 ) );
   return thePosition;
 }
 
-QRect   &MenuObject::boundingRect()
+QRect &MenuObject::boundingRect()
 {
   return m_boundingRect;
 }
 
-void MenuObject::setRect(QRect &rect)
+void MenuObject::setRect ( QRect &rect )
 {
-  m_rect = rect;
+  m_pData->m_rect = rect;
   // The bounding rect is set by drawContents.
   m_boundingRect = rect;
 }
 
 void MenuObject::setPosition ( QPoint &position )
 {
-  int iWidth  = m_rect.width  ( );
-  int iHeight = m_rect.height ( );
+  int iWidth  = m_pData->m_rect.width  ( );
+  int iHeight = m_pData->m_rect.height ( );
   int iX =  position.x () - (int)( (float) iWidth  / 2.0 );
   int iY =  position.y () - (int)( (float) iHeight / 2.0 );
-  m_rect = QRect ( iX, iY, iWidth, iHeight );
+  m_pData->m_rect = QRect ( iX, iY, iWidth, iHeight );
 
   iWidth  = m_boundingRect.width  ( );
   iHeight = m_boundingRect.height ( );
@@ -554,44 +651,44 @@ void MenuObject::setPosition ( QPoint &position )
   m_boundingRect = QRect ( iX, iY, iWidth, iHeight );
 }
 
-void MenuObject::setCurrentMousePos(QPoint &pos)
+void MenuObject::setCurrentMousePos ( QPoint &pos )
 {
-	m_currentMousePos = pos;
+  m_currentMousePos = pos;
 }
 
-void MenuObject::setParent(QWidget *pParent)
+void MenuObject::setParent ( QWidget *pParent )
 {
-	m_pParent = pParent;
+  m_pParent = pParent;
 }
 
-QWidget *MenuObject::parent()
+QWidget *MenuObject::parent ( )
 {
-	return m_pParent;
+  return m_pParent;
 }
 
-bool MenuObject::drawRect()
+bool MenuObject::drawRect ( )
 {
-	return m_bDrawRect;
+  return m_bDrawRect;
 }
 
 MenuObject *MenuObject::clone( QWidget *, MenuObject * )
-{  
+{
   return NULL;
 }
 
-void MenuObject::setName (QString &qsName)
+void MenuObject::setName ( QString &qsName )
 {
-	m_qsName = qsName;
+  m_pData->m_qsName = qsName;
 }
 
 void MenuObject::setExtractionPath (QString &qsPath)
 {
-	m_qsExtractionPath = qsPath;
+  m_qsExtractionPath = qsPath;
 }
 
 void MenuObject::setAnimation (QString &qsAnimation)
 {
-  m_qsAnimation = qsAnimation;
+  m_pData->m_qsAnimation = qsAnimation;
 }
 
 void MenuObject::slotDelete ()
@@ -605,7 +702,7 @@ void MenuObject::slotEmitDelete ( )
   emit ( signalDeleteMe ( this ) );
 }
 
-void MenuObject::slotAddShadow ( ) 
+void MenuObject::slotAddShadow ( )
 {
   // Send request back to be handled in MenuPreview ..
   emit ( signalShadowMe ( this ) );
@@ -613,13 +710,33 @@ void MenuObject::slotAddShadow ( )
 
 void MenuObject::slotDeleteShadow ( )
 {
+  UndoBuffer *pUndo = Global::pApp->getUndoBuffer ( );
+  if ( pUndo )
+       pUndo->push ( new DVDMenuUndoObject ( DVDMenuUndoObject::DELETED_SHADOW, this, shadow ( ) ) );
+
   setShadow ( NULL );
   emit ( signalUpdatePixmap ( ) );
 }
 
+void MenuObject::slotAddOverlay ( )
+{
+  // Send request back to be handled in MenuPreview ..
+  emit ( signalOverlayMe ( this ) );
+}
+
+void MenuObject::slotDeleteOverlay ( )
+{
+  UndoBuffer *pUndo = Global::pApp->getUndoBuffer ( );
+  if ( pUndo )
+       pUndo->push ( new DVDMenuUndoObject ( DVDMenuUndoObject::DELETED_OVERLAY, this, overlay ( ) ) );
+
+  setOverlay ( NULL );
+  emit ( signalUpdatePixmap ( ) );
+}
+
 void MenuObject::slotCut ( )
 {
-  slotCopy ( );
+  slotCopy       ( );
   slotEmitDelete ( );
 }
 
@@ -629,12 +746,12 @@ void MenuObject::slotCopy ( )
     delete Global::pCopyObject;
   Global::pCopyObject = clone ( );
 
-  if (m_pContextMenu)
+  if ( m_pContextMenu )
     delete m_pContextMenu;
   m_pContextMenu = NULL;
 }
 
-void MenuObject::slotToFront ()
+void MenuObject::slotToFront ( )
 {
 	// Send it all the way to the front
 	emit (signalMoveOnStack (this, 1000));
@@ -660,7 +777,7 @@ void MenuObject::loadAnimationData (QString &qsFileName, long iMaxNumberOfFrames
   // If no value for a prop at a frame is defined, 
   // then the last value is the winner.
   ///////////////////////////////////////////////
-  
+
   // Note, these values will change the true props 
   // of the object.
   // Once rendering is done the original values are re-set.
@@ -678,23 +795,24 @@ void MenuObject::loadAnimationData (QString &qsFileName, long iMaxNumberOfFrames
     delete m_listAnimationAttributes[t];
   }
   m_listAnimationAttributes.clear();
-  AnimationAttribute *pAnimAttr;
-  
+  AnimationAttribute *pAnimAttr = NULL;
+
   while ( !stream.atEnd() ) {
     line = stream.readLine(); // line of text excluding '\n'
     iPos = line.find ("::");
     iPos2 = line.find ("=");
     if ( (iPos == -1) || (iPos2 == -1) )
       continue;
-    qsProperty  = QString ("");
-    qsValue     = QString ("");
-    bFound      = false;
-    iFrameNumber = line.left (iPos).toInt();
+    qsProperty   = QString ("");
+    qsValue      = QString ("");
+    bFound       = false;
+    iFrameNumber = line.left ( iPos ).toInt ( &bFound );
     // Note: Frame position 0 is used to store the original values.
-    if ( (iFrameNumber < 1) || (iFrameNumber > iMaxNumberOfFrames) )
+    if ( !bFound || ( iFrameNumber < 0 ) || (iFrameNumber > iMaxNumberOfFrames) )
       continue;
+    bFound = false;
     qsProperty = line.mid (iPos+2, iPos2-iPos-2);
-    
+
     qsValue = line.right (line.length() - iPos2-1);
     if (qsValue == "SolidLine")
       qsValue = QString ("%1").arg(Qt::SolidLine);
@@ -711,7 +829,7 @@ void MenuObject::loadAnimationData (QString &qsFileName, long iMaxNumberOfFrames
     else if (qsValue == "RoundJoin")
       qsValue = QString ("%1").arg(Qt::RoundJoin);
     //printf ("MenuObject::loadAnimationData - load <%d> <%s>=<%s>\n", iFrameNumber, (const char *)qsProperty, (const char *)qsValue);
-    
+
     // This loop will check if this property is already taken care of and we have a array of values waiting.
     for (t=0;t<m_listAnimationAttributes.count ();t++)	{
       pAnimAttr = m_listAnimationAttributes[t];
@@ -738,59 +856,59 @@ void MenuObject::loadAnimationData (QString &qsFileName, long iMaxNumberOfFrames
 	  pAnimAttr->setNeedRefresh ( false );
 	}
 	else if (qsProperty == QString("rect.x1"))	{
-	  FunctionCallback<QRect> *callMeBack = new FunctionCallback<QRect>(m_rect, &QRect::setX);
-	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_rect.x(), callMeBack);
+	  FunctionCallback<QRect> *callMeBack = new FunctionCallback<QRect>(m_pData->m_rect, &QRect::setX);
+	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_pData->m_rect.x(), callMeBack);
 	  pAnimAttr->setNeedRefresh ( false );
 	}
 	else if (qsProperty == QString("rect.x2"))	{
-	  FunctionCallback<QRect> *callMeBack = new FunctionCallback<QRect>(m_rect, &QRect::setRight);
-	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_rect.right(), callMeBack);
+	  FunctionCallback<QRect> *callMeBack = new FunctionCallback<QRect>(m_pData->m_rect, &QRect::setRight);
+	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_pData->m_rect.right(), callMeBack);
 	  pAnimAttr->setNeedRefresh ( false );
 	}
 	else if (qsProperty == "rect.y1")	{
-	  FunctionCallback<QRect> *callMeBack = new FunctionCallback<QRect>(m_rect, &QRect::setY);
-	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_rect.y(), callMeBack);
+	  FunctionCallback<QRect> *callMeBack = new FunctionCallback<QRect>(m_pData->m_rect, &QRect::setY);
+	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_pData->m_rect.y(), callMeBack);
 	  pAnimAttr->setNeedRefresh ( false );
 	}
 	else if (qsProperty == "rect.y2")	{
-	  FunctionCallback<QRect> *callMeBack = new FunctionCallback<QRect>(m_rect, &QRect::setBottom);
-	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_rect.bottom(), callMeBack);
+	  FunctionCallback<QRect> *callMeBack = new FunctionCallback<QRect>(m_pData->m_rect, &QRect::setBottom);
+	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_pData->m_rect.bottom(), callMeBack);
 	  pAnimAttr->setNeedRefresh ( false );
 	}
 	else if (qsProperty == "rect.width")	{
-	  FunctionCallback<QRect> *callMeBack = new FunctionCallback<QRect>(m_rect, &QRect::setWidth);
-	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_rect.width(), callMeBack);
+	  FunctionCallback<QRect> *callMeBack = new FunctionCallback<QRect>(m_pData->m_rect, &QRect::setWidth);
+	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_pData->m_rect.width(), callMeBack);
 	  pAnimAttr->setNeedRefresh ( false );
 	}
 	else if (qsProperty == "rect.height")	{
-	  FunctionCallback<QRect> *callMeBack = new FunctionCallback<QRect>(m_rect, &QRect::setHeight);
-	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_rect.height(), callMeBack);
+	  FunctionCallback<QRect> *callMeBack = new FunctionCallback<QRect>(m_pData->m_rect, &QRect::setHeight);
+	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_pData->m_rect.height(), callMeBack);
 	  pAnimAttr->setNeedRefresh ( false );
 	}
 	else if (qsProperty == "visibleFromTo.startFrame") {
-	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_modifiers.iStartFrame, &m_modifiers.iStartFrame);
+	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, modifiers ( )->iStartFrame, &modifiers ( )->iStartFrame);
 	  pAnimAttr->setNeedRefresh ( false );
 	}
 	else if (qsProperty == "visibleFromTo.stopFrame") {
-	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_modifiers.iStopFrame, &m_modifiers.iStopFrame);
+	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, modifiers ( )->iStopFrame, &modifiers ( )->iStopFrame);
 	  pAnimAttr->setNeedRefresh ( false );
 	}
 	else if (qsProperty == "transparency")
-	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_modifiers.fTransparency, &m_modifiers.fTransparency);
+	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, modifiers ( )->fTransparency, &modifiers ( )->fTransparency);
 	else if (qsProperty == "zoom")
-	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_modifiers.fZoom, &m_modifiers.fZoom);
+	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, modifiers ( )->fZoom, &modifiers ( )->fZoom);
 	else if (qsProperty == "rotate")
-	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_modifiers.fRotate, &m_modifiers.fRotate);
+	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, modifiers ( )->fRotate, &modifiers ( )->fRotate);
 	else if (qsProperty == "shear.x")
-	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_modifiers.fShearX, &m_modifiers.fShearX);
+	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, modifiers ( )->fShearX, &modifiers ( )->fShearX);
 	else if (qsProperty == "shear.y")
-	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_modifiers.fShearY, &m_modifiers.fShearY);
+	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, modifiers ( )->fShearY, &modifiers ( )->fShearY);
 	else if (qsProperty == "scale.x") {
-	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_modifiers.fScaleX, &m_modifiers.fScaleX);
+	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, modifiers ( )->fScaleX, &modifiers ( )->fScaleX);
 	  pAnimAttr->setNeedRefresh ( false );
 	}
 	else if (qsProperty == "scale.y") {
-	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_modifiers.fScaleY, &m_modifiers.fScaleY);
+	  pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, modifiers ( )->fScaleY, &modifiers ( )->fScaleY);
 	  pAnimAttr->setNeedRefresh ( false );
 	}
       }
@@ -803,4 +921,10 @@ void MenuObject::loadAnimationData (QString &qsFileName, long iMaxNumberOfFrames
   file.close();
 }
 
+void MenuObject::slotDeleteMtxDlg ( )
+{
+  if ( m_pMatrixDialog )
+       delete m_pMatrixDialog;
+  m_pMatrixDialog = NULL;
+}
 
