@@ -36,6 +36,7 @@ class QDomNode;
 class QPopupMenu;
 class QDomElement;
 class StructureItem;
+class DialogMatrix;
 
 #define HANDLE_WIDTH  9
 #define HANDLE_HEIGHT 9
@@ -43,6 +44,24 @@ class StructureItem;
 class MenuObject : public QObject
 {
 	Q_OBJECT
+	friend class DVDMenuUndoObject;
+protected:
+	class Data
+	{
+	public:
+	  enum { DATA=0, IMAGE, TEXT, FRAME, MOVIE, BUTTON };
+	           Data ( );
+	  virtual ~Data ( );
+	  virtual  Data &operator  = ( Data & );
+	  virtual  bool  operator == ( Data & );
+	  virtual  Data *clone ( );
+
+	  int            m_iDataType; // is this for MenuObjects or ImageObjects or ...
+	  QString        m_qsName;
+	  QRect          m_rect;
+	  Modifiers      m_modifiers;
+	  QString        m_qsAnimation; // script or dataset holds also the datafilename
+	};
 public:
 	enum enState {
 	  StateUnknown = 0,
@@ -81,6 +100,10 @@ public:
 	virtual void replaceColor         ( Rgba, Rgba ) { };
 	virtual void updateShadow         ( );
 	virtual void setShadow            ( MenuObject * );
+	virtual void updateOverlay        ( );
+	virtual void setOverlay           ( MenuObject * );
+	virtual Data *data                ( );
+	virtual void  setData             ( Data * );
 
 	virtual void setModifiers   ( Modifiers &     );
 	virtual void setRect        ( QRect &         );
@@ -98,9 +121,10 @@ public:
 	void      loadAnimationData ( QString &, long );
 	enState   setObjectState    ( QPoint  &, Qt::ButtonState );
 	void      setObjectState    ( enState         );
-	
+
 	virtual Modifiers *modifiers( );
 	virtual MenuObject *shadow  ( );
+	virtual MenuObject *overlay ( );
 	virtual void resetStructureItem ( );
 	virtual QString &extractionPath ( );  // Used to store a unique path name for this object to be extracted into (movieObject)
 	QString   &objectType       ( );
@@ -123,12 +147,17 @@ public slots:
 	virtual void slotCopy         ( );
 	virtual void slotAddShadow    ( );
 	virtual void slotDeleteShadow ( );
+	virtual void slotAddOverlay   ( );
+	virtual void slotDeleteOverlay( );
+	virtual void slotDeleteMtxDlg ( );
 
 signals:
 	void signalMoveOnStack      ( MenuObject *, int );
 	void signalDefineAsButton   ( MenuObject * );
-	void signalDeleteMe         ( MenuObject * );
+	void signalDeleteMe         ( MenuObject * );  // used to delete the object
+	void signalUndoMe           ( MenuObject * );  // used for object to be put onto the UndoStack
 	void signalShadowMe         ( MenuObject * );
+	void signalOverlayMe        ( MenuObject * );
 	void signalAnimateMe        ( QString &, int, QString );
 	void signalUpdateStructure  ( );
 	void signalUpdatePixmap     ( );
@@ -136,21 +165,20 @@ signals:
 	void signalMaskColorChanged ( Rgba, Rgba );
 
 protected:
+	Data          *m_pData;
 	QPopupMenu    *m_pContextMenu;
+	DialogMatrix  *m_pMatrixDialog;
 	QString        m_qsObjectType;
-	QString        m_qsName;
 	QString        m_qsExtractionPath;
-	QRect	       m_rect;
 	QRect          m_boundingRect;
 	QWidget       *m_pParent;
 	bool           m_bDrawRect; // isActive, draw ActiveFrame (resize, rotate etc)
 	MenuObject    *m_pShadow;
+	MenuObject    *m_pOverlay;
 	StructureItem *m_pStructureItem;
-	Modifiers      m_modifiers;
-	QString        m_qsAnimation;	// script or dataset holds also the datafilename
 	QPoint         m_currentMousePos;
 	enState        m_objectState;
 	QValueList<AnimationAttribute *> m_listAnimationAttributes;
 };
 
-#endif	// MENUOBJECT_H
+#endif  // MENUOBJECT_H
