@@ -82,9 +82,11 @@ public slots:
 	virtual void slotAddFromLibrary        ( );
 	virtual void slotUpdateStructure       ( );
 	virtual void slotUpdatePixmap          ( );
+	virtual void slotAddOverlay            ( MenuObject   * );
 	virtual void slotAddShadow             ( MenuObject   * );
 	virtual void slotDefineAsButton        ( MenuObject   * );
 	virtual void slotDeleteObject          ( MenuObject   * );
+	virtual void slotUndoObject            ( MenuObject   * );
 	virtual void slotModifyObject          ( MenuObject   * );
 	virtual void slotUnbutton              ( ButtonObject * );
 	virtual void slotCreateButtonDialog    ( ButtonObject * );
@@ -97,6 +99,8 @@ public slots:
 	virtual void slotAlignSelection        ( );
 	virtual void slotConvertToPAL          ( );
 	virtual void slotConvertToNTSC         ( );
+	virtual void slotCut                   ( );
+	virtual void slotCopy                  ( );
 	virtual void slotPaste                 ( );
 
 signals:
@@ -127,12 +131,12 @@ protected:
 
 private:	// Private functions ...
 	MenuObject *insertDraggedObject     ( QDropEvent *, QImage  & );
-	MenuObject *insertDraggedImage      ( QDropEvent *, QImage  & );
-	MenuObject *insertDraggedMovie      ( QDropEvent *, QImage  &, long i=0 );
+	MenuObject *insertDraggedImage      ( QDropEvent *, QImage  &, bool b=true );
+	MenuObject *insertDraggedMovie      ( QDropEvent *, QImage  &, long i=0, bool b=true );
 	MenuObject *insertDraggedThumbnail  ( QDropEvent *, QImage  & );
 	MenuObject *insertDraggedChapter    ( QDropEvent *, QIconViewItem * );
 	void        insertDraggedChapters   ( QDropEvent *, QValueList<QIconViewItem *> & );
-	MenuObject *insertDraggedText       ( QDropEvent *, QString & );
+	MenuObject *insertDraggedText       ( QDropEvent *, QString &, bool b=true );
 	MenuObject *insertDraggedSourceInfo ( QDropEvent *, QImage  & );// ButtonObject
 	MenuObject *insertDraggedSourceEntry( QDropEvent *, QImage  & );// FullScale Layouts and new menus and all the bells and whistles (In future release anyhow ...SORRY)
 	QString     getMovieFileName        ( MovieObject*, QStringList & );
