@@ -33,17 +33,77 @@
 #include "messagebox.h"
 #include "qplayer/mediacreator.h"
 #include "qrender/startmenurender.h"
+#include "dvdmenuundoobject.h"
+
+
+MovieObject::MovieData::MovieData ( )
+  : ImageObject::ImageData ( )
+{
+  m_iDataType    = Data::MOVIE;
+  m_bMenuLength  = true; // default to the same length as the menu
+  m_enLoop       = Loop;
+  m_iMSecPreview = 0;
+}
+
+MovieObject::MovieData::~MovieData ( )
+{
+}
+
+MenuObject::Data &MovieObject::MovieData::operator = ( Data &theOther )
+{
+  // At this point we are sure both are acutally ImageManipulator objects.
+  ImageObject::ImageData::operator= ( theOther );
+  if ( m_iDataType   == theOther.m_iDataType )  {
+    MovieData *pOther = (MovieData *)&theOther;
+    m_iMSecPreview    = pOther->m_iMSecPreview;
+    m_timeOffset      = pOther->m_timeOffset;
+    m_timeDuration    = pOther->m_timeDuration;
+    m_bMenuLength     = pOther->m_bMenuLength;
+    m_enLoop          = pOther->m_enLoop;
+    m_bStartAfterDropEvent = pOther->m_bStartAfterDropEvent;
+  }
+
+  return *this;
+}
+
+bool MovieObject::MovieData::operator == ( Data &theOther )
+{
+  // At this point we are sure both are acutally ImageManipulator objects.
+  bool bReturn = ( ImageObject::ImageData::operator == ( theOther ) );
+  if ( ! bReturn )
+    return false;
+
+  if ( m_iDataType  == theOther.m_iDataType )  {
+    MovieData *pOther = (MovieData *)&theOther;
+    bReturn = ( ( m_iMSecPreview == pOther->m_iMSecPreview ) &&
+                ( m_timeOffset   == pOther->m_timeOffset   ) &&
+                ( m_timeDuration == pOther->m_timeDuration ) &&
+                ( m_bMenuLength  == pOther->m_bMenuLength  ) &&
+                ( m_enLoop       == pOther->m_enLoop       ) &&
+                ( m_bStartAfterDropEvent == pOther->m_bStartAfterDropEvent ) );
+  }
+  return bReturn;
+}
+
+MenuObject::Data *MovieObject::MovieData::clone ( )
+{
+  MovieData *pData = new MovieData;
+  *pData  = *this;
+  return pData;
+}
 
 MovieObject::MovieObject ( QWidget *pParent )
   : ImageObject ( pParent )
 {
-  m_qsObjectType  = QString (MOVIE_OBJECT);
-  m_pConvertTimer = NULL;
-  m_pDVDMenu      = NULL;
-  m_bExtracted    = false;
-  m_bMenuLength   = true; // default to the same length as the menu
-  m_enLoop        = Loop;
-  m_iMSecPreview  = 0;
+  if ( m_pData )
+    delete m_pData;        // part of the base class
+  m_pData = new MovieData; // will be deleted in base class
+
+  m_qsObjectType   = QString (MOVIE_OBJECT);
+  m_pConvertTimer  = NULL;
+  m_pDVDMenu       = NULL;
+  m_bExtracted     = false;
+  m_iHowNiceShouldIBe = 0;
 }
 
 MovieObject::~MovieObject ( )
@@ -58,18 +118,23 @@ MovieObject::~MovieObject ( )
   m_pDVDMenu = NULL;
 }
 
+MovieObject::MovieData *MovieObject::movieData ( )
+{
+  return (MovieData *)m_pData;
+}
+
 void MovieObject::setOffset ( QTime &timeOffset )
 {
-  m_timeOffset = timeOffset;
+  movieData ( )->m_timeOffset = timeOffset;
   createExtractionPath ( );
 }
 
 void MovieObject::setDuration ( QTime &timeDuration )
 {
-  m_timeDuration = timeDuration;
+  movieData ( )->m_timeDuration = timeDuration;
   createExtractionPath ( );
   if ( dvdMenu ( ) )
-    m_bMenuLength = ( timeDuration == dvdMenu ( )->getInterface ( )->timeDuration );
+    movieData ( )->m_bMenuLength = ( timeDuration == dvdMenu ( )->getInterface ( )->timeDuration );
 }
 
 void MovieObject::setMenuLength ( bool bMenuLength )
@@ -77,12 +142,12 @@ void MovieObject::setMenuLength ( bool bMenuLength )
   if ( bMenuLength && dvdMenu ( ) )
        setDuration  ( dvdMenu ( )->getInterface ( )->timeDuration );
 
-  m_bMenuLength = bMenuLength;
+  movieData ( )->m_bMenuLength = bMenuLength;
 }
 
 void MovieObject::setLoop ( MovieObject::enType enLoop )
 {
-  m_enLoop = enLoop;
+  movieData ( )->m_enLoop = enLoop;
 }
 
 void MovieObject::createExtractionPath ( )
@@ -93,7 +158,7 @@ void MovieObject::createExtractionPath ( )
   char      cBuffer[15];
 
   qsExtractionPath = fileInfo.baseName ( true );
-  qsOffset         = m_timeOffset.toString ( "hhmmsszzz"  );
+  qsOffset         = movieData ( )->m_timeOffset.toString ( "hhmmsszzz"  );
   int iDuration    = zeroTime.msecsTo      ( duration ( ) );
   sprintf ( cBuffer, "-%07d", iDuration );
   qsExtractionPath += "-"+qsOffset + cBuffer;
@@ -121,34 +186,34 @@ void MovieObject::setFile ( QString fileName )
 
 void MovieObject::setMSecPreview ( long iMSecPreview )
 {
-	m_iMSecPreview = iMSecPreview;
+	movieData ( )->m_iMSecPreview = iMSecPreview;
 }
 
 QTime &MovieObject::offset    ()
 {
-	return m_timeOffset;
+	return movieData ( )->m_timeOffset;
 }
 
 QTime &MovieObject::duration  ( )
 {
-  if ( m_bMenuLength && dvdMenu ( ) )
+  if ( movieData ( )->m_bMenuLength && dvdMenu ( ) )
     return dvdMenu ( )->getInterface ( )->timeDuration;
-  return m_timeDuration;
+  return movieData ( )->m_timeDuration;
 }
 
 bool MovieObject::isMenuLength ( )
 {
-  return m_bMenuLength;
+  return movieData ( )->m_bMenuLength;
 }
 
 MovieObject::enType MovieObject::loop ( )
 {
-  return m_enLoop;
+  return movieData ( )->m_enLoop;
 }
 
 long MovieObject::msecPreview  ()
 {
-	return m_iMSecPreview;
+	return movieData ( )->m_iMSecPreview;
 }
 
 void MovieObject::setMovieSize (QSize &sizeMovie)
@@ -213,13 +278,13 @@ void MovieObject::setPixmap ( QPixmap &thePixmap )
     m_imagePreview = m_imagePreview.smoothScale (rect().size(), QImage::ScaleFree);
 
   updatePixmap (); 
-  MediaCreator::registerWithMediaScanner (this, qsFileName, &m_imagePreview, m_iMSecPreview );
+  MediaCreator::registerWithMediaScanner (this, qsFileName, &m_imagePreview, movieData ( )->m_iMSecPreview );
 }
 
 void MovieObject::slotSetThumbnail ( long iMSecPreview )
 {
-  m_iMSecPreview = iMSecPreview;
-  MediaCreator::registerWithMediaScanner (this, m_qsFileName, &m_imagePreview, m_iMSecPreview );
+  movieData ( )->m_iMSecPreview = iMSecPreview;
+  MediaCreator::registerWithMediaScanner (this, m_qsFileName, &m_imagePreview, movieData ( )->m_iMSecPreview );
 }
 
 void MovieObject::timerEvent(QTimerEvent *pTimerEvent)
@@ -228,7 +293,7 @@ void MovieObject::timerEvent(QTimerEvent *pTimerEvent)
 	QSize theSize;
 	switch ( pTimerEvent->timerId ( ) )	{
 	case MEDIASCANNER_EVENT + 2: // ExecuteJob::TYPE_IMAGE
-		float fZoom = ((float)m_rect.width ( ) / m_imagePreview.width ( ) );
+		float fZoom = ((float)rect ( ).width ( ) / m_imagePreview.width ( ) );
 		theSize = m_imagePreview.size ( );
 		setZoom      ( fZoom );
 		setMovieSize ( theSize );
@@ -326,50 +391,59 @@ void MovieObject::slotEdit ()
 
 void MovieObject::slotProperties ()
 {
-	// Okay the user does want to make some changes ... so let us stop the timer ...
-	if ( m_pConvertTimer )
-		m_pConvertTimer->stop ( );
+  // Okay the user does want to make some changes ... so let us stop the timer ...
+  if ( m_pConvertTimer )
+       m_pConvertTimer->stop ( );
 
-	DialogMovie movieDialog (NULL);
-	connect ( &movieDialog, SIGNAL ( signalSetThumbnail ( long ) ), this, SLOT ( slotSetThumbnail ( long ) ) );
-	movieDialog.initMe ( this ); 
-	if ( movieDialog.exec ( ) == QDialog::Accepted )  {
-		int iX, iY, iWidth, iHeight;
-		iX      = movieDialog.m_pEditX->text ( ).toInt ( );
-		iY      = movieDialog.m_pEditY->text ( ).toInt ( );
-		iWidth  = movieDialog.m_pEditWidth ->text ( ).toInt ( );
-		iHeight = movieDialog.m_pEditHeight->text ( ).toInt ( );
-		QRect rect ( iX, iY, iWidth, iHeight );
-		setRect    ( rect );
-		// Next we hande the start/stop time.
-		QString qsFormat ("hh:mm:ss.zzz");
-		QTime startingTime, endingTime, duration;
-		startingTime = QTime::fromString ( movieDialog.m_pEditStarting2->text ( ) );
-		endingTime   = QTime::fromString ( movieDialog.m_pEditEnding2  ->text ( ) );
-		if ( startingTime.isValid ( ) )  {
-			setOffset ( startingTime );
-			if (endingTime.isValid ( ) )  {
-				int iMSecs = startingTime.msecsTo ( endingTime );
-				duration   = duration.addMSecs    (     iMSecs );
-				setDuration ( duration );
-			}
-		}
-		setMenuLength ( movieDialog.m_pCheckSameAsMenuLength->isChecked  ( ) );
-		setLoop ( Loop );
-		if ( movieDialog.m_pButtonLoop->text ( ) == "Stretch" )
-			setLoop ( Stretch );
-		if ( movieDialog.m_pButtonLoop->text ( ) == "Stop" )
-			setLoop ( Stop );
+  DialogMovie movieDialog (NULL);
+  connect ( &movieDialog, SIGNAL ( signalSetThumbnail ( long ) ), this, SLOT ( slotSetThumbnail ( long ) ) );
+  movieDialog.initMe ( this ); 
+  if ( movieDialog.exec ( ) == QDialog::Accepted )  {
 
-		emit (signalUpdatePixmap());
-		updatePixmap ();
-	}
-	disconnect ( &movieDialog, SIGNAL ( signalSetThumbnail ( long ) ), this, SLOT ( slotSetThumbnail ( long ) ) );
+    DVDMenuUndoObject *pUndo = new DVDMenuUndoObject ( this, MenuObject::StateUnknown );
+    int iX, iY, iWidth, iHeight;
+    iX      = movieDialog.m_pEditX->text ( ).toInt ( );
+    iY      = movieDialog.m_pEditY->text ( ).toInt ( );
+    iWidth  = movieDialog.m_pEditWidth ->text ( ).toInt ( );
+    iHeight = movieDialog.m_pEditHeight->text ( ).toInt ( );
+    QRect rect ( iX, iY, iWidth, iHeight );
+    setRect    ( rect );
+    // Next we hande the start/stop time.
+    QString qsFormat ("hh:mm:ss.zzz");
+    QTime startingTime, endingTime, duration;
+    startingTime = QTime::fromString ( movieDialog.m_pEditStarting2->text ( ) );
+    endingTime   = QTime::fromString ( movieDialog.m_pEditEnding2  ->text ( ) );
+    if ( startingTime.isValid ( ) )  {
+      setOffset ( startingTime );
+      if (endingTime.isValid ( ) )  {
+        int iMSecs = startingTime.msecsTo ( endingTime );
+        duration   = duration.addMSecs    (     iMSecs );
+        setDuration ( duration );
+      }
+    }
+    setMenuLength ( movieDialog.m_pCheckSameAsMenuLength->isChecked  ( ) );
+    setLoop ( Loop );
+    if ( movieDialog.m_pButtonLoop->text ( ) == "Stretch" )
+         setLoop ( Stretch );
+    if ( movieDialog.m_pButtonLoop->text ( ) == "Stop" )
+         setLoop ( Stop );
 
-	// And finally we re-start the timer (or start a new time to extract the propper images.
-	// Note: this can be done bacause I will check a status file first before extracting the frames, 
-	//       thus avoiding unneccesary extracting of the images in the movie.
-	registerToRender ( );
+    emit ( signalUpdatePixmap ( ) );
+    updatePixmap ( );
+    if ( pUndo->hasChanged ( ) )  {
+      UndoBuffer *pBuffer = Global::pApp->getUndoBuffer ( );
+      if ( pBuffer )
+           pBuffer->push ( pUndo );
+    }
+    else
+      delete pUndo;
+  }
+  disconnect ( &movieDialog, SIGNAL ( signalSetThumbnail ( long ) ), this, SLOT ( slotSetThumbnail ( long ) ) );
+
+  // And finally we re-start the timer (or start a new time to extract the propper images.
+  // Note: this can be done bacause I will check a status file first before extracting the frames, 
+  //       thus avoiding unneccesary extracting of the images in the movie.
+  registerToRender ( );
 }
 
 void MovieObject::registerToRender ()
@@ -448,7 +522,7 @@ void MovieObject::setDragNDropValues (DragNDropContainer *pDNDContainer)
 
 void MovieObject::setStartAfterDropEvent(bool bStartAfterDropEvent)
 {
-	m_bStartAfterDropEvent = bStartAfterDropEvent;
+	movieData ( )->m_bStartAfterDropEvent = bStartAfterDropEvent;
 }
 
 void MovieObject::setNiceness  (int iVeryNice)
@@ -530,17 +604,17 @@ bool MovieObject::writeProjectFile (QDomElement &theElement)
 	QDomElement movieNode = xmlDoc.createElement( MOVIE_OBJECT );	// <MovieObject>
 	// Here we set the attributes of the <dvdauthor> tag
 	QString qsType = "Loop";
-	if ( m_enLoop == Stretch )
+	if ( movieData ( )->m_enLoop == Stretch )
 	  qsType = "Stretch";
-	if ( m_enLoop == Stop )
+	if ( movieData ( )->m_enLoop == Stop )
 	  qsType = "Stop";
 	if (!m_qsFileName.isNull())
 		movieNode.setAttribute( MOVIE_OBJECT_FILE_NAME, m_qsFileName );
-	movieNode.setAttribute( MOVIE_OBJECT_OFFSET,   m_timeOffset.toString   (qsFormat) );
-	movieNode.setAttribute( MOVIE_OBJECT_DURATION, m_timeDuration.toString (qsFormat) );
-	movieNode.setAttribute( MOVIE_OBJECT_MENU_LENGTH,  m_bMenuLength ? "true" : "false" );
+	movieNode.setAttribute( MOVIE_OBJECT_OFFSET,   movieData ( )->m_timeOffset.toString   (qsFormat) );
+	movieNode.setAttribute( MOVIE_OBJECT_DURATION, movieData ( )->m_timeDuration.toString (qsFormat) );
+	movieNode.setAttribute( MOVIE_OBJECT_MENU_LENGTH,  movieData ( )->m_bMenuLength ? "true" : "false" );
 	movieNode.setAttribute( MOVIE_OBJECT_LOOP,         qsType );
-	movieNode.setAttribute( MOVIE_OBJECT_MSEC_PREVIEW, QString ( "%1" ).arg ( m_iMSecPreview ) );
+	movieNode.setAttribute( MOVIE_OBJECT_MSEC_PREVIEW, QString ( "%1" ).arg ( movieData ( )->m_iMSecPreview ) );
 	movieNode.setAttribute( MOVIE_OBJECT_SIZE, QString("%1x%2").arg(movieSize().width()).arg(movieSize().height()) );
 
 	// Store the ImageManipulator - variables (if neccesarry) ...
@@ -561,26 +635,39 @@ StructureItem *MovieObject::createStructure ( StructureItem *pParentItem )
   StructureItem *pTemp=NULL;
   QFileInfo fileInfo ( m_qsFileName );
   QString qsName;
-	
+
+  if ( ! pParentItem )  {
+    if ( m_pOverlay  )
+      pTemp = m_pOverlay->createStructure ( NULL );
+    if ( m_pShadow )
+      pTemp = m_pShadow->createStructure ( NULL );
+    manipulator ( ).createStructure ( NULL );
+    m_pStructureItem = NULL;
+    return NULL;
+  }
+
   qsName = QString ( "Movie Object" );
 
   if ( ! m_pStructureItem ) {
-    m_pStructureItem = new StructureItem ( this, pParentItem, pParentItem->lastChild ( ), qsName, fileInfo.fileName ( ) );
+    m_pStructureItem = new StructureItem ( this, StructureItem::Video, pParentItem, pParentItem->lastChild ( ), qsName, fileInfo.fileName ( ) );
     m_pStructureItem->setExpandable      ( TRUE );
   }
 
   ImageObject initialValues;
   // And finally we create the modifiers info ...
   pTemp = m_pStructureItem->setSibling ( this, pTemp, tr ("Geometry"),
-		      QString ( "%1, %2, %3, %4" ).arg ( rect ( ).x ( ) ).arg ( rect ( ).y ( ) ).
-		      arg ( rect ( ).width ( ) ).arg( rect ( ).height ( ) ) );
-  
+                                         QString ( "%1, %2, %3, %4" ).arg ( rect ( ).x ( ) ).arg ( rect ( ).y ( ) ).
+                                         arg ( rect ( ).width ( ) ).arg( rect ( ).height ( ) ) );
+
   pTemp = m_pStructureItem->setSibling ( this, pTemp, tr ("Duration"), duration ( ).toString ( ) );
-  pTemp = m_pStructureItem->setSibling ( this, pTemp, tr ("Offset"), m_timeOffset.toString   ( ) );
+  pTemp = m_pStructureItem->setSibling ( this, pTemp, tr ("Offset"), movieData ( )->m_timeOffset.toString   ( ) );
+
+  if ( m_pOverlay )
+    pTemp = m_pOverlay->createStructure ( m_pStructureItem );
 
   if ( m_pShadow )
     pTemp = m_pShadow->createStructure ( m_pStructureItem );
-  
+
   manipulator ( ).createStructure ( m_pStructureItem );
 
   return m_pStructureItem;
@@ -607,7 +694,7 @@ MenuObject *MovieObject::clone ( QWidget *pParent, MenuObject * )
   pNewObject->setNiceness   ( m_iHowNiceShouldIBe);
   //	pNewObject->setMenuName   (m_qsBelongsToMenuName);
   pNewObject->setDVDMenu    ( m_pDVDMenu, false );
-  pNewObject->setStartAfterDropEvent ( m_bStartAfterDropEvent );
+  pNewObject->setStartAfterDropEvent ( movieData ( )->m_bStartAfterDropEvent );
   //	QImage  m_imagePreview;	// used to create a preview in a background task.
   //	QTimer *m_pConvertTimer;
 
@@ -616,6 +703,11 @@ MenuObject *MovieObject::clone ( QWidget *pParent, MenuObject * )
     pShadow = pShadow->clone ( pParent, pNewObject );
   pNewObject->setShadow      ( pShadow );
 
+  MenuObject *pOverlay = overlay ( );
+  if ( pOverlay )
+       pOverlay = pOverlay->clone ( pParent, pNewObject );
+  pNewObject->setOverlay ( pOverlay );
+
   return pNewObject;
 }
 
@@ -693,6 +785,11 @@ void MovieObject::loadPixmap(QString qsFileName)
 
 void MovieObject::drawContents (QPainter *pPainter, int iRenderFrameNumber, int iTotalFramesToRender)
 {
+  if ( m_pOverlay )  {
+       m_pOverlay->drawContents ( pPainter, iRenderFrameNumber, iTotalFramesToRender );
+       return;
+  }
+
   QString qsFullPath, qsFileName, qsMenuName;
   int iCurrentClipFile = 0;
 
@@ -708,11 +805,11 @@ void MovieObject::drawContents (QPainter *pPainter, int iRenderFrameNumber, int
 //		m_listAnimationAttributes[t]->setValue (iRenderFrameNumber);
 
   // This one we use as a loop the video
-  if ( m_enLoop == Loop )
+  if ( movieData ( )->m_enLoop == Loop )
     iCurrentClipFile = ( iDirCount > 0 ) ? iRenderFrameNumber%iDirCount : 0;
-  else if ( m_enLoop == Stretch ) // Stretch : We have the full list of the frames, now we pick the appropriate frame ...
+  else if ( movieData ( )->m_enLoop == Stretch ) // Stretch : We have the full list of the frames, now we pick the appropriate frame ...
     iCurrentClipFile = (int)((float)theDir.count() * (float)iRenderFrameNumber/(float)iTotalFramesToRender);
-  else if ( m_enLoop == Stop )
+  else if ( movieData ( )->m_enLoop == Stop )
     iCurrentClipFile = ( iRenderFrameNumber > iDirCount ) ? iDirCount-1 : iRenderFrameNumber;
   qsFileName = QString ("%1/%2").arg(qsFullPath).arg(theDir[iCurrentClipFile]);
   //printf ("MovieObject::drawContents <%s> <%s> <%d> <%d>\n", (const char *)qsFileName, (const char *)qsFileName, theDir.count(), iCurrentClipFile);
@@ -722,7 +819,7 @@ void MovieObject::drawContents (QPainter *pPainter, int iRenderFrameNumber, int
     updateShadow ( );
 
   // then we call
-  ImageObject::drawContents(pPainter, iRenderFrameNumber, iTotalFramesToRender);
+  ImageObject::drawContents ( pPainter, iRenderFrameNumber, iTotalFramesToRender );
 }
 
 AnimationAttribute *MovieObject::getSpecificAttributes (long iMaxNumberOfFrames, QString qsProperty)
@@ -738,6 +835,3 @@ AnimationAttribute *MovieObject::getSpecificAttributes (long iMaxNumberOfFrames,
 
   return pAnimAttr;
 }
-
-
-
