@@ -62,6 +62,8 @@ OneClickWizard::OneClickWizard ( QWidget *pParent, const char *pName, bool bModa
   m_pConfigWidget        = NULL;
   m_pLibrary             = NULL;
   m_pPlugin              = NULL;
+  m_pToolBoxGroups       = NULL;
+  m_pScrollView          = NULL;
 
   //m_pGroupDVDParameters->hide ( ); // To be hidden for now I am not sure if this is required here ...
   m_pCheckCreateISO->hide ( );
@@ -71,6 +73,7 @@ OneClickWizard::OneClickWizard ( QWidget *pParent, const char *pName, bool bModa
 //  m_pButtonAddImages->setEnabled ( FALSE );
 
   // Can not create QTolbox without any item, so we have to remove the first item in the constructor.
+  createScrollView ( );
   m_pToolBoxGroups->removeItem ( m_pToolBoxGroups->item ( 0 ) );
   GroupView *pGroupView = new GroupView ( m_pToolBoxGroups, (SourceFileEntry *)NULL ); // or this 
   m_pToolBoxGroups->addItem ( pGroupView, QString ( "Groups" ) );
@@ -131,6 +134,42 @@ bool OneClickWizard::initMe ( )
   return ( iRet > 0 );
 }
 
+void OneClickWizard::createScrollView ( )
+{
+  QGridLayout *pLayout = new QGridLayout ( m_pFrameGroups, 1, 1, 0, 6, "m_pLayout1");
+  uint iFlags = Qt::WStaticContents;
+#if (QT_VERSION > 0x0301FF)
+  iFlags |= Qt::WNoAutoErase;
+#else
+  iFlags |= Qt::WPaintClever;
+#endif
+  m_pScrollView    = new QScrollView  ( m_pFrameGroups, "OWToolFRame", iFlags );
+  m_pToolBoxGroups = new QToolBox     ( m_pScrollView->viewport ( ), "OWScrollView", iFlags );
+  m_pScrollView->addChild             ( m_pToolBoxGroups );
+  m_pScrollView->setVScrollBarMode    ( QScrollView::AlwaysOn );
+  pLayout->addWidget ( m_pScrollView, 0, 0 );
+  QTimer::singleShot ( 10, this, SLOT ( slotResizeScrollView ( ) ) );
+}
+
+void OneClickWizard::slotResizeScrollView ( )
+{
+  int iWidth  = m_pOuterFrame->width   ( ) - 32;
+  int iHeight = m_pOuterFrame->height  ( ) - 54;
+  int iCount  = m_pToolBoxGroups->count( );
+  if ( iWidth  <  50 )
+       iWidth  = 200;
+  if ( iHeight <  50 )
+       iHeight = 200;
+  if ( iHeight < iCount * 28 + 120 )
+       iHeight = iCount * 28 + 120;
+  if ( ( m_pToolBoxGroups->width ( )  != iWidth ) ||
+       ( m_pToolBoxGroups->height ( ) != iHeight ) )
+         m_pToolBoxGroups->setFixedSize ( iWidth, iHeight );
+  // Ugly workaround to refresh to the proper extensions
+  // Why don't we have resize signals anywhere ?
+  QTimer::singleShot ( 1500, this, SLOT ( slotResizeScrollView ( ) ) );
+}
+
 QString OneClickWizard::uniqueGroupName ( QString qsName )
 {
   // This fnction will make sure all Groups hold a uniue name.
