@@ -13,6 +13,7 @@
 #include <qcombobox.h>
 #include <qlayout.h>
 #include <qcursor.h>
+#include <qimage.h>
 #include <qtimer.h>
 
 #include "uisimpledvd.h"
@@ -180,8 +181,11 @@ QString ComplexDVD::getImgCmd ( QString &qsTarget )
   }
   else  {
     // Get images from HDRCreme instead
-    iRandom   = ( rand ( ) % 2381 );
-    qsCommand=QString ( "PICURL=`wget -qO - 'hdrcreme.com/photo-gallery/%1/original/' | grep \"img src=\\\"../../../photo\" | sed -e \"s/.*src=\\\"..\\/..\\/..//g\" -e \"s/\\\".*//\"`; wget -q --user-agent=MSIE -O %2 hdrcreme.com$PICURL" ).arg ( iRandom ).arg ( qsTarget );
+    iRandom      = ( rand ( ) % 558 + 1 );
+    int iRandom2 = ( rand ( ) % 20  + 1 );
+    qsCommand=QString ( "PICURL=`wget -qO - 'hdrcreme.com/photo-gallery?page=%1' | grep imgGalleri | sed -e \"s,.*href=\\\",,\" -e \"s,\\\".*,,\" | head -%2 | tail -1`; PICURL=`wget -qO - hdrcreme.com$PICURL | grep showimagepic | sed -e \"s,.*src=\\\",,\" -e \"s,\\\".*,,\"`; wget -q --user-agent=MSIE -O %3 $PICURL" ).arg ( iRandom ).arg ( iRandom2 ).arg ( qsTarget );
+
+//    qsCommand=QString ( "PICURL=`wget -qO - 'hdrcreme.com/photo-gallery/%1/original/' | grep \"img src=\\\"../../../photo\" | sed -e \"s/.*src=\\\"..\\/..\\/..//g\" -e \"s/\\\".*//\"`; wget -q --user-agent=MSIE -O %2 hdrcreme.com$PICURL" ).arg ( iRandom ).arg ( qsTarget );
   }
 
 //printf ( "%s\n", qsCommand.ascii () );
@@ -321,8 +325,8 @@ QString ComplexDVD::getMenuXML ( QValueList<SourceGroup *> &list, int iMenuNumbe
   QFileInfo fileInfo;
   QString qsMenuXML, qsBackgroundFile, qsMenuName, qsFileName, qsGroupName;
   QString qsHeight = "480";
-  QString qsButtonName, qsFrameName, qsObjectName, qsJumpTo, qsX, qsY, qsTempPath;
-  int iCount = 0;
+  QString qsButtonName, qsFrameName, qsObjectName, qsJumpTo, qsX, qsY, qsW, qsH, qsTempPath;
+  int iX, iY, iCount = 0;
   SourceGroup *pGroup = NULL;
   if ( list.count ( ) < 1 )
     return qsMenuXML;
@@ -388,15 +392,24 @@ QString ComplexDVD::getMenuXML ( QValueList<SourceGroup *> &list, int iMenuNumbe
     qsButtonName  = QString ( "Button %1" ).arg ( ++iCount );
     qsObjectName  = QString ( "Object_%1" ).arg (   iCount );
     qsFrameName   = QString ( "Frame %1"  ).arg (   iCount );
-    qsX = QString( "%1" ).arg (( iCount-1 )%3 * 210 + 90 );
-    qsY = ( iCount > 3 ) ? "400" : "150";
+    iX  = ( iCount-1 )%3 * 210 + 90;
+    iY  = ( iCount > 3 ) ? 400 : 150;
+    qsX = QString ( "%1" ).arg ( iX );
+    qsY = QString ( "%1" ).arg ( iY );
 
     // First we create the assiciated TextObject
     qsMenuXML += "   <TextObject TextAlign=\"4,64\" ForegroundColor=\"#fff15735\" Font=\"Sans Serif,16,-1,5,75,0,0,0,0,0\" Text=\""+pGroup->qsGroupName+"\" >\n";
     qsMenuXML += "    <MenuObject Width=\"150\" X=\""+qsX+"\" Y=\""+qsY+"\" Height=\"41\" ObjectName=\""+pGroup->qsGroupName+"\" />\n";
     qsMenuXML += "   </TextObject>\n";
 
-    qsY = ( iCount > 3 ) ? "295" : "45";
+    iY = ( iCount > 3 ) ? 295 : 45;
+
+    QRect r = scaleScreenshot ( pGroup->qsScreenshot, 150, 115 );
+
+    qsX = QString ( "%1" ).arg ( iX + r.left ( ) );
+    qsY = QString ( "%1" ).arg ( iY + r.top  ( ) );
+    qsW = QString ( "%1" ).arg ( r.width  ( ) );
+    qsH = QString ( "%1" ).arg ( r.height ( ) );
 
     // Next we create the ButtonObject for the video jump+-+vmgm+-+
     qsMenuXML += "   <ButtonObject Action=\""+qsJumpTo+"\" ButtonName=\""+qsButtonName+"\"  SourceEntry=\""+qsGroupName+"\" >\n";
@@ -405,21 +418,21 @@ QString ComplexDVD::getMenuXML ( QValueList<SourceGroup *> &list, int iMenuNumbe
     qsMenuXML += "      <Manipulator>\n";
     qsMenuXML += "       <Modifiers/>\n";
     qsMenuXML += "      </Manipulator>\n";
-    qsMenuXML += "      <MenuObject Width=\"150\" X=\""+qsX+"\" Y=\""+qsY+"\" Height=\"115\" ObjectName=\""+qsObjectName+"\" ObjectExtractionPath=\""+qsObjectName+"\" />\n";
+    qsMenuXML += "      <MenuObject Width=\""+qsW+"\" X=\""+qsX+"\" Y=\""+qsY+"\" Height=\""+qsH+"\" ObjectName=\""+qsObjectName+"\" ObjectExtractionPath=\""+qsObjectName+"\" />\n";
     qsMenuXML += "     </ImageObject>\n";
     qsMenuXML += "    </NormalState>\n";
 
     qsMenuXML += "    <SelectedState>\n";
     qsMenuXML += "     <FrameObject Width=\"10\" Color=\"#a0ff1414\" >\n";
-    qsMenuXML += "      <MenuObject Width=\"150\" X=\""+qsX+"\" Y=\""+qsY+"\" Height=\"112\" ObjectName=\""+qsFrameName+"\" />\n";
+    qsMenuXML += "      <MenuObject Width=\""+qsW+"\" X=\""+qsX+"\" Y=\""+qsY+"\" Height=\""+qsH+"\" ObjectName=\""+qsFrameName+"\" />\n";
     qsMenuXML += "     </FrameObject>\n";
     qsMenuXML += "    </SelectedState>\n";
     qsMenuXML += "    <HighlightedState>\n";
     qsMenuXML += "     <FrameObject Width=\"10\" Color=\"#ff0f0fff\" >\n";
-    qsMenuXML += "      <MenuObject Width=\"150\" X=\""+qsX+"\" Y=\""+qsY+"\" Height=\"112\" ObjectName=\""+qsFrameName+"\" />\n";
+    qsMenuXML += "      <MenuObject Width=\""+qsW+"\" X=\""+qsX+"\" Y=\""+qsY+"\" Height=\""+qsH+"\" ObjectName=\""+qsFrameName+"\" />\n";
     qsMenuXML += "     </FrameObject>\n";
     qsMenuXML += "    </HighlightedState>\n";
-    qsMenuXML += "    <MenuObject Width=\"150\" X=\""+qsX+"\" Y=\""+qsY+"\" Height=\"112\" ObjectName=\""+qsButtonName+"\" />\n";
+    qsMenuXML += "    <MenuObject Width=\""+qsW+"\" X=\""+qsX+"\" Y=\""+qsY+"\" Height=\""+qsH+"\" ObjectName=\""+qsButtonName+"\" />\n";
     qsMenuXML += "   </ButtonObject>\n";
   }
   // Create prev/next buttons
@@ -509,5 +522,38 @@ QString ComplexDVD::getMenuButtons ( bool bPrev, bool bNext, int iMenuNumber )
   return   qsMenuXML;
 }
 
+QRect ComplexDVD::scaleScreenshot ( QString &qsScreenshot, int iWidth, int iHeight )
+{
+  QRect rect ( 0, 0, iWidth, iHeight );
+  QFileInfo fileInfo  ( qsScreenshot );
+  if ( ! fileInfo.exists ( ) )
+    return rect;
+
+  QString qsTemp = m_qsTempPath + "/" + fileInfo.baseName ( ) + ".png";
+  QImage image ( qsScreenshot );
+  if (   image.isNull ( ) )
+    return rect;
+
+  int x, y, w, h;
+  double fScaleX = ( (double)image.height ( ) / image.width  ( ) );
+  double fScaleY = ( (double)image.width  ( ) / image.height ( ) );
+  x = 0; y = 0; w = iWidth; h = iHeight;
+
+  if ( fScaleX <= fScaleY )  {
+    // Here we are scaling based on the width
+//    h = (int)( fScaleX * iHeight );
+    h = (int)( fScaleX * iWidth );
+    y = (int)( (double)( iHeight - h ) / 2.0 );
+  }
+  else  {
+    // Here we are scaling based on the height
+    w = (int)( fScaleY * iWidth );
+    x = (int)( (double)( iWidth - w ) / 2.0 );
+  }
+
+  rect = QRect ( x, y, w, h );
+  return rect;
+}
+
 }; // end namespace Plugin
 
