@@ -5,6 +5,7 @@
 #include <qapplication.h>
 #include <qpushbutton.h>
 #include <qfileinfo.h>
+#include <qimage.h>
 
 #include "uisimpledvd.h"
 #include "simpledvd.h"
@@ -112,8 +113,8 @@ QString SimpleDVD::getMenuXML ( QValueList<SourceGroup *> &list, int iMenuNumber
   QFileInfo fileInfo;
   QString qsMenuXML, qsBackgroundFile, qsMenuName, qsFileName, qsGroupName;
   QString qsHeight = "480";
-  QString qsButtonName, qsFrameName, qsObjectName, qsJumpTo, qsX, qsY;
-  int iCount = 0;
+  QString qsButtonName, qsFrameName, qsObjectName, qsJumpTo, qsX, qsY, qsW, qsH;
+  int iX, iY, iCount = 0;
   SourceGroup *pGroup = NULL;
   if ( list.count ( ) < 1 )
     return qsMenuXML;
@@ -175,16 +176,24 @@ QString SimpleDVD::getMenuXML ( QValueList<SourceGroup *> &list, int iMenuNumber
     qsButtonName  = QString ( "Button %1" ).arg ( ++iCount );
     qsObjectName  = QString ( "Object_%1" ).arg (   iCount );
     qsFrameName   = QString ( "Frame %1"  ).arg (   iCount );
-    qsX = QString( "%1" ).arg (( iCount-1 )%3 * 210 + 90 );
-    qsY = ( iCount > 3 ) ? "400" : "150";
+    iX = ( iCount-1 )%3 * 210 + 90;
+    iY = ( iCount > 3 ) ? 400 : 150;
+    qsX = QString ( "%1" ).arg ( iX );
+    qsY = QString ( "%1" ).arg ( iY );
 
     // First we create the assiciated TextObject
     qsMenuXML += "   <TextObject TextAlign=\"4,64\" ForegroundColor=\"#fff15735\" Font=\"Sans Serif,16,-1,5,75,0,0,0,0,0\" Text=\""+pGroup->qsGroupName+"\" >\n";
     qsMenuXML += "    <MenuObject Width=\"150\" X=\""+qsX+"\" Y=\""+qsY+"\" Height=\"41\" ObjectName=\""+pGroup->qsGroupName+"\" />\n";
     qsMenuXML += "   </TextObject>\n";
 
-    qsY = ( iCount > 3 ) ? "295" : "45";
+    iY = ( iCount > 3 ) ? 295 : 45;
 
+    QRect r = scaleScreenshot ( pGroup->qsScreenshot, 150, 115 );
+
+    qsX = QString ( "%1" ).arg ( iX + r.left ( ) );
+    qsY = QString ( "%1" ).arg ( iY + r.top  ( ) );
+    qsW = QString ( "%1" ).arg ( r.width  ( ) );
+    qsH = QString ( "%1" ).arg ( r.height ( ) );
     // Next we create the ButtonObject for the video jump+-+vmgm+-+
     qsMenuXML += "   <ButtonObject Action=\""+qsJumpTo+"\" ButtonName=\""+qsButtonName+"\"  SourceEntry=\""+qsGroupName+"\" >\n";
     qsMenuXML += "    <NormalState>\n";
@@ -192,21 +201,21 @@ QString SimpleDVD::getMenuXML ( QValueList<SourceGroup *> &list, int iMenuNumber
     qsMenuXML += "      <Manipulator>\n";
     qsMenuXML += "       <Modifiers/>\n";
     qsMenuXML += "      </Manipulator>\n";
-    qsMenuXML += "      <MenuObject Width=\"150\" X=\""+qsX+"\" Y=\""+qsY+"\" Height=\"115\" ObjectName=\""+qsObjectName+"\" ObjectExtractionPath=\""+qsObjectName+"\" />\n";
+    qsMenuXML += "      <MenuObject Width=\""+qsW+"\" X=\""+qsX+"\" Y=\""+qsY+"\" Height=\""+qsH+"\" ObjectName=\""+qsObjectName+"\" ObjectExtractionPath=\""+qsObjectName+"\" />\n";
     qsMenuXML += "     </ImageObject>\n";
     qsMenuXML += "    </NormalState>\n";
 
     qsMenuXML += "    <SelectedState>\n";
     qsMenuXML += "     <FrameObject Width=\"10\" Color=\"#a0ff1414\" >\n";
-    qsMenuXML += "      <MenuObject Width=\"150\" X=\""+qsX+"\" Y=\""+qsY+"\" Height=\"112\" ObjectName=\""+qsFrameName+"\" />\n";
+    qsMenuXML += "      <MenuObject Width=\""+qsW+"\" X=\""+qsX+"\" Y=\""+qsY+"\" Height=\""+qsH+"\" ObjectName=\""+qsFrameName+"\" />\n";
     qsMenuXML += "     </FrameObject>\n";
     qsMenuXML += "    </SelectedState>\n";
     qsMenuXML += "    <HighlightedState>\n";
     qsMenuXML += "     <FrameObject Width=\"10\" Color=\"#ff0f0fff\" >\n";
-    qsMenuXML += "      <MenuObject Width=\"150\" X=\""+qsX+"\" Y=\""+qsY+"\" Height=\"112\" ObjectName=\""+qsFrameName+"\" />\n";
+    qsMenuXML += "      <MenuObject Width=\""+qsW+"\" X=\""+qsX+"\" Y=\""+qsY+"\" Height=\""+qsH+"\" ObjectName=\""+qsFrameName+"\" />\n";
     qsMenuXML += "     </FrameObject>\n";
     qsMenuXML += "    </HighlightedState>\n";
-    qsMenuXML += "    <MenuObject Width=\"150\" X=\""+qsX+"\" Y=\""+qsY+"\" Height=\"112\" ObjectName=\""+qsButtonName+"\" />\n";
+    qsMenuXML += "    <MenuObject Width=\""+qsW+"\" X=\""+qsX+"\" Y=\""+qsY+"\" Height=\""+qsH+"\" ObjectName=\""+qsButtonName+"\" />\n";
     qsMenuXML += "   </ButtonObject>\n";
   }
   // Create prev/next buttons
@@ -292,5 +301,38 @@ QString SimpleDVD::getMenuButtons ( bool bPrev, bool bNext, int iMenuNumber )
   return   qsMenuXML;
 }
 
+QRect SimpleDVD::scaleScreenshot ( QString &qsScreenshot, int iWidth, int iHeight )
+{
+  QRect rect ( 0, 0, iWidth, iHeight );
+  QFileInfo fileInfo  ( qsScreenshot );
+  if ( ! fileInfo.exists ( ) )
+    return rect;
+
+  QString qsTemp = m_qsTempPath + "/" + fileInfo.baseName ( ) + ".png";
+  QImage image ( qsScreenshot );
+  if (   image.isNull ( ) )
+    return rect;
+
+  int x, y, w, h;
+  double fScaleX = ( (double)image.height ( ) / image.width  ( ) );
+  double fScaleY = ( (double)image.width  ( ) / image.height ( ) );
+  x = 0; y = 0; w = iWidth; h = iHeight;
+
+  if ( fScaleX <= fScaleY )  {
+    // Here we are scaling based on the width
+//    h = (int)( fScaleX * iHeight );
+    h = (int)( fScaleX * iWidth );
+    y = (int)( (double)( iHeight - h ) / 2.0 );
+  }
+  else  {
+    // Here we are scaling based on the height
+    w = (int)( fScaleY * iWidth );
+    x = (int)( (double)( iWidth - w ) / 2.0 );
+  }
+
+  rect = QRect ( x, y, w, h );
+  return rect;
+}
+
 }; // end namespace Plugin
 
