@@ -16,9 +16,12 @@
 #include "cachethumbs.h"
 #include "uipreviewer.h"
 
+class QTimer;
 class QIconView;
 class QIconViewItem;
 
+class MediaInterface;
+
 namespace Input
 {
 
@@ -40,17 +43,32 @@ protected slots:
   virtual void slotLast  ( );
   virtual void slotNext  ( );
   virtual void slotPrev  ( );
+  virtual void slotPlay  ( );
+  virtual void slotStop  ( );
   virtual void slotLoadWholeImage ( );
   virtual void slotShowImageInfo  ( );
+  virtual void slotAutoPlay       ( );
+  virtual void slotResizeM        ( );
 
 protected:
   virtual void mouseReleaseEvent  ( QMouseEvent * );
 
   void setButtonState ( );
+  bool isImage        ( QString & );
+  void setMovieFrame  ( );
+  void stop           ( );
 
-  QWMatrix      *m_pMatrix;
-  QIconView     *m_pIconView;
-  QIconViewItem *m_pCurrentItem;
+private:
+  bool            m_bImageItem;
+  bool            m_bPaused;
+  QWMatrix       *m_pMatrix;
+  QIconView      *m_pIconView;
+  QString         m_qsFileName;
+  QIconViewItem  *m_pCurrentItem;
+  QTimer         *m_pTimerAutoPlay;
+  MediaInterface *m_pMediaInterface;
+  QValueList<QPixmap *> m_listOfThumbs;
+  int m_iCurrentPix;
 };
 
 };  // End of namespace Input
