@@ -45,7 +45,11 @@ class DialogLibrary;
 class TemplateWizard;
 class CXmlDVDAuthor;
 class CXmlSpumux;
-namespace DVDLayout { class Gui; };
+class UndoBuffer;
+class DialogUndoStack;
+class ButtonTransition;
+namespace DVDLayout { class Gui;    };
+namespace Render    { class Client; };
 
 #define DVDAUTHOR_XML "dvdauthor.xml"
 
@@ -77,6 +81,7 @@ public slots:
 	virtual void editUndo     ( );
 	virtual void editRedo     ( );
 	virtual void editCut      ( );
+	virtual void editCopy     ( );
 	virtual void editPaste    ( );
 	virtual void editFind     ( );
 	virtual void helpIndex    ( );
@@ -96,12 +101,15 @@ public slots:
 	virtual void slotChangeLanguage         (  int );
 	virtual void slotVisibleRegion          ( bool );
 	virtual void slotSetup                       ( );
+	virtual void slotVersions                    ( );
 	virtual void slotDialogTemp                  ( );
 	virtual void slotTemplateDownload            ( );
 	virtual void slotExportDVDAuthorXML          ( );
 	virtual void slotEditAddMovie                ( );
 	virtual void slotAddFromLibrary              ( );
 	virtual void slotEditAddSlideshow            ( );
+	virtual void slotSelectZone                  ( );
+	virtual void slotZone                   ( QAction * );
 	virtual void slotStructureClicked ( StructureItem * );
 	virtual void slotGenerateMasks               ( );
 	virtual void slotCreateDVD                   ( );
@@ -140,9 +148,13 @@ public:
 	void addDVDMenu           ( DVDMenu *, bool b=false );
 	void returnFromRenderMenu ( );
 	bool checkForNewProject   ( );
+	void showUndoStack        ( );
+	void updateUndoStack      ( );
 	bool tabVisible           ( enTabs );
+	bool pleaseWait           ( );
 
 	int  getFreeSourceSlot                                ( );
+	void addDroppedSource                         ( QString );
 	void addMovieList      ( QStringList &, QString *p=NULL );
 	int  sourceFileCount                                  ( );
 	SourceFileEntry *sourceFileEntry                 ( uint );
@@ -152,20 +164,24 @@ public:
 	void removedSourceEntry             ( SourceFileEntry * );  // called after entry has been removed from SourceToolBar
 	void updateSourceFileInfo           ( SourceFileInfo  * );
 	void editSourceFileInfo             ( SourceFileInfo  * );
+	void updateDVDLayout                ( SourceFileEntry * );
+	void replaceMenuName             ( DVDMenu *, QString & );
 	void replaceSourceDisplayName    ( QString &, QString & );
 	bool isSourceEntryUsed                      ( QString & );
 	void addSound                           ( QStringList & );
 	void setColor                              ( uint, Rgba );
 	void updateStatusColor                                ( ); // updates the SourceToolBar color's
-	MediaInfo *getMediaInfo                               ( );
+	MediaInfo  *getMediaInfo                              ( );
+	UndoBuffer *getUndoBuffer                             ( );
 	DragNDropContainer *getDragNDropContainer             ( );
 
 	DVDMenu *findMenuFromSourceFileEntry( SourceFileEntry * );
-	DVDMenu         *getCurrentSubMenu                    ( );
-	DVDMenu         *getSubMenuByName             ( QString );
-	DVDMenu         *getVMGMenu                           ( );
+	DVDMenu *getCurrentSubMenu                            ( );
+	DVDMenu *getSubMenuByName                     ( QString );
+	DVDMenu *getVMGMenu                                   ( );
 	QValueList<DVDMenu *> &getSubMenus                    ( );
 	void    importDvdauthor                       (  void * );
+	void    createContextMenu ( SourceFileEntry *, QPoint & );
 
 	QString getUniqueMenuTitle  ( QString, int i=0, DVDMenu *p=NULL );
 	QString getVidFilter        ( );
@@ -179,6 +195,7 @@ private: // Private functions ...
 	void LoadIni                ( );
 	void SaveIni                ( );
 	void newWizard              ( );
+	void setRegionalZone        ( );
 	void buildHistoryEntries    ( );
 	void updateHistoryPix       ( );
 	void initTabs               ( );
@@ -196,13 +213,18 @@ private: // Private functions ...
 	void createTempDirStructure ( );
 	void createDvdauthorXml     ( );
 	void createSubtitlesXml     ( );
+	void createTransitionsXml   ( );
 	void createMenuXml          ( );
 	void createDVD              ( );
 	bool checkHealth            ( );
 	void engageCreateDVD        ( bool ); // Called to put QDVDAuthor in wait mode for slideshows to finish
 	void replaceTempPaths       ( QString );
-	QString getTempFile         ( QString );
-	QWidget *getTabWidget       ( enTabs, int i=0 );
+	QString  getTempFile        ( QString );
+	QWidget *getTabWidget       ( enTabs,    int i=0 );
+	unsigned long getAudioSize  ( SourceFileInfo   * );
+	void eventEntry             ( Render::Client   * );
+	void eventTrans             ( Render::Client   * );
+	bool buttonTransitionExists ( ButtonTransition * );
 
 private:
 	bool    m_bVisibleRegion;
@@ -218,6 +240,7 @@ private:
 	QTimer                 *m_pTimerAutosave;
 	bool                    m_bAutosaving;
 	bool                    m_bCreateDVDAfterSlideshows;
+	bool                    m_bWait;
 
 	Plugin::Interface      *m_pPluginInterface;
 	ColorToolBar           *m_pColorToolbar;
@@ -235,6 +258,7 @@ private:
 	DialogExecute          *m_pDialogExecute;
 	DialogLibrary          *m_pDialogLibrary;
 	TemplateWizard         *m_pTemplateWizard;
+	DialogUndoStack        *m_pUndoStackDialog;
 
 	QToolBar *m_pToolbar;
 	QAction  *m_pActionToolbarNew;
