@@ -39,6 +39,11 @@ public:
 	virtual void setPosition       (uint);
 	virtual void setPositionByTime (uint);
 
+	virtual void setNavigate( enNavigate );
+	virtual void setMenu        ( enMenu );
+	virtual void setSubtitleTrack  ( int );
+	virtual void setAudioTrack     ( int );
+
 	virtual float volume               ();
 	virtual float position             ();
 	virtual QString supportedExtensions();
@@ -49,7 +54,7 @@ public:
 
 
 	// Functions handled in the MediaInterface - calss
-	const QString& getMRL();
+//	const QString& getMRL();
 	void  slotTogglePause();
 
 	// Non mandatory functions. 
