@@ -176,8 +176,8 @@ void MPlayerInfo::queryValues ( bool bTryToLock )
   QFileInfo fileInfo ( QFile::encodeName ( m_qsFileName ) );
   QString qsAudioExtensions = getExtensions ( false );
   bool iIsVideo = 1;
-  if ( qsAudioExtensions.find ( fileInfo.extension ( ) ) > -1 )
-    iIsVideo = 0;
+  if ( qsAudioExtensions.find ( fileInfo.extension ( FALSE ) ) > -1 )
+       iIsVideo = 0;
 
   m_pSlaveProcess = new MPlayerInfoProcess ( this );
   debug_out ( "For this<%p> = new slave <%p>\n\n", this, m_pSlaveProcess );
@@ -212,10 +212,10 @@ void MPlayerInfo::queryValues ( bool bTryToLock )
     waitForProcess  ( );
 }
 
-void MPlayerInfo::waitForProcess ( ) 
+void MPlayerInfo::waitForProcess ( )
 {
   if ( ! m_pSlaveProcess )
-	  return;
+    return;
   int  iMaxCounter = 200;
   bool bContinue   = true;
   try {
@@ -230,12 +230,12 @@ debug_out ( "this<%p> slave<%p> iCount<%d>\n", this, m_pSlaveProcess, iMaxCounte
       else {
         bContinue = false;
         Unlock ( );
-        break;	 
+        break;
       }
     }
   }
   catch ( ... ) {
-	// Simply do not crash ...
+    // Simply do not crash ...
   }
 }
 
@@ -246,8 +246,10 @@ void MPlayerInfo::slotReadStdout ()
   Lock ( );
 //printf ("MPlayerInfo::slotReadStdout\n");
   QString qsWidth, qsHeight, qsWindowWidth, qsWindowHeight;
-  QString qsLength, qsBuffer;
+  QString qsLength, qsBuffer, qsAspect;
   QString qsStdout ( m_pSlaveProcess->readStdout ( ) );
+  float fAspect = 0.0f;
+  int iPos;
 
   if ( qsStdout.length ( ) == 0) {
     Unlock ( );
@@ -291,9 +293,39 @@ void MPlayerInfo::slotReadStdout ()
 		m_bAudioHandled = true;
 	}
 	// Movie-Aspect is 1.33:1 - prescaling to correct movie aspect.
-	regSize = QRegExp ("Movie-Aspect is (\\S+).*");	//  1.33:1 OR undefined ...
-	if (regSize.search(qsStdout) > -1)	{
-		m_qsRatio = regSize.cap(1);
+	regSize = QRegExp ("Movie-Aspect is (\\S+).*"); //  1.33:1 OR undefined ...
+	iPos = 0;
+	while ( ( iPos = regSize.search ( qsStdout, iPos ) ) != -1 )  {
+	  iPos += regSize.matchedLength ( );
+	  if ( fAspect == 0.0 )  {
+	    qsAspect = regSize.cap    ( 1 );
+	    fAspect  = qsAspect.toFloat ( );
+	    if ( fAspect > 0.0 )  {
+	      if ( fAspect < 1.5 )
+	        m_qsRatio = "4:3";
+	      else if ( fAspect < 2.0 )
+	        m_qsRatio = "16:9";
+	      else
+	        m_qsRatio = "4:3";
+	    }
+	  }
+	}
+	regSize = QRegExp ("ID_VIDEO_ASPECT=([0123456789.]*)"); //  1.33:1 OR undefined ...
+	iPos = 0;
+	while ( ( iPos = regSize.search ( qsStdout, iPos ) ) != -1 )  {
+	  iPos += regSize.matchedLength ( );
+	  if ( fAspect == 0.0 )  {
+	    qsAspect = regSize.cap    ( 1 );
+	    fAspect  = qsAspect.toFloat ( );
+	    if ( fAspect > 0.0 )  {
+	      if ( fAspect < 1.5 )
+	        m_qsRatio = "4:3";
+	      else if ( fAspect < 2.0 )
+	        m_qsRatio = "16:9";
+	      else
+	        m_qsRatio = "4:3";
+	    }
+	  }
 	}
 	// Selected video codec: [mpeg12] vfm:libmpeg2 (MPEG 1 or 2 (libmpeg2))
 	regSize = QRegExp ("ID_VIDEO_FORMAT=(\\S*)");
@@ -404,41 +436,48 @@ QImage MPlayerInfo::getScreenshot( long iMSecOffset )
 
         int iRet;
 	QImage theImage;
-	QString qsCommand, qsTempPath, m_qsScreenshotName;
+	QString qsCommand, qsTempPath, m_qsScreenshotName, qsFileName;
 	QTime timeOffset;
+
+	qsFileName = getFileName ( );
+	// Little surprise, that ` is not converted through QFile::convertName ...
+	qsFileName.replace ( "`", "\\`" );
+
 	timeOffset = timeOffset.addMSecs ( iMSecOffset );
-	QFileInfo fileInfo ( getFileName ( ) );
+		QFileInfo fileInfo ( qsFileName );
 
 	qsTempPath = QFile::encodeName ( QString ( "%1/%2" ).arg ( m_qsTempPathName ).arg ( fileInfo.baseName ( ) ) );
-	iRet = system ((const char *)QString ( "mkdir -p \"%1\"" ).arg ( qsTempPath ) );
+	qsCommand  = QString ( "mkdir -p \"%1\"" ).arg ( qsTempPath );
+	iRet = system ( qsCommand.utf8 ( ) );
 	
 	m_qsScreenshotName  = QString ("%1/00000001.jpg").arg ( qsTempPath );
 	if (iMSecOffset != 0)	// For some reasons Mplayer produces always the first image from start
 		m_qsScreenshotName  = QString ( "%1/00000002.jpg" ).arg ( qsTempPath );
 
 	// Next we verify that the file has been created.
-	fileInfo.setFile ( QFile::decodeName ( m_qsScreenshotName.latin1 ( ) ) );
-	if ( fileInfo.exists ( ) )
-		iRet = system ( QString ( "rm -f \"%1\"" ).arg ( m_qsScreenshotName ) );
-
+	fileInfo.setFile ( QFile::decodeName ( m_qsScreenshotName.utf8 ( ) ) );
+	if ( fileInfo.exists ( ) )  {
+		qsCommand = QString ( "rm -f \"%1\"" ).arg ( m_qsScreenshotName );
+		iRet = system ( qsCommand.utf8 ( ) );
+	}
 	// Here we generate the screenshot ...
 //	qsCommand = QString("mplayer -ao null -vo jpeg:outdir=\"%1/\" -vf spp,scale -frames 2 -quiet \"%2\" &>/dev/null").arg(qsTempPath).arg(getFileName());
 	if ( iMSecOffset == 0 )
-		qsCommand = QString ( "mplayer -ao null -vo jpeg:outdir=\"%1/\" -vf scale -frames 2 -quiet \"%2\" >/dev/null 2>&1" ).arg ( qsTempPath ).arg ( QFile::encodeName ( getFileName ( ) ) );
+		qsCommand = QString ( "mplayer -ao null -vo jpeg:outdir=\"%1/\" -vf scale -frames 2 -quiet \"%2\" >/dev/null 2>&1 &" ).arg ( qsTempPath ).arg ( QFile::encodeName ( qsFileName.utf8 ( ) ) );
 	else
-		qsCommand = QString("mplayer -ao null -vo jpeg:outdir=\"%1/\" -vf scale -ss %2 -forceidx -frames 3 -quiet \"%3\" >/dev/null 2>&1" ).arg ( qsTempPath ).arg ( timeOffset.toString ( ) ).arg ( QFile::encodeName ( getFileName ( ) ) );
+		qsCommand = QString ( "mplayer -ao null -vo jpeg:outdir=\"%1/\" -vf scale -ss %2 -forceidx -frames 3 -quiet \"%3\" >/dev/null 2>&1 &" ).arg ( qsTempPath ).arg ( timeOffset.toString ( ) ).arg ( QFile::encodeName ( qsFileName.utf8 ( ) ) );
 
-	iRet = system ( qsCommand );
+	iRet = system ( qsCommand.utf8 ( ) );
 	//printf ("MPlayerInfo::getScreenshot iRet<%d> = <%s>\n", iRet,(const char *)qsCommand);
 
 	int iMaxWait = 0;
-	while ( ( ! fileInfo.exists ( ) ) && 
+	while ( ( ! fileInfo.exists ( ) ) &&
 		( iMaxWait < 5 ) ) {
 		sleep ( 1 );
 		iMaxWait ++;
 	}
 	if ( fileInfo.exists ( ) )
-		theImage = QImage ( QFile::decodeName ( m_qsScreenshotName.latin1 ( ) ) );
+		theImage = QImage ( QFile::decodeName ( m_qsScreenshotName.utf8 ( ) ) );
 
 	//m_pMutex->unlock ( );
 	return theImage;
