@@ -61,7 +61,7 @@ QMPlayerWidget::~QMPlayerWidget ( )
   if ( m_pMasterProcess ) {
     if ( m_pMasterProcess->processIdentifier ( ) > 0 ) {
       QString qsCommand = QString ( "kill -9 `ps -ef | grep -e mplayer -e %1 | awk '{print $2}'`" ).arg ( winId ( ) ); 
-      int iRet = system ( qsCommand.ascii ( ) );
+      int iRet = system ( qsCommand.utf8 ( ) );
       iRet = iRet;
     }
     if ( m_pMasterProcess->isRunning ( ) )
@@ -430,16 +430,16 @@ void QMPlayerWidget::slotSetVolume(float fValue)
   writeToStdin ( qsBuffer );
 }
 */
-void QMPlayerWidget::slotScreenshot()
+void QMPlayerWidget::slotScreenshot ( )
 {
-
-	int iRet = system (QString("rm -f %1").arg(m_qsScreenshotName));
+	QString qsCommand = QString ( "rm -f %1" ).arg ( m_qsScreenshotName );
+	int iRet = system ( qsCommand.utf8 ( ) );
 	iRet = iRet;
-	QString qsBuffer = QString("screenshot %1\n").arg(m_qsScreenshotName);
-	writeToStdin (qsBuffer);
+	QString qsBuffer = QString ( "screenshot %1\n").arg ( m_qsScreenshotName );
+	writeToStdin ( qsBuffer );
 	// Next we verify that the file has been created.
-	QFileInfo fileInfo(m_qsScreenshotName);
-	if (fileInfo.exists())	{
+	QFileInfo fileInfo ( m_qsScreenshotName );
+	if ( fileInfo.exists ( ) )  {
 
 	}
 }
