@@ -149,12 +149,13 @@ void XineInfo::queryValues ()
 	int iReturn, iPosStream, iPosTime, iLengthTime;
 
 	// Sanity check ...
-	if (!xine_open(m_pXineStream, (const char *)m_qsFileName) || !xine_play(m_pXineStream, 0, 0)) {
-		printf("Unable to open mrl\n");
+	if ( ! xine_open ( m_pXineStream, m_qsFileName.ascii ( ) ) ||
+	     ! xine_play ( m_pXineStream, 0, 0 ) ) {
+		printf ( "Unable to open mrl\n" );
 		return;
 	}
 	// this is the main function, to query all values
-	if (!m_pXineStream)
+	if ( ! m_pXineStream )
 		return;
 	m_bHasAudio = (bool)xine_get_stream_info (m_pXineStream, XINE_STREAM_INFO_HAS_AUDIO);
 	m_bHasVideo = (bool)xine_get_stream_info (m_pXineStream, XINE_STREAM_INFO_HAS_VIDEO);
@@ -171,10 +172,16 @@ void XineInfo::queryValues ()
 	else
 		m_qsStatus = QObject::tr ("Not Ok");
 
-	iReturn = xine_get_pos_length (m_pXineStream, &iPosStream, &iPosTime, &iLengthTime);
+	iReturn      = 0;
+	int iCounter = 0;
+	while ( ( iReturn == 0 ) && ( iCounter++ < 3 ) )  {
+		usleep ( 1000 );
+		// Note: Xine is prone to returning wrong length information ( 2008 - 12 - 21 )
+		iReturn = xine_get_pos_length (m_pXineStream, &iPosStream, &iPosTime, &iLengthTime );
+	}
 	if (iReturn == 1)	{	// success
-		m_iLength = iLengthTime;
-		m_qsLength = msToTimeString (iLengthTime);
+		m_iLength  = iLengthTime;
+		m_qsLength = msToTimeString ( iLengthTime );
 	}
 	else
 		m_qsLength = QString ("0:00:00");
@@ -296,7 +303,6 @@ for (t=0;t<15;t++)
 	printf ("<%d = %d> ", iVideoArray[t], xine_get_stream_info (m_pXineStream, iVideoArray[t]));
 printf ("\n\n");
 */
-
 	xine_close(m_pXineStream);
 }
 
