@@ -29,7 +29,9 @@ public:
 	int getMax ();
 
 signals:
-	void signalNewValue(int);
+	void signalNewValue       ( int );
+	void signalSliderPressed  ( );
+	void signalSliderReleased ( );
 
 protected:
 	virtual void mouseMoveEvent    (QMouseEvent *);
@@ -59,15 +61,19 @@ public:
 	int  getMax                      ();
 
 signals:
-	void signalValuesChanged(int, int);
+	virtual void signalValuesChanged  ( int, int );
+	virtual void signalStartStopPressed   ( bool );
+	virtual void signalStartStopReleased  ( bool );
 
 protected slots:
-	virtual void show             ();
-	virtual void slotStartPressed ();
-	virtual void slotEndPressed   ();
-	virtual void slotNewStartValue(int);
-	virtual void slotNewEndValue  (int);
-	virtual void timeoutRepeat    ();
+	virtual void show              ( );
+	virtual void slotStartPressed  ( );
+	virtual void slotEndPressed    ( );
+	virtual void slotStartReleased ( );
+	virtual void slotEndReleased   ( );
+	virtual void slotNewStartValue ( int);
+	virtual void slotNewEndValue   ( int);
+	virtual void timeoutRepeat     ( );
 
 protected:
 	virtual void paintEvent       (QPaintEvent  *);
@@ -96,23 +102,30 @@ public:
 	 StartStopControl (QWidget* p=0, const char* n=0, WFlags f=0);
 	~StartStopControl ();
 	
-	void initMe          (QString, int, int l=0);
-	void showSlider  (bool);
-	void setProgress (float);
-	void setStart    (float); // [0.0 .. 1.0]
-	void setEnd      (float); // [0.0 .. 1.0]
+	void initMe      ( QString, int      );
+	void initMe      ( QString, int, int );
+	void createThumbnails ( int i=0 );
 
-	void updateMovieLine ();
-	void drawStartStopLines ();
+	void showSlider  ( bool  );
+	void setProgress ( float );
+	void setStart    ( float ); // [0.0 .. 1.0]
+	void setEnd      ( float ); // [0.0 .. 1.0]
+
+	void updateMovieLine   ( );
+	void drawStartStopLines( );
 
 public slots:
-	virtual void slotSliderValues (int, int);
+	virtual void slotSliderValues   ( int, int );
+	virtual void slotSliderPressed  ( bool );
+	virtual void slotSliderReleased ( bool );
 
 signals:
-	void signalSliderValues (float, float);
+	void signalSliderPressed  ( bool );
+	void signalSliderReleased ( bool );
+	void signalSliderValues   ( float, float );
 
 protected:
-	virtual void timerEvent  (QTimerEvent  *);
+	virtual void timerEvent   ( QTimerEvent  * );
 
 private:
 	StartStopSlider       *m_pStartStopSlider;
