@@ -31,6 +31,7 @@
 #include <math.h>
 #include <stdlib.h>
 #include <arpa/inet.h>
+#include <sys/stat.h>
 
 #include <qdir.h>
 #include <qfile.h>
@@ -40,12 +41,14 @@
 #include <qbuffer.h>
 #include <qpainter.h>
 #include <qapplication.h>
+#include <qsocketdevice.h>
 
 #include "render_client.h"
 
 #include "global.h"
 #include "messagebox.h"
 #include "qdvdauthor.h"
+#include "buttonobject.h"
 #include "xml_slideshow.h"
 #include "sourcefileentry.h"
 #include "../qrender/shared.h"
@@ -53,9 +56,15 @@
 #ifdef debug_out
 #undef debug_out
 #endif
+#if 1
 static void dummy(const char*,...){};
 #define debug_out dummy
-//#define debug_out printf
+#define sdebug_out dummy
+#else
+//#define debug_out printf ( "%s::%s::%d >", __FILE__, __FUNCTION__, __LINE__ ); printf
+#define debug_out printf ( "thr:%X obj:%p :: %s::%s::%d >", (uint)pthread_self(), this, __FILE__, __FUNCTION__, __LINE__ ); printf
+#define sdebug_out printf ( "thr:%X obj:          :: %s::%s::%d >", (uint)pthread_self(), __FILE__, __FUNCTION__, __LINE__ ); printf
+#endif
 
 namespace Render
 {
@@ -69,6 +78,7 @@ Manager::ServerInfo::ServerInfo ( unsigned long iServerIP, unsigned int iPort, u
   m_iServerPort        = iPort;
   m_iConcurrentThreads = iConcurrentThreads;
   m_bRecoveryMode      = false;
+  m_bStartingServer    = false;
 }
 
 Manager::ServerInfo::~ServerInfo ( )
@@ -105,6 +115,41 @@ bool Manager::ServerInfo::inRecovery ( )
   return m_bRecoveryMode;
 }
 
+bool Manager::ServerInfo::startingServer ( )
+{
+  return m_bStartingServer;
+}
+
+void Manager::ServerInfo::setStarting ( bool bStarting )
+{
+  m_bStartingServer = bStarting;
+}
+
+void Manager::ServerInfo::startServer ( )
+{
+  if ( startingServer ( ) )
+    return;
+  // We need to block other threads from trying the same to give qrender time to get started
+  setStarting ( true );
+
+  Utils theUtils;
+  QString qsCommand = theUtils.getToolPath ( "qrender" );
+  if ( qsCommand.isEmpty ( ) )  {
+    qsCommand = Global::qsSystemPath + "/bin/qrender";
+    QFileInfo fileInfo ( qsCommand );
+    if ( ! fileInfo.exists ( ) )  {
+      qsCommand = theUtils.checkForExe ( "qrender" );
+      if ( qsCommand.isEmpty ( ) )
+           qsCommand = "qrender";
+    }
+  }
+
+  qsCommand += " -v 5";
+  int iRet = system ( qsCommand.ascii ( ) );
+  iRet = iRet; // quiet down the compiler.
+  debug_out ( "ret<%d> = <%s>\n", iRet, qsCommand.ascii ( ) );
+}
+
 bool Manager::ServerInfo::hasClient ( Client *pClient )
 {
   QValueList<Client *>::iterator it = m_listOfClients.begin ( );
@@ -117,20 +162,60 @@ bool Manager::ServerInfo::hasClient ( Client *pClient )
 
 Client *Manager::ServerInfo::addClient ( SourceFileEntry *pEntry )
 {
+  if ( ! pEntry )
+    return NULL;
+
   Client *pClient = new Client ( pEntry  );
   m_listOfClients.append       ( pClient );
+
   return pClient;
 }
 
 bool Manager::ServerInfo::removeClient ( SourceFileEntry *pEntry )
 {
+  if ( ! pEntry )
+    return false;
+
   Client *pClient = NULL;
   QValueList<Client *>::iterator it = m_listOfClients.begin ( );
   while ( it != m_listOfClients.end ( ) )  {
     pClient = *it++;
     if ( pClient->sourceFileEntry ( ) == pEntry )  {
-      m_listOfClients.remove    ( pClient );
-      pClient->killServerThread ( );
+      m_listOfClients.remove      ( pClient );
+      pClient->killServerThread   ( );
+
+      delete pClient;
+      startNextClient ( );
+      return true;
+    }
+  }
+  return false;
+}
+
+Client *Manager::ServerInfo::addClient ( ButtonTransition *pTrans )
+{
+  if ( ! pTrans )
+    return NULL;
+
+  Client *pClient = new Client ( pTrans  );
+  m_listOfClients.append       ( pClient );
+
+  return pClient;
+}
+
+bool Manager::ServerInfo::removeClient ( ButtonTransition *pTrans )
+{
+  if ( ! pTrans )
+    return false;
+
+  Client *pClient = NULL;
+  QValueList<Client *>::iterator it = m_listOfClients.begin ( );
+  while ( it != m_listOfClients.end ( ) )  {
+    pClient = *it++;
+    if ( *pClient->buttonTransition ( ) == *pTrans )  {
+      m_listOfClients.remove        ( pClient );
+      pClient->killServerThread     ( );
+
       delete pClient;
       startNextClient ( );
       return true;
@@ -151,6 +236,7 @@ bool Manager::ServerInfo::startNextClient ( )
   QValueList<Client *>::iterator it = m_listOfClients.begin ( );
   // In the first run we count how many are currently running.
   unsigned int iCount = 0;
+
   while ( it != m_listOfClients.end ( ) )  {
     pClient = *it++;
     if ( ! pClient->waitingInQueue ( ) )
@@ -163,13 +249,18 @@ bool Manager::ServerInfo::startNextClient ( )
       break;
     pClient = *it++;
 
-    debug_out ( "%s : %d > Starting Client for <%s>\n", __FUNCTION__, __LINE__, pClient->sourceFileEntry()->qsDisplayName.ascii() );
+    if ( pClient->sourceFileEntry ( ) )  {
+      debug_out ( "Starting Client for <%s> waiting<%s>\n", pClient->sourceFileEntry()->qsDisplayName.ascii(), pClient->waitingInQueue ( ) ? "true" : "false");
+    }
+    else  {
+      debug_out ( "Starting Client for <%s> waiting<%s>\n", pClient->buttonTransition()->qsName.ascii(), pClient->waitingInQueue ( ) ? "true" : "false");
+    }
 
     if ( pClient->waitingInQueue ( ) )  {
-      pClient->setWaiting  ( false );
-      pClient->start ( );
-      iCount ++;
-      bReturn = true;
+         pClient->setWaiting ( false );
+         pClient->startMe    ( );
+         iCount ++;
+         bReturn = true;
     }
   }
   return bReturn;
@@ -177,7 +268,6 @@ bool Manager::ServerInfo::startNextClient ( )
 
 Manager::Manager ( )
 {
-
 }
 
 Manager::~Manager ( )
@@ -239,6 +329,7 @@ QStringList &Manager::getFilterNames ( )
   qsCommand += QString ( " -f >%1" ).arg   ( qsTempFile );
   int iRet   = system  ( qsCommand.ascii ( ) );
   iRet = iRet; // quiet down the compiler.
+  chmod ( qsTempFile.utf8 ( ), S_IRUSR | S_IWUSR | S_IWRITE | S_IROTH | S_IWOTH | S_IWGRP | S_IRGRP );
 
   // Next we read in the temp file
   QFileInfo fileInfo ( qsTempFile );
@@ -258,7 +349,7 @@ QStringList &Manager::getFilterNames ( )
   if ( m_listFilterNames.size ( ) < 1 )  {
     // Hardcode at least the Cross Fade and KenBurns filter.
     m_listFilterNames.append ( "Cross Fade" );
-    m_listFilterNames.append ( "Ken Burns"  );
+//    m_listFilterNames.append ( "Ken Burns"  );
   }
   return m_listFilterNames;
 }
@@ -277,6 +368,7 @@ bool Manager::registerEntry ( SourceFileEntry *pEntry )
        m_pSelf = new Manager;
 
 /* For as long as I develop this Client, we can disregard this check
+   This check will make sure a SourceFileEntry ( Slideshow ) is only rendered once.
   QValueList<Client *>::iterator it = m_pSelf->m_listOfClients.begin ( );
   while ( it != m_pSelf->m_listOfClients.end ( ) )  {
     if  ( pEntry == (*it++)->sourceFileEntry ( ) )
@@ -295,7 +387,8 @@ bool Manager::registerEntry ( SourceFileEntry *pEntry )
 //  pNewClient->start ( );
   pServerInfo->addClient ( pEntry );
   pServerInfo->startNextClient  ( );
-  debug_out ( "Manager::registerEntry - and start \n" );
+
+  sdebug_out ( " - and start <%s>\n", pEntry->qsDisplayName.ascii ( ) );
   return true;
 }
 
@@ -329,6 +422,61 @@ bool Manager::unregisterEntry ( SourceFileEntry *pEntry )
   return bReturn;
 }
 
+bool Manager::registerTrans ( ButtonTransition *pTrans )
+{
+  if ( ! pTrans || pTrans->bFinishedRendering )
+    return false;
+
+  // Check if we need to create a new object ...
+  if ( m_pSelf == NULL )
+       m_pSelf = new Manager;
+
+//  usleep ( 10000 ); // wait 10 ms to avoid double clicks to regiseter multiple instances
+  unsigned long iServerIP = 0x0F000001;  // 127.0.0.1
+  ServerInfo *pServerInfo = m_pSelf->getServerInfo ( iServerIP );
+  if ( ! pServerInfo )  {
+         pServerInfo =   new ServerInfo ( iServerIP, SERVER_PORT, Global::iMaxRunningThreads );
+         m_pSelf->m_listOfServer.append ( pServerInfo );
+  }
+
+  pServerInfo->addClient ( pTrans );
+  pServerInfo->startNextClient  ( );
+
+  sdebug_out ( " - and start <%s>\n", pTrans->qsName.ascii ( ) );
+
+  return true;
+}
+
+bool Manager::unregisterTrans ( ButtonTransition *pTrans )
+{
+  bool bReturn = false;
+  if ( ! pTrans )
+    return false;
+
+  // Check if we need to create a new object ...
+  if ( m_pSelf == NULL )
+       m_pSelf = new Manager;
+
+  m_pSelf->lock ( );
+
+  ServerInfo *pServerInfo = NULL;
+  QValueList<ServerInfo *>::iterator it = m_pSelf->m_listOfServer.begin ( );
+  while ( it != m_pSelf->m_listOfServer.end ( ) )  {
+    pServerInfo = *it++;
+    if ( pServerInfo->removeClient  ( pTrans ) )  {
+      if ( pServerInfo->clientCount ( )  < 1 )  {
+        m_pSelf->m_listOfServer.remove ( pServerInfo );
+        delete pServerInfo;
+      }
+      bReturn = true;
+      break;
+    }
+  }
+
+  m_pSelf->unlock ( );
+  return bReturn;
+}
+
 void Manager::lock ( )
 {
   m_mutex.lock ( );
@@ -347,16 +495,31 @@ void Manager::unlock ( )
 Client::Client ( SourceFileEntry *pEntry )
   : QObject ( ), QThread ( )
 {
+  initMe ( );
   m_pSourceFileEntry = pEntry;
-  m_bRemoteServer    = true;
-  m_bSendImageFiles  = true;
-  m_bWaitInQueue     = true;
-  m_fProgress        = 0.0f;
-  m_pFile            = NULL;
-  m_iMessageID       = 0;
-  m_iMessageLen      = 0;
-  m_iRetryCounter    = 3;  // Try three times to re-establish a lost connection
-  m_iKeepaliveTimer  = 0;
+}
+
+Client::Client ( ButtonTransition *pTrans )
+  : QObject ( ), QThread ( )
+{
+  initMe ( );
+  m_pButtonTransition = pTrans;
+}
+
+void Client::initMe ( )
+{
+  m_pSourceFileEntry  = NULL;
+  m_pButtonTransition = NULL;
+  m_bRemoteServer     = true;
+  m_bSendImageFiles   = true;
+  m_bWaitInQueue      = true;
+  m_bSentAllInfo      = false;
+  m_fProgress         = 0.0f;
+  m_pFile             = NULL;
+  m_iMessageID        = 0;
+  m_iMessageLen       = 0;
+  m_iRetryCounter     = 3;  // Try three times to re-establish a lost connection
+  m_iKeepaliveTimer   = 0;
   m_pSocket = new QSocket ( this );
   connect ( m_pSocket, SIGNAL ( connected        ( ) ), SLOT ( socketConnected ( ) ) );
   connect ( m_pSocket, SIGNAL ( readyRead        ( ) ), SLOT ( socketReadyRead ( ) ) );
@@ -369,7 +532,8 @@ Client::~Client ( )
   enableTimer ( false );
   debug_out ( "Client DESTRUCTOR\n" );
   if ( m_pSocket )   {
-    m_pSocket->close ( );
+    if ( ( m_pSocket->state ( ) != QSocket::Closing ) && ( m_pSocket->state ( ) != QSocket::Idle ) )
+           m_pSocket->close ( );
     delete m_pSocket;
     m_pSocket = NULL;
   }
@@ -389,12 +553,19 @@ void Client::enableTimer ( bool bEnable )
 
 void Client::slotReconnect ( )
 {
+  Manager::ServerInfo *pServerInfo = Manager::m_pSelf->getServerInfo ( this );
+  if ( pServerInfo )
+       pServerInfo->setStarting ( false );
+
   if ( m_pSocket )
        m_pSocket->connectToHost ( "localhost", 5066 );
+
+  debug_out ( "\n" );
 }
 
 void Client::closeConnection ( )
 {
+  debug_out ( "\n" );
   m_pSocket->close ( );
   if ( m_pSocket->state ( ) == QSocket::Closing ) // We have a delayed close.
       connect ( m_pSocket, SIGNAL ( delayedCloseFinished ( ) ), SLOT ( socketClosed ( ) ) );
@@ -420,7 +591,7 @@ void Client::socketReadyRead ( )
       if ( iRead1 == sizeof ( Q_UINT16 ) )
         m_iMessageID  = *(Q_UINT16 *)cMsgID;
     }
-    debug_out ( "Client::socketReadyRead prev<0x%04X> curr<0x%04X>\n", iPrev, m_iMessageID );
+    debug_out ( " prev<0x%04X> curr<0x%04X>\n", iPrev, m_iMessageID );
 
     switch ( m_iMessageID )  {
       case SERVER_YOUR_PROGRESS_SIR:
@@ -445,7 +616,7 @@ void Client::socketReadyRead ( )
         fileFromSocket ( );
         return;
       default:
-        debug_out ( "Client::socketReadyRead Unknown MsgID : prevMsgID<0x%04X> currMsgID<0x%04X>\n", iPrev, m_iMessageID );
+        debug_out ( " Unknown MsgID : prevMsgID<0x%04X> currMsgID<0x%04X>\n", iPrev, m_iMessageID );
       break; // don't do nothin'
     }        // end switch statement
     m_iMessageID  = 0;
@@ -495,7 +666,8 @@ printf ( "bRemoteServer<%d> IP=<0x%016X> = <%s>\n", m_bRemoteServer, p->getIP (
 */
 
   if ( pEvent->timerId ( ) == m_iKeepaliveTimer )  {
-    debug_out ( "Render::Client::timerEvent last<%s> Curr<%s>\n", m_lastPing.toString ( ).ascii ( ), QDateTime::currentDateTime ( ).toString ( ).ascii ( ) );
+    debug_out ( " last<%s> Curr<%s>\n", m_lastPing.toString ( ).ascii ( ), QDateTime::currentDateTime ( ).toString ( ).ascii ( ) );
+
     if ( m_lastPing.secsTo ( QDateTime::currentDateTime ( ) ) > WENT_MISSING_AFTER * 2 ) { //I_AM_ALIVE_INTERVAL )  {
       // Oh crap happened ... should I try to restart ?
       // Should I notify the user
@@ -534,8 +706,8 @@ printf ( "bRemoteServer<%d> IP=<0x%016X> = <%s>\n", m_bRemoteServer, p->getIP (
         m_pSocket->flush ( );
       }
     }
-//    else
-//      m_lastPing = QDateTime::currentDateTime ( );
+    else
+      m_lastPing = QDateTime::currentDateTime ( );
   }
 }
 
@@ -573,23 +745,35 @@ void Client::receivedVersionAndHost ( )
   // On a local host the wakeAll can come in before the wait is entered, thus we give it a shot break here
   usleep     ( 10000 );  // 10 msec
   m_waiter.wakeAll ( );
-
-//printf ( "Client::receivedVersionAndHost  Host<%s> Version<%s> Len<%ld>\n", qsHostName.ascii ( ), qsVersion.ascii ( ), m_iMessageLen );
 }
 
 void Client::receivedServerState ( )
 {
   QString qsStatus;
   char cMsgLen [ sizeof ( Q_UINT64 ) ];
-  //Q_LONG iRead2 = m_pSocket->readBlock ( cMsgLen, sizeof ( Q_UINT64 ) );
-  m_pSocket->readBlock ( cMsgLen, sizeof ( Q_UINT64 ) );
-  m_iMessageLen = *(Q_UINT64 *)cMsgLen;
 
-  QDataStream ds ( m_pSocket );
-  ds >> qsStatus;
+  m_iMessageLen  = 0L;
+  Q_ULONG iAvail = 0L;
+  Q_ULONG iRead  = 0L;
+  if ( m_pSocket->state ( ) == QSocket::Connected )  {
+    iAvail  = m_pSocket->bytesAvailable ( );
+    if ( iAvail > 0 )  {
+      iRead = m_pSocket->readBlock ( cMsgLen, sizeof ( Q_UINT64 ) );
+      if ( iRead > 0 )  {
+        m_iMessageLen = *(Q_UINT64 *)cMsgLen;
 
-  if ( qsStatus == "SameXML" ) 
-    m_bSendImageFiles = false;
+        QDataStream ds ( m_pSocket );
+        ds >> qsStatus;
+
+        if ( qsStatus == "SameXML" ) 
+          m_bSendImageFiles = false;
+      }
+      else  {
+        QSocketDevice::Error err = m_pSocket->socketDevice ( )->error ( );
+        printf ( "%s::%d > There was an Socket Error <%d>\n", __FILE__, __LINE__, err );
+      }
+    }
+  }
 
   usleep ( 100000 ); // 10 ms break ...
   qApp->processEvents ( );
@@ -602,10 +786,28 @@ void Client::receivedServerDone ( )
 
   QString qsStatus;
   char cMsgLen [ sizeof ( Q_UINT64 ) ];
+  memset ( &cMsgLen, 0, sizeof ( Q_UINT64 ) );
 
-  m_pSocket->readBlock ( cMsgLen, sizeof ( Q_UINT64 ) );
-  QDataStream ds ( m_pSocket );
-  ds >> qsStatus;
+  Q_ULONG iAvail = 0L;
+  Q_ULONG iRead  = 0L;
+  if ( m_pSocket->state ( ) == QSocket::Connected )  {
+    iAvail  = m_pSocket->bytesAvailable ( );
+    if ( iAvail > 0 )  {
+      iRead = m_pSocket->readBlock ( cMsgLen, sizeof ( Q_UINT64 ) );
+      if ( iRead > 0 )  {
+        Q_UINT64 iLen = *(Q_UINT64 *)cMsgLen;
+
+        if ( iLen < 10 )  { // Should be a length of 4 ( "Done" )
+          QDataStream ds ( m_pSocket );
+          ds >> qsStatus;
+        }
+        else  {
+          QSocketDevice::Error err = m_pSocket->socketDevice ( )->error ( );
+          printf ( "%s::%d > There was an Socket Error <%d>\n", __FILE__, __LINE__, err );
+        }
+      }
+    }
+  }
 
   QCustomEvent *pNewEvent = new QCustomEvent ( EVENT_RENDER_EXIT );
   pNewEvent->setData      ( (void *) this );
@@ -650,25 +852,22 @@ void Client::sendNextPacket ( )
   char data[BLOCK_LEN];
   if ( ! m_pFile )
     return;
-printf ( "%s : %d > \n", __FUNCTION__, __LINE__ );
+
   int  iLen  = m_pFile->readBlock ( data, BLOCK_LEN );
   if ( iLen != -1 )  {
     m_pSocket->writeBlock ( data, iLen );
     Q_ULONG iPos  = m_pFile->at   ( );
     Q_ULONG iSize = m_pFile->size ( );
 
-//    float fPercentage =  ( iPos * 100.0 / iSize );
     if ( iPos >= iSize ) {
       m_pFile->close   ( );
       m_pSocket->flush ( );
       delete m_pFile;
       m_pFile = NULL;
       usleep ( 100 );
-//printf ( "Client::sendNextPacket WAKE ALL\n" );
       m_waiter.wakeAll ( );
     }
 //    emit sendingNonAckPercent ( fPercentage );
-//printf ( "Client::sendNextPacket thr<%X> <%f%%>\n", (unsigned)pthread_self ( ), fPercentage );
   }
 }
 
@@ -682,13 +881,47 @@ bool Client::sendVersionAndHostRequest ( )
   stream << (Q_UINT16)CLIENT_WHO_AND_WHERE_ARE_YOU << (Q_UINT64)qsHostName.length ( );
   stream << qsHostName;
   m_pSocket->flush ( );
-  debug_out ( "Client::sendVersionAndHostRequest <0x%04X>\n", CLIENT_WHO_AND_WHERE_ARE_YOU );
+  debug_out ( " <0x%04X>\n", CLIENT_WHO_AND_WHERE_ARE_YOU );
   return m_waiter.wait ( 30000 ); // wait for 30 seconds max
 }
 
+void Client::startMe ( )
+{
+  m_iRetryCounter = 3;
+  QTimer::singleShot ( 10, this, SLOT ( slotConnectAndStart ( ) ) );
+}
+
+void Client::slotConnectAndStart ( )
+{
+  if ( m_pSocket->state ( ) == QSocket::Connected )  {
+    usleep ( 10000 ); // in case we are not yet out of run()
+    start  ( ); // -=> Execue run() in new thread
+    return;
+  }
+  else if ( m_pSocket->state ( ) == QSocket::Idle )  {
+    disconnect ( m_pSocket, SLOT   ( socketConnected ( ) ) );
+    connect    ( m_pSocket, SIGNAL ( connected ( ) ), SLOT ( socketConnectAndStart ( ) ) );
+    startServer( );
+    usleep  ( 10000 );  // wait 10 milli second for the connection to happen.
+    return;
+  }
+  if ( m_iRetryCounter-- > 0 )
+    QTimer::singleShot ( 1000, this, SLOT ( slotConnectAndStart ( ) ) );
+  else  {
+    debug_out ( "Giving up trying to connect to server.\n" );
+  }
+}
+
+void Client::socketConnectAndStart ( )
+{
+  disconnect ( m_pSocket, SLOT   ( socketConnectAndStart ( ) ) );
+  connect    ( m_pSocket, SIGNAL ( connected ( ) ), SLOT ( socketConnected ( ) ) );
+  start ( );
+}
+
 void Client::socketConnected ( )
 {
-  debug_out ( "Client::socketConnected \n" );
+  debug_out ( "\n" );
   m_waiter.wakeAll ( );
 }
 
@@ -701,38 +934,92 @@ void Client::socketConnectionClosed ( )
 
 void Client::socketClosed ( )
 {
-  debug_out ( "Client::socketClosed : Connection closed\n" );
-
+  debug_out ( "Connection closed\n" );
 }
 
 void Client::socketError ( int e )
 {
-  debug_out ( "Client::socketError ErroNumber<%d>\n", e );
+  debug_out ( "ErroNumber<%d>\n", e );
   if ( m_iRetryCounter > 0 )  {
     m_iRetryCounter --;
-    debug_out ( "Client::socketError Retry<%d>\n", m_iRetryCounter );
+    // Here is the reconnection protocol
+    // 1) Try to re-establish connection with server
+    //    - If server is dead, then ( 3 times )
+    //      o check if we have already sent all info.
+    //      o try to restart server
+    //      o try to reconnect
+    //    - If connection is re-established, then
+    //      o send msg RECONNECT ( worker id or worker name )
+    //      o If server finds info, then send current progress
+    //      o If server cant find info, then delete this Client and re-register
+//    if ( ! m_bSentAllInfo )  {
+      Client::reRequest ( this );
+      return;
+//    }
+    debug_out ( "Retry<%d>\n", m_iRetryCounter );
+
     startServer ( );
+    m_iRetryCounter = 5;
+    QTimer::singleShot ( 1000, this, SLOT ( slotRecovered ( ) ) );
+  }
+}
+
+void Client::slotRecovered ( )
+{
+  // Called afer socket error.
+  if ( m_pSocket && m_pSocket->state ( ) == QSocket::Connected )  {
+    QString qsName;
+    if ( sourceFileEntry ( ) )
+      qsName = sourceFileEntry  ( )->qsDisplayName;
+    else if (  buttonTransition ( ) )
+      qsName = buttonTransition ( )->qsName;
+
+    debug_out ( "Reconnecting for <%s>[%u]\n", qsName.ascii ( ), (Q_UINT16)qsName.length ( ) );
+    QDataStream stream ( m_pSocket );
+    stream << (Q_UINT16)CLIENT_RECONNECTING << (Q_UINT16)qsName.length ( );
+    stream << qsName;
+    m_pSocket->flush ( );
+    m_waiter.wait ( 30000 ); // wait for 30 seconds max
+  }
+  else if ( m_iRetryCounter > 0 )  {
+    m_iRetryCounter --;
+    debug_out ( "Trying again <%d> left\n",  m_iRetryCounter );
+    QTimer::singleShot  ( 1000, this, SLOT ( slotRecovered ( ) ) );
+  }
+  else  {
+    // Can not establish connection, try the whole rendering again
+    Client::reRequest ( this );
+  }
+}
+
+void Client::reRequest ( Client *pSelf )
+{
+  if ( pSelf->sourceFileEntry(  )  )  {
+    sdebug_out ( "\n" );
+    SourceFileEntry *pEntry = pSelf->sourceFileEntry ( );
+    Manager::unregisterEntry ( pEntry );
+    Manager::registerEntry   ( pEntry );
+  }
+  else if ( pSelf->buttonTransition ( ) )  {
+    sdebug_out ( "\n" );
+    ButtonTransition *pTransition = pSelf->buttonTransition ( );
+    Manager::unregisterTrans ( pTransition );
+    Manager::registerTrans   ( pTransition );
   }
 }
 
 void Client::startServer ( )
 {
-  Utils theUtils;
-  QString qsCommand = theUtils.getToolPath ( "qrender" );
-  if ( qsCommand.isEmpty ( ) )  {
-    qsCommand = Global::qsSystemPath + "/bin/qrender";
-    QFileInfo fileInfo ( qsCommand );
-    if ( ! fileInfo.exists ( ) )  {
-      qsCommand = theUtils.checkForExe ( "qrender" );
-      if ( qsCommand.isEmpty ( ) )
-           qsCommand = "qrender";
-    }
-  }
-  qsCommand += " -v 5";
-  int iRet = system ( qsCommand.ascii ( ) );
-  iRet = iRet; // quiet down the compiler.
-  // Wait a second and then try to re-connect
-  QTimer::singleShot ( 1000, this, SLOT ( slotReconnect ( ) ) );
+  Manager::ServerInfo *pServerInfo = Manager::m_pSelf->getServerInfo ( this );
+  if ( pServerInfo )
+       pServerInfo->startServer ( );
+
+//  QTimer::singleShot ( 1000, this, SLOT ( slotReconnect ( ) ) );
+  // We must be blocking here otherwise the timer would be fired way later
+  // if the main loop is busy E.g. creating meus from a plugin
+  usleep ( 100000 );
+  slotReconnect  ( );
+  debug_out ( "start server\n" );
 }
 
 void Client::restartServer ( )
@@ -742,6 +1029,7 @@ void Client::restartServer ( )
     slotReconnect ( );
     if ( ( m_pSocket->state ( ) == QSocket::Connected  ) ||
          ( m_pSocket->state ( ) == QSocket::Connecting ) ) {
+       debug_out ( "Reconnected\n" );
        return;
     }
     usleep ( 1000000 );  // Now wait a second ...
@@ -750,11 +1038,31 @@ void Client::restartServer ( )
   usleep ( 10000 );  // give it 10 milli seconds
   if ( running ( ) )
     terminate  ( );
+
   start  ( );
+  debug_out ( "restart\n" );
 }
 
 void Client::run ( )
 {
+  debug_out ( "\n" );
+  m_bSentAllInfo = false;
+
+  if ( m_pSocket->state ( ) == QSocket::Idle )  {
+    startMe ( );
+    return;
+//    startServer  ( );
+//    usleep ( 10000 );  // wait 10 milli second for the connection to happen.
+//    for ( int t=0; t<5; t++ )  {
+//      if ( m_pSocket->state ( ) != QSocket::Connected )
+//        usleep ( 1000000 );  // Now wait a second ...
+//    }
+  }
+  if ( m_pSocket->state ( ) != QSocket::Connected )  {
+    QString qsName = m_pSourceFileEntry ? m_pSourceFileEntry->qsDisplayName : m_pButtonTransition->qsName;
+    debug_out ( "Giving up connecting to the server for %s.\n", qsName.ascii ( ) );
+    return;
+  }
   // Executed in background thread and active as long as the rendering process is active for 
   // this SourceFileEntry.
   // Here is what we have to do;
@@ -768,29 +1076,35 @@ void Client::run ( )
   //    - Note: if on same server, then we can omit this step
   // 4) receive progress notification
   // 5) Finish up.
-  if ( ! m_pSourceFileEntry || ( m_pSourceFileEntry->listFileInfos.count ( ) < 1 ) )
-    return;
+  if ( m_pSourceFileEntry )
+    runEntry ( );
+  else if ( m_pButtonTransition )
+    runTrans ( );
 
-  Utils theUtils;
-  CXmlSlideshow *pSlideshow = m_pSourceFileEntry->listFileInfos[0]->pSlideshow;
+  m_bSentAllInfo = true;
+}
+
+void Client::runSlideshow ( CXmlSlideshow *pSlideshow )
+{
   if ( ! pSlideshow )
     return;
 
- slotReconnect ( );
- if ( m_pSocket->state ( ) == QSocket::Idle )  {
-      startServer  ( );
-      usleep ( 10000 );  // give it 10 milli seconds
-  }
+  Utils theUtils;
   // Now that the connection is/should be established, we can start the Keepalive - timer.
-  enableTimer   ( true );
+  enableTimer ( true );
 
-  debug_out ( "Client::run \n" );
+  if ( m_pSourceFileEntry )  {
+    debug_out ( "Run for entry<%s> \n", m_pSourceFileEntry->qsDisplayName.ascii ( ) );
+  }
+  else if ( m_pButtonTransition )  {
+    debug_out ( "Run for entry<%s> \n", m_pButtonTransition->qsName.ascii ( ) );
+  }
 
   int iCounter = 0;
   while ( iCounter++ <  10 )  {
     if  ( m_pSocket->state ( ) != QSocket::Connected )  {
       // wait for the connection event to kick in.
-      debug_out ( "Client::run before WAIT <%d> \n", iCounter );
+      debug_out ( "before WAIT <%d> \n", iCounter );
       m_waiter.wait  ( 1000 ); // in mSec or ULONG_MAX
       usleep ( 100000 );  // Socket just connected. Give it 10 msec.
     }
@@ -798,14 +1112,18 @@ void Client::run ( )
 
   if ( ! sendVersionAndHostRequest ( ) )  {
     // Timed out, Assume local host.
-    debug_out ( "Client::run After 30 seconds : did not receive ServerVersionAndHost response\n" );
+    debug_out ( "After 30 seconds : did not receive ServerVersionAndHost response\n" );
     m_bRemoteServer = false;
     m_waiter.wakeAll ( );
   }
 
   QString qsFileName = pSlideshow->slideshow_name;
   qsFileName.replace ( "/", "_" );
-  qsFileName = theUtils.getTempFile ( qsFileName + ".xml" );
+  if ( m_pButtonTransition )
+    qsFileName = theUtils.getTempFile ( "" ) + m_pButtonTransition->qsName + "/" + qsFileName + ".xml";
+  else
+    qsFileName = theUtils.getTempFile ( qsFileName + ".xml" );
+
   pSlideshow->writeXml ( qsFileName );
 
   QDataStream stream   (  m_pSocket );
@@ -838,6 +1156,8 @@ void Client::run ( )
   }
   else  {
     QString qsTempPath = theUtils.getTempFile ( QString ( ) );
+    if ( m_pButtonTransition )
+      qsTempPath += m_pButtonTransition->qsName + "/";
     stream << (Q_UINT16)CLIENT_TAKE_THAT_XML << (Q_UINT64)( qsFileName.length ( ) + qsTempPath.length ( ) );
     stream << qsFileName << qsTempPath;
     m_pSocket->flush ( );
@@ -845,11 +1165,33 @@ void Client::run ( )
   }
   stream << (Q_UINT16)CLIENT_GO_TO_WORK << (Q_UINT64)0LL;
 
-  debug_out ( "Client::run Get the server to work ... msgID<0x%04X>\n",  CLIENT_GO_TO_WORK );
+  debug_out ( "Get the server to work ... msgID<0x%04X>\n",  CLIENT_GO_TO_WORK );
   m_pSocket->flush ( );
   setWaiting ( false );
 }
 
+void Client::runEntry ( )
+{
+  if ( ! m_pSourceFileEntry || ( m_pSourceFileEntry->listFileInfos.count ( ) < 1 ) )
+    return;
+
+  CXmlSlideshow *pSlideshow = m_pSourceFileEntry->listFileInfos[0]->pSlideshow;
+
+  runSlideshow ( pSlideshow );
+}
+
+void Client::runTrans ( )
+{
+  if ( ! m_pButtonTransition )
+    return;
+
+  CXmlSlideshow *pSlideshow = m_pButtonTransition->createSlideshow ( );
+  runSlideshow ( pSlideshow );
+
+  // Finally we can destroy the temp slideshow object. Thanks for existing.
+  delete pSlideshow;
+}
+
 void Client::lock ( )
 {
   m_mutex.lock ( );
@@ -865,6 +1207,11 @@ SourceFileEntry *Client::sourceFileEntry ( )
   return m_pSourceFileEntry;
 }
 
+ButtonTransition *Client::buttonTransition ( )
+{
+  return m_pButtonTransition;
+}
+
 float Client::progress ( )
 {
   float fReturn;
@@ -907,7 +1254,7 @@ Q_UINT64 Client::fileFromSocket ( )
     QDataStream ds ( m_pSocket );
     ds >> qsFileName;
 
-    debug_out ( "Render::fileFromSocket <%s>\n", (const char *)qsFileName.ascii ( ) );
+    debug_out ( "<%s>\n", (const char *)qsFileName.ascii ( ) );
     // Store under e.g. /tmp/qrender/MySlide/MySlide.vob
     QFileInfo fileInfo ( qsFileName );
     QString qsExt, qsNewFileName;
