@@ -30,7 +30,9 @@ class QFile;
 // Throttling transfere over socket for better debugging (delay in usec)
 #define THROTTLE 0
 
+class CXmlSlideshow;
 class SourceFileEntry;
+class ButtonTransition;
 
 namespace Render
 {
@@ -51,23 +53,30 @@ class Client : public QObject, public QThread
   };
 
 public:
-  Client ( SourceFileEntry * );
+  Client ( SourceFileEntry  * );
+  Client ( ButtonTransition * );
   virtual  ~Client ( );
 
   virtual void run ( );
   void lock   ( );
   void unlock ( );
+  void initMe ( );
+  void startMe( );
 
   float progress         ( );
   bool  waitingInQueue   ( );
   void  killServerThread ( );
   void  setWaiting  ( bool );
-  SourceFileEntry *sourceFileEntry ( );
+  SourceFileEntry  *sourceFileEntry  ( );
+  ButtonTransition *buttonTransition ( );
 
 protected:
-  virtual void timerEvent ( QTimerEvent * );
+  virtual void timerEvent ( QTimerEvent   * );
+  void runSlideshow       ( CXmlSlideshow * );
   bool sendFile ( QString );
   void enableTimer ( bool );
+  void runEntry         ( );
+  void runTrans         ( );
   void startServer      ( );
   void restartServer    ( );
   void receivedPing     ( );
@@ -80,6 +89,7 @@ protected:
   void receivedServerState       ( );
   void receivedVersionAndHost    ( );
   bool sendVersionAndHostRequest ( );
+  static void reRequest ( Client * );
 
 private slots:
   virtual void closeConnection ( );
@@ -87,25 +97,30 @@ private slots:
   virtual void socketConnected ( );
   virtual void socketClosed    ( );
   virtual void slotReconnect   ( );
+  virtual void slotRecovered   ( );
   virtual void socketError ( int );
   virtual void socketConnectionClosed ( );
+  virtual void slotConnectAndStart    ( );
+  virtual void socketConnectAndStart  ( );
 
 private:
   // Something for the server connection
-  QFile           *m_pFile;
-  QSocket         *m_pSocket;
-  SourceFileEntry *m_pSourceFileEntry;
-  QWaitCondition   m_waiter;
-  QMutex           m_mutex;
-  float            m_fProgress;
-  Q_ULONG          m_iMessageLen;     // to keep track of the incoming slideshow video file
-  Q_UINT16         m_iMessageID;      // The current MessageID being processed
-  bool             m_bRemoteServer;   // The server is on a different computer
-  bool             m_bSendImageFiles; // The server needs the image files ( def. true )
-  bool             m_bWaitInQueue;    // If the server has currently more then the set threads running
-  int              m_iKeepaliveTimer; // Checks if the server has a heartbeat
-  QDateTime        m_lastPing;        // Last received ping
-  int              m_iRetryCounter;   // The numer of times to try to re-establish the connection
+  QFile            *m_pFile;
+  QSocket          *m_pSocket;
+  SourceFileEntry  *m_pSourceFileEntry;
+  ButtonTransition *m_pButtonTransition;
+  QWaitCondition    m_waiter;
+  QMutex            m_mutex;
+  float             m_fProgress;
+  Q_ULONG           m_iMessageLen;     // to keep track of the incoming slideshow video file
+  Q_UINT16          m_iMessageID;      // The current MessageID being processed
+  bool              m_bRemoteServer;   // The server is on a different computer
+  bool              m_bSendImageFiles; // The server needs the image files ( def. true )
+  bool              m_bWaitInQueue;    // If the server has currently more then the set threads running
+  bool              m_bSentAllInfo;    // has the function run() completed ?
+  int               m_iKeepaliveTimer; // Checks if the server has a heartbeat
+  QDateTime         m_lastPing;        // Last received ping
+  int               m_iRetryCounter;   // The numer of times to try to re-establish the connection
 };
 
 // Singleton class Manager
@@ -120,12 +135,17 @@ public:
 
     unsigned long  getIP ( );
     unsigned int getPort ( );
-    Client    *addClient ( SourceFileEntry * );
-    bool    removeClient ( SourceFileEntry * );
+    Client    *addClient ( SourceFileEntry  * );
+    bool    removeClient ( SourceFileEntry  * );
+    Client    *addClient ( ButtonTransition * );
+    bool    removeClient ( ButtonTransition * );
     bool startNextClient ( );
+    bool  startingServer ( );
     int      clientCount ( );
+    void     startServer ( );
     bool      inRecovery ( ); // connection was lost and now trying to re-eastablish
     void     setRecovery ( bool );
+    void     setStarting ( bool );
     bool       hasClient ( Client * );
 
   private:
@@ -133,17 +153,22 @@ public:
     unsigned int         m_iServerPort;  // Defaults to Render::SERVER_PORT
     unsigned int         m_iConcurrentThreads;
     bool                 m_bRecoveryMode;
+    bool                 m_bStartingServer; // After issuing the bash command through system() we should prevent other threads
+
     QValueList<Client *> m_listOfClients;
   };
 
 public:
    Manager ( );
   ~Manager ( );
-  static bool registerEntry   ( SourceFileEntry * );
-  static bool unregisterEntry ( SourceFileEntry * );
+  static bool registerEntry   ( SourceFileEntry  * );
+  static bool unregisterEntry ( SourceFileEntry  * );
 
-  ServerInfo  *getServerInfo  ( unsigned long     );
-  ServerInfo  *getServerInfo  ( Render::Client  * );
+  static bool registerTrans   ( ButtonTransition * );
+  static bool unregisterTrans ( ButtonTransition * );
+
+  ServerInfo  *getServerInfo  ( unsigned long      );
+  ServerInfo  *getServerInfo  ( Render::Client   * );
   QStringList &getFilterNames ( );
 
   void lock   ( );
