@@ -182,7 +182,8 @@ ShadowObject::ShadowObject ( MenuObject *pMenuObject, QWidget *pParent )
 ShadowObject::~ShadowObject ( ) 
 {
   m_pMenuObject    = NULL;
-  m_pStructureItem = NULL;
+  resetStructureItem ( );
+//  m_pStructureItem = NULL;
 }
 
 ShadowObject &ShadowObject::operator = ( ShadowObject &theOther )
@@ -285,7 +286,6 @@ void ShadowObject::drawShadow ( QPainter *pPainter )
 
   theMatrix.reset ( );
   pPainter->setWorldMatrix ( theMatrix );
-
 }
 
 void ShadowObject::drawMirror ( QPainter *pPainter )
@@ -372,6 +372,11 @@ bool ShadowObject::mouseDoubleClickEvent ( QMouseEvent * )
 
 StructureItem *ShadowObject::createStructure ( StructureItem *pParentItem )
 {
+  if ( ! pParentItem )  {
+    m_pStructureItem = NULL;
+    return NULL;
+  }
+
   StructureItem *pShadowItem, *pTemp=NULL;
   enType theType = type ( );
   QString qsItem = tr   ( "Shadow Object" );
@@ -384,7 +389,7 @@ StructureItem *ShadowObject::createStructure ( StructureItem *pParentItem )
 
   pShadowItem = m_pStructureItem;
   if ( ! pShadowItem ) {
-    pShadowItem = new StructureItem ( this, pParentItem, pParentItem->lastChild ( ), qsName, qsInfo );
+    pShadowItem = new StructureItem ( this, StructureItem::Attributes, pParentItem, pParentItem->lastChild ( ), qsName, qsInfo );
     pShadowItem->setExpandable ( TRUE );
     m_pStructureItem = pShadowItem;
   }
@@ -477,15 +482,12 @@ void ShadowObject::updateShadow ( bool bRecreate )
   int         iFrameWidth = 0;
   QString     emptyAnimation;
   Modifiers   emptyModifier;
-  TextObject  tempText;
-  FrameObject tempFrame;
   MenuObject      *pMenuObject = NULL;
   SelectionObject *pSelection  = NULL;
   QColor      backgroundColor (   1,   1,   1 );
   QColor      foregroundColor ( 255, 255, 255 );
 
-  //printf ( "%s::%s::%d type<%s>\n", __FILE__, __FUNCTION__, __LINE__, m_pMenuObject->objectType ().ascii() );
-  if ( m_pMenuObject->objectType ( ) == tempText.objectType ( ) ) {
+  if ( m_pMenuObject->objectType ( ) == TEXT_OBJECT )  {
     pMenuObject = m_pMenuObject->clone ( );
     if ( ! pMenuObject )
       return;
@@ -500,7 +502,7 @@ void ShadowObject::updateShadow ( bool bRecreate )
       pText->setBackgroundColor ( backgroundColor );
     }
   }
-  else if ( m_pMenuObject->objectType  ( ) == tempFrame.objectType ( ) ) {
+  else if ( m_pMenuObject->objectType  ( ) == FRAME_OBJECT )  {  //tempFrame.objectType ( ) ) {
      pMenuObject = m_pMenuObject->clone ( );
     if ( ! pMenuObject )
       return;
@@ -515,31 +517,26 @@ void ShadowObject::updateShadow ( bool bRecreate )
     if ( m_enType != TYPE_MIRROR ) 
       pFrame->setFrameColor ( foregroundColor );
   }
-  else if ( m_pMenuObject->objectType ( ) == QString ( SELECTION_OBJECT ) ) {
+  else if ( m_pMenuObject->objectType ( ) == SELECTION_OBJECT ) {
     pSelection = (SelectionObject *)m_pMenuObject;
     pSelection->updateShadows ( this, bRecreate );
   }
   else {
     // ImageObject, MoveObject
     // Otherwise we simply want to draw a rect in the right color / size
-    if ( bRecreate &&  m_enType == TYPE_MIRROR ) {
+    if ( ( TYPE_MIRROR == m_enType && bRecreate ) ||
+         ( TYPE_SHADOW == m_enType ) )  {
       pMenuObject = m_pMenuObject->clone ( );
-      //printf ( "%s::%s::%d obj<%p>\n", __FILE__, __FUNCTION__, __LINE__, pMenuObject );
       if ( ! pMenuObject )
-	return;
+        return;
+
+      ImageObject *pImage = (ImageObject *)pMenuObject;
       pMenuObject->setShadow ( NULL ); 
       // Remember that the modifiers will be part of the Imagebject
       pMenuObject->setModifiers ( emptyModifier  );
       pMenuObject->setAnimation ( emptyAnimation );
-      ImageObject *pImage = (ImageObject *)pMenuObject;
       pImage->updatePixmap ( );
     }
-    else if ( m_enType == TYPE_SHADOW ) {
-      TextObject *pText = new TextObject;
-      pMenuObject = pText;
-      pText->setRect ( m_pMenuObject->rect ( ) );
-      pText->setBackgroundColor ( foregroundColor );
-    }
   }
 
   if ( ! pSelection ) { // selectio objects already called this fnc for all seelcted objects inside.
@@ -587,14 +584,14 @@ void ShadowObject::createMirror ( MenuObject *pMenuObject, int iFrameWidth, bool
 
     // Lastly we should filter out color keys if present (MovieObject, and Imagebject only)
     filterColorKeys ( theImage );
-    
+
     enMType mType = mirrorType ( );
     // Okay flip the image aongside the Y - Axis
     if ( ( mType == MIRROR_TOP ) || ( mType == MIRROR_BOTTOM ) )
       theImage = theImage.mirror ( false, true );
     else
       theImage = theImage.mirror ( true, false );
-    
+
     theImage.setAlphaBuffer ( true );
     cutOffImage ( theImage, mType  );
     m_imageBuffer = theImage.copy( );
@@ -616,12 +613,12 @@ void ShadowObject::filterColor ( QImage &theImage, QColor theColor )
   colorToFilter    = theColor.rgb ( ) | 0xff000000;
   for ( y=0;  y < theImage.height ( ); y++ ) {
     pLine = (QRgb *) theImage.scanLine ( y );
-	if ( ! pLine )
-		continue;
+    if ( ! pLine )
+      continue;
     for ( x=0; x < theImage.width ( ); x++ ) {
       currentPixel = *(pLine + x);
       if ( currentPixel == colorToFilter )
-	*(pLine + x) = colorTransparent;
+        *(pLine + x) = colorTransparent;
     }
   }
 }
@@ -637,8 +634,8 @@ void ShadowObject::applyTransparency ( )
 
   for ( y=0;  y < m_imageShadow.height ( ); y++ ) {
     pLine = (QRgb *) m_imageShadow.scanLine ( y );
-	if ( ! pLine )
-		continue;
+    if ( ! pLine )
+      continue;
     for ( x=0; x < m_imageShadow.width ( ); x++ ) {
       currentPixel = *(pLine + x);
       lAlpha   = (unsigned long ) ( ( currentPixel & 0xff000000 ) * ( 1.0 - m_fTransparency ) );
@@ -682,8 +679,8 @@ void ShadowObject::applyColorFading ( )
   // transparency.
   for ( y=0;  y < m_imageShadow.height ( ); y++ ) {
     pLine = (QRgb *) m_imageShadow.scanLine ( y );
-	if ( ! pLine )
-		continue;
+    if ( ! pLine )
+      continue;
     for ( x=0; x < m_imageShadow.width ( ); x++ ) {
       currentPixel = *(pLine + x);
       iRed   = lookup [0][ qRed   ( currentPixel ) ];
@@ -701,23 +698,41 @@ void ShadowObject::createShadow ( MenuObject *pMenuObject, int iFrameWidth )
     return;
   QColor backgroundColor (   1,   1,   1 );
   QRect r = m_pMenuObject->rect ( );
+  bool bImageOrText = true;
   // Now that we have the object to draw we should create the QImage - object ...
   QImage theImage;
   {
     int iOffset1 = (int) ( iFrameWidth/2.0 + m_iBlurRadius + 10 );
     int iOffset2 = (int) ( iFrameWidth + 2.0*m_iBlurRadius + 10 );
     QRect    theRect     ( iOffset1, iOffset1, r.width ( ), r.height ( ) );
-    QPixmap  thePix      ( r.width ( ) + iOffset2, r.height ( ) + iOffset2 );
-    QPainter thePainter  ( &thePix );
+    if ( ( pMenuObject->objectType ( ) == IMAGE_OBJECT ) ||
+         ( pMenuObject->objectType ( ) == MOVIE_OBJECT ) )  {
+      ImageObject *pImageObject = (ImageObject *)pMenuObject;
+      QImage img = pImageObject->image ( ).copy ( );
+      img = img.convertDepth ( 32 );
+      theImage = QImage ( img.width ( ) + iOffset1, img.height ( ) + iOffset2, 32 ); //theRect.size ( ), 32 );
+      theImage.setAlphaBuffer ( true );
+      QRgb  *pBits = (QRgb *)theImage.bits ( );
+      int    iSize = (int)( theImage.numBytes ( ) / 4.0 );
+      for ( int t=0; t<iSize; t++ )
+        pBits[t] = 0x00FFFFFF;
 
-    thePainter.fillRect  ( thePix.rect ( ), backgroundColor );
-    pMenuObject->setRect ( theRect );
-    pMenuObject->drawContents ( &thePainter );
-    // At this point we should have thePix as we want/need with a shadowee backdrop
-    theImage = thePix.convertToImage ( );
-    theImage.setAlphaBuffer ( true );
+      bitBlt ( &theImage, iOffset1, iOffset2, &img );
+    }
+    else  {  // Text, or Frame Objects
+      QPixmap  thePix      ( r.width ( ) + iOffset2, r.height ( ) + iOffset2 );
+      QPainter thePainter  ( &thePix );
+
+      thePainter.fillRect  ( thePix.rect ( ), backgroundColor );
+      pMenuObject->setRect ( theRect );
+      pMenuObject->drawContents ( &thePainter );
+      // At this point we should have thePix as we want/need with a shadowee backdrop
+      theImage = thePix.convertToImage ( );
+      theImage.setAlphaBuffer ( true );
+      bImageOrText = false;
+    }
   }
-  
+
   // Lastly we should filter out color keys if present (MovieObject, and Imagebject only)
   filterColorKeys ( theImage );
 
@@ -734,18 +749,33 @@ void ShadowObject::createShadow ( MenuObject *pMenuObject, int iFrameWidth )
   iRed   = m_color.red   ( );
   iGreen = m_color.green ( );
   iBlue  = m_color.blue  ( );
-  for ( y=0;  y < m_imageShadow.height ( ); y++ ) {
-    pLine = (QRgb *) m_imageShadow.scanLine ( y );
-	if ( ! pLine )
-		continue;
-    for ( x=0; x < m_imageShadow.width ( ); x++ ) {
-      currentPixel = *(pLine + x);
-      //iAlpha = (int)( (1.0 - m_fTransparency) / 3.0 * ( qRed( currentPixel ) + qGreen( currentPixel ) + qBlue( currentPixel ) ) );
-      iAlpha = (int)( (1.0 - m_fTransparency) * qRed( currentPixel ) );
-      newPixel = qRgba ( iRed, iGreen, iBlue, iAlpha );
-      *(pLine + x) = newPixel;
+  if ( bImageOrText )    {  // For performance reasons I check only once and have the loop in here twice
+    for ( y=0;  y < m_imageShadow.height ( ); y++ ) {
+      pLine = (QRgb *) m_imageShadow.scanLine ( y );
+      if ( ! pLine )
+        continue;
+      for ( x=0; x < m_imageShadow.width ( ); x++ ) {
+        currentPixel = *(pLine + x);
+        iAlpha = (int)( (1.0 - m_fTransparency) * qAlpha ( currentPixel ) );
+        newPixel = qRgba ( iRed, iGreen, iBlue, iAlpha );
+        *(pLine + x) = newPixel;
+      }
     }
   }
+  else  {
+    for ( y=0;  y < m_imageShadow.height ( ); y++ ) {
+      pLine = (QRgb *) m_imageShadow.scanLine ( y );
+      if ( ! pLine )
+        continue;
+      for ( x=0; x < m_imageShadow.width ( ); x++ ) {
+        currentPixel = *(pLine + x);
+        iAlpha = (int)( (1.0 - m_fTransparency) * qRed ( currentPixel ) );
+        newPixel = qRgba ( iRed, iGreen, iBlue, iAlpha );
+        *(pLine + x) = newPixel;
+      }
+    }
+  }
+
   blurShadow ( );
 }
 
@@ -758,7 +788,7 @@ void ShadowObject::blurShadow ( )
     QRgb *pLine, currentPixel, newPixel;
     int iRed, iGreen, iBlue, iAlpha, iWidth, iHeight, iPhaseZone;
     float fPhaseOut;
-      
+
     // What we have now is a greyscale image. The values actually represent the 
     // transparency of the final shadow.
     // Thus we will create an image that has only one color and only modulate the 
@@ -768,8 +798,8 @@ void ShadowObject::blurShadow ( )
     iPhaseZone = 5;
     for ( y=0;  y < m_imageShadow.height ( ); y++ ) {
       pLine = (QRgb *) m_imageShadow.scanLine ( y );
-	  if ( ! pLine )
-		  continue;
+      if ( ! pLine )
+        continue;
       for ( x=0; x < m_imageShadow.width ( ); x++ ) {
 	currentPixel = *(pLine + x);
 	iRed   = qRed   ( currentPixel );
@@ -801,11 +831,10 @@ void ShadowObject::blurShadow ( )
 
 bool ShadowObject::filterColorKeys ( QImage &theShadow )
 {
-  ImageObject tempImage, *pImageObject;
-  MovieObject tempMovie;
+  ImageObject *pImageObject = NULL;
   // Only Image, or Movie objects of interest for color keying.
-  if ( ( m_pMenuObject->objectType ( ) != tempImage.objectType ( ) ) &&
-       ( m_pMenuObject->objectType ( ) != tempMovie.objectType ( ) ) )
+  if ( ( m_pMenuObject->objectType ( ) != IMAGE_OBJECT ) &&
+       ( m_pMenuObject->objectType ( ) != MOVIE_OBJECT ) )
     return false;
 
   // Note MovieObject is derived off ImageObject.
@@ -821,7 +850,7 @@ bool ShadowObject::filterColorKeys ( QImage &theShadow )
   QValueList<QColor *>listColorKeyMax;
   QColor *pColorMin, *pColorMax, theColor, targetColor;
   QRgb thePixel, *pLine, *pShadowLine;
-  
+
   int t, x, y, iX, iY, iDelta, iRed, iGreen, iBlue, iColorKeyCount;
   iColorKeyCount = pManipulator->listColorKeys.count();
   for (t=0;t<iColorKeyCount;t++)	{
@@ -833,7 +862,7 @@ bool ShadowObject::filterColorKeys ( QImage &theShadow )
     iRed = theColor.red() + iDelta; iGreen = theColor.green() + iDelta; iBlue = theColor.blue() + iDelta;
     iRed = (iRed > 255) ? 255 : iRed; iGreen = (iGreen > 255) ? 255 : iGreen; iBlue = (iBlue > 255) ? 255 : iBlue;
     pColorMax = new QColor (iRed, iGreen, iBlue);
-    
+
     listColorKeyMin.append (pColorMin);
     listColorKeyMax.append (pColorMax);
   }
@@ -843,8 +872,8 @@ bool ShadowObject::filterColorKeys ( QImage &theShadow )
     if ( iY >= theImage.height ( ) )
          iY  = theImage.height ( )-1;
     pLine = (QRgb *)theImage.scanLine  ( iY );
-	if ( ! pLine )
-		continue;
+    if ( ! pLine )
+      continue;
     iY = y + 10; 
     if ( ( iY >= 0 ) && ( iY < theShadow.height ( ) ) ) {
       pShadowLine = (QRgb *)theShadow.scanLine  ( iY  );
@@ -897,7 +926,7 @@ void ShadowObject::cutOffImage ( QImage &theImage, enMType theType )
   unsigned char iTransparency;
   float fValue, fHalfCutRadius, fCutOff;
   QRgb *pLine;
-  
+
   fCutOff = m_fCutOff;
   if ( ( theType == MIRROR_LEFT ) || ( theType == MIRROR_TOP ) )
     fCutOff = 1.0 - m_fCutOff;
