@@ -27,7 +27,7 @@
 AudioEntry::AudioEntry ( )
 {
   iAudioNumber = -1;
-  qsIso639 = Global::qsDefaultLanguage;
+  qsIso639     = Global::qsDefaultLanguage;
 }
 
 AudioEntry &AudioEntry::operator= ( AudioEntry &theOther )
@@ -39,7 +39,7 @@ AudioEntry &AudioEntry::operator= ( AudioEntry &theOther )
 
 Audio::Audio ( )
 {
-  iAudioNumber = -1;
+  iAudioNumber        =   -1;
   pTranscodeInterface = NULL;
 }
 
@@ -65,6 +65,23 @@ Audio &Audio::operator = ( Audio &theOther )
   return *this;
 }
 
+bool Audio::operator == ( Audio &theOther )
+{
+  bool bReturn = (
+    ( iAudioNumber == theOther.iAudioNumber ) &&
+    ( qsFileName   == theOther.qsFileName   ) &&
+    ( qsIso639     == theOther.qsIso639     ) );
+  if ( ! bReturn )
+    return false;
+
+  if ( pTranscodeInterface && theOther.pTranscodeInterface )
+    return *pTranscodeInterface == *theOther.pTranscodeInterface;
+  if ( ! pTranscodeInterface && ! theOther.pTranscodeInterface )
+    return true;
+
+  return false;
+}
+
 bool Audio::readProjectFile  ( QDomNode &xmlNode )
 {
   QString     qsAttribute, qsFile, qsTagName;
@@ -235,10 +252,10 @@ bool TranscodeInterface::readProjectFile  (QDomNode &xmlNode)
   qsAttribute = theElement.attributeNode ( TRANSCODE_SUBTITLE_FROM_META ).value();
   if ( ! qsAttribute.isEmpty ( ) )
     iSubtitleFromMetaInfo = qsAttribute.toInt ();
-  
+
   // Always get the TempFileName, empty or not ... here I come ... you can't run ...
   qsTempFile = theElement.text ();    
-  
+
   return true;
 }
 
@@ -288,6 +305,7 @@ SourceFileInfo::SourceFileInfo ()
   pPreview       = NULL;
   bAutotranscode = true;
   bMetaSubtitles = true;
+  bHasAudioTrack = true;
   pTranscodeInterface = NULL;
   bUpdateInfo    = false;
   iPause         =  0;
@@ -312,6 +330,18 @@ SourceFileInfo::~SourceFileInfo ()
   deleteAudio     ( );
 }
 
+int SourceFileInfo::format ( )
+{
+  Utils theUtils;
+  int iFormat = 0;
+  if ( pTranscodeInterface )
+    iFormat = theUtils.getFormat ( pTranscodeInterface->qsResolution );
+  else
+    iFormat = theUtils.getFormat ( qsResolution );
+
+  return iFormat;
+}
+
 int SourceFileInfo::audioCount ( )
 {
   int t, iCount = 0;
@@ -384,7 +414,7 @@ QString SourceFileInfo::getFileName ()
     QString qsReturn;
     for ( t=0; t<MAX_SUBTITLES; t++ ) {
       if ( arraySubtitles[t] ) // the last file name wins.
-	qsReturn = arraySubtitles[t]->m_qsTempFile;
+        qsReturn = arraySubtitles[t]->m_qsTempFile;
     }
     if ( qsReturn.isEmpty ( ) )
       qsReturn = QString ( "%1/subtitles.vob" ).arg ( theUtils.getTempPath ( fileInfo.baseName ( TRUE ) ) );
@@ -402,6 +432,305 @@ QString SourceFileInfo::getFileName ()
   return qsFileName;
 }
 
+bool SourceFileInfo::readProjectFile ( QDomNode &xmlNode )
+{
+  QDomElement theElement = xmlNode.toElement  ( );
+
+  // Okay, here we read the stored data from the xml file.
+  QString qsAttribute;
+
+  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_SIZE).value();
+  if ( ! qsAttribute.isEmpty ( ) )
+    qsSize = qsAttribute;
+
+  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_RESOLUTION).value();
+  if ( ! qsAttribute.isEmpty ( ) )
+    qsResolution = qsAttribute;
+
+  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_FPS).value();
+  if ( ! qsAttribute.isEmpty ( ) )
+    qsFPS = qsAttribute;
+
+  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_RATIO).value();
+  if ( ! qsAttribute.isEmpty ( ) )
+    qsRatio = qsAttribute;
+
+  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_LENGTH).value();
+  if ( ! qsAttribute.isEmpty ( ) )
+    qsLength = qsAttribute;
+
+  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_VIDEO_FORMAT).value();
+  if ( ! qsAttribute.isEmpty ( ) )
+    qsVideoFormat = qsAttribute;
+
+  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_VIDEO_CODEC).value();
+  if ( ! qsAttribute.isEmpty ( ) )
+    qsVideoCodec = qsAttribute;
+
+  qsAttribute = theElement.attributeNode ( SOURCE_OBJECT_HAS_AUDIO ).value ( );
+  if ( ! qsAttribute.isEmpty ( ) )
+    bHasAudioTrack = ( qsAttribute == "true" );
+
+  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_AUDIO_FORMAT).value();
+  if ( ! qsAttribute.isEmpty ( ) )
+    qsAudioFormat = qsAttribute;
+
+  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_AUDIO_CODEC).value();
+  if ( ! qsAttribute.isEmpty ( ) )
+    qsAudioCodec  = qsAttribute;
+
+  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_STATUS).value();
+  if ( ! qsAttribute.isEmpty ( ) )
+    qsStatus = qsAttribute;
+
+  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_BITS).value();
+  if ( ! qsAttribute.isEmpty ( ) )
+    qsBits = qsAttribute;
+
+  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_SAMPLE).value();
+  if ( ! qsAttribute.isEmpty ( ) )
+    qsSample = qsAttribute;
+
+  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_BPS).value();
+  if ( ! qsAttribute.isEmpty ( ) )
+    qsBPS = qsAttribute;
+
+  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_PAUSE).value();
+  if ( ! qsAttribute.isEmpty ( ) )
+    iPause = qsAttribute.toInt();
+
+  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_TITLESET).value();
+  if ( ! qsAttribute.isEmpty ( ) )
+    iTitleset = qsAttribute.toInt();
+
+  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_AUTOTRANSCODE).value();
+  bAutotranscode = true;
+  if ( ! qsAttribute.isEmpty ( ) )
+    bAutotranscode = ( qsAttribute == QString ("true") );
+
+  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_METASUBTITLES).value();
+  bMetaSubtitles = true;
+  if ( ! qsAttribute.isEmpty ( ) )
+    bMetaSubtitles = ( qsAttribute == QString ("true") );
+
+  qsAttribute = theElement.attributeNode ( SOURCE_OBJECT_MSEC_OFFSET ).value ( );
+  if ( ! qsAttribute.isEmpty ( ) )
+    iMSecPreview = qsAttribute.toInt ( );
+
+  QDomNode infoNode = theElement.firstChild ( );
+  deleteSubtitles ( );
+  while ( ! infoNode.isNull ( ) )  {
+    QDomElement infoElement = infoNode.toElement( );
+    // Okay, here we read the stored data from the xml file.
+    QString infoName = infoElement.tagName  ( );
+    QString infoText = infoElement.text     ( );
+    if (SOURCE_OBJECT_FILE_NAME == infoName)	{
+      if ( ! infoText.isEmpty ( ) )
+        qsFileName = infoText;
+    }
+    else if ( SOURCE_OBJECT_CHAPTERS == infoName )  {
+      if ( ! infoText.isEmpty ( ) )
+        listChapters = QStringList::split (",", infoText);
+    }
+    else if ( SOURCE_OBJECT_PROPERTIES == infoName )  {
+      qsAttribute = infoElement.attributeNode ( SOURCE_PROPERTIES_POST ).value ( );
+      if ( ! qsAttribute.isEmpty ( ) )
+        qsPostCommand = qsAttribute;
+
+      qsAttribute = infoElement.attributeNode (SOURCE_PROPERTIES_PRE).value();
+      if ( ! qsAttribute.isEmpty ( ) )
+        qsPreCommand = qsAttribute;
+
+      qsAttribute = infoElement.attributeNode (SOURCE_PROPERTIES_PAUSE).value();
+      if ( ! qsAttribute.isEmpty ( ) )
+        iPause = qsAttribute.toInt ( );
+    }
+    else if (SOURCE_OBJECT_TRANSCODE == infoName)	{
+      if ( pTranscodeInterface)
+        delete pTranscodeInterface;
+      pTranscodeInterface = new TranscodeInterface;
+      pTranscodeInterface->readProjectFile (infoNode);
+    }
+    else if ( SLIDESHOW_TAG == infoName )	{
+      if ( pSlideshow )
+        delete pSlideshow;
+      pSlideshow = new CXmlSlideshow;
+      QDomElement theElement = infoNode.toElement ( );
+      pSlideshow->readXml ( &theElement );
+    }
+    else if (SOURCE_OBJECT_SUBTITLES == infoName)	{
+      Subtitles *pSubtitle = new Subtitles ( 0, 0 );
+      pSubtitle->readProjectFile ( infoNode );
+      addSubtitles   ( pSubtitle );
+    }
+    else if (SOURCE_OBJECT_AUDIO == infoName)	{
+      Audio *pAudio = new Audio;
+      pAudio->readProjectFile ( infoNode );
+      addAudio    ( pAudio   );
+    }
+    infoNode = infoNode.nextSibling ();
+  }
+  return true;
+}
+
+bool SourceFileInfo::writeProjectFile ( QDomElement &sourceNode, bool /* bAutosave */ )
+{
+  uint i;
+  QDomDocument xmlDoc = sourceNode.ownerDocument ( );
+  QDomElement  fileElement, infoElement, chapterElement, propertiesElement;
+  QDomElement  tag,  slideshowElement, transcodeElement, subtitlesElement;
+  QDomText     text;
+  // Here we set the attributes of the <SourceInfo> tag
+
+  infoElement = xmlDoc.createElement( SOURCE_OBJECT_INFO );
+  if ( ! qsSize.isNull ( ) )
+    infoElement.setAttribute ( SOURCE_OBJECT_SIZE, qsSize );
+  if ( ! qsResolution.isNull ( ) )
+    infoElement.setAttribute ( SOURCE_OBJECT_RESOLUTION, qsResolution );
+  if ( ! qsFPS.isNull        ( ) )
+    infoElement.setAttribute ( SOURCE_OBJECT_FPS, qsFPS );
+  if ( ! qsRatio.isNull      ( ) )
+    infoElement.setAttribute ( SOURCE_OBJECT_RATIO, qsRatio );
+  if ( ! qsLength.isNull     ( ) )
+    infoElement.setAttribute ( SOURCE_OBJECT_LENGTH, qsLength );
+  if ( ! qsVideoFormat.isNull( ) )
+    infoElement.setAttribute ( SOURCE_OBJECT_VIDEO_FORMAT, qsVideoFormat );
+  if ( ! qsVideoCodec.isNull ( ) )
+    infoElement.setAttribute ( SOURCE_OBJECT_VIDEO_CODEC,  qsVideoCodec  );
+  infoElement.setAttribute   ( SOURCE_OBJECT_HAS_AUDIO, bHasAudioTrack ? "true" : "false" );
+  if ( ! qsAudioFormat.isNull( ) )
+    infoElement.setAttribute ( SOURCE_OBJECT_AUDIO_FORMAT, qsAudioFormat );
+  if ( ! qsAudioCodec.isNull ( ) )
+    infoElement.setAttribute ( SOURCE_OBJECT_AUDIO_CODEC,  qsAudioCodec  );
+  if ( ! qsStatus.isNull     ( ) )
+    infoElement.setAttribute ( SOURCE_OBJECT_STATUS, qsStatus );
+  if ( ! qsBits.isNull       ( ) )
+    infoElement.setAttribute ( SOURCE_OBJECT_BITS, qsBits );
+  if ( ! qsSample.isNull     ( ) )
+    infoElement.setAttribute ( SOURCE_OBJECT_SAMPLE, qsSample );
+  if ( ! qsBPS.isNull        ( ) )
+    infoElement.setAttribute ( SOURCE_OBJECT_BPS, qsBPS );
+  if ( iPause != -1 )
+    infoElement.setAttribute ( SOURCE_OBJECT_PAUSE, QString ("%1").arg(iPause) );
+  if ( iTitleset != -1 )
+    infoElement.setAttribute ( SOURCE_OBJECT_TITLESET, QString ("%1").arg(iTitleset) );
+  if ( bAutotranscode == false )
+    infoElement.setAttribute ( SOURCE_OBJECT_AUTOTRANSCODE, "false" );
+  if ( bMetaSubtitles == false )
+    infoElement.setAttribute ( SOURCE_OBJECT_METASUBTITLES, "false" );
+  if ( iMSecPreview != -1 )
+    infoElement.setAttribute ( SOURCE_OBJECT_MSEC_OFFSET, QString ("%1").arg(iMSecPreview) );
+
+  // Here we create another level in the XML tree for name and chapter ...
+  // First the file name
+  fileElement = xmlDoc.createElement( SOURCE_OBJECT_FILE_NAME );
+  text = xmlDoc.createTextNode ( qsFileName );
+  fileElement.appendChild( text );
+  infoElement.appendChild( fileElement );
+  // Second the properties (Post / Pre, and Pause)
+  propertiesElement = xmlDoc.createElement( SOURCE_OBJECT_PROPERTIES );
+  if (!qsPostCommand.isNull())
+    propertiesElement.setAttribute( SOURCE_PROPERTIES_POST, qsPostCommand );
+  if (!qsPreCommand.isNull())
+    propertiesElement.setAttribute( SOURCE_PROPERTIES_PRE, qsPreCommand );
+  propertiesElement.setAttribute( SOURCE_PROPERTIES_PAUSE, QString ("%1").arg(iPause ));
+  infoElement.appendChild( propertiesElement );
+  // Then chapters if avail.
+  if (listChapters.count() > 0)	{
+    chapterElement = xmlDoc.createElement( SOURCE_OBJECT_CHAPTERS );
+    text = xmlDoc.createTextNode ( listChapters.join(QString(",")));
+    chapterElement.appendChild( text );
+    infoElement.appendChild( chapterElement );
+  }
+  // next to last the TranscodeInterface if any.
+  if (pTranscodeInterface)	{
+    transcodeElement = xmlDoc.createElement( SOURCE_OBJECT_TRANSCODE );
+    pTranscodeInterface->writeProjectFile (transcodeElement);
+    infoElement.appendChild( transcodeElement );
+  }
+  if ( pSlideshow )  {
+    slideshowElement = xmlDoc.createElement (  SLIDESHOW_TAG );
+    pSlideshow->writeXml ( &slideshowElement );
+    infoElement.appendChild ( slideshowElement );
+  }
+
+  // Next to last we add the audio channels.
+  for ( i=0; i<MAX_AUDIO; i++ ) {
+    if ( arrayAudio[i] )
+      arrayAudio[i]->writeProjectFile ( infoElement );
+  }
+  // And finally the Subtitles
+  for ( i=0; i<MAX_SUBTITLES; i++ ) {
+    if ( arraySubtitles[i] )
+      arraySubtitles[i]->writeProjectFile ( infoElement );
+  }
+  sourceNode.appendChild ( infoElement );
+
+  return true;
+}
+
+SourceFileInfo & SourceFileInfo::operator = (SourceFileInfo &theOther)
+{
+  int t;
+  qsFileName     = theOther.qsFileName;
+  qsSize         = theOther.qsSize;
+  qsResolution   = theOther.qsResolution;
+  qsFPS          = theOther.qsFPS;
+  qsRatio        = theOther.qsRatio;
+  qsLength       = theOther.qsLength;
+  bHasAudioTrack = theOther.bHasAudioTrack;
+  qsAudioFormat  = theOther.qsAudioFormat;
+  qsAudioCodec   = theOther.qsAudioCodec;
+  qsVideoFormat  = theOther.qsVideoFormat;
+  qsVideoCodec   = theOther.qsVideoCodec;
+  qsStatus       = theOther.qsStatus;
+  qsBits         = theOther.qsBits;
+  qsSample       = theOther.qsSample;
+  qsBPS          = theOther.qsBPS;
+  listChapters   = theOther.listChapters;
+  qsPreCommand   = theOther.qsPreCommand;
+  qsPostCommand  = theOther.qsPostCommand;
+  iPause         = theOther.iPause;
+  iMSecPreview   = theOther.iMSecPreview;
+  iTitleset      = theOther.iTitleset;
+  bAutotranscode = theOther.bAutotranscode;
+  bMetaSubtitles = theOther.bMetaSubtitles;
+  if (theOther.pTranscodeInterface)	{
+    if (!pTranscodeInterface)
+      pTranscodeInterface  = new TranscodeInterface ();
+    *(pTranscodeInterface) = *(theOther.pTranscodeInterface);
+  }
+  else if ( pTranscodeInterface )	{
+    delete pTranscodeInterface;
+    pTranscodeInterface = NULL;
+  }
+
+  deleteSubtitles ( );
+  for ( t=0; t<MAX_SUBTITLES; t++ ) {
+    if ( theOther.arraySubtitles [ t ] ) {
+        arraySubtitles [ t ]  = new Subtitles ( 0, 0 );
+      *(arraySubtitles [ t ]) = *(theOther.arraySubtitles [ t ]);
+    }
+  }
+  deleteAudio ( );
+  for ( t=0; t<MAX_AUDIO; t++ ) {
+    if ( theOther.arrayAudio [ t ] ) {
+        arrayAudio [ t ]  = new Audio;
+      *(arrayAudio [ t ]) = *(theOther.arrayAudio [ t ]);
+    }
+  }
+
+  if ( pSlideshow )
+    delete pSlideshow;
+  pSlideshow = NULL;
+  if ( theOther.pSlideshow )  {
+     pSlideshow = new CXmlSlideshow;
+    *pSlideshow = *theOther.pSlideshow;
+  }
+
+  return *this;
+}
+
 SourceFileEntry::SourceFileEntry ( )
 {
   iTitleset     = -1;
@@ -677,12 +1006,102 @@ int SourceFileEntry::subtitleCount ( bool bMaxCount )
       }
     }
   }
-  if ( iMaxInfoSubtitle < iMaxEntrySubtitle )
-    iMaxEntrySubtitle = iMaxInfoSubtitle;
+  if ( iMaxInfoSubtitle  < iMaxEntrySubtitle )
+       iMaxEntrySubtitle = iMaxInfoSubtitle;
 
   return iMaxEntrySubtitle;
 }
 
+void SourceFileEntry::changeAudioTrack ( uint iAudioNumber, QString qsCode )
+{
+  SourceFileInfo *pInfo;
+  bool bFound = false;
+
+  if ( iAudioNumber >= MAX_AUDIO )
+    return;
+
+  QValueList<SourceFileInfo *>::iterator it = listFileInfos.begin ( );
+  while ( it != listFileInfos.end ( ) )  {
+    pInfo = *it++;
+    if ( pInfo->arrayAudio[iAudioNumber] )  {
+         pInfo->arrayAudio[iAudioNumber]->qsIso639 = qsCode;
+         bFound = true;
+    }
+  }
+
+  if ( bFound )  {
+    arrayAudioEntries[iAudioNumber].qsIso639     = qsCode;
+    arrayAudioEntries[iAudioNumber].iAudioNumber = iAudioNumber;
+  }
+  else  {
+    arrayAudioEntries[iAudioNumber].qsIso639     = "";
+    arrayAudioEntries[iAudioNumber].iAudioNumber = -1;
+  }
+}
+
+void SourceFileEntry::changeSubtitleTrack ( uint iSubtitleNumber, QString qsCode )
+{
+  SourceFileInfo *pInfo;
+  bool bFound = false;
+
+  if ( iSubtitleNumber >= MAX_SUBTITLES )
+    return;
+
+  QValueList<SourceFileInfo *>::iterator it = listFileInfos.begin ( );
+  while ( it != listFileInfos.end ( ) )  {
+    pInfo = *it++;
+    if ( pInfo->arraySubtitles[iSubtitleNumber] )  {
+      pInfo->arrayAudio[iSubtitleNumber]->qsIso639 = qsCode;
+      bFound = true;
+    }
+  }
+
+  if ( bFound )  {
+    arraySubtitleEntries[iSubtitleNumber].m_qsIso639        = qsCode;
+    arraySubtitleEntries[iSubtitleNumber].m_iSubtitleNumber = iSubtitleNumber;
+  }
+  else  {
+    arraySubtitleEntries[iSubtitleNumber].m_qsIso639        = "";
+    arraySubtitleEntries[iSubtitleNumber].m_iSubtitleNumber = -1;
+  }
+}
+
+void SourceFileEntry::synchTracks ( bool bSynchAudioTracks, bool bSynchSubtitleTracks )
+{
+  int t;
+  Audio      *pAudio;
+  AudioEntry *pAudioEntry;
+  Subtitles *pSubtitle;
+  SubtitleEntry *pSubtitleEntry;
+  SourceFileInfo *pInfo;
+
+  QValueList<SourceFileInfo *>::iterator it = listFileInfos.begin ( );
+  while ( it != listFileInfos.end ( ) )  {
+    pInfo = *it++;
+    if ( bSynchAudioTracks )  {
+      for ( t=0; t<MAX_AUDIO; t++ )  {
+        pAudio = pInfo->arrayAudio[t];
+        if ( pAudio )  {
+          pAudioEntry = &arrayAudioEntries[pAudio->iAudioNumber];
+          pAudioEntry->iAudioNumber = pAudio->iAudioNumber;
+          pAudioEntry->qsIso639     = pAudio->qsIso639;
+        }
+      }
+    }
+
+    if ( bSynchSubtitleTracks )  {
+      for ( t=0; t<MAX_SUBTITLES; t++ )  {
+        pSubtitle = pInfo->arraySubtitles[t];
+        if ( pSubtitle )  {
+          pSubtitleEntry = &arraySubtitleEntries[pSubtitle->m_iSubtitleNumber];
+          pSubtitleEntry->m_iSubtitleNumber = pSubtitle->m_iSubtitleNumber;
+          pSubtitleEntry->m_qsIso639        = pSubtitle->m_qsIso639;
+        }
+      }
+    }
+  }
+}
+
 SourceFileEntry & SourceFileEntry::operator = (SourceFileEntry &theOther)
 {
 	uint t;
@@ -719,297 +1138,3 @@ SourceFileEntry & SourceFileEntry::operator = (SourceFileEntry &theOther)
 
 	return *this;
 }
-
-bool SourceFileInfo::readProjectFile ( QDomNode &xmlNode )
-{
-  QDomElement theElement = xmlNode.toElement  ( );
-
-  // Okay, here we read the stored data from the xml file.
-  QString qsAttribute;
-
-  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_SIZE).value();
-  if ( ! qsAttribute.isEmpty ( ) )
-    qsSize = qsAttribute;
-
-  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_RESOLUTION).value();
-  if ( ! qsAttribute.isEmpty ( ) )
-    qsResolution = qsAttribute;
-
-  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_FPS).value();
-  if ( ! qsAttribute.isEmpty ( ) )
-    qsFPS = qsAttribute;
-
-  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_RATIO).value();
-  if ( ! qsAttribute.isEmpty ( ) )
-    qsRatio = qsAttribute;
-
-  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_LENGTH).value();
-  if ( ! qsAttribute.isEmpty ( ) )
-    qsLength = qsAttribute;
-
-  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_VIDEO_FORMAT).value();
-  if ( ! qsAttribute.isEmpty ( ) )
-    qsVideoFormat = qsAttribute;
-
-  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_VIDEO_CODEC).value();
-  if ( ! qsAttribute.isEmpty ( ) )
-    qsVideoCodec = qsAttribute;
-
-  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_AUDIO_FORMAT).value();
-  if ( ! qsAttribute.isEmpty ( ) )
-    qsAudioFormat = qsAttribute;
-
-  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_AUDIO_CODEC).value();
-  if ( ! qsAttribute.isEmpty ( ) )
-    qsAudioCodec  = qsAttribute;
-
-  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_STATUS).value();
-  if ( ! qsAttribute.isEmpty ( ) )
-    qsStatus = qsAttribute;
-
-  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_BITS).value();
-  if ( ! qsAttribute.isEmpty ( ) )
-    qsBits = qsAttribute;
-
-  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_SAMPLE).value();
-  if ( ! qsAttribute.isEmpty ( ) )
-    qsSample = qsAttribute;
-
-  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_BPS).value();
-  if ( ! qsAttribute.isEmpty ( ) )
-    qsBPS = qsAttribute;
-
-  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_PAUSE).value();
-  if ( ! qsAttribute.isEmpty ( ) )
-    iPause = qsAttribute.toInt();
-
-  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_TITLESET).value();
-  if ( ! qsAttribute.isEmpty ( ) )
-    iTitleset = qsAttribute.toInt();
-
-  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_AUTOTRANSCODE).value();
-  bAutotranscode = true;
-  if ( ! qsAttribute.isEmpty ( ) )
-    bAutotranscode = ( qsAttribute == QString ("true") );
-
-  qsAttribute = theElement.attributeNode (SOURCE_OBJECT_METASUBTITLES).value();
-  bMetaSubtitles = true;
-  if ( ! qsAttribute.isEmpty ( ) )
-    bMetaSubtitles = ( qsAttribute == QString ("true") );
-
-  qsAttribute = theElement.attributeNode ( SOURCE_OBJECT_MSEC_OFFSET ).value ( );
-  if ( ! qsAttribute.isEmpty ( ) )
-    iMSecPreview = qsAttribute.toInt ( );
-
-  QDomNode infoNode = theElement.firstChild ( );
-  deleteSubtitles ( );
-  while ( ! infoNode.isNull ( ) )  {
-    QDomElement infoElement = infoNode.toElement( );
-    // Okay, here we read the stored data from the xml file.
-    QString infoName = infoElement.tagName  ( );
-    QString infoText = infoElement.text     ( );
-    if (SOURCE_OBJECT_FILE_NAME == infoName)	{
-      if ( ! infoText.isEmpty ( ) )
-        qsFileName = infoText;
-    }
-    else if ( SOURCE_OBJECT_CHAPTERS == infoName )  {
-      if ( ! infoText.isEmpty ( ) )
-        listChapters = QStringList::split (",", infoText);
-    }
-    else if ( SOURCE_OBJECT_PROPERTIES == infoName )  {
-      qsAttribute = infoElement.attributeNode ( SOURCE_PROPERTIES_POST ).value ( );
-      if ( ! qsAttribute.isEmpty ( ) )
-        qsPostCommand = qsAttribute;
-
-      qsAttribute = infoElement.attributeNode (SOURCE_PROPERTIES_PRE).value();
-      if ( ! qsAttribute.isEmpty ( ) )
-        qsPreCommand = qsAttribute;
-
-      qsAttribute = infoElement.attributeNode (SOURCE_PROPERTIES_PAUSE).value();
-      if ( ! qsAttribute.isEmpty ( ) )
-        iPause = qsAttribute.toInt ( );
-    }
-    else if (SOURCE_OBJECT_TRANSCODE == infoName)	{
-      if ( pTranscodeInterface)
-        delete pTranscodeInterface;
-      pTranscodeInterface = new TranscodeInterface;
-      pTranscodeInterface->readProjectFile (infoNode);
-    }
-    else if ( SLIDESHOW_TAG == infoName )	{
-      if ( pSlideshow )
-        delete pSlideshow;
-      pSlideshow = new CXmlSlideshow;
-      QDomElement theElement = infoNode.toElement ( );
-      pSlideshow->readXml ( &theElement );
-    }
-    else if (SOURCE_OBJECT_SUBTITLES == infoName)	{
-      Subtitles *pSubtitle = new Subtitles;
-      pSubtitle->readProjectFile ( infoNode );
-      addSubtitles   ( pSubtitle );
-    }
-    else if (SOURCE_OBJECT_AUDIO == infoName)	{
-      Audio *pAudio = new Audio;
-      pAudio->readProjectFile ( infoNode );
-      addAudio    ( pAudio   );
-    }
-    infoNode = infoNode.nextSibling ();
-  }
-  return true;
-}
-
-bool SourceFileInfo::writeProjectFile ( QDomElement &sourceNode, bool /* bAutosave */ )
-{
-  uint i;
-  QDomDocument xmlDoc = sourceNode.ownerDocument ( );
-  QDomElement  fileElement, infoElement, chapterElement, propertiesElement;
-  QDomElement  tag,  slideshowElement, transcodeElement, subtitlesElement;
-  QDomText     text;
-  // Here we set the attributes of the <SourceInfo> tag
-
-  infoElement = xmlDoc.createElement( SOURCE_OBJECT_INFO );
-  if ( ! qsSize.isNull ( ) )
-    infoElement.setAttribute ( SOURCE_OBJECT_SIZE, qsSize );
-  if ( ! qsResolution.isNull ( ) )
-    infoElement.setAttribute ( SOURCE_OBJECT_RESOLUTION, qsResolution );
-  if ( ! qsFPS.isNull        ( ) )
-    infoElement.setAttribute ( SOURCE_OBJECT_FPS, qsFPS );
-  if ( ! qsRatio.isNull      ( ) )
-    infoElement.setAttribute ( SOURCE_OBJECT_RATIO, qsRatio );
-  if ( ! qsLength.isNull     ( ) )
-    infoElement.setAttribute ( SOURCE_OBJECT_LENGTH, qsLength );
-  if ( ! qsVideoFormat.isNull( ) )
-    infoElement.setAttribute ( SOURCE_OBJECT_VIDEO_FORMAT, qsVideoFormat );
-  if ( ! qsVideoCodec.isNull ( ) )
-    infoElement.setAttribute ( SOURCE_OBJECT_VIDEO_CODEC,  qsVideoCodec  );
-  if ( ! qsAudioFormat.isNull( ) )
-    infoElement.setAttribute ( SOURCE_OBJECT_AUDIO_FORMAT, qsAudioFormat );
-  if ( ! qsAudioCodec.isNull ( ) )
-    infoElement.setAttribute ( SOURCE_OBJECT_AUDIO_CODEC,  qsAudioCodec  );
-  if ( ! qsStatus.isNull     ( ) )
-    infoElement.setAttribute ( SOURCE_OBJECT_STATUS, qsStatus );
-  if ( ! qsBits.isNull       ( ) )
-    infoElement.setAttribute ( SOURCE_OBJECT_BITS, qsBits );
-  if ( ! qsSample.isNull     ( ) )
-    infoElement.setAttribute ( SOURCE_OBJECT_SAMPLE, qsSample );
-  if ( ! qsBPS.isNull        ( ) )
-    infoElement.setAttribute ( SOURCE_OBJECT_BPS, qsBPS );
-  if ( iPause != -1 )
-    infoElement.setAttribute ( SOURCE_OBJECT_PAUSE, QString ("%1").arg(iPause) );
-  if ( iTitleset != -1 )
-    infoElement.setAttribute ( SOURCE_OBJECT_TITLESET, QString ("%1").arg(iTitleset) );
-  if ( bAutotranscode == false )
-    infoElement.setAttribute ( SOURCE_OBJECT_AUTOTRANSCODE, "false" );
-  if ( bMetaSubtitles == false )
-    infoElement.setAttribute ( SOURCE_OBJECT_METASUBTITLES, "false" );
-  if ( iMSecPreview != -1 )
-    infoElement.setAttribute ( SOURCE_OBJECT_MSEC_OFFSET, QString ("%1").arg(iMSecPreview) );
-
-  // Here we create another level in the XML tree for name and chapter ...
-  // First the file name
-  fileElement = xmlDoc.createElement( SOURCE_OBJECT_FILE_NAME );
-  text = xmlDoc.createTextNode ( qsFileName );
-  fileElement.appendChild( text );
-  infoElement.appendChild( fileElement );
-  // Second the properties (Post / Pre, and Pause)
-  propertiesElement = xmlDoc.createElement( SOURCE_OBJECT_PROPERTIES );
-  if (!qsPostCommand.isNull())
-    propertiesElement.setAttribute( SOURCE_PROPERTIES_POST, qsPostCommand );
-  if (!qsPreCommand.isNull())
-    propertiesElement.setAttribute( SOURCE_PROPERTIES_PRE, qsPreCommand );
-  propertiesElement.setAttribute( SOURCE_PROPERTIES_PAUSE, QString ("%1").arg(iPause ));
-  infoElement.appendChild( propertiesElement );
-  // Then chapters if avail.
-  if (listChapters.count() > 0)	{
-    chapterElement = xmlDoc.createElement( SOURCE_OBJECT_CHAPTERS );
-    text = xmlDoc.createTextNode ( listChapters.join(QString(",")));
-    chapterElement.appendChild( text );
-    infoElement.appendChild( chapterElement );
-  }
-  // next to last the TranscodeInterface if any.
-  if (pTranscodeInterface)	{
-    transcodeElement = xmlDoc.createElement( SOURCE_OBJECT_TRANSCODE );
-    pTranscodeInterface->writeProjectFile (transcodeElement);
-    infoElement.appendChild( transcodeElement );
-  }
-  if ( pSlideshow )  {
-    slideshowElement = xmlDoc.createElement (  SLIDESHOW_TAG );
-    pSlideshow->writeXml ( &slideshowElement );
-    infoElement.appendChild ( slideshowElement );
-  }
-
-  // Next to last we add the audio channels.
-  for ( i=0; i<MAX_AUDIO; i++ ) {
-    if ( arrayAudio[i] )
-      arrayAudio[i]->writeProjectFile ( infoElement );
-  }
-  // And finally the Subtitles
-  for ( i=0; i<MAX_SUBTITLES; i++ ) {
-    if ( arraySubtitles[i] )
-      arraySubtitles[i]->writeProjectFile ( infoElement );
-  }
-  sourceNode.appendChild ( infoElement );
-
-  return true;
-}
-
-SourceFileInfo & SourceFileInfo::operator = (SourceFileInfo &theOther)
-{
-  int t;
-  qsFileName     = theOther.qsFileName;
-  qsSize         = theOther.qsSize;
-  qsResolution   = theOther.qsResolution;
-  qsFPS          = theOther.qsFPS;
-  qsRatio        = theOther.qsRatio;
-  qsLength       = theOther.qsLength;
-  qsAudioFormat  = theOther.qsAudioFormat;
-  qsAudioCodec   = theOther.qsAudioCodec;
-  qsVideoFormat  = theOther.qsVideoFormat;
-  qsVideoCodec   = theOther.qsVideoCodec;
-  qsStatus       = theOther.qsStatus;
-  qsBits         = theOther.qsBits;
-  qsSample       = theOther.qsSample;
-  qsBPS          = theOther.qsBPS;
-  listChapters   = theOther.listChapters;
-  qsPreCommand   = theOther.qsPreCommand;
-  qsPostCommand  = theOther.qsPostCommand;
-  iPause         = theOther.iPause;
-  iMSecPreview   = theOther.iMSecPreview;  
-  iTitleset      = theOther.iTitleset;
-  bAutotranscode = theOther.bAutotranscode;
-  bMetaSubtitles = theOther.bMetaSubtitles;
-  if (theOther.pTranscodeInterface)	{
-    if (!pTranscodeInterface)
-      pTranscodeInterface  = new TranscodeInterface ();
-    *(pTranscodeInterface) = *(theOther.pTranscodeInterface);
-  }
-  else if ( pTranscodeInterface )	{
-    delete pTranscodeInterface;
-    pTranscodeInterface = NULL;
-  }
-
-  deleteSubtitles ( );
-  for ( t=0; t<MAX_SUBTITLES; t++ ) {
-    if ( theOther.arraySubtitles [ t ] ) {
-        arraySubtitles [ t ]  = new Subtitles;
-      *(arraySubtitles [ t ]) = *(theOther.arraySubtitles [ t ]);
-    }
-  }
-  deleteAudio ( );
-  for ( t=0; t<MAX_AUDIO; t++ ) {
-    if ( theOther.arrayAudio [ t ] ) {
-        arrayAudio [ t ]  = new Audio;
-      *(arrayAudio [ t ]) = *(theOther.arrayAudio [ t ]);
-    }
-  }
-
-  if ( pSlideshow )
-    delete pSlideshow;
-  pSlideshow = NULL;
-  if ( theOther.pSlideshow )  {
-     pSlideshow = new CXmlSlideshow;
-    *pSlideshow = *theOther.pSlideshow;
-  }
-
-  return *this;
-}
-
