@@ -41,13 +41,13 @@ public:
 	QString qsVideoFormat;
 	QString qsVideoType;
 	QString qsResolution;
-	int     iVideoBitrate;
+	int     iVideoBitrate;  // in bps
 	float   fFrameRate;
 	QString qsRatio;
 
 	QString qsAudioFormat;  // "Stereo"
 	QString qsAudioType;    // One of [ mp2, ac3, or pcm ]
-	int     iAudioBitrate;  // in kbps E.g. 192 (instead of 196608 or 192000)
+	int     iAudioBitrate;  // in bps E.g. 192000 (instead of 196608 or 192k)
 	int     iSample;
 
 	int     iStartingAt;	// The movie should start here (in mSec)
@@ -65,7 +65,7 @@ public:
 	QString qsTempFile;
 };
 
-class AudioEntry 
+class AudioEntry
 {
 public:
   AudioEntry ( );
@@ -80,6 +80,7 @@ public:
    Audio ( );
   ~Audio ( );
   Audio &operator =      ( Audio       & );
+  bool   operator ==     ( Audio       & );
   bool  readProjectFile  ( QDomNode    & );
   bool  writeProjectFile ( QDomElement & );
 
@@ -105,6 +106,7 @@ public:
 	int     subtitleCount   ( );
 	void    deleteSubtitles ( );
 	void    addSubtitles    ( Subtitles * );
+	int     format          ( );
 
 	QString qsFileName;	  // E.g. video.mpg
 	QString qsSize;		  // E.g. 2.2M
@@ -116,15 +118,16 @@ public:
 	QString qsVideoCodec;	  // E.g. 'mpeg'
 	QString qsStatus;	  // E.g. Ok	-=> to say this file we can autmatically convert to a DVD video
 	// For Audio Sources :
+	bool    bHasAudioTrack;   // Elemntary streams only have Video
 	QString qsAudioFormat;    // E.g. 'Stereo'
 	QString qsAudioCodec;     // E.g. "mp2"
-	QString qsBits;		  // E.g. 16
-	QString qsSample;	  // E.g. 48kHz
-	QString qsBPS;		  // E.g. 00:05:32
+	QString qsBits;           // E.g. 16
+	QString qsSample;         // E.g. 48kHz
+	QString qsBPS;            // E.g. 00:05:32
 
 	long    iMSecPreview;     // The preview is to be generated iMSec from the start.
-	QImage *pPreview;	  // Image of the size for the Button
-	bool    bUpdateInfo;	  // if the preview has been generated we set this flag to indicate this.
+	QImage *pPreview;         // Image of the size for the Button
+	bool    bUpdateInfo;      // if the preview has been generated we set this flag to indicate this.
 
 	QStringList listChapters; // The chapters of this SourceFile entry
 	QString     qsPreCommand; // The Pre  Command tag for dvdauthor
@@ -151,8 +154,11 @@ public:
 	bool writeProjectFile ( QDomElement &, bool b=false );
 	SourceFileEntry & operator = ( SourceFileEntry & );
 	SourceFileEntry * flatClone  ( );
-	int audioCount    ( bool );
-	int subtitleCount ( bool );
+	int  audioCount    ( bool );
+	int  subtitleCount ( bool );
+	void synchTracks   ( bool a=true, bool s=true );
+	void changeAudioTrack    ( uint, QString );
+	void changeSubtitleTrack ( uint, QString );
 
 	QString qsDisplayName;          // The name that appears on the display
 	QValueList <SourceFileInfo *> listFileInfos;
