@@ -38,7 +38,7 @@
 #include "uisourcewidget.h"
 #include "sourcefileentry.h"
 
-class QDVDAuthor;
+//class QDVDAuthor;
 class MediaInfo;
 class DialogThumbnail;
 class ListViewItemMedia;
@@ -66,22 +66,24 @@ public:
 	void slideshowDone                         ( SourceFileEntry * );
 	bool sourceFileEntryExists                 ( SourceFileEntry * );
 	void editSourceFileInfo                    ( SourceFileInfo  * );
-	void updateListViewItem                    ( SourceFileInfo  * );
+	bool updateListViewItem                    ( SourceFileInfo  * );
 	bool checkRightAudioFormat                 ( SourceFileInfo  * );
 	bool checkRightFileFormat                  ( SourceFileInfo  * );
+	void createContextMenu                     ( SourceFileEntry *, QPoint & );
 	void appendSourceFileEntry                 ( SourceFileEntry *, bool b=false );
 	bool setRenderProgress                     ( SourceFileEntry *, float );
 	void removeEntry                           ( SourceFileEntry *, bool b=true  );
 	void removeEntry                           ( QString &, bool  b=true  );
 	void updateStatusColor                     ( QListViewItem   *p=NULL  );
-
+	QListView   *getStructureListView          ( );
+	QTabWidget  *getTabCtrl                    ( );
+	QPushButton *getButton                     ( bool );
 	virtual QSize sizeHint                     ( ) const;
 	QString convertFileSize                    ( uint );
-
 	MediaInfo *getMediaInfo                    ( );
 
 protected:
-	virtual void  timerEvent      ( QTimerEvent * );
+	virtual void timerEvent       ( QTimerEvent     * );
 	ListViewItemMedia *getItem    ( SourceFileEntry * );
 
 private:
@@ -99,12 +101,12 @@ public slots:
 	virtual void slotPlayMovie       ( );
 	virtual void slotEditMovie       ( );
 	virtual void slotEditSlideshow   ( );
+	virtual void slotRerenderSlide   ( );
 	virtual void slotEditAudio       ( );
 	virtual void slotDeleteSource    ( );
 	virtual void slotTranscodeAll    ( );
 	virtual void slotTranscodeNone   ( );
 	virtual void slotAllToTitleset   ( );
-	virtual void slotToNewSubmenu    ( );
 	virtual void slotAddMovieOld     ( );
 	virtual void slotAddMovie        ( );
 	virtual void slotAddSound        ( );
@@ -119,9 +121,10 @@ private:
 	QMutex             m_mutexThumbnails;
 	MediaInfo         *m_pMediaInfo;
 	DialogThumbnail   *m_pThumbnailDialog;
-	QDVDAuthor        *m_pDVDAuthor;
 	QListView         *m_pListViewAllSources;
-	QValueList<SourceFileEntry *>	m_listSourceFileEntries;
+	QValueList<SourceFileEntry *> m_listSourceFileEntries;
+	SourceFileEntry   *m_pContexMenuEntry;
+	SourceFileInfo    *m_pContexMenuInfo;
 };
 
 #endif // SOURCETOOLBAR_H
