@@ -30,6 +30,7 @@
 #include <qlcdnumber.h>
 #include <qtabwidget.h>
 #include <qscrollview.h>
+#include <qfiledialog.h>
 #include <qdragobject.h>
 #include <qpushbutton.h>
 #include <qinputdialog.h>
@@ -39,14 +40,16 @@
 #include "utils.h"
 #include "global.h"
 #include "importsrt.h"
+#include "importssa.h"
 #include "exportsrt.h"
+#include "exportssa.h"
 #include "messagebox.h"
 #include "qdvdauthor.h"
 #include "subtitlesgui.h"
 #include "listviewmedia.h"
 #include "sourcefileentry.h"
-#include "qplayer/mediacreator.h"
 #include "uidialogsubtitle.h"
+#include "qplayer/mediacreator.h"
 
 MediaLabel::MediaLabel ( SubtitlesGui *pSubtitlesGui, QWidget *pParent, const char *pName, WFlags flags )
   : QLabel ( pParent, pName, flags )
@@ -288,11 +291,12 @@ void SubtitlesGui::initMe ( )
   connect ( m_pButtonStart,      SIGNAL ( clicked ( ) ), this, SLOT ( slotStart         ( ) ) );
   connect ( m_pButtonEnd,        SIGNAL ( clicked ( ) ), this, SLOT ( slotEnd           ( ) ) );
   connect ( m_pButtonImport,     SIGNAL ( clicked ( ) ), this, SLOT ( slotImport        ( ) ) );
-  connect ( m_pButtonExport,     SIGNAL ( clicked ( ) ), this, SLOT ( slotExport        ( ) ) ); 
-  connect ( m_pButtonClear,      SIGNAL ( clicked ( ) ), this, SLOT ( slotClear         ( ) ) ); 
-  connect ( m_pButtonDelete,     SIGNAL ( clicked ( ) ), this, SLOT ( slotDelete        ( ) ) ); 
-  connect ( m_pButtonDeleteLine, SIGNAL ( clicked ( ) ), this, SLOT ( slotDeleteLine    ( ) ) ); 
-  connect ( m_pButtonChange,     SIGNAL ( clicked ( ) ), this, SLOT ( slotChange        ( ) ) ); 
+  connect ( m_pButtonExport,     SIGNAL ( clicked ( ) ), this, SLOT ( slotExport        ( ) ) );
+  connect ( m_pButtonClear,      SIGNAL ( clicked ( ) ), this, SLOT ( slotClear         ( ) ) );
+  connect ( m_pButtonDelete,     SIGNAL ( clicked ( ) ), this, SLOT ( slotDelete        ( ) ) );
+  connect ( m_pButtonDeleteLine, SIGNAL ( clicked ( ) ), this, SLOT ( slotDeleteLine    ( ) ) );
+  connect ( m_pButtonChange,     SIGNAL ( clicked ( ) ), this, SLOT ( slotChange        ( ) ) );
+  connect ( m_pButtonApplyAll,   SIGNAL ( clicked ( ) ), this, SLOT ( slotApplyToAll    ( ) ) );
   connect ( m_pButtonSearchText, SIGNAL ( clicked ( ) ), this, SLOT ( slotSearchText    ( ) ) );
   connect ( m_pButtonAddSubtitle,SIGNAL ( clicked ( ) ), this, SLOT ( slotAddSubtitle   ( ) ) );
   connect ( m_pCheckOutline,     SIGNAL ( toggled (bool)),this,SLOT ( slotOutline  ( bool ) ) );
@@ -602,42 +606,37 @@ void SubtitlesGui::displayPreview ( QImage &theSubtitle, QRect &subtitleRect )
   QImage  imgPreview;
   QPixmap preview;
   int dx, dy, iWidth, iHeight;
-  double fScale = 2.0;
+  float fScaleX, fScaleY;
+
   dx      = subtitleRect.x      ( );
   dy      = subtitleRect.y      ( );
   iWidth  = subtitleRect.width  ( );
   iHeight = subtitleRect.height ( );
-//printf ( "%s::%d > x<%d> y<%d> w<%d> h<%d> pos<%d x %d>\n", __FUNCTION__, __LINE__, dx, dy, iWidth, iHeight, subtitlePos.x ( ), subtitlePos.y ( ) );
+  fScaleX = fScaleY = 2.0;
 
   // Next we get the default.jpg from mimeSource
   imgPreview = QImage::fromMimeSource   ( "default.jpg" ).copy ( );
   if ( m_enPreviewMode == ModeScroll )  {
-    fScale = 1.0;
-    imgPreview = imgPreview.smoothScale ( m_iVideoWidth, m_iVideoHeight, QImage::ScaleMin );
+    fScaleX = fScaleY = 1.0;
+    imgPreview = imgPreview.smoothScale ( m_iVideoWidth, m_iVideoHeight );
   }
   else if ( m_enPreviewMode == ModeHalfScroll )   {
-    fScale = 0.5;
-    int iW = (int)( fScale * m_iVideoWidth  );
-    int iH = (int)( fScale * m_iVideoHeight );
-    theSubtitle = theSubtitle.smoothScale ( (int)(fScale * theSubtitle.width ( ) ), (int)(fScale * theSubtitle.height ( ) ), QImage::ScaleMin );
+    fScaleX = fScaleY = 0.5;
+    int iW = (int)( fScaleX * m_iVideoWidth  );
+    int iH = (int)( fScaleY * m_iVideoHeight );
+    theSubtitle = theSubtitle.smoothScale ( (int)(fScaleX * theSubtitle.width ( ) ), (int)(fScaleY * theSubtitle.height ( ) ) );
     if ( ( imgPreview.width ( ) != iW ) || ( imgPreview.height ( ) != iH ) )
-           imgPreview = imgPreview.smoothScale ( iW,  iH, QImage::ScaleMin );
+           imgPreview = imgPreview.smoothScale ( iW,  iH );
   }
   else if ( m_enPreviewMode == ModeFitWindow )   {
-    float fScaleX, fScaleY;
     // In this part, fScale has to be the ration of the defImgRes to the videoRes
     // bc bitBlt copies the subtitle to the full sized defImg
-    fScaleX = (float)imgPreview.width  ( ) / m_iVideoWidth;  // imgPreview.width  ( );
-    fScaleY = (float)imgPreview.height ( ) / m_iVideoHeight; // imgPreview.height ( );
-    fScale  = fScaleX;
-    if ( fScale > fScaleY )
-         fScale = fScaleY;
-    theSubtitle = theSubtitle.smoothScale ( (int)(fScale * iWidth), (int)(fScale * iHeight) );
-//printf ( "%s::%d >   scaleX<%f> scaleY<%f> scale<%f>\n", __FUNCTION__, __LINE__, fScaleX, fScaleY, fScale );
+    fScaleX = (float)imgPreview.width  ( ) / m_iVideoWidth;
+    fScaleY = (float)imgPreview.height ( ) / m_iVideoHeight;
+    theSubtitle = theSubtitle.smoothScale ( (int)(fScaleX * iWidth), (int)(fScaleY * iHeight) );
   }
 
-  bitBlt ( &imgPreview, (int)(dx * fScale), (int)(dy * fScale), &theSubtitle );
-//printf ( "%s::%d >    dx<%d> dy<%d> mode<%d> scale<%f> imgW<%d> imgH<%d>\n", __FUNCTION__, __LINE__, (int)(dx*fScale), (int)(dy*fScale), (int)m_enPreviewMode, fScale, theSubtitle.width ( ), theSubtitle.height ( ) );
+  bitBlt ( &imgPreview, (int)(dx * fScaleX), (int)(dy * fScaleY), &theSubtitle );
 
   if ( m_enPreviewMode == ModeFitWindow )
     imgPreview = imgPreview.smoothScale ( m_pScrollView->width ( ) - 6, m_pScrollView->height ( ) - 6, QImage::ScaleMin );
@@ -695,10 +694,10 @@ void SubtitlesGui::resetMediaWidget ()
 //////////////////////////////////////////////////////////
 void SubtitlesGui::slotAddSubtitle ( )
 {
-  addSubtitle ( true );
+  addSubtitle ( QString::null );
 }
 
-bool SubtitlesGui::addSubtitle ( bool )
+bool SubtitlesGui::addSubtitle ( QString qsFileName )
 {
   int  t,  iValue, iFirstNewSubtitleNumber;
   QString  qsText, qsLang, qsCode;
@@ -745,8 +744,10 @@ bool SubtitlesGui::addSubtitle ( bool )
     // First we attach the new subtitle - object to the SourceFileInfo ...
     if ( m_pSourceFileInfo->arraySubtitles[iValue] )
       delete m_pSourceFileInfo->arraySubtitles[iValue];
-    m_pSourceFileInfo->arraySubtitles [ iValue ] = new Subtitles;
+
+    m_pSourceFileInfo->arraySubtitles [ iValue ] = new Subtitles ( m_iVideoWidth, m_iVideoHeight );
     m_pSourceFileInfo->arraySubtitles [ iValue ]->m_iSubtitleNumber = iValue;
+    m_pSourceFileInfo->arraySubtitles [ iValue ]->m_qsFileName      = qsFileName;
     m_pSourceFileInfo->arraySubtitles [ iValue ]->m_qsIso639        = qsCode;
     // And finally also set the SourceFileEntry ( which is the parent )
     pEntry = sourceFileEntry ( m_pSourceFileInfo );
@@ -869,11 +870,11 @@ void SubtitlesGui::slotSubtitleChanged ( QListViewItem * )
   // Check if no subtitle exists yet for this subtitle index, then let us create one.
   if ( m_pSourceFileInfo ) {
     if ( ! m_pSourceFileInfo->arraySubtitles [ m_iCurrentSubtitleNumber ] )
-           m_pSourceFileInfo->arraySubtitles [ m_iCurrentSubtitleNumber ] = new Subtitles;
+           m_pSourceFileInfo->arraySubtitles [ m_iCurrentSubtitleNumber ] = new Subtitles ( m_iVideoWidth, m_iVideoHeight );
     m_pSourceFileInfo->arraySubtitles [ m_iCurrentSubtitleNumber ]->m_iSubtitleNumber = m_iCurrentSubtitleNumber;
   }
 
-  m_pTextEdit->setText ( tr ( "Language changed" ) );
+  m_pTextEdit->setText  ( tr ( "Language changed" ) );
   initNewSourceFileInfo ( false );
 }
 
@@ -1057,8 +1058,10 @@ void SubtitlesGui::slotAddNew ()
   QTime timeStart, timeEnd, nullTime;
   Subtitles::entry *pEntry, *pNewEntry, *pNextEntry = NULL;
 
-  if ( m_pTextEdit->text ().length () < 2 )
+  if ( m_pTextEdit->text ().length () < 2 )  {
+    MessageBox::warning ( NULL, tr ( "No text" ), tr ( "Please enter text first."), QMessageBox::Ok, QMessageBox::NoButton );
     return;
+  }
 
   timeStart = QTime::fromString ( m_strStartTime );
   timeEnd   = QTime::fromString ( m_strEndTime   );
@@ -1069,11 +1072,11 @@ void SubtitlesGui::slotAddNew ()
   if ( iStartTime > iEndTime ) {
     long iTemp = iStartTime;
     iStartTime = iEndTime;
-    iEndTime   = iTemp;  
+    iEndTime   = iTemp;
   }
 
   if ( ! m_pSubtitles )
-    m_pSubtitles = new Subtitles;
+    m_pSubtitles = new Subtitles ( m_iVideoWidth, m_iVideoHeight );
 
   for (t=0;t<m_pSubtitles->m_listOfSubtitles.count ();t++) {
     pEntry = m_pSubtitles->m_listOfSubtitles[t];
@@ -1129,8 +1132,8 @@ void SubtitlesGui::slotAddNew ()
   for ( t=0;t< m_pSubtitles->m_listOfSubtitles.count ();t++)
     m_pSubtitles->m_listOfSubtitles[t]->iIndex = t;
 
-  createTableEntries ();
-  
+  createTableEntries ( );
+
   // Oh and we should copy the end-time to the start-time
   m_pLCDNumberStart->display ( m_strEndTime );
   m_strStartTime = m_strEndTime;
@@ -1154,33 +1157,63 @@ void SubtitlesGui::slotImport ()
 {
   uint t;
   Subtitles::entry *pEntry;
-  Import::Srt theImporter;
+  QString qsImportFile = QFileDialog::getOpenFileName ( Global::qsCurrentPath, QObject::tr ( "Subtitle files (*.srt *.SRT *.ssa *.SSA *.ass *.ASS)"), NULL, QObject::tr ( "Open Subtitles File" ) );
 
-  if ( theImporter.import ( ) ) {
+  QFileInfo fileInfo ( qsImportFile );
+  Global::qsCurrentPath = fileInfo.dirPath ( true );
+
+  Import::Srt srtImporter;
+  Import::Ssa ssaImporter;
+  Import::BaseSubtitle *pImporter = &ssaImporter;
+  if ( fileInfo.extension ( false ).lower ( ) == "srt" )
+    pImporter = &srtImporter;
+
+  if ( pImporter->readFile ( qsImportFile ) )  {
     // first we should store the current table in the subtitles
     SourceFileInfo *pInfo = m_pSourceFileInfo;
     releaseSourceFileInfo ( );
     m_pSourceFileInfo = pInfo;
 
     // Okay we can now proceed ...
-    if ( !addSubtitle ( true ) ) // ask user of subtitle number, sets all required vars
+    if ( !addSubtitle ( pImporter->getFileName ( ) ) ) // ask user of subtitle number, sets all required vars
       return;
 
-    m_pTableSubtitles->setNumRows ( theImporter.m_listOfEntries.count() );
-    for (t=0;t<theImporter.m_listOfEntries.count();t++) {
-      pEntry = theImporter.m_listOfEntries[t];
-      m_pTableSubtitles->setText (t, 0, QString ("%1").arg(pEntry->iIndex ) );
-      m_pTableSubtitles->setText (t, 1, pEntry->qsTimeStart );
-      m_pTableSubtitles->setText (t, 2, pEntry->qsTimeStop  );
-      m_pTableSubtitles->setText (t, 3, pEntry->qsText      );
+    QValueList<Subtitles::entry *> list = pImporter->getSubtitles ( );
+    m_pTableSubtitles->setNumRows ( list.count ( ) );
+    for ( t=0; t<list.count ( ); t++ )  {
+      pEntry = list[t];
+      m_pTableSubtitles->setText ( t, 0, QString ( "%1" ).arg ( pEntry->iIndex ) );
+      m_pTableSubtitles->setText ( t, 1, pEntry->qsTimeStart );
+      m_pTableSubtitles->setText ( t, 2, pEntry->qsTimeStop  );
+      m_pTableSubtitles->setText ( t, 3, pEntry->qsText      );
     }
-    createSubtitles ();
+    createSubtitles ( );
   }
 }
 
+void SubtitlesGui::slotExport ()
+{
+  QString qsExportFile = QFileDialog::getSaveFileName ( Global::qsCurrentPath, tr ( "Subtitle files (*.srt *.SRT *.ass *.ASS)"), NULL, tr ( "Save Subtitles File" ) );
+  if ( qsExportFile.isEmpty ( ) )
+    return;
+
+  QFileInfo fileInfo ( qsExportFile );
+  Global::qsCurrentPath = fileInfo.dirPath ( true );
+
+  Export::Srt srtExporter;
+  Export::Ssa ssaExporter;
+  Export::BaseSubtitle *pExporter = &ssaExporter;
+  if ( fileInfo.extension ( false ) == "srt" )
+    pExporter = &srtExporter;
+
+  if ( createSubtitles ( ) )
+       pExporter->writeFile ( m_pSubtitles, qsExportFile );
+  m_pSubtitles->m_qsFileName = qsExportFile;
+}
+
 void SubtitlesGui::slotClear ()
 {
-  if ( m_pTableSubtitles->numRows () > 10 ) 
+  if ( m_pTableSubtitles->numRows () > 10 )
     if ( MessageBox::warning ( NULL, tr("Attention:"), tr("Are you sure you want to delete the current subtitles ?"), QMessageBox::Yes, QMessageBox::No ) == QMessageBox::No )
       return;
 
@@ -1197,9 +1230,10 @@ void SubtitlesGui::slotClear ()
 bool SubtitlesGui::createSubtitles ( )
 {
   uint t;
-  QString qsIso639;
+  QString qsIso639, qsFileName;
   if ( m_pSubtitles ) {
-    qsIso639 = m_pSubtitles->m_qsIso639;
+    qsIso639   = m_pSubtitles->m_qsIso639;
+    qsFileName = m_pSubtitles->m_qsFileName;
     delete m_pSubtitles;
   }
   m_pSubtitles = NULL;
@@ -1228,16 +1262,17 @@ bool SubtitlesGui::createSubtitles ( )
   else if ( m_pButtonJustify->isOn ( ) )
     iHAlignment = 4;
 
-  m_pSubtitles = new Subtitles;
-  m_pSubtitles->m_bFit              = m_pButtonFit->isOn ( );
-  m_pSubtitles->m_iOutline          = -1;
+  m_pSubtitles = new Subtitles ( m_iVideoWidth, m_iVideoHeight );
+  m_pSubtitles->m_bFit             = m_pButtonFit->isOn ( );
+  m_pSubtitles->m_iOutline         = -1;
   if ( m_pCheckOutline->isChecked ( ) )
-    m_pSubtitles->m_iOutline        = m_pEditOutline->text( ).toInt ( );
-  m_pSubtitles->m_font              = m_qfFont;
-  m_pSubtitles->m_rect              = theRect;
-  m_pSubtitles->m_qsIso639          = qsIso639;
-  m_pSubtitles->m_iSubtitleNumber   = m_iCurrentSubtitleNumber;
-  m_pSubtitles->m_alignment         = QPoint ( iHAlignment, iVAlignment );
+       m_pSubtitles->m_iOutline    = m_pEditOutline->text( ).toInt ( );
+  m_pSubtitles->m_font             = m_qfFont;
+  m_pSubtitles->m_rect             = theRect;
+  m_pSubtitles->m_qsFileName       = qsFileName;
+  m_pSubtitles->m_qsIso639         = qsIso639;
+  m_pSubtitles->m_iSubtitleNumber  = m_iCurrentSubtitleNumber;
+  m_pSubtitles->m_alignment        = QPoint ( iHAlignment, iVAlignment );
 
   m_backgroundColor.setAlpha ( (unsigned char)( ( 1.0 - m_pEditBGTransparency->text().toFloat () ) * 255.0 ) );
   m_foregroundColor.setAlpha ( (unsigned char)( ( 1.0 - m_pEditTransparency  ->text().toFloat () ) * 255.0 ) );
@@ -1419,13 +1454,6 @@ void SubtitlesGui::slotDeleteLine ()
   createSubtitles ();
 }
 
-void SubtitlesGui::slotExport ()
-{
-  Export::Srt theExporter;
-  if ( createSubtitles () )
-    theExporter.exportSrt ( m_pSubtitles );
-}
-
 void SubtitlesGui::slotTabChanged ( QWidget * )
 {
   // PageIndex = 2 => Ta with preview ...
@@ -1447,6 +1475,33 @@ void SubtitlesGui::slotTextOnly ( bool bChecked )
   }
 }
 
+void SubtitlesGui::slotApplyToAll ( )
+{
+  // This function will take the settings from the currebt selected subtitle and apply it to all subtitles.
+  if ( ! m_pSubtitles )  {
+    MessageBox::warning ( NULL, tr ( "No Subtitle selected" ), tr ( "Please select the subtitle you want to apply the setting to all other subtitles first.\n"), QMessageBox::Ok, QMessageBox::NoButton);
+    return;
+  }
+
+  createSubtitles ( );
+  int t;
+  Subtitles *pSub = NULL;
+  for ( t=0; t<MAX_SUBTITLES; t++ )  {
+    pSub = m_pSourceFileInfo->arraySubtitles [ t ];
+    if ( ( m_iCurrentSubtitleNumber != t ) && pSub )  {
+      pSub->m_bFit         = m_pSubtitles->m_bFit;
+      pSub->m_iOutline     = m_pSubtitles->m_iOutline;
+      pSub->m_font         = m_pSubtitles->m_font;
+      pSub->m_rect         = m_pSubtitles->m_rect;
+//      pSub->m_qsFileName   = m_pSubtitles->m_qsFileName;
+//      pSub->m_qsIso639     = m_pSubtitles->m_qsIso639;
+      pSub->m_alignment    = m_pSubtitles->m_alignment;
+      pSub->m_subColors[0] = m_pSubtitles->m_subColors[0];
+      pSub->m_subColors[1] = m_pSubtitles->m_subColors[1];
+    }
+  }
+}
+
 void SubtitlesGui::slotSearchText ()
 {
   QString qsText, qsCellText;
@@ -1458,8 +1513,8 @@ void SubtitlesGui::slotSearchText ()
     m_pTableSubtitles->clearSelection ( );
     for ( t=0; t<m_pTableSubtitles->numRows ( ); t++ ) {
       qsCellText = m_pTableSubtitles->text ( t, 3 );
-      if ( qsCellText.find ( qsText ) > -1 )
-	m_pTableSubtitles->selectRow ( t );
+      if ( qsCellText.find  ( qsText ) > -1 )
+           m_pTableSubtitles->selectRow ( t );
     }
   }
 }
@@ -1913,14 +1968,17 @@ void SubtitlesGui::releaseSourceFileInfo ()
 Subtitles *SubtitlesGui::getSubtitles ( bool bCreateNew )
 {
   if ( bCreateNew ) {
-    QString qsLang;
+    QString qsLang, qsFileName;
     // We will create a vanilla Subtitles - object
     if ( m_pSubtitles ) {
-      qsLang = m_pSubtitles->m_qsIso639;
-      delete   m_pSubtitles;
+      qsFileName = m_pSubtitles->m_qsFileName;
+      qsLang     = m_pSubtitles->m_qsIso639;
+      delete       m_pSubtitles;
     }
-    m_pSubtitles = new Subtitles;
-    m_pSubtitles->m_qsIso639 = qsLang;
+
+    m_pSubtitles = new Subtitles ( m_iVideoWidth, m_iVideoHeight );
+    m_pSubtitles->m_qsFileName = qsFileName;
+    m_pSubtitles->m_qsIso639   = qsLang;
     m_pTableSubtitles->setNumRows ( 0 );
   }
   else // Or the default we create the Subtitles from the table.
@@ -1978,12 +2036,13 @@ void SubtitlesGui::initNewSourceFileInfo ( bool bRecreateListView )
     Rgba rgbInitialForeground ( START_SUBTITLEFG_COLOR );
     m_iCurrentSubtitleNumber      = 0;
     m_pTableSubtitles->setNumRows ( 0 );
+    m_pListViewSubtitles->clear    ( );
     setBackgroundColor ( rgbInitialBackground );
     setForegroundColor ( rgbInitialForeground );
     return;
   }
 
-  m_pSubtitles =  new Subtitles;
+  m_pSubtitles =  new Subtitles ( m_iVideoWidth, m_iVideoHeight );
  *m_pSubtitles = *m_pSourceFileInfo->arraySubtitles [ m_iCurrentSubtitleNumber ];
 
   m_pButtonFit->setOn         ( m_pSubtitles->m_bFit );
