@@ -361,7 +361,7 @@ void TemplateDownload::downloadThumbnail ( int iCurrentEntry )
   if ( iCurrentEntry < m_xml.entryCount ( ) ) {
     Xml::Entry *pEntry = m_xml.entry ( iCurrentEntry );
     if ( pEntry ) {
-      qsStatus = tr ( "Retrieving Thumbnail : %1" ).arg ( pEntry->qsThumbnail );
+      qsStatus = tr ( "Retrieving Thumbnail<br> %1" ).arg ( pEntry->qsThumbnail );
       m_pEditConnect->append ( tr ( "Getting Thumbnail : %1" ).arg ( pEntry->qsThumbnail ) );
       displayStatus ( qsStatus );
 
@@ -583,7 +583,7 @@ void TemplateDownload::downloadTemplate ( int iCurrentEntry )
       return;
     }
     if ( pEntry ) {
-      m_pEditDownload->append ( tr ( "Retrieving Template : %1. Size = %2" ).arg ( pEntry->qsName ).arg ( pEntry->iLocationSize ) );
+      m_pEditDownload->append ( tr ( "Retrieving Template<br>%1. Size = %2" ).arg ( pEntry->qsName ).arg ( pEntry->iLocationSize ) );
 
       QString qsUrl = m_pComboServer->currentText ( ) + "/" + pEntry->qsLocation;
       if ( ! verifyUrl ( qsUrl ) ) { 
@@ -613,7 +613,7 @@ void TemplateDownload::installTemplates ( )
   qsTemplatePath = "~/.qdvdauthor/";
   if ( m_pRadioSystemWide->isChecked ( ) )
     qsTemplatePath = Global::qsSystemPath + "/share/qdvdauthor/";
-  
+
   if ( file.open( IO_WriteOnly ) ) {
     QTextStream stream( &file );
     stream << Global::qsBashString;
@@ -623,41 +623,20 @@ void TemplateDownload::installTemplates ( )
 
     stream << "cd " + qsTemplatePath + "\n";
 
-    for (t=0;t<m_xml.entryCount ();t++) {
+    for ( t=0; t<m_xml.entryCount ( ); t++ ) {
       Xml::Entry *pEntry = m_xml.entry ( t );
-      if ( pEntry && pEntry->bSelectedForInstall )
-	stream << "tar -xzf /tmp/" + pEntry->qsLocation + "\n";
+      if( pEntry && pEntry->bSelectedForInstall )
+        stream << "tar -xzf /tmp/" + pEntry->qsLocation + "\n";
     }
     file.close();
   }
   // And here we grant read/write/execute permission.
-  chmod ((const char *)QString( qsExecutable ), S_IEXEC | S_IRUSR | S_IWRITE | S_IROTH | S_IXOTH | S_IWOTH );
-  
+  chmod ((const char *)QString( qsExecutable ), S_IXGRP | S_IWGRP | S_IRGRP | S_IEXEC | S_IRUSR | S_IWRITE | S_IROTH | S_IXOTH | S_IWOTH );
+
   // install the Templates ...
   installToSystem ( qsExecutable, m_pRadioLocally->isChecked ( ) );
 }
 
-QString TemplateDownload::findGuiSU ( )
-{
-  int iRet = 0;
-  uint t;
-  QString qsReturn;
-  QString qsTmpOut = QString ( "%1/%2_out.txt" ).arg ( Global::qsTempPath ).arg ( getlogin ( ) );
-  char arraySUs[3][10] = {"kdesu", "xsu", "gnomesu"};
-  QFileInfo fileInfo( qsTmpOut );
-  for (t=0;t<3;t++)	{
-    // the next check is to see if we can find kdesu
-    iRet = system ((const char *)QString ("which %1 > %2 2>/dev/null").arg( arraySUs[t] ).arg ( qsTmpOut ) );
-    fileInfo.setFile( qsTmpOut );
-    // Okay we cna not find it, so we wont bother asking the user ...
-    if (fileInfo.size() > 4)	{
-      qsReturn = QString ("%1").arg(arraySUs[t]);
-      return qsReturn;
-    }
-  }
-  return qsReturn;
-}
-
 void TemplateDownload::installToSystem ( QString &qsCommand, bool bInstallLocally )
 {
   if (m_pNewProcess)	{
@@ -667,6 +646,7 @@ void TemplateDownload::installToSystem ( QString &qsCommand, bool bInstallLocall
       delete m_pNewProcess;
   }
 
+  Utils theUtils;
   m_pNewProcess  = new QProcess ( this );
   connect( m_pNewProcess, SIGNAL ( readyReadStdout ( ) ), this, SLOT ( slotReadFromProcess ( ) ) );
   connect( m_pNewProcess, SIGNAL ( readyReadStderr ( ) ), this, SLOT ( slotReadFromProcess ( ) ) );
@@ -674,7 +654,7 @@ void TemplateDownload::installToSystem ( QString &qsCommand, bool bInstallLocall
 
   if ( ! bInstallLocally ) {
     // the next check is to see if we can find kdesu
-    QString qsGuiSU = findGuiSU ( );
+    QString qsGuiSU = theUtils.findGuiSU ( );
     if ( qsGuiSU.isEmpty ( ) ) {
       MessageBox::information (NULL, tr ( "Could not find kdesu, xsu, or gnomesu" ),
         tr ( "Without kdesu, xsu, or gnomesu I can not istall them templates in %1/share/qdvdauthor/.\n").
@@ -705,8 +685,8 @@ void TemplateDownload::installToSystem ( QString &qsCommand, bool bInstallLocall
     }
   }
   m_pNewProcess->addArgument ( qsCommand );
-  
-  m_pNewProcess->start ();
+
+  m_pNewProcess->start ( );
 }
 
 void TemplateDownload::slotReadFromProcess  ( )
