@@ -17,7 +17,7 @@
 ** template.xml -> holds information about the template, such as comment
 **                 NrOfDropZones, coordinates of DropZones etc.
 ** thumbnail.png-> Image to display in selectionBox ( width=200 )
-** background/backgroundXXXXX.jpg where XXXXX = sequenceNo. 
+** background/backgroundXXXXX.jpg where XXXXX = sequenceNo.
 ** background/background00000.jpg REQUIRED.
 ** template.mpg -> preview for menus
 ** dropZone[X].mpg -> default for the dropZone, with X = [1 .. N]
@@ -506,20 +506,18 @@ void TemplateWizard::droppedSomething (QDropEvent *pDropEvent)
       }
     }
   }
-//  DragNDropContainer *pDND = Global::pApp->getDragNDropContainer ( );
-//  pDropZone->bMovie = pDND ? ( ! pDND->bImageButton ) : true;
 
-  if ( ( pDropZone->iType != DropZone::TextObject ) && ( pDropZone->iType != DropZone::FrameObject ) )  {
-    if (pInfo && pInfo->pPreview) {
+//  if ( ( pDropZone->iType != DropZone::TextObject ) && ( pDropZone->iType != DropZone::FrameObject ) )  {
+    if ( pInfo && pInfo->pPreview )  {
       float fRatio = (float)pInfo->pPreview->height ( ) / pInfo->pPreview->width ( );
       // for the templates we want to use the width as our ruler (I.e. adjust the height of the image according to the ratio
-      iHeight = (int)(fRatio * pDropZone->rect.width ( ) );
-      theImage = pInfo->pPreview->smoothScale ((int)(m_fScale*pDropZone->rect.width ( ) ),(int)(m_fScale * iHeight), QImage::ScaleMin);
+      iHeight = (int)( fRatio * pDropZone->rect.width ( ) );
+      theImage = pInfo->pPreview->smoothScale ( (int)( m_fScale * pDropZone->rect.width ( ) ),(int)( m_fScale * iHeight ), QImage::ScaleMin );
       //theImage = pInfo->pPreview->smoothScale ((int)(m_fScale*pDropZone->rect.width()), (int)(m_fScale * pDropZone->rect.height()), QImage::ScaleMin);
     }
-    else	{
-      theImage = QImage().fromMimeSource( "error.jpg" );
-      theImage = theImage.smoothScale ((int)(m_fScale*pDropZone->rect.width()), (int)(m_fScale * pDropZone->rect.height()), QImage::ScaleFree );
+    else  {
+      theImage = QImage ( ).fromMimeSource( "error.jpg" );
+      theImage = theImage.smoothScale ( (int)( m_fScale*pDropZone->rect.width ( ) ), (int)( m_fScale * pDropZone->rect.height ( ) ), QImage::ScaleFree );
     }
 
     DragNDropContainer *pDND = Global::pApp->getDragNDropContainer ( );
@@ -533,12 +531,12 @@ void TemplateWizard::droppedSomething (QDropEvent *pDropEvent)
     // Set the DropZoneValues ...
     // Note: theText is "$FileName : $size" so that does not work.
     // pDropZone->qsFileName = theText;
-    if (pDropZone->pPixmap)
+    if ( pDropZone->pPixmap )
       delete pDropZone->pPixmap;
     pDropZone->pPixmap = new QPixmap;
-    pDropZone->pPixmap->convertFromImage (theImage);
-  }
-  refreshBackground ();
+    pDropZone->pPixmap->convertFromImage ( theImage );
+//  }
+  refreshBackground ( );
 }
 
 void TemplateWizard::slotTargetMenuChanged (const QString &qsMenuName)
@@ -585,25 +583,25 @@ void TemplateWizard::slotTemplateClicked (QMouseEvent *pMouseEvent)
   Global::qsCurrentPath = backgroundInfo.dirPath ( TRUE );
 
   pDropZone->qsFileName = qsFileName;
-  qsExtensions = pMediaInfo->getExtensions (true);
+  qsExtensions = pMediaInfo->getExtensions ( true );
   if (qsExtensions.find ("."+backgroundInfo.extension(FALSE), 0, FALSE) > -1)	{
     pDropZone->bMovie = true;
-    if ( ( pDropZone->iType == DropZone::TextObject ) || ( pDropZone->iType == DropZone::FrameObject ) ) {
-      refreshBackground ( );
-      return;
-    }
+//    if ( ( pDropZone->iType == DropZone::TextObject ) || ( pDropZone->iType == DropZone::FrameObject ) ) {
+//      refreshBackground ( );
+//      return;
+//    }
     // Okay let us get the first frame of the game as a screenshot ...
-    pMediaInfo->setFileName(qsFileName);
-    theImage = pMediaInfo->getScreenshot(0);
+    pMediaInfo->setFileName ( qsFileName );
+    theImage = pMediaInfo->getScreenshot ( 0 );
     // The we save the screenshot in the temp dir
-    qsFileName = theUtil.getTempFile(backgroundInfo.baseName())+QString (".png");
-    theImage.save (qsFileName, "PNG");
+    qsFileName = theUtil.getTempFile ( backgroundInfo.baseName ( ) ) + QString ( ".png" );
+    theImage.save ( qsFileName, "PNG" );
     DragNDropContainer *pDND = Global::pApp->getDragNDropContainer ( );
     if ( pDND && pDND->bImageButton )
          pDropZone->qsImagePreview = qsFileName;
   }
-  else	// Not a movie file, then we should reset this ...
-    theImage = QImage(qsFileName);
+  else  // Not a movie file, then we should reset this ...
+    theImage = QImage ( qsFileName );
 
   if ( ( pDropZone->iType != DropZone::TextObject ) && ( pDropZone->iType != DropZone::FrameObject ) ) {
     theImage = theImage.smoothScale ((int)(m_fScale*pDropZone->rect.width()), (int)(m_fScale * pDropZone->rect.height()), QImage::ScaleMin);
@@ -613,7 +611,7 @@ void TemplateWizard::slotTemplateClicked (QMouseEvent *pMouseEvent)
     pDropZone->pPixmap = new QPixmap;
     pDropZone->pPixmap->convertFromImage (theImage);
   }
-  refreshBackground ();
+  refreshBackground ( );
 }
 
 QString TemplateWizard::readProjectFile (QString qsFileName, bool bHeaderOnly)
@@ -831,13 +829,22 @@ void TemplateWizard::reject ()
 	QWizard::reject();
 }
 
+// +DROP_ZONE_x_IMOV_TYPE+   Replaces Object with either Image or Movie Object
+// +DROP_ZONE_x_OBJECT_TYPE+ Replaces Object with img/mov/text/frame
+// +DROP_ZONE_x_BUTTON+      Replaces ButtonObject with IgnoreObject if required
+// +DROP_ZONE_x_FILE_NAME+   Replaces with proper file name from the drop
+// +DROP_ZONE_x_BASE_NAME+   Replaces the object name
+// +DROP_ZONE_x_GEOMETRY+    Replaces the geometry such that it stays cenetered
+// +DROP_ZONE_x_MOVIE_SIZE+  
+// +DROP_ZONE_x_ACTION+      Replaces with the proper buton action string.
+// +DROP_ZONE_SOURCE_ENTRY+  Replaces 
 void TemplateWizard::accept ()
 {
 	// First we are going to replace the DropZone placeholder and then create a DVDMenu.
 	uint t;
 	int i, iFreeSlot, iX, iY, iWidth, iHeight;
 	QString qsAction, qsEntryName, qsEntry, qsPrefixDir, qsMenuName;
-	QString qsGeometry, qsCurrentPath, qsType, qsExtensions;
+	QString qsGeometry, qsCurrentPath, qsType, qsTypeImov, qsExtensions;
 	QFileInfo fileInfo;
 	QStringList listAddMovies;
 	DropZone *pDropZone;
@@ -859,19 +866,21 @@ void TemplateWizard::accept ()
 		i = pDropZone->iNumber;
 
 		fileInfo.setFile ( pDropZone->qsFileName );
+
 		bDropToImage = false;
 		qsExtensions = Global::pApp->getImageFilter ( );
 		if ( qsExtensions.find ( "."+fileInfo.extension ( FALSE ), 0, FALSE ) > -1 )
 		  bDropToImage = true;
 
 		if ( pDropZone->qsFileName.isEmpty())	{
+			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_IMOV_TYPE+"  ).arg(i), QString ("IgnoreObject"));
 			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_OBJECT_TYPE+").arg(i), QString ("IgnoreObject"));
-			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_BUTTON+").arg(i), QString ("IgnoreObject"));
-			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_FILE_NAME+").arg(i), QString ());
-			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_BASE_NAME+").arg(i), QString ());
-			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_GEOMETRY+").arg(i), QString ());
-			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_MOVIE_SIZE+").arg(i), QString ());
-			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_ACTION+").arg(i), QString ());
+			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_BUTTON+"     ).arg(i), QString ("IgnoreObject"));
+			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_FILE_NAME+"  ).arg(i), QString ());
+			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_BASE_NAME+"  ).arg(i), QString ());
+			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_GEOMETRY+"   ).arg(i), QString ());
+			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_MOVIE_SIZE+" ).arg(i), QString ());
+			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_ACTION+"     ).arg(i), QString ());
 			continue;
 		}
 		
@@ -885,15 +894,15 @@ void TemplateWizard::accept ()
 		  qsSourceEntry.replace ( "<", "&lt;" );
 		  qsSourceEntry.replace ( ">", "&gt;" );
 		  rx.setMinimal ( true );
-		  qsPattern = QString ( "DROP_ZONE_%1_BUTTON[^>]*>.*</.DROP_ZONE_%2_BUTTON.>" ).arg ( i ).arg ( i );		  
+		  qsPattern = QString ( "DROP_ZONE_%1_BUTTON[^>]*>.*</.DROP_ZONE_%2_BUTTON.>" ).arg ( i ).arg ( i );
 		  rx.setPattern ( qsPattern );
 
 		  if ( m_qsXMLDVDMenu.find ( rx ) > -1 ) {
-		    qsButtonXml = rx.cap ( 0 );
-		    if ( qsButtonXml.find ( "+DROP_ZONE_SOURCE_ENTRY+" ) > -1 )
-		      qsButtonXml.replace ( "+DROP_ZONE_SOURCE_ENTRY+", qsSourceEntry );
+		    qsButtonXml = rx.cap   ( 0 );
+		    if ( qsButtonXml.find  ( "+DROP_ZONE_SOURCE_ENTRY+" ) > -1 )
+		      qsButtonXml.replace  ( "+DROP_ZONE_SOURCE_ENTRY+", qsSourceEntry );
 		    else // Could not find in template ... create on the spot.
-		      qsButtonXml.replace ( "Action=", "SourceEntry=\"" + qsSourceEntry + "\" Action=" );
+		      qsButtonXml.replace  ( "Action=", "SourceEntry=\"" + qsSourceEntry + "\" Action=" );
 		  }
 		  // TODO: fix this function ...
 		  adjustPixmapHeight ( pDropZone->pPixmap, qsButtonXml );
@@ -928,6 +937,7 @@ void TemplateWizard::accept ()
 		  if ( ! bDropToImage )
 		    qsType = "MovieObject";
 		}
+		qsTypeImov = qsType;
 		// Note we do not want to change Frames or text ...
 		if ( pDropZone->iType == DropZone::TextObject )
 		  qsType = "TextObject";
@@ -944,12 +954,14 @@ void TemplateWizard::accept ()
 			qsEntry.replace ("<", "&lt;");
 			qsEntry.replace (">", "&gt;");
 			qsAction = QString ( "jump+-+%1+-+%2+-+00:00:00.000").arg(qsEntry).arg(fileInfo.fileName ( ) );
-			m_qsXMLDVDMenu.replace ( QString ( "+DROP_ZONE_%1_OBJECT_TYPE+").arg( i ),  qsType  );
+			m_qsXMLDVDMenu.replace ( QString ( "+DROP_ZONE_%1_IMOV_TYPE+"  ).arg( i ),  qsTypeImov  );
+			m_qsXMLDVDMenu.replace ( QString ( "+DROP_ZONE_%1_OBJECT_TYPE+").arg( i ),  qsType      );
 			m_qsXMLDVDMenu.replace ( QString ( "+DROP_ZONE_%1_MOVIE_SIZE+" ).arg( i ),  QString ( "720x480" ) );
 		}
 		else	{	// The image button should jump to the MainMenu  jump+-+vmgm+-+
 			qsAction =  QString  ( "jump+-+vmgm+-+" );
-			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_OBJECT_TYPE+").arg(i), qsType );
+			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_IMOV_TYPE+"  ).arg ( i ), qsTypeImov );
+			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_OBJECT_TYPE+").arg ( i ), qsType );
 		}
 		if ( ! bDropToImage && pDND ? pDND->bImageButton : false )
 			fileInfo.setFile ( pDropZone->qsImagePreview );
