@@ -29,7 +29,6 @@
 
 class TemplateWizard;
 class MediaInterface;
-class QDVDAuthor;
 class DVDMenu;
 
 // This class was derived to have the mouseClick and drop events covered
@@ -83,13 +82,14 @@ public:
 		QString  qsText;	// E.g "Movie 1" used for ???
 		QRect    rect;
 		QString  qsFileName;
+		QString  qsImagePreview;
 		QPixmap *pPixmap;
 	};
 
 	 TemplateWizard( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
 	~TemplateWizard();
 
-	void initMe             ( QDVDAuthor * );
+	void initMe             ( );
 	void droppedSomething   ( QDropEvent * );
 	void showDialog         ( );
 
@@ -117,9 +117,8 @@ protected slots:
 	virtual void accept   ( );
 	virtual void hide     ( );
 	virtual void slotHelp ( );
-    
+
 private:
-	QDVDAuthor              *m_pDVDAuthor;
 	TemplateFrame           *m_pFrameProperties;
 	MediaInterface          *m_pMediaInterface;
 	ListBoxToolTip          *m_pListBoxToolTip;
