@@ -24,15 +24,77 @@
 #include "menuobject.h"
 #include "textobject.h"	// derived from menuobject
 #include "structuretoolbar.h"
+#include "qdvdauthor.h"
+#include "dvdmenuundoobject.h"
+
+TextObject::TextData::TextData ( )
+  : MenuObject::Data ( )
+{
+  m_iDataType       = Data::TEXT;
+  m_colorBackground = Rgba ( TRANSPARENT_COLOR ); // indicate the transparent color.
+  m_iStyleStrategy  = QFont::PreferAntialias;
+  m_bAllowWordBreak = true;
+}
+
+TextObject::TextData::~TextData ( )
+{
+}
+
+MenuObject::Data &TextObject::TextData::operator = ( Data &theOther )
+{
+  // At this point we are sure both are acutally ImageManipulator objects.
+  MenuObject::Data::operator= ( theOther );
+  if ( m_iDataType == theOther.m_iDataType )  {
+       TextData *pOther  = (TextData *)&theOther;
+       m_qsText          = pOther->m_qsText;
+       m_Font            = pOther->m_Font;
+       m_ptTextAlign     = pOther->m_ptTextAlign;
+       m_colorForeground = pOther->m_colorForeground;
+       m_colorBackground = pOther->m_colorBackground;
+       m_iStyleStrategy  = pOther->m_iStyleStrategy;
+       // don't need ot be stored. Only set after DialogTextFont
+       m_bAllowWordBreak = pOther->m_bAllowWordBreak;
+  }
+
+  return *this;
+}
+
+bool TextObject::TextData::operator == ( Data &theOther )
+{
+  // At this point we are sure both are acutally ImageManipulator objects.
+  bool bReturn = ( MenuObject::Data::operator == ( theOther ) );
+  if ( ! bReturn )
+    return false;
+
+  if ( m_iDataType  == theOther.m_iDataType )  {
+    TextData *pOther = (TextData *)&theOther;
+    bReturn = ( ( m_qsText          == pOther->m_qsText          ) &&
+                ( m_Font            == pOther->m_Font            ) &&
+                ( m_ptTextAlign     == pOther->m_ptTextAlign     ) &&
+                ( m_colorForeground == pOther->m_colorForeground ) &&
+                ( m_colorBackground == pOther->m_colorBackground ) &&
+                ( m_iStyleStrategy  == pOther->m_iStyleStrategy  ) &&
+                ( m_bAllowWordBreak == pOther->m_bAllowWordBreak ) );
+  }
+  return bReturn;
+}
+
+MenuObject::Data *TextObject::TextData::clone ( )
+{
+  TextData *pData = new TextData;
+  *pData  = *this;
+  return pData;
+}
 
 TextObject::TextObject ( QWidget *pParent )
-	: MenuObject   ( pParent )
+  : MenuObject ( pParent )
 {
-	m_qsObjectType    = QString ( TEXT_OBJECT );
-	m_colorBackground = Rgba    ( TRANSPARENT_COLOR );   // indicate the transparent color.
-	m_iStyleStrategy  = QFont::PreferAntialias;
-	m_pContextMenu    = NULL;
-	m_bAllowWordBreak = true;
+  if ( m_pData )
+    delete m_pData;        // part of the base class
+  m_pData = new TextData; // will be deleted in base class
+
+  m_qsObjectType    = QString ( TEXT_OBJECT );
+  m_pContextMenu    = NULL;
 }
 
 TextObject::~TextObject ( )
@@ -42,25 +104,30 @@ TextObject::~TextObject ( )
   m_pContextMenu = NULL;
 }
 
+TextObject::TextData *TextObject::textData ( )
+{
+  return (TextData *)m_pData;
+}
+
 void TextObject::setFont ( QFont &theFont )
 {
-  m_Font = theFont;
+  textData ( )->m_Font = theFont;
 }
 
 QFont &TextObject::getFont ()
 {
-  return m_Font;
+  return textData ( )->m_Font;
 }
 
 void TextObject::setWordBreak ( bool bBreak )
 {
-  m_bAllowWordBreak = bBreak;
+  textData ( )->m_bAllowWordBreak = bBreak;
 }
 
 void TextObject::setText (QString &theText)
 {
-  m_qsText = theText;
-  
+  textData ( )->m_qsText = theText;
+
   // Here we seal off after 20 char and remove special characters from the Object Name.
   QString qsTemp = theText;
   qsTemp = qsTemp.remove  ( '\n' );
@@ -73,28 +140,28 @@ void TextObject::setText (QString &theText)
 
 QString &TextObject::getText ()
 {
-	return m_qsText;
+	return textData ( )->m_qsText;
 }
 
 void TextObject::setTextAlign (QPoint &newAlign)
 {
-	m_ptTextAlign = newAlign;
+	textData ( )->m_ptTextAlign = newAlign;
 }
 
 QPoint &TextObject::getTextAlign ()
 {
-	return m_ptTextAlign;
+	return textData ( )->m_ptTextAlign;
 }
 
 void TextObject::setStyleStrategy (int iStyleStrategy)
 {
-	m_iStyleStrategy = iStyleStrategy;
-	m_Font.setStyleStrategy ((QFont::StyleStrategy)iStyleStrategy);
+	textData ( )->m_iStyleStrategy = iStyleStrategy;
+	textData ( )->m_Font.setStyleStrategy ((QFont::StyleStrategy)iStyleStrategy);
 }
 
 int TextObject::getStyleStrategy ()
 {
-  return (int)m_iStyleStrategy;
+  return (int)textData ( )->m_iStyleStrategy;
 }
 
 void TextObject::replaceColor ( Rgba theColor, Rgba oldColor )
@@ -107,22 +174,22 @@ void TextObject::replaceColor ( Rgba theColor, Rgba oldColor )
 
 void TextObject::setForegroundColor ( const Rgba &theColor)
 {
-  m_colorForeground = theColor;
+  textData ( )->m_colorForeground = theColor;
 }
 
 void TextObject::setBackgroundColor ( const Rgba &theColor)
 {
-  m_colorBackground = theColor;
+  textData ( )->m_colorBackground = theColor;
 }
 
 Rgba &TextObject::getForegroundColor ()
 {
-  return m_colorForeground;
+  return textData ( )->m_colorForeground;
 }
 
 Rgba &TextObject::getBackgroundColor ()
 {
-  return m_colorBackground;
+  return textData ( )->m_colorBackground;
 }
 
 void TextObject::drawSemiTransparent (QPainter *pPainter)
@@ -144,7 +211,7 @@ void TextObject::drawSemiTransparent (QPainter *pPainter)
   fTargetTrans = 1.0 - modifiers()->fTransparency;
   fSourceTrans = modifiers()->fTransparency;
 
-  QPen thePen ( m_colorForeground.color ( ), 2, Qt::DashDotLine );
+  QPen thePen ( textData ( )->m_colorForeground.color ( ), 2, Qt::DashDotLine );
   // Here we define the cenetered rect.
   theRect = QRect((int)-(rect().width()/ 2.0), (int)-(rect().height()/2.0), rect().width(), rect().height());
 
@@ -174,19 +241,19 @@ void TextObject::drawSemiTransparent (QPainter *pPainter)
   pmPainter.setWorldMatrix(theMatrix);
 
   pmPainter.setPen   ( Qt::NoPen );
-  pmPainter.setBrush ( QBrush    ( m_colorBackground.color ( ) ) );
+  pmPainter.setBrush ( QBrush    ( textData ( )->m_colorBackground.color ( ) ) );
   pmPainter.drawRect ( theRect   );
 
-  m_Font.setStyleStrategy ((QFont::StyleStrategy)m_iStyleStrategy);
-  pmPainter.setFont(m_Font);
+  textData ( )->m_Font.setStyleStrategy ((QFont::StyleStrategy)textData ( )->m_iStyleStrategy);
+  pmPainter.setFont(textData ( )->m_Font);
 
   // Okay, I abused a QPoint class to hold some other infos ... so what ???
-  //iFlags = Qt::DontClip|Qt::WordBreak|m_ptTextAlign.x()|m_ptTextAlign.y();
-  iFlags = Qt::DontClip|m_ptTextAlign.x ( )|m_ptTextAlign.y ( );
-  if ( m_bAllowWordBreak )
+  //iFlags = Qt::DontClip|Qt::WordBreak|textData ( )->m_ptTextAlign.x()|textData ( )->m_ptTextAlign.y();
+  iFlags = Qt::DontClip|textData ( )->m_ptTextAlign.x ( )|textData ( )->m_ptTextAlign.y ( );
+  if ( textData ( )->m_bAllowWordBreak )
     iFlags |= Qt::WordBreak;
 
-  pmPainter.drawText(theRect, iFlags, m_qsText, -1, &dummyRect);
+  pmPainter.drawText(theRect, iFlags, textData ( )->m_qsText, -1, &dummyRect);
 
   // At this point we should have the text on a memory Bitmap object, and now 
   // we should copy this over, bit for bit. 
@@ -235,7 +302,7 @@ void TextObject::drawSemiTransparent (QPainter *pPainter)
   }
 }
 
-void TextObject::drawContents (QPainter *painter)
+void TextObject::drawContents ( QPainter *painter )
 {
   QWMatrix theMatrix;
   // If we have a complete transparent text we should not draw at all.
@@ -267,7 +334,7 @@ void TextObject::drawContents (QPainter *painter)
   theMatrix.rotate ( modifiers ( )->fRotate);
   theMatrix.shear  ( modifiers ( )->fShearX, modifiers ( )->fShearY );
 
-  QPen thePen ( m_colorForeground.color ( ), 2, Qt::DashDotLine);
+  QPen thePen ( textData ( )->m_colorForeground.color ( ), 2, Qt::DashDotLine );
   painter->setPen ( thePen );
 
   // Here we get the rect that encompasses the button for this object.
@@ -279,10 +346,10 @@ void TextObject::drawContents (QPainter *painter)
   painter->setWorldMatrix ( theMatrix );
 
   // and continue to draw the contents ...
-  if ( m_colorBackground.alpha ( ) > 0 )  {
+  if ( textData ( )->m_colorBackground.alpha ( ) > 0 )  {
     // No frame wanted, we take care of that ourself if requested ...
     painter->setPen   ( Qt::NoPen );
-    painter->setBrush ( QBrush ( m_colorBackground.color ( ) ) );
+    painter->setBrush ( QBrush ( textData ( )->m_colorBackground.color ( ) ) );
     painter->drawRect ( theRect );
     // and re-set the PEN.
     painter->setPen   ( thePen );
@@ -290,26 +357,26 @@ void TextObject::drawContents (QPainter *painter)
   else
     painter->setBrush ( Qt::NoBrush );
 
-  m_Font.setStyleStrategy ( (QFont::StyleStrategy)m_iStyleStrategy );
-  painter->setFont ( m_Font );
+  textData ( )->m_Font.setStyleStrategy ( (QFont::StyleStrategy)textData ( )->m_iStyleStrategy );
+  painter->setFont ( textData ( )->m_Font );
   QRect boundingRect;
   bool  bUpdateAgain = false;
   // Okay, I abused a QPoint class to hold some other infos ... so what ???
-  int iFlags = Qt::DontClip|m_ptTextAlign.x ( )|m_ptTextAlign.y ( );
-  if ( m_bAllowWordBreak )
+  int iFlags = Qt::DontClip|textData ( )->m_ptTextAlign.x ( )|textData ( )->m_ptTextAlign.y ( );
+  if ( textData ( )->m_bAllowWordBreak )
     iFlags |= Qt::WordBreak;
 
   if ( m_bDrawRect )
     painter->drawRect ( theRect );
 
-  painter->drawText ( theRect, iFlags, m_qsText, -1, &boundingRect );
+  painter->drawText ( theRect, iFlags, textData ( )->m_qsText, -1, &boundingRect );
   // Here we expand the rectangle if needs be, and update again, to re-draw.
-  if ( m_rect.width ( ) < boundingRect.width ( ) )  {
-    m_rect.setWidth ( boundingRect.width ( ) );
+  if ( rect ( ).width ( ) < boundingRect.width ( ) )  {
+    rect ( ).setWidth ( boundingRect.width ( ) );
     bUpdateAgain = true;
   }
-  if ( m_rect.height ( ) < boundingRect.height ( ) )  {
-    m_rect.setHeight ( boundingRect.height ( ) );
+  if ( rect ( ).height ( ) < boundingRect.height ( ) )  {
+    rect ( ).setHeight ( boundingRect.height ( ) );
     bUpdateAgain = true;
   }
 
@@ -328,9 +395,9 @@ void TextObject::drawContents (QPainter *pPainter, int iRenderFrameNumber, int)
   // Next is to set the AnimationAttributes
   for (uint t=0;t<m_listAnimationAttributes.count();t++)
     m_listAnimationAttributes[t]->setValue (iRenderFrameNumber);
-  
+
   // If start stop have been set then we check the range
-  if ( ( ( iRenderFrameNumber < modifiers ( )->iStartFrame ) && ( modifiers ( )->iStartFrame != -1 ) ) || 
+  if ( ( ( iRenderFrameNumber < modifiers ( )->iStartFrame ) && ( modifiers ( )->iStartFrame != -1 ) ) ||
        ( ( iRenderFrameNumber > modifiers ( )->iStopFrame  ) && ( modifiers ( )->iStopFrame  != -1 ) ) ) 
     return;
 
@@ -341,11 +408,11 @@ bool TextObject::readProjectFile  (QDomNode &theNode)
 {
   QDomElement theElement = theNode.toElement();
   QDomAttr attribute;
-  
+
   attribute = theElement.attributeNode ( TEXT_OBJECT_TEXT );
   if (!attribute.isNull())
-    m_qsText = attribute.value();
-  
+    textData ( )->m_qsText = attribute.value();
+
   attribute = theElement.attributeNode ( TEXT_OBJECT_TEXT_ALIGN );
   if (!attribute.isNull())	{
     // QString "x,y" - we abused a QPoint object for the Align info.
@@ -355,10 +422,10 @@ bool TextObject::readProjectFile  (QDomNode &theNode)
       textAlign = QPoint(list[0].toInt(), list[1].toInt());
     setTextAlign (textAlign);
   }
-  
+
   attribute = theElement.attributeNode ( TEXT_OBJECT_STYLE_STRATEGY );
   if (!attribute.isNull())
-    m_iStyleStrategy = attribute.value().toInt();
+    textData ( )->m_iStyleStrategy = attribute.value().toInt();
   
   attribute = theElement.attributeNode ( TEXT_OBJECT_FONT);
   if (!attribute.isNull())	{
@@ -393,7 +460,7 @@ bool TextObject::writeProjectFile (QDomElement &theElement)
 	TextObject initialValues;
 	// Here we set the attributes of the <dvdauthor> tag
 	if (!getText().isNull())
-		textNode.setAttribute( TEXT_OBJECT_TEXT, m_qsText );
+		textNode.setAttribute( TEXT_OBJECT_TEXT, textData ( )->m_qsText );
 	if (getTextAlign() != initialValues.getTextAlign())
 		textNode.setAttribute( TEXT_OBJECT_TEXT_ALIGN, QString("%1,%2").arg(getTextAlign().x()).arg(getTextAlign().y()) );
 	if (getStyleStrategy() != initialValues.getStyleStrategy())
@@ -426,9 +493,9 @@ bool TextObject::createContextMenu (QPoint globalPos)
 	if (m_pContextMenu)
 		delete m_pContextMenu;
 	m_pContextMenu = new QPopupMenu(m_pParent);
-	m_pContextMenu->insertItem ( tr ("Edit ...")  , this,  SLOT(slotEdit      ( ) ) );
-	m_pContextMenu->insertItem ( tr ("Matrix ..."), this,  SLOT(slotMatrix    ( ) ) );
-	m_pContextMenu->insertItem ( tr ("Delete")    , this,  SLOT(slotDelete    ( ) ) );
+	m_pContextMenu->insertItem ( tr ("Edit ...")  , this,  SLOT ( slotEdit   ( ) ) );
+	m_pContextMenu->insertItem ( tr ("Matrix ..."), this,  SLOT ( slotMatrix ( ) ) );
+	m_pContextMenu->insertItem ( tr ("Delete")    , this,  SLOT ( slotDelete ( ) ) );
 	m_pContextMenu->insertSeparator();
 	if ( shadow ( ) )  {
 	  m_pContextMenu->insertItem ( tr ( "Edit Shadow ..." ), this, SLOT ( slotAddShadow    ( ) ) );
@@ -437,11 +504,8 @@ bool TextObject::createContextMenu (QPoint globalPos)
 	else
 	  m_pContextMenu->insertItem ( tr ( "Add Shadow ..." ),  this, SLOT ( slotAddShadow ( ) ) );
 
-	//	m_pContextMenu->insertItem ( tr ("Add Frame"), this,   SLOT(slotAddFrame  ( ) ) );
-	//	m_pContextMenu->insertItem ( tr ("Add Image"), this,   SLOT(slotAddImage  ( ) ) );
-	//	m_pContextMenu->insertItem ( tr ("Add Movie"), this,   SLOT(slotAddMovie  ( ) ) );
-	m_pContextMenu->insertSeparator();
-	m_pContextMenu->insertItem ( tr ("Define as Button") , this, SLOT(slotDefineAsButton()));
+	m_pContextMenu->insertSeparator ( );
+	m_pContextMenu->insertItem ( tr ( "Define as Button" ),  this, SLOT ( slotDefineAsButton ( ) ) );
 	// Blocking call
 	m_pContextMenu->exec(globalPos2, 3);
 	// Here we mark that the user called a menu item thus we don't want the
@@ -458,84 +522,99 @@ StructureItem *TextObject::createStructure ( StructureItem *pParentItem )
 {
   StructureItem *pTemp=NULL;
 
+  if ( ! pParentItem )  {
+    if ( m_pOverlay  )
+      pTemp = m_pOverlay->createStructure ( NULL );
+    if ( m_pShadow )
+      pTemp = m_pShadow->createStructure  ( NULL );
+    modifiers ( )->createStructure  ( NULL, NULL );
+    m_pStructureItem = NULL;
+    return NULL;
+  }
+
   QString qsName = tr ( "Text Object" );
   if ( ! m_pStructureItem ) {
-    m_pStructureItem = new StructureItem ( this, pParentItem, qsName, m_qsText );
+    m_pStructureItem = new StructureItem ( this, StructureItem::Text, pParentItem, qsName, textData ( )->m_qsText );
     m_pStructureItem->setExpandable      ( TRUE );
   }
   else
-    m_pStructureItem->setText ( qsName, m_qsText );
+    m_pStructureItem->setText ( qsName, textData ( )->m_qsText );
 
   pTemp = m_pStructureItem->setSibling ( this, pTemp, tr ( "Geometry" ), QString ( "%1, %2, %3, %4" ).
        	         arg ( rect().x()).arg(rect().y()).arg(rect().width()).arg(rect().height()) );
 
   pTemp = m_pStructureItem->setSibling ( this, pTemp, tr ("Foreground"),
                 tr ("color(%1, %2, %3)").
-		arg ( m_colorForeground.red   ( ) ).
-		arg ( m_colorForeground.green ( ) ).
-		arg ( m_colorForeground.blue  ( ) ) );
-  
-  if ( m_colorBackground == Rgba ( TRANSPARENT_COLOR ) )
+		arg ( textData ( )->m_colorForeground.red   ( ) ).
+		arg ( textData ( )->m_colorForeground.green ( ) ).
+		arg ( textData ( )->m_colorForeground.blue  ( ) ) );
+
+  if ( textData ( )->m_colorBackground == Rgba ( TRANSPARENT_COLOR ) )
     pTemp = m_pStructureItem->setSibling ( this, pTemp, tr ("Background"), tr ("Transparent") );
   else
     pTemp = m_pStructureItem->setSibling ( this, pTemp, tr ("Background"),
 			tr  ( "color(%1, %2, %3)").
-			arg ( m_colorBackground.red   ( ) ).
-			arg ( m_colorBackground.green ( ) ).
-			arg ( m_colorBackground.blue  ( ) ) );
+			arg ( textData ( )->m_colorBackground.red   ( ) ).
+			arg ( textData ( )->m_colorBackground.green ( ) ).
+			arg ( textData ( )->m_colorBackground.blue  ( ) ) );
   if ( m_pShadow )
     pTemp = m_pShadow->createStructure ( m_pStructureItem );
-  
-  m_modifiers.createStructure ( m_pStructureItem, pTemp );
+
+  modifiers ( )->createStructure ( m_pStructureItem, pTemp );
 
   return m_pStructureItem;
 }
 
-bool TextObject::mousePressEvent (QMouseEvent *pEvent)
+bool TextObject::mousePressEvent ( QMouseEvent *pEvent )
 {
-	m_currentMousePos = pEvent->pos();
-	if (pEvent->button() == Qt::RightButton)	{
-		QPoint globalPos = pEvent->globalPos();
-		return createContextMenu ( globalPos );
-	}
-	else
-		m_bDrawRect = true;
-	return false;
+  m_currentMousePos  = pEvent->pos ( );
+  if ( pEvent->button ( ) == Qt::RightButton )  {
+    QPoint globalPos = pEvent->globalPos   ( );
+    return createContextMenu ( globalPos     );
+  }
+
+  m_bDrawRect = true;
+  return false;
 }
 
-bool TextObject::mouseReleaseEvent (QMouseEvent *)
+bool TextObject::mouseReleaseEvent ( QMouseEvent * )
 {
-	m_bDrawRect = false;
-	emit (signalUpdatePixmap());
-	emit (signalUpdateStructure());
-	return false;
+  UndoBuffer *pUndo = Global::pApp->getUndoBuffer ( );
+  pUndo->removeUnchanged ( );
+
+  m_bDrawRect = false;
+  emit ( signalUpdatePixmap    ( ) );
+  emit ( signalUpdateStructure ( ) );
+  return false;
 }
 
-bool TextObject::mouseDoubleClickEvent (QMouseEvent *)
+bool TextObject::mouseDoubleClickEvent ( QMouseEvent * )
 {
   QFont theFont;
-  DialogTextFont fontDialog ( m_pParent, this );
-	
+  DialogTextFont fontDialog  ( m_pParent, this );
+
   if ( fontDialog.exec ( ) == QDialog::Rejected )
     return false;
 
-  setFont(fontDialog.getFont());
-  QString theString = fontDialog.getText();
-  setText(theString);
-  setAnimation (fontDialog.getAnimation ());
-  if (fontDialog.getBackgroundColor() != getBackgroundColor())	{
-    // This emit signal is only caught when in ButtonDialog and the Selected, or Highlighted mask 
+  DVDMenuUndoObject *pUndo = new DVDMenuUndoObject ( this, MenuObject::StateUnknown );
+
+  setFont ( fontDialog.getFont ( ) );
+  QString theString = fontDialog.getText ( );
+  setText ( theString );
+  setAnimation ( fontDialog.getAnimation ( ) );
+  if ( fontDialog.getBackgroundColor ( ) != getBackgroundColor ( ) )  {
+    // This emit signal is only caught when in ButtonDialog and the Selected, or Highlighted mask
     // was changed (I.e. This Object is in one of those two frames).
     // Otherwise the signal is ignored.
-    emit (signalMaskColorChanged(fontDialog.getBackgroundColor(), getBackgroundColor()));
+    emit ( signalMaskColorChanged ( fontDialog.getBackgroundColor ( ), getBackgroundColor ( ) ) );
     // P.s. replaceColor (QColor, QColor) of all ButtonObjects will be called, inclusive this object)
   }
-  setBackgroundColor (fontDialog.getBackgroundColor());
-  if (fontDialog.getForegroundColor() != getForegroundColor())	{
+  setBackgroundColor ( fontDialog.getBackgroundColor ( ) );
+  if ( fontDialog.getForegroundColor ( ) != getForegroundColor ( ) )  {
     // This emit signal is only caught when in ButtonDialog and the Selected, or Highlighted mask 
     // was changed (I.e. This Object is in one of those two frames).
     // Otherwise the signal is ignored.
-    emit (signalMaskColorChanged(fontDialog.getForegroundColor(), getForegroundColor()));
+    emit ( signalMaskColorChanged ( fontDialog.getForegroundColor ( ), getForegroundColor ( ) ) );
     // P.s. replaceColor (QColor, QColor) of all ButtonObjects will be called, inclusive this object)
   }
   setForegroundColor ( fontDialog.getForegroundColor ( ) );
@@ -548,50 +627,45 @@ bool TextObject::mouseDoubleClickEvent (QMouseEvent *)
   }
   else
     setRect ( fontDialog.getRect ( ) );
-  
-  m_pParent->update ( );
-  
+
+  if ( m_pParent )
+       m_pParent->update ( );
+
   QRect orig = rect ( );
   updateShadow      ( );
   emit ( signalUpdatePixmap    ( ) );
-  
+
   if ( rect ( ) != orig ) // if the rect has changed ...
     emit ( signalUpdatePixmap  ( ) );
-  
+
   emit ( signalUpdateStructure ( ) );
-  
+
   setWordBreak ( true );
+
+  if ( pUndo->hasChanged ( ) )  {
+    UndoBuffer *pBuffer = Global::pApp->getUndoBuffer ( );
+    if ( pBuffer )
+         pBuffer->push ( pUndo );
+  }
+  else
+    delete pUndo;
+
   return false;
 }
 
-void TextObject::slotEdit ()
+void TextObject::slotEdit ( )
 {
-	mouseDoubleClickEvent(NULL);
+  mouseDoubleClickEvent ( NULL );
 }
 
-void TextObject::slotMatrix()
+void TextObject::slotMatrix ( )
 {
-	DialogMatrix *pDialog = new DialogMatrix(MenuObject::parent());
-	pDialog->initMe(this);
-	pDialog->show();
-}
+  if ( m_pMatrixDialog )
+    return;
 
-void TextObject::slotAddFrame ()
-{
-	MessageBox::warning (NULL, tr ("Not Implemented yet."), tr ("Not Implemented yet."),
-		QMessageBox::Ok ,  QMessageBox::Cancel);
-}
-
-void TextObject::slotAddImage ()
-{
-	MessageBox::warning (NULL, tr ("Not Implemented yet."), tr ("Not Implemented yet."),
-		QMessageBox::Ok ,  QMessageBox::Cancel);
-}
-
-void TextObject::slotAddMovie ()
-{
-	MessageBox::warning (NULL, tr ("Not Implemented yet."), tr ("Not Implemented yet."),
-		QMessageBox::Ok ,  QMessageBox::Cancel);
+  m_pMatrixDialog = new DialogMatrix ( MenuObject::parent ( ) );
+  m_pMatrixDialog->initMe ( this );
+  m_pMatrixDialog->show   ( );
 }
 
 void TextObject::slotDefineAsButton ()
@@ -604,27 +678,27 @@ void TextObject::slotDefineAsButton ()
 
 MenuObject *TextObject::clone ( QWidget *pParent, MenuObject * )
 {
-  if ( !pParent )
+  if ( ! pParent )
     pParent = MenuObject::parent ( );
-  
+
   TextObject *pNewObject = new TextObject ( pParent );
-  pNewObject->setFont (getFont());
-  pNewObject->setText (getText());
-  pNewObject->setTextAlign (getTextAlign());
-  pNewObject->setForegroundColor (getForegroundColor());
-  pNewObject->setBackgroundColor (getBackgroundColor());
-  pNewObject->setStyleStrategy   ((QFont::StyleStrategy)m_iStyleStrategy);
-  
+  pNewObject->setFont ( getFont ( ) );
+  pNewObject->setText ( getText ( ) );
+  pNewObject->setTextAlign ( getTextAlign ( ) );
+  pNewObject->setForegroundColor ( getForegroundColor ( ) );
+  pNewObject->setBackgroundColor ( getBackgroundColor ( ) );
+  pNewObject->setStyleStrategy   ( (QFont::StyleStrategy)textData ( )->m_iStyleStrategy );
+
   // The following two are not really needed, since we get those 
   // informations solely from the Normal State - objects ...
-  pNewObject->setRect(rect());
-  pNewObject->setModifiers(*modifiers());
-  
-  MenuObject *pShadow = shadow ( );
+  pNewObject->setRect ( rect ( ) );
+  pNewObject->setModifiers ( *modifiers ( ) );
+
+  MenuObject *pShadow = shadow  ( );
   if ( pShadow )
-    pShadow = pShadow->clone ( pParent, pNewObject );
-  pNewObject->setShadow      ( pShadow );
-  
+       pShadow = pShadow->clone ( pParent, pNewObject );
+  pNewObject->setShadow ( pShadow );
+
   return pNewObject;
 }
 
@@ -634,44 +708,44 @@ AnimationAttribute *TextObject::getSpecificAttributes (long iMaxNumberOfFrames,
   static int m_iTestInt = 0;
 
   if (qsProperty == "text")
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_qsText, &m_qsText);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, textData ( )->m_qsText, &textData ( )->m_qsText);
   else if (qsProperty == "font")	{
-    FunctionCallback<QFont> *callMeBack = new FunctionCallback<QFont>(m_Font, &QFont::fromString);
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_Font.toString(), callMeBack);
+    FunctionCallback<QFont> *callMeBack = new FunctionCallback<QFont>(textData ( )->m_Font, &QFont::fromString);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, textData ( )->m_Font.toString(), callMeBack);
   }
   else if (qsProperty == "textAlign")
     pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_iTestInt, &m_iTestInt);
   else if (qsProperty == "foregroundColor.red")	{
-    FunctionCallback<Rgba> *callMeBack = new FunctionCallback<Rgba>(m_colorForeground, TYPE_COLOR_RED );
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_colorForeground.red(), callMeBack);
+    FunctionCallback<Rgba> *callMeBack = new FunctionCallback<Rgba>(textData ( )->m_colorForeground, TYPE_COLOR_RED );
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, textData ( )->m_colorForeground.red(), callMeBack);
   }
   else if (qsProperty == "foregroundColor.green")	{
-    FunctionCallback<Rgba> *callMeBack = new FunctionCallback<Rgba>(m_colorForeground, TYPE_COLOR_GREEN);
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_colorForeground.green(), callMeBack);
+    FunctionCallback<Rgba> *callMeBack = new FunctionCallback<Rgba>(textData ( )->m_colorForeground, TYPE_COLOR_GREEN);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, textData ( )->m_colorForeground.green(), callMeBack);
   }
   else if (qsProperty == "foregroundColor.blue")	{
-    FunctionCallback<Rgba> *callMeBack = new FunctionCallback<Rgba>(m_colorForeground, TYPE_COLOR_BLUE);
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_colorForeground.blue(), callMeBack);
+    FunctionCallback<Rgba> *callMeBack = new FunctionCallback<Rgba>(textData ( )->m_colorForeground, TYPE_COLOR_BLUE);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, textData ( )->m_colorForeground.blue(), callMeBack);
   }
   else if (qsProperty == "foregroundColor.alpha")	{
-    FunctionCallback<Rgba> *callMeBack = new FunctionCallback<Rgba>(m_colorForeground, TYPE_COLOR_ALPHA);
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_colorForeground.alpha(), callMeBack);
+    FunctionCallback<Rgba> *callMeBack = new FunctionCallback<Rgba>(textData ( )->m_colorForeground, TYPE_COLOR_ALPHA);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, textData ( )->m_colorForeground.alpha(), callMeBack);
   }
   else if (qsProperty == "backgroundColor.red")	{
-    FunctionCallback<Rgba> *callMeBack = new FunctionCallback<Rgba>(m_colorBackground, TYPE_COLOR_RED);
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_colorBackground.red(), callMeBack);
+    FunctionCallback<Rgba> *callMeBack = new FunctionCallback<Rgba>(textData ( )->m_colorBackground, TYPE_COLOR_RED);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, textData ( )->m_colorBackground.red(), callMeBack);
   }
   else if (qsProperty == "backgroundColor.green")	{
-    FunctionCallback<Rgba> *callMeBack = new FunctionCallback<Rgba>(m_colorBackground, TYPE_COLOR_GREEN);
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_colorBackground.green(), callMeBack);
+    FunctionCallback<Rgba> *callMeBack = new FunctionCallback<Rgba>(textData ( )->m_colorBackground, TYPE_COLOR_GREEN);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, textData ( )->m_colorBackground.green(), callMeBack);
   }
   else if (qsProperty == "backgroundColor.blue")	{
-    FunctionCallback<Rgba> *callMeBack = new FunctionCallback<Rgba>(m_colorBackground, TYPE_COLOR_BLUE);
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_colorBackground.blue(), callMeBack);
+    FunctionCallback<Rgba> *callMeBack = new FunctionCallback<Rgba>(textData ( )->m_colorBackground, TYPE_COLOR_BLUE);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, textData ( )->m_colorBackground.blue(), callMeBack);
   }
   else if (qsProperty == "backgroundColor.alpha")	{
-    FunctionCallback<Rgba> *callMeBack = new FunctionCallback<Rgba>(m_colorBackground, TYPE_COLOR_ALPHA);
-    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, m_colorBackground.alpha(), callMeBack);
+    FunctionCallback<Rgba> *callMeBack = new FunctionCallback<Rgba>(textData ( )->m_colorBackground, TYPE_COLOR_ALPHA);
+    pAnimAttr = new AnimationAttribute (iMaxNumberOfFrames, qsProperty, textData ( )->m_colorBackground.alpha(), callMeBack);
   }
   return pAnimAttr;
 }
