@@ -21,9 +21,27 @@ class QPopupMenu;
 class TextObject : public MenuObject
 {
 	Q_OBJECT
+	class TextData : public MenuObject::Data
+	{
+	public:
+	           TextData ( );
+	  virtual ~TextData ( );
+	  virtual  Data &operator  = ( Data & );
+	  virtual  bool  operator == ( Data & );
+	  virtual  Data *clone ( );
+
+	  QString  m_qsText;
+	  QFont    m_Font;
+	  QPoint   m_ptTextAlign;
+	  Rgba     m_colorForeground;
+	  Rgba     m_colorBackground;
+	  int      m_iStyleStrategy;
+	  bool     m_bAllowWordBreak; // don't need ot be stored. Only set after DialogTextFont
+	};
+
 public:
-	TextObject (QWidget *pParent = 0);
-	virtual ~TextObject ();
+	TextObject ( QWidget *p=0 );
+	virtual ~TextObject ( );
 	virtual void drawContents          ( QPainter * );
 	virtual void drawContents          ( QPainter *, int , int );
 	virtual bool mousePressEvent       ( QMouseEvent * );
@@ -52,27 +70,25 @@ public:
 	virtual Rgba    &getBackgroundColor( );
 
 private:
-	void drawSemiTransparent           ( QPainter * );
+	void drawSemiTransparent( QPainter * );
+	TextData *textData      ( );
 
 protected:
-	virtual AnimationAttribute *getSpecificAttributes (long, QString);
+	virtual AnimationAttribute *getSpecificAttributes ( long, QString );
 
 public slots:
 	virtual void slotEdit              ( );
 	virtual void slotMatrix            ( );
-	virtual void slotAddFrame          ( );
-	virtual void slotAddImage          ( );
-	virtual void slotAddMovie          ( );
 	virtual void slotDefineAsButton    ( );
 
 protected:
-	QString     m_qsText;
-	QFont       m_Font;
-	QPoint      m_ptTextAlign;
-	Rgba        m_colorForeground;
-	Rgba        m_colorBackground;
-	int         m_iStyleStrategy;
-	bool        m_bAllowWordBreak; // don't need ot be stored. Only set after DialogTextFont
+//	QString     m_qsText;
+//	QFont       m_Font;
+//	QPoint      m_ptTextAlign;
+//	Rgba        m_colorForeground;
+//	Rgba        m_colorBackground;
+//	int         m_iStyleStrategy;
+//	bool        m_bAllowWordBreak; // don't need ot be stored. Only set after DialogTextFont
 };
 
 
