@@ -19,13 +19,17 @@
 #include <qpixmap.h>
 #include <qprogressbar.h>
 
+#include "global.h"
+
 void uiDialogRenderPreview::setProgressData (QPixmap *pPixmap, long iCurrentFrameNumber, long iTotalFramesToRender)
 {
+	bool bPNG = Global::bPNGRender;
 	QString qsFileName;
 	int iProgress = (int)((float)iCurrentFrameNumber /(float) iTotalFramesToRender * 1000.0);
 	m_pProgress->setProgress (iProgress);
 	m_pLabelPreview->setPaletteBackgroundPixmap (*pPixmap);
 //	m_pLabelPreview->setPixmap (*pPixmap);
-	qsFileName.sprintf ("<p align=\"center\">rendered_%08ld.png</p>", iCurrentFrameNumber);
+	qsFileName.sprintf ("<p align=\"center\">rendered_%08ld.%s</p>", iCurrentFrameNumber, bPNG ? "png" : "jpg" );
 	m_pLabelFileName->setText (qsFileName);
 }
+
