@@ -17,12 +17,14 @@
 #endif
 
 #include <qfileinfo.h>
+#include <qregexp.h>
 #include <qimage.h>
 #include <qfile.h>
 #include <qdir.h>
 
 #include <qstylefactory.h>
 #include <qapplication.h>
+#include <qurloperator.h>
 #include <qmetaobject.h>
 #include <qstyle.h>
 
@@ -42,6 +44,238 @@ Utils::~Utils ()
 
 }
 
+float Utils::getVersionInfo ( QString &qsFileName )
+{
+  // Read in the file and parses or the version information
+  // Note: This is a single floating point value in the first line of the file
+  float fVersion = 1.0f;  // defaults to 1.0
+
+  QRegExp rx  ( "\\d*\\.\\d+"  ); // floating point
+  QString qsLine, qsPath;
+  QFileInfo fileInfo ( qsFileName );
+  qsPath = fileInfo.dirPath( true );
+  fileInfo.setFile   ( qsPath );
+  if ( ! fileInfo.exists  ( ) )
+    return 0.0;
+
+  QFile     file ( qsFileName  );
+  if ( file.open ( IO_ReadOnly ) )  {
+    QTextStream stream ( &file );
+    while ( ! stream.atEnd ( ) )  {
+      qsLine= stream.readLine( );
+      if ( rx.search ( qsLine, 0 ) == -1 ) // line must start with a character
+        continue;
+      fVersion = atof ( qsLine );
+      break;
+    }
+    file.close ( );
+  }
+
+  return fVersion;
+}
+
+bool Utils::webToFile ( QString &qsURL, QString &qsLocal )
+{
+  QUrlOperator op; // ( qsServerVersion );
+  QPtrList<QNetworkOperation> list = op.copy ( qsURL, qsLocal );
+  // Next we make sure we get the whole file ( less then 1k )
+  bool bDone = false;
+  int iMaxCounter = 30; // 30 * 100 msec = 3sec
+  while ( bDone == false )  {
+    bDone = false;
+    QPtrList<QNetworkOperation>::iterator it = list.begin ( );
+    while ( it != list.end ( ) )  {
+      QNetworkOperation *pNO = *it++;
+      if ( pNO->state ( ) != QNetworkProtocol::StDone )
+        break;
+      if ( pNO->state ( ) == QNetworkProtocol::StFailed )
+        return false;
+      bDone = true;
+    }
+    if ( ! bDone )  {
+      // Not yet done.
+      qApp->processEvents ( );
+      iMaxCounter --;
+      usleep ( 100000 ); // sleep for a 100 milli seconds.
+      if ( iMaxCounter == 0 )  // exit after max attempts.
+           bDone = true;
+    }
+  }
+
+  return ( iMaxCounter > 0 );
+}
+
+QStringList Utils::localTemplates ( )
+{
+  // This function will look into the local, and system folder
+  // and create a list of templates
+  QString qsSubPaths[3], qsBasePath, qsPath, qsTemplate;
+  QStringList list;
+  QStringList::iterator it2;
+  QDir theDir;
+  bool bFound;
+  const QFileInfoList *pList;
+  QFileInfo *pFileInfo;
+
+  qsBasePath = QDir::homeDirPath ( ) + "/.qdvdauthor/";
+  qsSubPaths[0] = "static";
+  qsSubPaths[1] = "animated";
+  qsSubPaths[2] = "transition";
+
+  int t, i;
+  for ( t=0; t<2; t++ )  {
+    for ( i=0; i<3; i++ )  {
+      qsPath = qsBasePath + qsSubPaths[i];
+      theDir.setPath    ( qsPath );
+      theDir.setFilter  ( QDir::Dirs );
+      pList = theDir.entryInfoList ( );
+      if ( ! pList || pList->count ( ) < 1 )
+        continue;
+
+      QFileInfoListIterator  it ( *pList );
+      while( (pFileInfo = it.current ( ) ) != 0 )  {
+        ++it;
+        qsTemplate = pFileInfo->fileName ( );
+        if ( ( qsTemplate == "."  ) ||
+             ( qsTemplate == ".." ) )
+          continue;
+        bFound = false;
+        it2 = list.begin ( );
+        while ( it2 != list.end ( ) )  {
+          if ( *it2++ == qsTemplate )  {
+            bFound = true;
+            break;
+          }
+        }
+
+        if ( ! bFound )
+          list.append ( qsTemplate );
+      }
+    }
+    qsBasePath = Global::qsSystemPath + "/share/qdvdauthor/";
+  }
+//for (t=0; t<list.size ( ); t++ )
+//  printf ( "%s \n", list[t].ascii() );
+//printf ( "2 list<%d>\n", list.size() );
+
+  return list;
+}
+
+QString Utils::findGuiSU ( )
+{
+  int iRet = 0;
+  uint t;
+  QString qsReturn;
+  QString qsTmpOut = QString ( "%1/%2_out.txt" ).arg ( Global::qsTempPath ).arg ( getlogin ( ) );
+  char arraySUs[3][10] = {"kdesu", "xsu", "gnomesu"};
+  QFileInfo fileInfo ( qsTmpOut );
+  for ( t=0; t<3; t++ )     {
+    // the next check is to see if we can find kdesu
+    iRet = system ((const char *)QString ( "which %1 > %2 2>/dev/null").arg ( arraySUs[t] ).arg ( qsTmpOut ) );
+    fileInfo.setFile ( qsTmpOut );
+    // Okay we cna not find it, so we wont bother asking the user ...
+    if ( fileInfo.size ( ) > 4 )  {
+      qsReturn = QString ( "%1" ).arg ( arraySUs[t] );
+      return qsReturn;
+    }
+  }
+  return qsReturn;
+}
+
+void Utils::createStructureIcons ( )
+{
+// Unknown=0, DVDProject=1, MainMenu=2, SubMenu=3, Intro=4, Extro=5, 
+// Background=6, Audio=7, Buttons=8, Button=9, Frame=10, 
+// Text=11, Image=12, Video=13, Highlighted=14, Selected=15, 
+// Geometry=16, Modifiers=17, Attributes=18
+  const char files[20][20] = { "unknown.png", "1.png", "2.png", "submenu.png", "intros.png", "intros.png", "background.png", "audio.png", "buttons.png", "button.png", "frame.png", "text.png", "image.png", "video.png", "highlighted.png", "selected.png", "geometry.png", "modifiers.png", "attributes.png", "unknown.png" };
+
+  QPixmap final ( 32*20, 32 );
+  final.fill ( 0xFF0000 );
+  QPixmap temp;
+  for ( int t=0; t<20; t++ )  {
+    temp = QPixmap ( QString ( "/home/varol/structure_icons/" ) + files[t] );
+    if ( temp.isNull ( ) )
+      continue;
+    copyBlt ( &final, t*32, 0, &temp );
+  }
+
+  final.save ( "/tmp/structure_icons.png", "PNG", 100 );
+}
+
+// Copied over from qrender -> filter.cpp
+void Utils::augmentAlpha ( int iDeltaA, QImage &filter, QImage &start, QImage &stop, QImage &result )
+{
+  int   r, g, b, x, y;
+  float fVal;
+  QRgb  alpha, pix1, pix2;
+  // First we copy the filter to start and end ...
+  // Next we run through every pixel to increase / decrease the filters
+  for  ( y=0; y<filter.height ( ); y++ )  {
+    for ( x=0; x<filter.width ( ); x++ )  {
+      alpha = filter.pixel ( x, y );
+      pix1  = start.pixel  ( x, y );
+      pix2  = stop.pixel   ( x, y );
+      fVal = (float)( qRed ( alpha ) - 256 + iDeltaA ) / 255.0f; // represents the alpha value ...
+      if ( fVal < 0.0f )
+           fVal = 0.0f;
+      if ( fVal > 1.0f )
+           fVal = 1.0f;
+
+      r = qRed   ( pix1 ) * (int)( ( 1.0f - fVal ) + qRed   ( pix2 ) * fVal );
+      g = qGreen ( pix1 ) * (int)( ( 1.0f - fVal ) + qGreen ( pix2 ) * fVal );
+      b = qBlue  ( pix1 ) * (int)( ( 1.0f - fVal ) + qBlue  ( pix2 ) * fVal );
+
+      pix1 = qRgb     ( r, g, b );
+      result.setPixel ( x, y, pix1 );
+    }
+  }
+}
+
+void Utils::augmentAlpha ( int iDeltaA, int iDeltaB, QImage &filter, QImage &start, QImage &stop, QImage &result, QImage *pBackground )
+{
+  int   r, g, b, x, y;
+  float fVal1, fVal2, fVal3 = 0.0f;
+  QRgb alpha, pix1, pix2, pix3;
+  // First we copy the filter to start and end ...
+  // Next we run through every pixel to increase / decrease the filters
+  for  ( y=0; y<filter.height ( ); y++ )  {
+    for ( x=0; x<filter.width ( ); x++ )  {
+      alpha = filter.pixel ( x, y );
+      pix1  = start.pixel  ( x, y );
+      pix2  = stop.pixel   ( x, y );
+      fVal1 = (float)( qRed ( alpha ) - 256 + iDeltaA ) / 255.0f; // represents the alpha value ...
+      if ( fVal1 < 0.0f )
+           fVal1 = 0.0f;
+      if ( fVal1 > 1.0f )
+           fVal1 = 1.0f;
+      fVal2 = (float)( qRed ( alpha ) - 256 + iDeltaB ) / 255.0f; // represents the alpha value ...
+      if ( fVal2 < 0.0f )
+           fVal2 = 0.0f;
+      if ( fVal2 > 1.0f )
+           fVal2 = 1.0f;
+
+      r = qRed   ( pix1 ) * (int)( ( 1.0f - fVal1 ) + qRed   ( pix2 ) * fVal2 );
+      g = qGreen ( pix1 ) * (int)( ( 1.0f - fVal1 ) + qGreen ( pix2 ) * fVal2 );
+      b = qBlue  ( pix1 ) * (int)( ( 1.0f - fVal1 ) + qBlue  ( pix2 ) * fVal2 );
+
+      if ( pBackground )  {
+        //fVal3 = 1.0f-fVal1 + fVal2;
+        fVal3 = fVal2 - fVal1;
+        if ( fVal3 < 0.0f )
+             fVal3 = -fVal3;
+        pix3  = pBackground->pixel ( x, y );
+        r += (int)( qRed   ( pix3 ) * fVal3 );
+        g += (int)( qGreen ( pix3 ) * fVal3 );
+        b += (int)( qBlue  ( pix3 ) * fVal3 );
+      }
+
+      pix1 = qRgb     ( r, g, b );
+      result.setPixel ( x, y, pix1 );
+    }
+  }
+}
+
 QString Utils::currentStyleName ( )
 {
   QStringList list = QStyleFactory::keys();
@@ -199,8 +433,8 @@ QRgb Utils::nearestColor (QRgb *rgbPalette, int iNumColors, QRgb rgbSeeking)
   return rgbReturn;
 }
 
-/** 
- * After a lot of frustration I decided to write this little (slow) routine, to 
+/**
+ * After a lot of frustration I decided to write this little (slow) routine, to
  * - reduce the used colors to 4 AND
  * - merge all used colors to the lower parts of the palette
  * - reduce the number of colors in the palette.
@@ -218,20 +452,12 @@ void Utils::reduceColors ( QImage &theImage, int iNumColors, QRgb *pPalette )
   // First we have to ensure that we do have a color indexed image.
   theImage = theImage.convertDepthWithPalette (8, (QRgb *)pPalette, iNumColors, Qt::ThresholdDither);
 
-//  theImage.save ( "/tmp/Unnamed/Main Menu VMGM/3.png", "PNG", 100);
-//printf ( "Image 3 has <%d> diff colors. Color Table size=<%d>\n", (int)countColors ( theImage ), //theImage.numColors ( ) );
-//for (x=0;x<theImage.numColors ();x++)
-//  printf ("colorTable[%d]=<%X>\n", x, theImage.color ( x ) );
-
   // This will create a image which might have more then the requested colors.
   // So we replace all colors with the nearest color of the four.
   pImageColors = theImage.colorTable ();
   iImageColors = theImage.numColors  ();
-//  printf ( "%s::%s:%d  <%d> <%d>\n",  __FILE__, __FUNCTION__, __LINE__, iImageColors, iNumColors );
   for (t=0;(int)t<iImageColors; t++) {
     theNearestColor = nearestColor (pPalette, iNumColors, pImageColors[t]);
-//printf ( "color[%d]<%X> nearestFound<%X>\n", t, pImageColors[t], theNearestColor );
-//    theImage.setColor (t, theNearestColor & 0x00ffffff);
     theImage.setColor (t, theNearestColor );
   }
 
@@ -241,7 +467,6 @@ void Utils::reduceColors ( QImage &theImage, int iNumColors, QRgb *pPalette )
     for (x=0;x<theImage.width();x++) {
       iIndex = pLine[x];
       theColor = theImage.color ( iIndex );
-//      theColor &= 0x00ffffff; // disregard the alpha channel for now ...
       pLine[x] = 0;
       bFound = false;
       for ( t=0;t<iNumColors;t++ ) {
@@ -300,7 +525,7 @@ long Utils::countColors ( QImage &theImage )
     return listFoundColors.count ();
   }
   else  { // 32 bits
-static bool bFirst=true;
+//static bool bFirst=true;
 
     QRgb *pLine, rgb;
     QValueList<QRgb> listFoundColors;
@@ -319,13 +544,13 @@ static bool bFirst=true;
           listFoundColors.append ( rgb );
       }
     }
-if ( bFirst )  {
-  qHeapSort ( listFoundColors );
-  for (z=0;z<(int)listFoundColors.count ();z++)  {
-    printf ( "[%d]=<%X>\n", z, listFoundColors[z] );
-  }
-  bFirst = false;
-}
+//if ( bFirst )  {
+//  qHeapSort ( listFoundColors );
+//  for (z=0;z<(int)listFoundColors.count ();z++)  {
+//    printf ( "[%d]=<%X>\n", z, listFoundColors[z] );
+//  }
+//  bFirst = false;
+//}
     return listFoundColors.count ();
   }
   return 0L;
@@ -333,20 +558,23 @@ if ( bFirst )  {
 
 QString Utils::getVideoFormat (int iWidth, int iHeight)
 {
-	QString qsFormat = ("custom");
-	if ( ((iWidth == 720) && (iHeight == 480) ) ||
-	     ((iWidth == 704) && (iHeight == 480) ) ||
-	     ((iWidth == 352) && (iHeight == 480) ) ||
-	     ((iWidth == 352) && (iHeight == 240) ) )
-		 qsFormat = QString ("ntsc");
+  QString qsFormat = ( "custom" );
+  int iFormat = getFormat ( iWidth, iHeight );
+  if ( iFormat != FORMAT_NONE )  {
+    if ( iFormat > FORMAT_NTSC4 )
+      qsFormat = "pal";
+    else
+      qsFormat = "ntsc";
+  }
+  return qsFormat;
+}
 
-	if ( ((iWidth == 720) && (iHeight == 576) ) ||
-	     ((iWidth == 704) && (iHeight == 576) ) ||
-	     ((iWidth == 352) && (iHeight == 576) ) ||
-	     ((iWidth == 352) && (iHeight == 288) ) )
-		 qsFormat = QString ("pal");
+int Utils::getFormat ( QString &qsResolution )
+{
+  int iWidth  = getWHFromResolution ( qsResolution, true  );
+  int iHeight = getWHFromResolution ( qsResolution, false );
 
-	return qsFormat;
+  return getFormat ( iWidth, iHeight );
 }
 
 int Utils::getFormat (int iWidth, int iHeight)
@@ -371,6 +599,46 @@ int Utils::getFormat (int iWidth, int iHeight)
 	return iFormat;
 }
 
+bool Utils::getResolution ( int iFormat, int &iWidth, int &iHeight )
+{
+  bool bFound = true;
+  iWidth  = 720;
+  iHeight = 480;
+  switch ( iFormat )  {
+  case FORMAT_NTSC1:
+  break;
+  case FORMAT_NTSC2:
+    iWidth = 704;
+  break;
+  case FORMAT_NTSC3:
+    iWidth = 352;
+  break;
+  case FORMAT_NTSC4:
+    iWidth  = 352;
+    iHeight = 240;
+  break;
+  case FORMAT_PAL1:
+    iHeight = 576;
+  break;
+  case FORMAT_PAL2:
+    iWidth  = 704;
+    iHeight = 576;
+  break;
+  case FORMAT_PAL3:
+    iWidth  = 352;
+    iHeight = 576;
+  break;
+  case FORMAT_PAL4:
+    iWidth  = 352;
+    iHeight = 288;
+  break;
+
+  default:
+    bFound = false;
+  }
+  return bFound;
+}
+
 QString Utils::iso639 (QString qsCode, bool bReverse, int iWhich )
 {
 	// This function will return the Country name of the found two-letter - code
@@ -735,7 +1003,7 @@ bool Utils::recMkdir ( const QString &qsDir )
   return true;
 }
 
-long Utils::getMsFromString(QString qsInput)
+long Utils::getMsFromString ( QString qsInput )
 {
   long iMilliSeconds;
   uint iHours, iMinutes, iSeconds, iStepper;
@@ -829,7 +1097,7 @@ int Utils::getWHFromResolution ( QString qsResolution, bool bWidth )
 
 bool Utils::isMpeg2 ( QString qsFileName )
 {
-  unsigned char Mpeg2Header[4] = {0x00, 0x00, 0x01, 0xba };
+  unsigned char Mpeg2Header[4] = { 0x00, 0x00, 0x01, 0xba };
   int t, iByte;
   unsigned char c;
   QFile theFile ( qsFileName );
@@ -850,7 +1118,7 @@ bool Utils::isMpeg2 ( QString qsFileName )
     if ( Mpeg2Header[t] != c )
       return false;
   }
-  theFile.close ();
+  theFile.close ( );
 
   // Okay getting here means the first 4 bytes are mpeg2 header compliant ... Greeeeeat !!!
   return true;
@@ -921,10 +1189,10 @@ bool Utils::checkCommand ( const char *pCommand )
 {
   // executes a command and checks if there was a result
   int iRet = 0;
-  char tmpName[11] =  { "out_XXXXXX" };
+  char tmpName[20] =  { "/tmp/out_XXXXXX" };
   int iFD = mkstemp ( (char *)&tmpName );
   iFD = iFD; // Silence the compiler warnings.
-  QString qsTmpFile = QString ( "/tmp/%1"  ).arg ( tmpName  );
+  QString qsTmpFile = QString ( "%1"  ).arg ( tmpName  );
   QString qsCommand = QString ( "%1 > %2 " ).arg ( pCommand ).arg ( qsTmpFile );
 
   iRet = system ( qsCommand.ascii ( ) );
@@ -953,9 +1221,10 @@ QString Utils::getDefaultParameter ( enDefaultParameter enWhich )
     qsReturn = QString ( "48000" ); // E.g. toolame 0.2m
     // execute [toolame -h 2>&1 | grep sfrq | grep "rate in Hz" > /tmp/test ] and check if result is empty.
     if ( checkCommand ( "toolame -h 2>&1 | grep sfrq | grep \"rate in kHz\"" ) )
-	 qsReturn = QString ( "48" ); // E.g.  toolame 0.2i
+       qsReturn = QString ( "48" ); // E.g.  toolame 0.2i
   }
   break;
   };
   return qsReturn;
 }
+
