@@ -43,6 +43,9 @@ public:
 	 Utils ();
 	~Utils ();
 
+	void augmentAlpha ( int, QImage &, QImage &, QImage &, QImage & );
+	void augmentAlpha ( int, int, QImage &, QImage &, QImage &, QImage &, QImage *p=NULL );
+	void createStructureIcons    ( );
 	QString currentStyleName     ( );
 	QString formatChapter        ( QString );
 	QString longToNorm           ( long );
@@ -65,11 +68,16 @@ public:
 	QRgb    nearestColor         ( QRgb *, int, QRgb );
 	void    reduceColors         ( QImage &, int, QRgb * );
 	long    countColors          ( QImage &  );
-
+	float   getVersionInfo       ( QString & );
 	QImage  convertStringToImage ( QString & );
 	QString convertImageToString ( QImage  & );
 	QString getVideoFormat       ( int,  int );
 	int     getFormat            ( int,  int );
+	int     getFormat            ( QString & );
+	bool    getResolution        ( int, int &, int & );
+	bool    webToFile            ( QString &, QString & );
+	QStringList localTemplates   ( );
+	QString findGuiSU            ( );
 
 	long    getMsFromString      ( QString       );
 	QString getStringFromMs      ( long          );
