@@ -21,11 +21,11 @@
 // Version 1.1 RC3 = 1. 01 03
 // Note the minor version is one behind the RC version
 #define QDVDAUTHOR_VERSION                     "XMLVersion"
-#define QDVDAUTHOR_VERSION_NUMBER              "1.0800"
+#define QDVDAUTHOR_VERSION_NUMBER              "2.0100"
 
 // Here we goof around with the Templates
 // Nodes
-#define TEMPLATE_VERSION_NUMBER                1.2
+#define TEMPLATE_VERSION_NUMBER                1.3
 #define TEMPLATE_DOCTYPE                       "QDVDAuthor"
 #define XML_TEMPLATE                           "QDVDTemplate"
 #define TEMPLATE_DATA                          "DVDTemplateData"
@@ -50,6 +50,7 @@
 // Here are the defines for qdvdauthor.
 #define DVDAUTHOR                               "DVDAuthor"
 #define DVDAUTHOR_TEMP_PATH                     "TempPath"
+#define DVDAUTHOR_ZONE                          "Zone"
 #define DVDAUTHOR_PROJECT_PATH                  "ProjectPath"
 #define DVDAUTHOR_CURRENT_PATH                  "CurrentPath"
 #define DVDAUTHOR_ROOT_VMGM                     "MainVMGM"
@@ -128,6 +129,19 @@
 #define SHADOW_OBJECT_COLOR_FADING              "ColorFading"
 #define SHADOW_OBJECT_COLOR_FADE_TO             "ColorFadeTo"
 
+// Overlay is an Mask from the mask library which
+// masks out the underlying image/video
+#define OVERLAY_OBJECT                           "OverlayObject"
+#define OVERLAY_OBJECT_COLOR                     "Color"
+#define OVERLAY_OBJECT_IS_MASK                   "isMask"
+#define OVERLAY_OBJECT_OFFSET                    "Offset"
+#define OVERLAY_OBJECT_ROTATE                    "Rotate"
+#define OVERLAY_OBJECT_SHEAR_X                   "ShearX"
+#define OVERLAY_OBJECT_SHEAR_Y                   "ShearY"
+#define OVERLAY_OBJECT_SCALE_X                   "ScaleX"
+#define OVERLAY_OBJECT_SCALE_Y                   "ScaleY"
+#define OVERLAY_OBJECT_FILE_NAME                 "FileName"
+
 // Mask is a subset of a shadow object.
 #define MASK_OBJECT                             "MaskObject"
 #define MASK_OBJECT_COLOR                       "Color"
@@ -175,6 +189,16 @@
 #define BUTTON_OBJECT_MULTIPLE                  "PlayMe"
 #define BUTTON_OBJECT_LOOP_MULTIPLE             "Loop"
 
+// The ButtonTransition - class
+#define BUTTON_TRANSITION                       "Transition"
+#define BUTTON_TRANS_DURATION                   "Duration"
+#define BUTTON_TRANS_USER_FILE                  "UserFile"
+#define BUTTON_TRANS_FORMAT                     "Format"
+#define BUTTON_TRANS_TYPE                       "Type"
+#define BUTTON_TRANS_TRANSITION_NAME            "TransName"
+#define BUTTON_TRANS_NAME                       "Name"
+#define BUTTON_TRANS_RENDERED                   "Rendered"
+
 // The SourceFileEntry - class
 #define SOURCE_OBJECT                           "SourceEntry"
 #define SOURCE_OBJECT_DISPLAY_NAME              "DisplayName"
@@ -191,6 +215,7 @@
 #define SOURCE_OBJECT_LENGTH                    "Length"
 #define SOURCE_OBJECT_VIDEO_FORMAT              "Format"
 #define SOURCE_OBJECT_VIDEO_CODEC               "Codec"
+#define SOURCE_OBJECT_HAS_AUDIO                 "HasAudio"
 #define SOURCE_OBJECT_AUDIO_FORMAT              "AudioFormat"
 #define SOURCE_OBJECT_AUDIO_CODEC               "AudioCodec"
 #define SOURCE_OBJECT_STATUS                    "Status"
@@ -229,6 +254,7 @@
 #define SUBTITLES_SUBTITLE_ENTRY                "Entry"
 #define SUBTITLES_NUMBER                        "SubtitleNumber"
 #define SUBTITLES_LANG                          "Lang"
+#define SUBTITLES_FILE_NAME                     "File"
 #define SUBTITLES_SUBTITLE_FILE                 "TempFile"
 #define SUBTITLES_FONT                          "Font"
 #define SUBTITLES_FIT                           "Fit"
@@ -238,6 +264,7 @@
 #define SUBTITLES_TEXT_SIZE                     "TextSize"
 #define SUBTITLES_OUTLINE                       "Outline"
 #define SUBTITLES_RECT                          "Rect"
+#define SUBTITLES_VIDEO_SIZE                    "size"
 #define SUBTITLES_ALIGNMENT                     "Alignment"
 #define SUBTITLES_COLORS                        "Colors"
 #define SUBTITLES_SUBTITLE_INDEX                "Index"
