@@ -603,7 +603,9 @@ CXmlSlideshow::CXmlSlideshow ( )
 	filter_delay  = -1.0;
 	xres          =  720;
 	yres          =  480; // NTSC for now
+	aspect        =    0; // 4:3
 	intro_page    = true;
+	fade_out      = true;
 	id            = NULL;
 	total_audio_length = 0;
 	bImgBackgroundImg  = false;
@@ -640,10 +642,10 @@ uint CXmlSlideshow::audioLength ( )
 uint CXmlSlideshow::slideLength ( )
 {
 	uint iTotalPlayTime = 0;
-	iTotalPlayTime += count      ( ) * filter_delay;
-	iTotalPlayTime += countImg   ( ) * delay;
-	iTotalPlayTime += vidsLength ( );
-	iTotalPlayTime += filter_delay;
+	iTotalPlayTime += (int)( count      ( ) * filter_delay );
+	iTotalPlayTime += (int)( countImg   ( ) * delay );
+	iTotalPlayTime += (int)( vidsLength ( ) );
+	iTotalPlayTime += (int)( filter_delay );
 
 	return iTotalPlayTime;
 }
@@ -654,8 +656,10 @@ void CXmlSlideshow::clear ( )
 	delay        = -1.0f;
 	filter_delay = -1.0f;
 	xres         = 720;
-	yres         = 480;	// NTSC for now
+	yres         = 480;  // NTSC for now
+	aspect       =   0;  // 4:3
 	intro_page   = true;
+	fade_out     = true;
 	id           = NULL;
 	background   = QString ( "" );
 	audio_list.clear ( );
@@ -716,6 +720,8 @@ CXmlSlideshow &CXmlSlideshow::operator = (CXmlSlideshow &theOther)
 	xres           = theOther.xres;
 	yres           = theOther.yres;
 	intro_page     = theOther.intro_page;
+	aspect         = theOther.aspect;
+	fade_out       = theOther.fade_out;
 	id             = theOther.id;
 	background     = theOther.background;
 	slideshow_name = theOther.slideshow_name;
@@ -748,6 +754,8 @@ bool CXmlSlideshow::operator == ( CXmlSlideshow &theOther )
 		( xres             == theOther.xres             ) &&
 		( yres             == theOther.yres             ) &&
 		( intro_page       == theOther.intro_page       ) &&
+		( aspect           == theOther.aspect           ) &&
+		( fade_out         == theOther.fade_out         ) &&
 		( background       == theOther.background       ) &&
 		( slideshow_name   == theOther.slideshow_name   ) &&
 		( imgBkgImg ( )    == theOther.imgBkgImg ( )    ) &&
@@ -848,6 +856,8 @@ bool CXmlSlideshow::readXml ( QDomElement *pElement )
 	time_object   *pTimeObject = NULL;
 	bool bReturn = false;
 	intro_page   = true;
+	aspect       = 0;
+	fade_out     = true;
 	QString audioList, filterList;
 
 	QDomNode xmlNode = pElement->firstChild ( );
@@ -856,6 +866,12 @@ bool CXmlSlideshow::readXml ( QDomElement *pElement )
 	a = pElement->attributeNode ( SLIDESHOW_INTRO );
 	if ( !  a.value ( ).isEmpty ( ) )
 		intro_page = a.value( ).toInt ( );
+	a = pElement->attributeNode ( SLIDESHOW_ASPECT );
+	if ( !  a.value ( ).isEmpty ( ) )
+		aspect = a.value( ).toInt ( );
+	a = pElement->attributeNode ( SLIDESHOW_FADE_OUT );
+	if ( !  a.value ( ).isEmpty ( ) )
+		fade_out = a.value  ( ).toInt ( );
 	a = pElement->attributeNode ( SLIDESHOW_FILTER_DELAY );
 	filter_delay    = a.value ( ).toFloat ( );
 	a = pElement->attributeNode ( SLIDESHOW_BACKGROUND );
@@ -989,7 +1005,11 @@ bool CXmlSlideshow::writeXml ( QDomElement *pElement )
 	if ( bImgBackgroundImg )
 		pElement->setAttribute( SLIDESHOW_IMG_BKG_IMG, "true" );
 	if ( ! intro_page )
-		pElement->setAttribute( SLIDESHOW_INTRO,  QString ("%1").arg ( (int)intro_page ) );
+		pElement->setAttribute( SLIDESHOW_INTRO, QString ("%1").arg ( (int)intro_page ) );
+	if ( aspect )
+		pElement->setAttribute( SLIDESHOW_ASPECT, QString ("%1").arg ( aspect ) );
+	if ( ! fade_out )
+		pElement->setAttribute( SLIDESHOW_FADE_OUT, QString ("%1").arg ( (int)fade_out ) );
 	if ( audio_list.count ( ) > 0 )
 		pElement->setAttribute( SLIDESHOW_AUDIO_LIST, audio_list.join (QString ("+-+")) );
 	if ( validFilterNames.count ( ) > 0 )
