@@ -33,6 +33,8 @@
 #define SLIDESHOW_TAG		"slideshow"
 #define SLIDESHOW_DELAY		"delay"
 #define SLIDESHOW_INTRO		"intro"
+#define SLIDESHOW_ASPECT	"aspect"
+#define SLIDESHOW_FADE_OUT	"fade_out"
 #define SLIDESHOW_FILTER_DELAY	"filter_delay"
 #define SLIDESHOW_NAME		"name"
 #define SLIDESHOW_XRES		"xres"
@@ -210,9 +212,11 @@ public:
 	QString     slideshow_name;
 	float delay;             // in seconds
 	float filter_delay;      // in seconds
-	int   xres;               // Determines the resolution as well as PAL vs NTSC
+	int   xres;              // Determines the resolution as well as PAL vs NTSC
 	int   yres;
+	int   aspect;            // Currently only 0=4:3 or 1=16:9
 	bool  intro_page;
+	bool  fade_out;
 	uint  total_audio_length; // in seconds.
 	bool  bImgBackgroundImg;
 	// Note: filter and img are on the same level, to get a continous structure as stored in the file we need to check the iPosition of those structures ...
