@@ -44,9 +44,10 @@ void Encoder::endStream ( )
 {
 }
 
-void Encoder::setVideoParam ( uint iBitrate, enAspect )
+void Encoder::setVideoParam ( uint iBitrate, enAspect aspect )
 {
   m_iVideoBitrate = iBitrate;
+  m_AspectRatio   = aspect;
 }
 
 void Encoder::setAudioParam ( uint iBitrate, int iSample ) // 192000, 4800
