@@ -16,6 +16,7 @@ extern "C" {
 #define __STDC_LIMIT_MACROS
 #include <libavformat/avformat.h>
 #include <libswscale/swscale.h>
+#include <engine/colorspace.h>
 }
 
 #define AUDIO_BUF_SIZE 192 * 1024 * 4
@@ -100,7 +101,7 @@ bool FFmpeg::Media::openStream ( QString qsStreamName )
   if ( av_find_stream_info ( m_pFormatCtx ) < 0 )
     return false; // Couldn't find stream information
 
-  // pFormatCtx->streams is just an array of pointers, of size pFormatCtx->nb_streams, 
+  // pFormatCtx->streams is just an array of pointers, of size pFormatCtx->nb_streams,
   // so let's walk through it until we find a video stream.
   bool bHasAudio = false;
   bool bHasVideo = false;
@@ -130,6 +131,12 @@ bool FFmpeg::Media::openStream ( QString qsStreamName )
       }
       unlockEngine ( );
 
+//      if ( ( pCodecCtx->sample_aspect_ratio.num == 0 ) && ( pCodecCtx->sample_aspect_ratio.den == 1 ) )  {
+//        // Assume input aspect ratio of 4:3
+//        float fAspectRatio =  16.0f / 9.0f;
+//        float fAspectRatio =  4.0f / 3.0f;
+//        pCodecCtx->sample_aspect_ratio = av_d2q ( fAspectRatio * pCodecCtx->height / pCodecCtx->width, 255 );
+//      }
       bHasVideo = true;
       m_pVideoCodecCtx = pCodecCtx;
       m_iVideoStreamID = iStreamID;
@@ -192,6 +199,20 @@ void FFmpeg::Media::setOutputContext ( AVCodecContext *pVideoContext, AVCodecCon
 {
   m_pOutputVideo = pVideoContext;
   m_pOutputAudio = pAudioContext;
+
+// debug for 16:9
+if ( pVideoContext )  {
+  AVRational sample_aspect_ratio = pVideoContext->sample_aspect_ratio;
+
+  int i = 0;
+  i++;
+}
+//if ( ( m_pVideoCodecCtx->sample_aspect_ratio.num == 0 ) && ( m_pVideoCodecCtx->sample_aspect_ratio.den == 1 ) )  {
+//  // Assume input aspect ratio of 4:3
+//  float fAspectRatio =  4.0f / 3.0f;
+//  m_pVideoCodecCtx->sample_aspect_ratio = av_d2q ( fAspectRatio * m_pVideoCodecCtx->height / m_pVideoCodecCtx->width, 255 );
+//}
+
 }
 
 void FFmpeg::Media::resetStream ( )
@@ -297,23 +318,7 @@ FFmpeg::Buffer *FFmpeg::Audio::decode ( bool &bEOF )
 
   return m_pSampleBuffer;
 }
-/*
-FFmpeg::Buffer *FFmpeg::Audio::decode ( bool &bEOF )
-{
-  bEOF = false;
-  if ( ! m_pAudioCodecCtx )
-    return NULL;
 
-  AVPacket packet;
-  int  iRet  = av_read_frame ( m_pFormatCtx, &packet );
-  if ( iRet == 0 )
-    decode ( &packet  );
-  else  if ( iRet < 0 )
-    bEOF = true;
-
-  return m_pSampleBuffer;
-}
-*/
 FFmpeg::Buffer *FFmpeg::Audio::decode ( AVPacket *pPacket )
 {
   if ( ! pPacket )
@@ -331,9 +336,11 @@ FFmpeg::Buffer *FFmpeg::Audio::decode ( AVPacket *pPacket )
       iDecodedSize  = m_iSampleSize;
       memset ( pDecodedAudio, 0, m_iSampleSize );
 
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT ( 52, 26, 0 )
       iLength = avcodec_decode_audio2 ( m_pAudioCodecCtx, pDecodedAudio, &iDecodedSize, pPacketData, iPacketSize );
-//printf ( "  %s::%d > len<%d> decoded<%d> iPacketSize<%d>\n", __FILE__, __LINE__, iLength, iDecodedSize, iPacketSize );
-
+#else
+      iLength = avcodec_decode_audio3 ( m_pAudioCodecCtx, pDecodedAudio, &iDecodedSize, pPacket );
+#endif
       if ( iDecodedSize < 1 ) // Something completely strange happened ( E.g. ogg causes this )
         continue;
 
@@ -344,7 +351,6 @@ FFmpeg::Buffer *FFmpeg::Audio::decode ( AVPacket *pPacket )
       else  {
         iPacketSize -= iLength;
         pPacketData += iLength;
-
         memcpy ( (uint8_t*)(m_pSampleBuffer->pBuffer+iOffset), pDecodedAudio, iDecodedSize );
         iOffset += iDecodedSize;
       }
@@ -366,6 +372,8 @@ FFmpeg::Buffer *FFmpeg::Audio::decode ( AVPacket *pPacket )
   m_pSampleBuffer->iSampleRate        = m_pAudioCodecCtx->sample_rate;
   m_pSampleBuffer->iChannels          = m_pAudioCodecCtx->channels;  // # of input Channels
   m_pSampleBuffer->iSamplesPerChannel = (unsigned int)iOffset / ( sizeof ( short ) * getChannels ( ) );
+  if ( m_pAudioCodecCtx->sample_fmt  == SAMPLE_FMT_U8 )
+       m_pSampleBuffer->iSamplesPerChannel *= 2;
 
   return m_pSampleBuffer;
 }
@@ -457,14 +465,21 @@ bool FFmpeg::Audio::openStream ( QString qsStreamName )
 
 FFmpeg::Buffer *FFmpeg::Audio::decodeAudio ( bool &bEOF )
 {
-  int    iSamples       = 0;
-  int    iSampleSize    = sizeof ( short );
-  int    iOutChannels   = m_pOutputAudio->channels;
-  int    iOutSampleRate = m_pOutputAudio->sample_rate;
-  int    iOutFrameSize  = m_pOutputAudio->frame_size;
+  int    iSamples          = 0;
+  int    iSampleSize       = sizeof ( short );
+  int    iOutChannels      = m_pOutputAudio->channels;
+  int    iOutSampleRate    = m_pOutputAudio->sample_rate;
+  int    iOutFrameSize     = m_pOutputAudio->frame_size;
 
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT ( 52, 15, 0 )
   if ( ! m_pResample && m_pAudioCodecCtx )
-         m_pResample    = audio_resample_init ( iOutChannels, m_pAudioCodecCtx->channels, iOutSampleRate, m_pAudioCodecCtx->sample_rate );
+         m_pResample  = audio_resample_init ( iOutChannels, m_pAudioCodecCtx->channels, iOutSampleRate, m_pAudioCodecCtx->sample_rate );
+#else
+  SampleFormat enOutFormat = m_pOutputAudio->sample_fmt;
+  SampleFormat enInFormat  = m_pAudioCodecCtx->sample_fmt;
+  if ( ! m_pResample && m_pAudioCodecCtx )
+         m_pResample  = av_audio_resample_init ( iOutChannels, m_pAudioCodecCtx->channels, iOutSampleRate, m_pAudioCodecCtx->sample_rate, enOutFormat, enInFormat, 16, 10, 0, 0.8 );
+#endif
 
   FFmpeg::Buffer *pBuffer = NULL;
   char *pData = (char *)av_malloc ( AUDIO_BUF_SIZE );
@@ -514,14 +529,21 @@ FFmpeg::Buffer *FFmpeg::Audio::decodeAudio ( bool &bEOF )
 
 FFmpeg::Buffer *FFmpeg::Audio::decodeAudio ( AVPacket *pPacket )
 {
-  int    iSampleSize    = sizeof ( short );
-  int    iOutChannels   = m_pOutputAudio->channels;
-  int    iOutSampleRate = m_pOutputAudio->sample_rate;
-  int    iOutFrameSize  = m_pOutputAudio->frame_size;
+  int    iSampleSize       = sizeof ( short );
+  int    iOutChannels      = m_pOutputAudio->channels;
+  int    iOutSampleRate    = m_pOutputAudio->sample_rate;
+  int    iOutFrameSize     = m_pOutputAudio->frame_size;
 
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT ( 52, 15, 0 )
+  if ( ! m_pResample && m_pAudioCodecCtx )
+         m_pResample  = audio_resample_init ( iOutChannels, m_pAudioCodecCtx->channels, iOutSampleRate, m_pAudioCodecCtx->sample_rate );
+#else
+  SampleFormat enOutFormat = m_pOutputAudio->sample_fmt;
+  SampleFormat enInFormat  = m_pAudioCodecCtx->sample_fmt;
   // First we ensure we have a resample context if required ...
   if ( ! m_pResample && m_pAudioCodecCtx )
-         m_pResample    = audio_resample_init ( iOutChannels, m_pAudioCodecCtx->channels, iOutSampleRate, m_pAudioCodecCtx->sample_rate );
+         m_pResample    = av_audio_resample_init ( iOutChannels, m_pAudioCodecCtx->channels, iOutSampleRate, m_pAudioCodecCtx->sample_rate, enOutFormat, enInFormat, 16, 10, 0, 0.8 );
+#endif
 
   // Alloc some memory ...
   FFmpeg::Buffer *pBuffer = NULL;
@@ -597,6 +619,11 @@ FFmpeg::Vid::Vid ( )
   m_pAudioBuffer   = (uint8_t *) av_malloc ( AUDIO_BUF_SIZE );
   m_pOutputVideo   = NULL;
   m_pOutputAudio   = NULL;
+  m_iPaddingYUV[0] = 16;
+  m_iPaddingYUV[1] = 128;
+  m_iPaddingYUV[2] = 128;
+  m_iPadding[0]    = 0;
+  m_iPadding[1]    = 0;
 }
 
 FFmpeg::Vid::~Vid ( )
@@ -636,21 +663,77 @@ void FFmpeg::Vid::closeStream ( )
   FFmpeg::Audio::closeStream ( );
 }
 
-void FFmpeg::Vid::setSWScale ( int iTargetWidth, int iTargetHeight )
+void FFmpeg::Vid::setSWScale ( int iTargetWidth, int iTargetHeight, int iAspectRatio, int iPaddingColor )
 {
   if ( ! m_pVideoCodecCtx )
     return;
 
+  int r = ( ( iPaddingColor >> 16 ) & 0xFF );
+  int g = ( ( iPaddingColor >>  8 ) & 0xFF );
+  int b = (   iPaddingColor         & 0xFF );
+
+  m_iPaddingYUV[0] = RGB_TO_Y ( r, g, b    );
+  m_iPaddingYUV[1] = RGB_TO_U ( r, g, b, 0 );
+  m_iPaddingYUV[2] = RGB_TO_V ( r, g, b, 0 );
+
+  m_iPadding[0]    = 0;
+  m_iPadding[1]    = 0;
+
   // as we only generate a YUV420P picture, we must convert it to the codec pixel format if needed
-  int iCWidth  = m_pVideoCodecCtx->width;
-  int iCHeight = m_pVideoCodecCtx->height;
-  if ( ( m_pVideoCodecCtx->pix_fmt != PIX_FMT_YUV420P ) || ( iCWidth != iTargetWidth ) || ( iCHeight != iTargetHeight ) )
-    m_pImgConvertCtx = sws_getContext ( iCWidth, iCHeight, m_pVideoCodecCtx->pix_fmt, iTargetWidth, iTargetHeight, PIX_FMT_YUV420P, SWS_BICUBIC, NULL, NULL, NULL );
+  int iSourceWidth  = m_pVideoCodecCtx->width;
+  int iSourceHeight = m_pVideoCodecCtx->height;
+
+  int iAspectNum = m_pVideoCodecCtx->sample_aspect_ratio.num;
+  int iAspectDen = m_pVideoCodecCtx->sample_aspect_ratio.den;
+  if ( iAspectRatio == 1 )  {
+    // δ = 16/9 x 480/720 = (16x480)/(9x720) = 7680/6480 = ( 2^9 x 3 x 5 ) / ( 2^4 x 3^4 x 5 ) = 2^5 / 3^3 == 32/27
+    // we want to create a 16:9 video so ...
+    // if the source is not also 16:9 we need to prescale and pad the final video accordingly.
+    if ( ( iAspectNum != 32 ) || ( iAspectDen != 27 ) )  {
+      float fScale = 27.0 / 32.0;
+      if  ( ( iAspectDen != 0 ) && ( iAspectNum != 0 ) )
+            fScale   *= (float)iAspectNum / iAspectDen;
+      fScale = (float)iTargetWidth - ( fScale * iSourceWidth );
+      if  ( fScale  < 0.0 )  {
+        m_iPadding[1] = -(int)( fScale / 2.0 );
+        iTargetWidth  = iTargetHeight - 2 * m_iPadding[1];
+      }
+      else  {
+        m_iPadding[0] = (int)( fScale / 2.0 );
+        iTargetWidth  = iTargetWidth - 2 * m_iPadding[0];
+      }
+    }
+  }
+  else  {
+    // δ' = 4/3 / (720/480) = ( 480 x 4 ) / ( 720 x 3 ) = 160 / 180 = 8/9
+    // We want to create a 4:3 video, so ...
+    if ( ( iAspectNum != 9 ) || ( iAspectDen != 8 ) )  {
+      // if the source is not also 4:3 we need to prescale and pad the video accordingly
+      float fScale = 9.0 / 8.0;
+      if  ( ( iAspectDen != 0 ) && ( iAspectNum != 0 ) )
+            fScale   *= (float)iAspectNum / iAspectDen;
+//      fScale = (float)iTargetHeight - ( fScale * iSourceHeight );
+      fScale = (float)iTargetWidth - ( fScale * iSourceWidth );
+      if  ( fScale    < 0.0 )  {
+        // Note since we have a negative number we need to scale width (like 16:9)
+        //      and NOT the Height as you might expect.
+        m_iPadding[0] = -(int)( fScale / 2.0 );
+        iTargetWidth  = iTargetWidth - 2 * m_iPadding[0];
+      }
+      else  {
+        m_iPadding[1] = (int)( fScale / 2.0 );
+        iTargetHeight = iTargetHeight - 2 * m_iPadding[1];
+      }
+    }
+  }
+
+  if ( ( m_pVideoCodecCtx->pix_fmt != PIX_FMT_YUV420P ) || ( iSourceWidth != iTargetWidth ) || ( iSourceHeight != iTargetHeight ) )
+    m_pImgConvertCtx = sws_getContext ( iSourceWidth, iSourceHeight, m_pVideoCodecCtx->pix_fmt, iTargetWidth, iTargetHeight, PIX_FMT_YUV420P, SWS_BICUBIC, NULL, NULL, NULL );
   else
     m_pImgConvertCtx = NULL;
 
   // allocate the encoded raw picture
-  m_pFrame = allocPicture ( m_pVideoBuffer, PIX_FMT_YUV420P, iTargetWidth, iTargetHeight );
+  m_pFrame = allocPicture ( m_pVideoBuffer, PIX_FMT_YUV420P, iTargetWidth + 2 * m_iPadding[0], iTargetHeight + 2 * m_iPadding[1] );
 
   if ( ! m_pFrame )
     printf ( "Could not allocate picture" );
@@ -696,9 +779,10 @@ AVPacket *FFmpeg::Vid::hasRemainingAudio ( AVStream *pOutputAudioStream )
   m_pPacket->size = pOutputAudioStream->codec->codec->encode ( pOutputAudioStream->codec, m_pAudioBuffer, iSize, (void *)pSamples->pBuffer );
   pOutputAudioStream->codec->frame_number++;
 
-  m_pPacket->pts  = av_rescale_q ( pOutputAudioStream->codec->coded_frame->pts, pOutputAudioStream->codec->time_base, pOutputAudioStream->time_base );
+//  if ( pOutputAudioStream->codec->coded_frame && pOutputAudioStream->codec->coded_frame->pts != AV_NOPTS_VALUE )
+    m_pPacket->pts  = av_rescale_q ( pOutputAudioStream->codec->coded_frame->pts, pOutputAudioStream->codec->time_base, pOutputAudioStream->time_base );
 
-  m_pPacket->flags |= PKT_FLAG_KEY;
+  m_pPacket->flags       |= PKT_FLAG_KEY;
   m_pPacket->stream_index = pOutputAudioStream->index;
   m_pPacket->data         = m_pAudioBuffer;
 
@@ -761,27 +845,54 @@ AVPacket *FFmpeg::Vid::getNextPacket ( AVStream *pOutputVideoStream, AVStream *p
       iWidth    = m_pVideoCodecCtx->width;
       iHeight   = m_pVideoCodecCtx->height;
       // Determine required buffer size and allocate buffer
-      iNumBytes = avpicture_get_size ( PIX_FMT_YUV420P, iWidth, iHeight );
+//    iNumBytes = avpicture_get_size ( PIX_FMT_YUV420P, iWidth, iHeight ) + FF_INPUT_BUFFER_PADDING_SIZE;
+      int  x = m_pOutputVideo->width;
+      int  y = m_pOutputVideo->height;
+      if ( x < iWidth  )
+           x = iWidth;
+      if ( y < iHeight )
+           y = iHeight;
+      iNumBytes = avpicture_get_size ( PIX_FMT_YUV420P, x, y ) + FF_INPUT_BUFFER_PADDING_SIZE;
       pBuffer   = new uint8_t[iNumBytes];
+      pBuffer[iNumBytes-1] = 0;
 
       // Assign appropriate parts of buffer to image planes in pFrameRGB
       avpicture_fill ( (AVPicture *)pFrame, pBuffer, PIX_FMT_YUV420P, iWidth, iHeight );
 
       iFrameFinished = 0;
       // Decode input video frame
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT ( 52, 26, 0 )
       int  iLen = avcodec_decode_video ( m_pVideoCodecCtx, pFrame, &iFrameFinished, m_pPacket->data, m_pPacket->size );
       if ( iFrameFinished == 0 ) // Mpeg streams might need this double encoding phase ...
            iLen = avcodec_decode_video ( m_pVideoCodecCtx, pFrame, &iFrameFinished, m_pPacket->data, m_pPacket->size );
-
+#else
+      int  iLen = avcodec_decode_video2 ( m_pVideoCodecCtx, pFrame, &iFrameFinished, m_pPacket );
+      if ( iFrameFinished == 0 ) // Mpeg streams might need this double encoding phase ...
+           iLen = avcodec_decode_video2 ( m_pVideoCodecCtx, pFrame, &iFrameFinished, m_pPacket );
+#endif
       // Did we get a video frame?
       // Note : Instead of running the decoding again, it seems to be agreed on
       //        that simply ignoring the iFrameFinished check. However it works for me !
       if ( iFrameFinished )  {
         int iOutSize = -1;
-        if ( m_pImgConvertCtx )
-          sws_scale ( m_pImgConvertCtx, pFrame->data, pFrame->linesize, 0, iHeight, m_pFrame->data, m_pFrame->linesize );
-        else // av_picture_copy ( AVPicture *dst, const AVPicture *src, int pix_fmt, int width, int height);
-          av_picture_copy ( (AVPicture *)m_pFrame, (AVPicture *)pFrame, PIX_FMT_YUV420P, iWidth, iHeight );
+        if ( ( m_iPadding[0] == 0 ) && ( m_iPadding[1] == 0 ) )  {
+          if ( m_pImgConvertCtx )
+            sws_scale ( m_pImgConvertCtx, pFrame->data, pFrame->linesize, 0, iHeight, m_pFrame->data, m_pFrame->linesize );
+          else // av_picture_copy ( AVPicture *dst, const AVPicture *src, int pix_fmt, int width, int height);
+            av_picture_copy ( (AVPicture *)m_pFrame, (AVPicture *)pFrame, PIX_FMT_YUV420P, iWidth, iHeight );
+        }
+        else  {
+          int iOutWidth  = m_pOutputVideo->width;
+          int iOutHeight = m_pOutputVideo->height;
+
+          if ( m_pImgConvertCtx )  {
+            sws_scale ( m_pImgConvertCtx, pFrame->data, pFrame->linesize, 0, iHeight, m_pFrame->data, m_pFrame->linesize );
+
+            av_picture_copy ( (AVPicture *)pFrame, (AVPicture *)m_pFrame, PIX_FMT_YUV420P, iOutWidth-2*m_iPadding[0], iOutHeight-2*m_iPadding[1] );
+          }
+
+          av_picture_pad ( (AVPicture *)m_pFrame, (const AVPicture *)pFrame, iOutHeight, iOutWidth, PIX_FMT_YUV420P, m_iPadding[1], m_iPadding[1], m_iPadding[0], m_iPadding[0], m_iPaddingYUV );
+        }
 
         // encode the image
         iOutSize = avcodec_encode_video ( pOutputVideoStream->codec, m_pVideoBuffer, VIDEO_BUF_SIZE, m_pFrame );
@@ -819,7 +930,8 @@ AVPacket *FFmpeg::Vid::getNextPacket ( AVStream *pOutputVideoStream, AVStream *p
       m_pPacket->size = pOutputAudioStream->codec->codec->encode ( pOutputAudioStream->codec, m_pAudioBuffer, iSize, (void *)pSamples->pBuffer );
       pOutputAudioStream->codec->frame_number++;
 
-      m_pPacket->pts  = av_rescale_q ( pOutputAudioStream->codec->coded_frame->pts, pOutputAudioStream->codec->time_base, pOutputAudioStream->time_base );
+//      if ( pOutputAudioStream->codec->coded_frame && pOutputAudioStream->codec->coded_frame->pts != AV_NOPTS_VALUE )
+        m_pPacket->pts = av_rescale_q ( pOutputAudioStream->codec->coded_frame->pts, pOutputAudioStream->codec->time_base, pOutputAudioStream->time_base );
 
       m_pPacket->flags       |= PKT_FLAG_KEY;
       m_pPacket->stream_index = pOutputAudioStream->index;
@@ -854,7 +966,7 @@ QImage &FFmpeg::Vid::getFrame ( AVFrame *pFrame, int iWidth, int iHeight )
   // Assign appropriate parts of buffer to image planes in pFrameRGB
   avpicture_fill ( (AVPicture *)pFrameRGB, pBuffer, PIX_FMT_RGB32, iWidth, iHeight );
 
-  pSwsCtx = sws_getContext ( iWidth, iHeight, m_pVideoCodecCtx->pix_fmt, iWidth, iHeight, PIX_FMT_RGB32, SWS_BILINEAR, NULL, NULL, NULL);
+  pSwsCtx = sws_getContext ( iWidth, iHeight, m_pVideoCodecCtx->pix_fmt, iWidth, iHeight, PIX_FMT_RGB32, SWS_BILINEAR, NULL, NULL, NULL );
 
   // Convert the image from its native format to RGB
   sws_scale ( pSwsCtx, pFrame->data, pFrame->linesize, 0, iHeight, pFrameRGB->data, pFrameRGB->linesize );
@@ -893,12 +1005,18 @@ QImage &FFmpeg::Vid::getFrame ( double fStreamPos )
   iWidth    = m_pVideoCodecCtx->width;
   iHeight   = m_pVideoCodecCtx->height;
   // Determine required buffer size and allocate buffer
-  iNumBytes = avpicture_get_size ( PIX_FMT_RGB32, iWidth, iHeight );
+  iNumBytes = avpicture_get_size ( PIX_FMT_RGB32, iWidth, iHeight ) + FF_INPUT_BUFFER_PADDING_SIZE;
 
   if ( fStreamPos < 0.0 )  {
     // get the last decoded frame, else invert the number ...
     fStreamPos *= -1.0;
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT ( 52, 26, 0 )
     avcodec_decode_video ( m_pVideoCodecCtx, pFrame, &iFrameFinished, NULL, 0 );
+#else
+    packet.data = NULL;
+    packet.size = 0;
+    avcodec_decode_video2 ( m_pVideoCodecCtx, pFrame, &iFrameFinished, &packet );
+#endif
 
     // Did we get a video frame?
     if ( iFrameFinished )  {
@@ -916,13 +1034,17 @@ QImage &FFmpeg::Vid::getFrame ( double fStreamPos )
         ; //av_log ( NULL, AV_LOG_ERROR, "  duration is unknown: %.2f\n", duration);
     }
     AVStream *pVideoStream = m_pFormatCtx->streams[m_iVideoStreamID];
-    int64_t iEndPosition = fDuration * fStreamPos / av_q2d ( pVideoStream->time_base ); // in time_base uni
+    int64_t iEndPosition = (int64_t)( fDuration * fStreamPos / av_q2d ( pVideoStream->time_base ) ); // in time_base uni
     iSeekPos = reallySeek ( m_pFormatCtx, m_iVideoStreamID, iEndPosition, 0, fDuration );
   }
 
   while ( !bFound && av_read_frame ( m_pFormatCtx, &packet ) >= 0 )  {
     // Decode video frame
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT ( 52, 26, 0 )
     avcodec_decode_video ( m_pVideoCodecCtx, pFrame, &iFrameFinished, packet.data, packet.size );
+#else
+    avcodec_decode_video2 ( m_pVideoCodecCtx, pFrame, &iFrameFinished, &packet );
+#endif
 
     // Did we get a video frame?
     if ( iFrameFinished )  {
@@ -975,9 +1097,9 @@ int FFmpeg::Vid::reallySeek ( AVFormatContext *pFormatCtx, int index, int64_t ti
     /* and then we try seeking by byte (AVSEEK_FLAG_BYTE) */
     // here we assume that the whole file has duration seconds.
     // so we'll interpolate accordingly.
-    AVStream *pStream = pFormatCtx->streams[index];
-    int64_t duration_tb = duration / av_q2d(pStream->time_base); // in time_base unit
-    double start_time = (double) pFormatCtx->start_time / AV_TIME_BASE; // in seconds
+    AVStream *pStream   = pFormatCtx->streams[index];
+    int64_t duration_tb = (int64_t)( duration / av_q2d ( pStream->time_base ) ); // in time_base unit
+    double start_time   = (double) pFormatCtx->start_time / AV_TIME_BASE; // in seconds
     // if start_time is negative, we ignore it; FIXME: is this ok?
     if (start_time < 0) {
         start_time = 0;
@@ -985,7 +1107,7 @@ int FFmpeg::Vid::reallySeek ( AVFormatContext *pFormatCtx, int index, int64_t ti
 
     // normally when seeking by timestamp we add start_time to timestamp
     // before seeking, but seeking by byte we need to subtract the added start_time
-    timestamp -= start_time / av_q2d(pStream->time_base);
+    timestamp -= (int64_t)( start_time / av_q2d ( pStream->time_base ) );
     if (pFormatCtx->file_size <= 0) {
         return -1;
     }
@@ -1026,48 +1148,6 @@ double FFmpeg::Vid::guessDuration ( AVFormatContext *pFormatCtx, int index )
     }
     return -1;
 }
-/* NON Seeking version.
-QImage &FFmpeg::Vid::getFrame ( double fStreamPos )
-{
-  if ( ( fStreamPos < 0.0f ) || ( fStreamPos > 1.0f ) || ( ! m_pVideoCodecCtx ) )  {
-    // TODO: Create Error Image ...
-    m_frame = QImage ( );
-    return m_frame;
-  }
-
-  AVPacket   packet;
-  AVFrame   *pFrame    = NULL;
-  bool bFound = false;
-
-  int iFrameFinished;
-  int iWidth, iHeight, iNumBytes;
-
-  pFrame    = avcodec_alloc_frame ( );
-  iWidth    = m_pVideoCodecCtx->width;
-  iHeight   = m_pVideoCodecCtx->height;
-  // Determine required buffer size and allocate buffer
-  iNumBytes = avpicture_get_size ( PIX_FMT_RGB32, iWidth, iHeight );
-
-  while ( !bFound && av_read_frame ( m_pFormatCtx, &packet ) >= 0 )  {
-    // Decode video frame
-    avcodec_decode_video ( m_pVideoCodecCtx, pFrame, &iFrameFinished, packet.data, packet.size );
-
-    // Did we get a video frame?
-    if ( iFrameFinished )  {
-      getFrame ( pFrame, iWidth, iHeight );
-      bFound = true;
-    }
-
-    // Free the packet that was allocated by av_read_frame
-    av_free_packet ( &packet );
-  }
-
-  if ( pFrame )
-    av_free ( pFrame );
-
-  return m_frame;
-}
-*/
 
 bool FFmpeg::Vid::isValidStream ( )
 {
@@ -1139,7 +1219,11 @@ bool FFmpeg::initStream ( QString qsFileName, enVideo videoFormat, enAudio audio
   else
     pOutputFormat->audio_codec = CODEC_ID_MP2;
 
+#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT ( 52, 27, 0 )
   m_pOutputCtx = av_alloc_format_context ( );
+#else
+  m_pOutputCtx = avformat_alloc_context ( );
+#endif
   if ( ! m_pOutputCtx )
     return false;
 
@@ -1172,6 +1256,8 @@ bool FFmpeg::initStream ( QString qsFileName, enVideo videoFormat, enAudio audio
   // write the stream header
   m_pOutputCtx->packet_size = 2048;
   m_pOutputCtx->mux_rate    = 10080000;
+  m_pOutputCtx->preload     = (int)( 0.5*AV_TIME_BASE );
+  m_pOutputCtx->max_delay   = (int)( 0.7*AV_TIME_BASE );
   av_write_header ( m_pOutputCtx );
 
   // The last step we init all input audio streams ...
@@ -1283,25 +1369,19 @@ bool FFmpeg::addVideoStream ( int iCodecID, enVideo videoFormat, int iBitrate )
   if ( ! m_pVideoStream )
     return false;
 
+//  if ( m_threadCount > 1 )
+//    avcodec_thread_init ( m_videoStm->codec, m_threadCount );
+
+  float fAspectRatio =  4.0f / 3.0f;
+  if ( m_AspectRatio == arAspect16to9 )
+        fAspectRatio = 16.0f / 9.0f;
+
   AVCodecContext *pVideo  = m_pVideoStream->codec;
   pVideo->codec_id        = (CodecID)iCodecID;
   pVideo->codec_type      = CODEC_TYPE_VIDEO;
   pVideo->bit_rate        = iBitrate * 1024;
   pVideo->width           = 720;
-  pVideo->sample_aspect_ratio.den = 4;
-  pVideo->sample_aspect_ratio.num = 3;
-
-// The following field was introduced in 52.21.0
-#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(52,21,0)
-//#if LIBAVFORMAT_VERSION_MAJOR >= 52
-//#if LIBAVFORMAT_VERSION_MINOR >= 21
-  m_pVideoStream->sample_aspect_ratio.den = 4;
-  m_pVideoStream->sample_aspect_ratio.num = 3;
-//#endif
-#endif
-
-//  pVideo->dtg_active_format = FF_DTG_AFD_4_3; only used for decoding
-
+//pVideo->dtg_active_format = FF_DTG_AFD_4_3; only used for decoding
   if ( videoFormat == vfPAL )  {
     pVideo->height        = 576;
     pVideo->time_base.den =  25;
@@ -1314,10 +1394,15 @@ bool FFmpeg::addVideoStream ( int iCodecID, enVideo videoFormat, int iBitrate )
     pVideo->time_base.num =  1001;
     pVideo->gop_size      =    18;
   }
+
   pVideo->pix_fmt         = PIX_FMT_YUV420P;
   pVideo->rc_buffer_size  = VIDEO_BUF_SIZE;
   pVideo->rc_max_rate     = 9 * 1024 * 1024;
   pVideo->rc_min_rate     = 0;
+  pVideo->sample_aspect_ratio = av_d2q ( fAspectRatio * pVideo->height / pVideo->width, 255 );
+#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(52,21,0)
+  m_pVideoStream->sample_aspect_ratio = pVideo->sample_aspect_ratio;
+#endif
 
   return true;
 }
@@ -1359,7 +1444,7 @@ bool FFmpeg::OpenAudioEncoder ( )
     }
   }
 
-  m_iSampleSize = iAudioInputFrameSize * 2 * pAudio->channels;
+  m_iSampleSize = iAudioInputFrameSize * 2 * pAudio->channels + FF_INPUT_BUFFER_PADDING_SIZE;
   m_pSamples    = (int16_t*) av_malloc ( m_iSampleSize );
   // Zero out the audio so we have silence ...
   memset ( m_pSamples, 0, m_iSampleSize );
@@ -1368,8 +1453,9 @@ bool FFmpeg::OpenAudioEncoder ( )
 
 void FFmpeg::CloseAudioEncoder ( )
 {
-  if ( ! m_pAudioStream )
+  if ( ( ! m_pAudioStream ) || ( ! m_pAudioStream->codec ) || ( ! m_pAudioStream->codec->codec ) )
     return;
+
   lockEngine    ( );
   avcodec_close ( m_pAudioStream->codec );
   unlockEngine  ( );
@@ -1426,7 +1512,7 @@ bool FFmpeg::OpenVideoEncoder ( )
 
 void FFmpeg::CloseVideoEncoder ( )
 {
-  if ( ! m_pVideoStream )
+  if ( ( ! m_pVideoStream ) || ( ! m_pVideoStream->codec ) || ( ! m_pVideoStream->codec->codec ) )
     return;
 
   lockEngine    ( );
@@ -1457,9 +1543,13 @@ bool FFmpeg::addAudioStream ( int iCodecID )
   if ( ! m_pAudioStream )
     return false;
 
+//  if ( m_threadCount > 1 )
+//    avcodec_thread_init ( m_audioStm->codec, m_threadCount );
+
   AVCodecContext *pAudio = m_pAudioStream->codec;
   pAudio->codec_id       = (CodecID)iCodecID;
   pAudio->codec_type     = CODEC_TYPE_AUDIO;
+  pAudio->sample_fmt     = SAMPLE_FMT_S16;
   pAudio->bit_rate       = 448000;
   pAudio->sample_rate    = 48000;
   pAudio->channels       = 2;
@@ -1536,7 +1626,7 @@ bool FFmpeg::writeAudioFrame ( float fVolume )
 
   pAudioCodec->frame_number++;
   pkt.pts  = av_rescale_q ( pAudioCodec->coded_frame->pts, pAudioCodec->time_base, m_pAudioStream->time_base );
-  pkt.flags |= PKT_FLAG_KEY;
+  pkt.flags       |= PKT_FLAG_KEY;
   pkt.stream_index = m_pAudioStream->index;
   pkt.data         = m_pAudioBuffer;
 
@@ -1570,7 +1660,6 @@ bool FFmpeg::writeVideoFrame ( )
     pkt.data         = m_pVideoBuffer;
     pkt.size         = out_size;
 
-//printf ( " ... addVideoFrame : OutVideoStream -> CurDts=%lld\n", m_pOutputCtx->streams[0]->cur_dts );
     // write the compressed frame in the media file
     int  ret = av_write_frame ( m_pOutputCtx, &pkt );
     if ( ret != 0 ) {
@@ -1583,7 +1672,7 @@ bool FFmpeg::writeVideoFrame ( )
 
 bool FFmpeg::addImage ( QImage *pImage, int iFrames, float fVolume )
 {
-  if ( ! pImage )
+  if ( ( ! pImage ) || ( ! m_pFrame ) )
     return false;
 
   AVCodecContext *pVideo = m_pVideoStream->codec;
@@ -1632,8 +1721,11 @@ bool FFmpeg::addImage ( QImage *pImage, int iFrames, float fVolume )
   return true;
 }
 
-bool FFmpeg::addVid ( QString qsFileName, int iTargetWidth, int iTargetHeight )
+bool FFmpeg::addVid ( QString qsFileName, int iTargetWidth, int iTargetHeight, int iAspectRatio, int iPaddingColor )
 {
+  // iAspect=0 ==  4:3
+  // iAspect=1 == 16:9
+
   // This function transcodes the VID into mpeg2 compatible material and adds it to the stream.
   // First cause of action is to ensure we have already an open stream with m_vid.
   if ( ! m_vid.isValidStream ( ) || ( m_vid.streamName ( ) != qsFileName ) )  {
@@ -1644,7 +1736,7 @@ bool FFmpeg::addVid ( QString qsFileName, int iTargetWidth, int iTargetHeight )
       return false;
     }
   }
-  m_vid.setSWScale ( iTargetWidth, iTargetHeight );
+  m_vid.setSWScale ( iTargetWidth, iTargetHeight, iAspectRatio, iPaddingColor );
 
   double fTargetFrameRate = (double)m_pVideoStream->codec->time_base.den / m_pVideoStream->codec->time_base.num;
   double fSourceFrameRate = m_vid.getFrameRate ( ); //10.0;
