@@ -139,7 +139,7 @@ class FFmpeg : public Encoder
     bool   isValidStream    ( );
     double getFrameRate     ( );
 
-    void        setSWScale  ( int, int );
+    void        setSWScale  ( int, int, int a=0, int c=0 );
     SwsContext *getSWScale  ( );
     AVFrame   *allocPicture ( uint8_t *&, PixelFormat, int, int   );
     AVPacket *getNextPacket ( AVStream *, AVStream  *, int & );
@@ -158,6 +158,8 @@ class FFmpeg : public Encoder
     uint8_t    *m_pVideoBuffer;
     uint8_t    *m_pAudioBuffer;
     int         m_iVideoOutSize;
+    int         m_iPadding[2];
+    int         m_iPaddingYUV[3];
     AVFrame    *m_pFrame;         // Current Frame ???
   };
 
@@ -169,7 +171,7 @@ public:
   virtual bool addImage       ( QImage *, int, float fVolume=1.0f );
   virtual void endStream      ( );
 
-  virtual bool addVid         ( QString, int, int );
+  virtual bool addVid         ( QString, int, int, int a=0, int c=0 );
   virtual bool initVid        ( QString );
   virtual void closeVid       (         );
   virtual QImage &getVidFrame (  double );
