@@ -45,6 +45,11 @@ Logger::~Logger ( )
   m_pLogFile = NULL;
 }
 
+int Logger::logLevel ( )
+{
+  return (int)m_enLogLevel;
+}
+
 void Logger::setLogInfo (int iLogLevel, QString qsLogFileName )
 {
   m_enLogLevel    = (type)iLogLevel;
