@@ -44,6 +44,8 @@ public:
 #define eLOG(fmt, args...) line (this, pthread_self (), __FILE__, __FUNCTION__, __LINE__, ::Render::Logger::logError, fmt, ## args );
 // fLOG for fatal errors ( unrecoverable )
 #define fLOG(fmt, args...) line (this, pthread_self (), __FILE__, __FUNCTION__, __LINE__, ::Render::Logger::logFatal, fmt, ## args );
+// fLOG for fatal errors ( unrecoverable )
+#define LOG(lvl, fmt, args...) line (this, pthread_self (), __FILE__, __FUNCTION__, __LINE__, lvl, fmt, ## args );
 
   void log  (       const char *, ... );
   void log  ( type, const char *, ... );
@@ -51,10 +53,11 @@ public:
   void log  ( type, QString );
 
   void line ( void *, unsigned long, const char *, const char *, long, type, const char *, ... );
+  int logLevel ( );
 
 protected:
-  void lock   ( );
-  void unlock ( );
+  void lock    ( );
+  void unlock  ( );
 
 private:
   static  QMutex  m_mutex;
