@@ -27,8 +27,10 @@ public:
 
   bool    parseCommandLine ( int, char ** );
   QString getLogfileName   ( );
+  QString getPath          ( );
   int     getServerPort    ( );
   int     getLogLevel      ( );
+  int     getRegionalZone  ( );
   bool    isTTY            ( );
 
 protected:
@@ -40,10 +42,12 @@ private:
   QString m_qsTempPath;
   QString m_qsInputName;
   QString m_qsOutputName;
+  int     m_iRegionalZone;
   int     m_iServerPort;
   bool    m_bTTY;
   int     m_iLogLevel;
   QString m_qsLogFileName;
+  QString m_qsPath;
 };
 
 }; // end of namespace Render
