@@ -26,8 +26,9 @@
 #include <QFile>
 #include <QImage>
 #include <QFileInfo>
-#include <QTimerEvent>
 #include <QByteArray>
+#include <QTimerEvent>
+#include <QApplication>
 
 #include "utils.h"
 #include "shared.h"
@@ -148,6 +149,10 @@ void Render::slotReadSocket ( )
     case CLIENT_GOT_A_PIECE:
       sendNextPacket ( );
     break;
+    case CLIENT_RECONNECTING:
+      // This render object was created after a socket error
+      receivedReconnect ( );
+    break;
     case CLIENT_KILL_YOURSELF:
       m_bKillClient = true;  // slideshow will eventually terminate
 //      exit ( CLIENT_KILL_YOURSELF );
@@ -165,9 +170,14 @@ void Render::slotReadSocket ( )
 
 void Render::myExit ( int iExitCode )
 {
+  m_tcpSocket.flush   ( );
   m_tcpSocket.disconnectFromHost ( );
+  qApp->processEvents ( );
+
   if ( m_tcpSocket.state ( ) != QAbstractSocket::UnconnectedState )
        m_tcpSocket.waitForDisconnected ( 5000 );  // wait for up to 5 seconds to be disconnected
+  qApp->processEvents ( );
+
   QThread::exit ( iExitCode );
   logger ( ).iLOG ( "Server exited \n" );
   status ( ).log  ( "EXIT %d\n", iExitCode );
@@ -219,6 +229,12 @@ void Render::sendMessage ( quint16 iMessageID, QString qsMessage )
   m_tcpSocket.write  ( Message<quint16>::create ( iMessageID ) );
   m_tcpSocket.write  ( Message<quint64>::create ( (quint64)qsMessage.length ( ) ) );
 
+/*
+quint64 iLen = (quint64)qsMessage.length ( );
+char *cLen = (char *)&iLen;
+printf ( "%s::%d > len<%llu> [%02x %02x %02x %02x]\n", __FILE__, __LINE__, iLen, cLen[0], cLen[1], cLen[2], cLen[3] );
+*/
+
   QDataStream stream ( &m_tcpSocket );
   stream << qsMessage;
 
@@ -419,6 +435,23 @@ quint64 Render::fileFromSocket ( )
   return ( iBytesAvailable - iActualRead + iDelta );
 }
 
+void Render::receivedReconnect ( )
+{
+  logger ( ).iLOG ( "receiveReconnect \n" );
+  QString qsName;
+
+//  qint64  iBytesAvail = m_tcpSocket.bytesAvailable ( );
+  quint16 iLength;
+
+  QDataStream ds ( &m_tcpSocket );
+  ds >> iLength;
+  ds >> qsName;
+
+//printf ( "%s::%d > bytesAvail<%lld> len<%u> Reconnect Name<%s>\n", __FILE__, __LINE__, iBytesAvail, iLength, (const char *)qsName.toUtf8 ( ) );
+  logger ( ).iLOG ( "Reconnect Name<%s>\n", (const char *)qsName.toUtf8 ( ) );
+  status ( ).log  ( "RECONNECT\n" );
+}
+
 void Render::receivedXMLName ( )
 {
   QString qsFileName, qsTempPath;
@@ -530,12 +563,13 @@ void Render::createSlideshow ( )
         sendFile ( qsFileName );
       else  {
         sendMessage ( SERVER_STICK_A_FORK_IN_ME, tr ( "Done" ) );
-        exit ( 0 );
+        myExit ( 0 ); // exi the thread
       }
     }
     else  {
       sendMessage ( (quint16)SERVER_SHIT_HAPPENED, tr ( "Final slideshow file <%1> not found.\n" ).arg ( qsFileName ) );
-      exit ( -1 ); // exit the EventLoop. This will cause the function run to exit
+      myExit ( -1 );
+      //exit ( -1 ); // exit the EventLoop. This will cause the function run to exit
     }
   }
 }
