@@ -37,6 +37,9 @@ namespace Render
 // Tell the server to get started as he has all images / infos required.
 #define CLIENT_GO_TO_WORK 0xAEEE
 
+// there was an problem with the socket. Client tries to re-establish connection
+#define CLIENT_RECONNECTING 0x1003
+
 // KeepAlive messages sent every 20 seconds from server to QDVDAuthor
 // when no rendering is done. Simply to know if server is still there.
 #define SERVER_I_AM_ALIVE  0x5555
