@@ -16,6 +16,8 @@
 #include <QFileInfo>
 
 #include "utils.h"
+#include "logger.h"
+#include "parser.h"
 #include "render.h"
 #include "filter.h"
 #include "slideshow.h"
@@ -23,15 +25,17 @@
 namespace Render
 {
 
-Slideshow::Slideshow ( Render::Render *pParent )
+Slideshow::Slideshow ( Render *pParent )
 {
+  m_pParser = NULL;
   m_pParent = pParent;
-  m_size = QSize ( 720, 480 ); // NTSC
+  m_size    = QSize ( 720, 480 ); // NTSC
 }
 
-Slideshow::Slideshow ( )
+Slideshow::Slideshow ( Parser *pParser )
 {
   m_pParent = NULL;
+  m_pParser = pParser;
   m_size    = QSize ( 720, 480 ); // NTSC
 }
 
@@ -61,16 +65,34 @@ void Slideshow::setFileName ( QString &qsFileName )
 
 void Slideshow::displayProgress ( float fProgress )
 {
-  if ( m_pParent )
+  if ( m_pParent )  {
     m_pParent->sendProgress ( fProgress );
+    if ( m_pParent->logger ( ).logLevel ( ) <= Logger::logInformation )
+         m_pParent->logger ( ).iLOG ( "Progress : %f\n", fProgress );
+  }
   else {
-    printf ( "." );
+    if ( m_pParser && m_pParser->getLogLevel ( ) <= Logger::logInformation )
+      printf ( "Progress : %f\n", fProgress );
+    else
+      printf ( "." );
     if ( fProgress >= 100.0f )
       printf ( "\n" );
     fflush ( stdout );
   }
 }
 
+void Slideshow::log ( QString qsLogMessage, int iLogLevel )
+{
+ if ( m_pParent )  {
+    if ( m_pParent->logger ( ).logLevel ( ) <= iLogLevel )
+         m_pParent->logger ( ).LOG ( (Logger::type)iLogLevel, (const char *)qsLogMessage.toUtf8 ( ) );
+  }
+  else {
+    if ( m_pParser && m_pParser->getLogLevel ( ) <= iLogLevel )
+      printf ( "%s", (const char *)qsLogMessage.toUtf8 ( ) );
+  }
+}
+
 QString Slideshow::getHashName ( QString qsInput )
 {
   if ( m_pParent )
@@ -78,6 +100,23 @@ QString Slideshow::getHashName ( QString qsInput )
   return qsInput;
 }
 
+void Slideshow::prescaleImage ( QImage &theImage )
+{
+  float fAspect = 720.0/480.0 / ( 16.0/9.0 );
+  if  ( m_pSlideshow && ( m_pSlideshow->aspect == 0 ) )
+        fAspect = 720.0/480.0 / (  4.0/3.0 );
+
+  // Current only other aspect ratio is 16:9.
+  // Since the DVD spec uses the same resolution for both ratios
+  // the 16:9 images are being distorted.
+  // To counter this effect we will have to pre-scale the image to
+  // eliminate this effect
+  // For details check: http://lipas.uwasa.fi/~f76998/video/conversion/
+  int   iWidth  = (int)( fAspect * theImage.width ( ) );
+  int   iHeight = theImage.height ( );
+  theImage      = theImage.scaled ( iWidth, iHeight, Qt::IgnoreAspectRatio, Qt::SmoothTransformation );
+}
+
 bool Slideshow::exec ( )
 {
   if ( ! m_pSlideshow )
@@ -110,28 +149,33 @@ bool Slideshow::exec ( )
   CXmlSlideshow::img_struct  *pXmlCurrent, *pXmlPrevious;
   CXmlSlideshow::time_object *pTimeObject;
 
+  QString qsLog = QString ( "Starting slideshow %1\n" ).arg ( m_pSlideshow->slideshow_name );
+  log ( qsLog, Logger::logInformation );
   createBackground ( );
 
   Encoder::FFmpeg theEncoder;
-  theEncoder.setAudioList ( m_pSlideshow->audio_list );
-  theEncoder.initStream   ( m_qsFileName, enFormat, Encoder::Encoder::afAC3, 1000 );
+  theEncoder.setAudioList    ( m_pSlideshow->audio_list );
+  if ( m_pSlideshow->aspect == 1 ) // 16:9
+    theEncoder.setVideoParam ( 6000, Encoder::Encoder::arAspect16to9 );
+  theEncoder.initStream      ( m_qsFileName, enFormat, Encoder::Encoder::afAC3, 1000 );
 
   pXmlPrevious     = pXmlStart;
   uint t, iCount   = m_pSlideshow->count ( );
-  uint iAudioCount = m_pSlideshow->audio_list.count ( );
   float fProgress  = 0.0f;
-  float fCount2    = 2.0f * ( iCount + iAudioCount ) + 2.0f;
+  float fCount2    = 2.0f * iCount + 2.0f;
   int iProgress    = 0;
   int iStart       = 0;
 
   if ( ! m_pSlideshow->intro_page )  {
     // The user chose to skip the intro page with the slideshow title
     iStart        = 1;
-    pXmlPrevious  = m_pSlideshow->getImg ( 0 );
+    pXmlPrevious  = (CXmlSlideshow::img_struct *)m_pSlideshow->getTimeObject ( 0 );
     fCount2      -= 2.0f;
   }
+  if ( ! m_pSlideshow->fade_out )
+    fCount2      -= 2.0f;
 
-  for  ( t=iStart; t<iCount+1; t++ )  {
+  for  ( t=iStart; t<iCount + (int)m_pSlideshow->fade_out; t++ )  {
     if ( m_pParent && m_pParent->killClient ( ) )  {
       // The user terminated this slideshow generation
       theEncoder.endStream ( );
@@ -153,6 +197,8 @@ bool Slideshow::exec ( )
 
     pXmlCurrent = (CXmlSlideshow::img_struct *)pTimeObject;
     fProgress = 100.0f * ( (float)iProgress++ / fCount2 );
+    if ( fProgress == 0.0 )
+         fProgress  = 0.0001;
     displayProgress ( fProgress );
     if ( ( pXmlPrevious->node_name == "img" ) && 
          ( pXmlCurrent ->node_name == "img" ) )
@@ -172,17 +218,18 @@ bool Slideshow::exec ( )
 
     pXmlPrevious = pXmlCurrent;
   }
-  // The frame is to be displayed for a bit ...
-//printf ( "Encoding               <%s>\n", (const char *)pXmlPrevious->src.toUtf8 ( ) );
-  createFromImage  ( pXmlPrevious, &theEncoder, fFPS );
-  fProgress = 100.0f * ( (float)iProgress++ / fCount2 );
-  displayProgress ( fProgress );
-//printf ( "Encoding Transition to <%s>\n", (const char *)pXmlEnd->src.toUtf8 ( ) );
-  if ( pXmlPrevious->node_name == "img" )
-    createIITransition ( pXmlPrevious, pXmlEnd, &theEncoder, fFPS );
-  else
-    createVITransition ( (CXmlSlideshow::vid_struct *)pXmlPrevious, pXmlEnd, &theEncoder, fFPS );
 
+  // The frame is to be displayed for a bit ...
+  createFromImage  ( pXmlPrevious, &theEncoder, fFPS );
+
+  if ( m_pSlideshow->fade_out )  {
+    fProgress = 100.0f * ( (float)iProgress++ / fCount2 );
+    displayProgress ( fProgress );
+    if ( pXmlPrevious->node_name == "img" )
+      createIITransition ( pXmlPrevious, pXmlEnd, &theEncoder, fFPS );
+    else
+      createVITransition ( (CXmlSlideshow::vid_struct *)pXmlPrevious, pXmlEnd, &theEncoder, fFPS );
+  }
   displayProgress ( 100.0f );
 
   theEncoder.endStream  ( );
@@ -193,6 +240,8 @@ bool Slideshow::exec ( )
 
 void Slideshow::createBackground ( )
 {
+  log ( "Creating Background\n", Logger::logInformation );
+
   m_background = QImage ( m_size, QImage::Format_ARGB32_Premultiplied );
   QString qsBackground = m_pSlideshow->background;
   bool bTryImageBackground = true;
@@ -224,6 +273,11 @@ void Slideshow::createBackground ( )
 
 CXmlSlideshow::img_struct *Slideshow::createStartEnd ( bool bStart )
 {
+  if ( bStart )
+    log ( "Creating intro page\n", Logger::logInformation );
+  else
+    log ( "Creating extro page\n", Logger::logInformation );
+
   // This frunction creates the first frame and the last frame.
   int iX = m_size.width  ( );
   int iY = m_size.height ( );
@@ -239,7 +293,7 @@ CXmlSlideshow::img_struct *Slideshow::createStartEnd ( bool bStart )
   QString qsFileName = m_pSlideshow->slideshow_name;
   qsFileName.replace ( "/", "_" );
   if ( bStart )  {
-    thePainter.drawText ( iX / 3.0, iY / 2.0 - 10, m_pSlideshow->slideshow_name );
+    thePainter.drawText ( (int)( iX / 3.0 ), (int)( iY / 2.0 - 10 ), m_pSlideshow->slideshow_name );
     qsFileName = m_qsTempPath + qsFileName + "-start.jpg";
   }
   else
@@ -262,6 +316,10 @@ void Slideshow::createFromImage ( CXmlSlideshow::img_struct *pXmlImage, Encoder:
   if ( ! pXmlImage )
     return;
 
+  QFileInfo fileInfo ( pXmlImage->src );
+  QString qsLog = QString ( "Creating from Image %1\n" ).arg ( fileInfo.fileName ( ) );
+  log ( qsLog, Logger::logInformation );
+
   QPainter thePainter;
   QString  qsFileName = getHashName ( pXmlImage->src );
   QImage   canvas   ( m_size, QImage::Format_ARGB32_Premultiplied );
@@ -269,7 +327,8 @@ void Slideshow::createFromImage ( CXmlSlideshow::img_struct *pXmlImage, Encoder:
 
   if ( pXmlImage->pMatrix )
        imgStart = imgStart.transformed ( *pXmlImage->pMatrix, Qt::SmoothTransformation );
-  imgStart = imgStart.scaled ( m_size, Qt::KeepAspectRatio );
+  prescaleImage ( imgStart );
+  imgStart = imgStart.scaled ( m_size, Qt::KeepAspectRatio, Qt::SmoothTransformation );
 
   int iX, iY;
   iX = (int)( ( m_size.width  ( ) - imgStart.width  ( ) ) / 2.0 );
@@ -297,9 +356,19 @@ void Slideshow::createFromVid ( CXmlSlideshow::vid_struct *pXmlVideo, Encoder::F
   if ( ! pXmlVideo )
     return;
 
+  QFileInfo fileInfo ( pXmlVideo->src );
+  QString qsLog = QString ( "Creating from vid %1\n" ).arg ( fileInfo.fileName ( ) );
+  log ( qsLog, Logger::logInformation );
+
   QString qsFileName = getHashName ( pXmlVideo->src );
+
+  unsigned int iColor = 0x000000;
+  QColor theColor ( m_pSlideshow->background );
+  if ( theColor.isValid  ( ) )
+    iColor = theColor.rgba ( );
+
   // I do not think we'll be able to support Matrix operations. ( pXmlVideo->pMatrix )
-  pEncoder->addVid ( qsFileName, m_size.width ( ), m_size.height ( ) );
+  pEncoder->addVid ( qsFileName, m_size.width ( ), m_size.height ( ), m_pSlideshow->aspect, (int)iColor );
 }
 
 // Image to Image Transition
@@ -308,14 +377,17 @@ void Slideshow::createIITransition ( CXmlSlideshow::img_struct *pStart, CXmlSlid
   if ( ! pStart || ! pStop )
     return;
 
+  log ( "Creating II Transition\n", Logger::logInformation );
+
   QPainter thePainter;
   QString  qsFileName = getHashName ( pStart->src );
   QImage   canvas   ( m_size, QImage::Format_ARGB32_Premultiplied );
   QImage   imgStart ( qsFileName );
 
   if ( pStart->pMatrix )
-       imgStart = imgStart.transformed ( *pStart->pMatrix, Qt::SmoothTransformation );
-  imgStart = imgStart.scaled ( m_size, Qt::KeepAspectRatio );
+       imgStart = imgStart.transformed  ( *pStart->pMatrix, Qt::SmoothTransformation );
+  prescaleImage ( imgStart );
+  imgStart = imgStart.scaled ( m_size, Qt::KeepAspectRatio, Qt::SmoothTransformation );
 
   int iX, iY;
   iX = (int)( ( m_size.width  ( ) - imgStart.width  ( ) ) / 2.0 );
@@ -331,8 +403,9 @@ void Slideshow::createIITransition ( CXmlSlideshow::img_struct *pStart, CXmlSlid
   QImage imgStop ( qsFileName );
 
   if ( pStop->pMatrix )
-       imgStop = imgStop.transformed ( *pStop->pMatrix, Qt::SmoothTransformation );
-  imgStop = imgStop.scaled ( m_size, Qt::KeepAspectRatio );
+       imgStop  = imgStop.transformed ( *pStop->pMatrix, Qt::SmoothTransformation );
+  prescaleImage ( imgStop );
+  imgStop = imgStop.scaled ( m_size, Qt::KeepAspectRatio, Qt::SmoothTransformation );
 
   iX = (int)( ( m_size.width  ( ) - imgStop.width  ( ) ) / 2.0 );
   iY = (int)( ( m_size.height ( ) - imgStop.height ( ) ) / 2.0 );
@@ -369,6 +442,9 @@ void Slideshow::createIVTransition ( CXmlSlideshow::img_struct *pStart, CXmlSlid
 {
   if ( ! pStart || ! pStop )
     return;
+
+  log ( "Creating IV Transition\n", Logger::logInformation );
+
   // Here we have to 
   // o Get the first frame of the Video
   // o turn down the background music during the duration of the transition
@@ -380,7 +456,8 @@ void Slideshow::createIVTransition ( CXmlSlideshow::img_struct *pStart, CXmlSlid
 
   if ( pStart->pMatrix )
        imgStart = imgStart.transformed ( *pStart->pMatrix, Qt::SmoothTransformation );
-  imgStart = imgStart.scaled ( m_size, Qt::KeepAspectRatio );
+  prescaleImage ( imgStart );
+  imgStart = imgStart.scaled ( m_size, Qt::KeepAspectRatio, Qt::SmoothTransformation );
 
   int iX, iY;
   iX = (int)( ( m_size.width  ( ) - imgStart.width  ( ) ) / 2.0 );
@@ -399,14 +476,13 @@ void Slideshow::createIVTransition ( CXmlSlideshow::img_struct *pStart, CXmlSlid
   if ( imgStop.isNull ( ) )
      imgStop = imgStart;
 
-//  m_vid.setSWScale ( m_size.width ( ), m_size.height ( ) );
   // We need a image with transparency ...
   imgStop = imgStop.convertToFormat ( QImage::Format_ARGB32 );//_Premultiplied );
 
   if ( pStop->pMatrix )
-       imgStop = imgStop.transformed ( *pStop->pMatrix, Qt::SmoothTransformation );
-
-  imgStop = imgStop.scaled ( m_size, Qt::KeepAspectRatio );
+       imgStop  = imgStop.transformed  ( *pStop->pMatrix, Qt::SmoothTransformation );
+  prescaleImage ( imgStop );
+  imgStop = imgStop.scaled ( m_size, Qt::KeepAspectRatio, Qt::SmoothTransformation );
 
   iX = (int)( ( m_size.width  ( ) - imgStop.width  ( ) ) / 2.0 );
   iY = (int)( ( m_size.height ( ) - imgStop.height ( ) ) / 2.0 );
@@ -415,7 +491,6 @@ void Slideshow::createIVTransition ( CXmlSlideshow::img_struct *pStart, CXmlSlid
   thePainter.begin ( &canvas );
   thePainter.drawImage ( iX, iY, imgStop, 0, 0 );
   thePainter.end   ( );
-
   imgStop = canvas;
 
   float fDuration = (float)m_pSlideshow->filter_delay;
@@ -444,6 +519,9 @@ void Slideshow::createVITransition ( CXmlSlideshow::vid_struct *pStart, CXmlSlid
 {
   if ( ! pStart || ! pStop )
     return;
+
+  log ( "Creating VI Transition\n", Logger::logInformation );
+
   // Here we have to 
   // o Get the last frame of the Video
   // o turn up the background music during the duration of the transition
@@ -451,10 +529,15 @@ void Slideshow::createVITransition ( CXmlSlideshow::vid_struct *pStart, CXmlSlid
   QPainter thePainter;
   QString  qsFileName = getHashName ( pStart->src );
   QImage   canvas   ( m_size, QImage::Format_ARGB32_Premultiplied );
-  QImage   imgStart = pEncoder->getVidFrame ( -1.0 );
+  QImage   imgStart;
+
+  if ( pEncoder->initVid   ( qsFileName ) )
+       imgStart = pEncoder->getVidFrame ( -1.0 );
 //  // In case something went bad ...
   if ( imgStart.isNull ( ) )
        imgStart = QImage ( qsFileName );
+  if ( imgStart.isNull ( ) )
+       imgStart = QImage ( qsFileName );
 
   // Don't forget to close the current Vid stream in the encoder.
   pEncoder->closeVid ( );
@@ -462,10 +545,10 @@ void Slideshow::createVITransition ( CXmlSlideshow::vid_struct *pStart, CXmlSlid
   if ( imgStart.isNull ( ) )
     return;
 
-//  imgStart = QImage ( "/tmp/z.jpg" );
   if ( pStart->pMatrix )
        imgStart = imgStart.transformed ( *pStart->pMatrix, Qt::SmoothTransformation );
-  imgStart = imgStart.scaled ( m_size, Qt::KeepAspectRatio );
+  prescaleImage ( imgStart );
+  imgStart = imgStart.scaled ( m_size, Qt::KeepAspectRatio, Qt::SmoothTransformation );
 
   int iX, iY;
   iX = (int)( ( m_size.width  ( ) - imgStart.width  ( ) ) / 2.0 );
@@ -483,8 +566,9 @@ void Slideshow::createVITransition ( CXmlSlideshow::vid_struct *pStart, CXmlSlid
        imgStop = imgStart;
 
   if ( pStop->pMatrix )
-       imgStop = imgStop.transformed ( *pStop->pMatrix, Qt::SmoothTransformation );
-  imgStop = imgStop.scaled ( m_size, Qt::KeepAspectRatio );
+       imgStop  = imgStop.transformed ( *pStop->pMatrix, Qt::SmoothTransformation );
+  prescaleImage ( imgStop );
+  imgStop = imgStop.scaled ( m_size, Qt::KeepAspectRatio, Qt::SmoothTransformation );
 
   iX = (int)( ( m_size.width  ( ) - imgStop.width  ( ) ) / 2.0 );
   iY = (int)( ( m_size.height ( ) - imgStop.height ( ) ) / 2.0 );
@@ -521,6 +605,9 @@ void Slideshow::createVVTransition ( CXmlSlideshow::vid_struct *pStart, CXmlSlid
 {
   if ( ! pStart || ! pStop )
     return;
+
+  log ( "Creating VV Transition\n", Logger::logInformation );
+
   // Here we have to 
   // o Get the last frame of the Video
   // o turn up the background music during the duration of the transition
@@ -528,8 +615,11 @@ void Slideshow::createVVTransition ( CXmlSlideshow::vid_struct *pStart, CXmlSlid
   QPainter thePainter;
   QString  qsFileName = getHashName ( pStart->src );
   QImage   canvas   ( m_size, QImage::Format_ARGB32_Premultiplied );
-  QImage   imgStart = pEncoder->getVidFrame ( -1.0 );
+  QImage   imgStart;
 //  // In case something went bad ...
+  if ( pEncoder->initVid ( qsFileName ) )
+       imgStart = pEncoder->getVidFrame ( -1.0 );
+
   if ( imgStart.isNull ( ) )
        imgStart = QImage ( qsFileName );
 
@@ -539,10 +629,10 @@ void Slideshow::createVVTransition ( CXmlSlideshow::vid_struct *pStart, CXmlSlid
   if ( imgStart.isNull ( ) )
     return;
 
-//  imgStart = QImage ( "/tmp/z.jpg" );
   if ( pStart->pMatrix )
        imgStart = imgStart.transformed ( *pStart->pMatrix, Qt::SmoothTransformation );
-  imgStart = imgStart.scaled ( m_size, Qt::KeepAspectRatio );
+  prescaleImage ( imgStart );
+  imgStart = imgStart.scaled ( m_size, Qt::KeepAspectRatio, Qt::SmoothTransformation );
 
   int iX, iY;
   iX = (int)( ( m_size.width  ( ) - imgStart.width  ( ) ) / 2.0 );
@@ -557,13 +647,14 @@ void Slideshow::createVVTransition ( CXmlSlideshow::vid_struct *pStart, CXmlSlid
   qsFileName = getHashName ( pStop->src );
   QImage imgStop;
   if ( pEncoder->initVid   ( qsFileName ) )
-     imgStop = pEncoder->getVidFrame( 0.0 );
+       imgStop = pEncoder->getVidFrame ( 0.0 );
   if ( imgStop.isNull ( ) )
-     imgStop = imgStart;
+       imgStop = imgStart;
 
   // We need a image with transparency ...
   imgStop = imgStop.convertToFormat ( QImage::Format_ARGB32 );//_Premultiplied );
-  imgStop = imgStop.scaled ( m_size, Qt::KeepAspectRatio );
+  prescaleImage ( imgStop );
+  imgStop = imgStop.scaled ( m_size, Qt::KeepAspectRatio, Qt::SmoothTransformation );
 
   iX = (int)( ( m_size.width  ( ) - imgStop.width  ( ) ) / 2.0 );
   iY = (int)( ( m_size.height ( ) - imgStop.height ( ) ) / 2.0 );
