@@ -63,3 +63,26 @@ bool Utils::recMkdir ( const QString &qsDir )
   return true;
 }
 
+void Utils::writeZoneCode ( unsigned char iZoneCode, QString qsDVDPath )
+{
+  const char arrayCodes[] = { 0x00, 0xFE, 0xFD, 0xFB, 0xF7, 0xEF, 0xDF };
+  // * Region 1 - The U.S., U.S. territories and Canada
+  // * Region 2 - Europe, Japan, the Middle East, Egypt, South Africa, Greenland
+  // * Region 3 - Taiwan, Korea, the Philippines, Indonesia, Hong Kong
+  // * Region 4 - Mexico, South America, Central America, Australia, New Zealand, Pacific Islands, Caribbean
+  // * Region 5 - Russia (okay, former Russia), Eastern Europe, India, most of Africa, North Korea, Mongolia
+  // * Region 6 - China 
+  if ( ( iZoneCode > 6 ) || ( qsDVDPath.isEmpty ( ) ) )
+    return;
+
+  QFile ifo ( qsDVDPath + "/VIDEO_TS/VIDEO_TS.IFO" );
+  for ( int t=0; t<2; t++ )  {
+    if  ( ifo.open ( QIODevice::ReadWrite ) == true )  {
+      ifo.seek     ( 0x23 );
+      ifo.putChar  ( arrayCodes[iZoneCode] );
+    }
+    ifo.close      ( );
+    ifo.setFileName( qsDVDPath + "/VIDEO_TS/VIDEO_TS.BUP" );
+  }
+}
+
